use std::collections::HashMap;
use super::LocalisedText;

lazy_static::lazy_static! {
    pub static ref AUTO_TRANSLATE: HashMap<(u32, u32), LocalisedText> = maplit::hashmap! {
        (1, 100) => LocalisedText {
            en: "【Languages】",
            ja: "【言葉】",
            de: "【Sprachen】",
            fr: "【Langues】",
        },
        (1, 101) => LocalisedText {
            en: "Please use the auto-translate function.",
            ja: "定型文辞書を使ってください。",
            de: "Bitte benutz die Auto-Übersetzung.",
            fr: "Utilisez la traduction automatique.",
        },
        (1, 102) => LocalisedText {
            en: "Japanese language",
            ja: "日本語",
            de: "Japanisch",
            fr: "Japonais",
        },
        (1, 103) => LocalisedText {
            en: "English language",
            ja: "英語",
            de: "Englisch",
            fr: "Anglais",
        },
        (1, 104) => LocalisedText {
            en: "French language",
            ja: "フランス語",
            de: "Französisch",
            fr: "Français",
        },
        (1, 105) => LocalisedText {
            en: "German language",
            ja: "ドイツ語",
            de: "Deutsch",
            fr: "Allemand",
        },
        (1, 106) => LocalisedText {
            en: "Can you speak Japanese?",
            ja: "日本語は話せますか？",
            de: "Sprichst du Japanisch?",
            fr: "Parlez-vous japonais ?",
        },
        (1, 107) => LocalisedText {
            en: "Can you speak English?",
            ja: "英語は話せますか？",
            de: "Sprichst du Englisch?",
            fr: "Parlez-vous anglais ?",
        },
        (1, 108) => LocalisedText {
            en: "Can you speak French?",
            ja: "フランス語は話せますか？",
            de: "Sprichst du Französisch?",
            fr: "Parlez-vous français ?",
        },
        (1, 109) => LocalisedText {
            en: "Can you speak German?",
            ja: "ドイツ語は話せますか？",
            de: "Sprichst du Deutsch?",
            fr: "Parlez-vous allemand ?",
        },
        (1, 110) => LocalisedText {
            en: "I don't speak any English.",
            ja: "私は英語が話せません。",
            de: "Ich spreche kein Englisch.",
            fr: "Je ne parle pas anglais.",
        },
        (1, 111) => LocalisedText {
            en: "I don't speak any Japanese.",
            ja: "私は日本語が話せません。",
            de: "Ich spreche kein Japanisch.",
            fr: "Je ne parle pas japonais.",
        },
        (1, 112) => LocalisedText {
            en: "I don't speak any French.",
            ja: "私はフランス語が話せません。",
            de: "Ich spreche kein Französisch.",
            fr: "Je ne parle pas français.",
        },
        (1, 113) => LocalisedText {
            en: "I don't speak any German.",
            ja: "私はドイツ語が話せません。",
            de: "Ich spreche kein Deutsch.",
            fr: "Je ne parle pas allemand.",
        },
        (1, 114) => LocalisedText {
            en: "Please listen.",
            ja: "聞いてください。",
            de: "Hör mir bitte zu.",
            fr: "Veuillez écouter.",
        },
        (1, 115) => LocalisedText {
            en: "Can you hear me?",
            ja: "聞こえていますか？",
            de: "Kannst du mich hören?",
            fr: "M'entendez-vous ?",
        },
        (1, 116) => LocalisedText {
            en: "I can speak a little.",
            ja: "少し話せます。",
            de: "Ich spreche es ein bisschen.",
            fr: "Je parle un peu.",
        },
        (1, 117) => LocalisedText {
            en: "I can understand a little.",
            ja: "少しなら理解できます。",
            de: "Ich verstehe es ein bisschen.",
            fr: "Je comprends un peu.",
        },
        (1, 118) => LocalisedText {
            en: "Please use simple words.",
            ja: "簡単な単語で話してください。",
            de: "Benutz bitte einfache Wörter.",
            fr: "Utilisez des mots simples.",
        },
        (1, 119) => LocalisedText {
            en: "Please do not abbreviate your words.",
            ja: "省略形ではなくフルスペルでお願いします。",
            de: "Bitte benutz keine Abkürzungen.",
            fr: "Évitez les abréviations.",
        },
        (1, 120) => LocalisedText {
            en: "I need some time to put together my answer.",
            ja: "返事するのでちょっと待ってください。",
            de: "Ich brauche ein wenig Zeit, um zu antworten.",
            fr: "J'ai besoin de temps pour répondre.",
        },
        (1, 121) => LocalisedText {
            en: "What is your native language?",
            ja: "母国語は何ですか？",
            de: "Was ist deine Muttersprache?",
            fr: "Quelle est votre langue maternelle ?",
        },
        (2, 200) => LocalisedText {
            en: "【Greetings】",
            ja: "【あいさつ】",
            de: "【Begrüßung】",
            fr: "【Salutations】",
        },
        (2, 201) => LocalisedText {
            en: "Nice to meet you.",
            ja: "はじめまして。",
            de: "Sehr erfreut.",
            fr: "Enchanté(e).",
        },
        (2, 202) => LocalisedText {
            en: "Good morning!",
            ja: "おはよう。",
            de: "Guten Morgen!",
            fr: "Bonjour !",
        },
        (2, 203) => LocalisedText {
            en: "Hello!",
            ja: "こんにちは。",
            de: "Hallo!",
            fr: "Salut !",
        },
        (2, 204) => LocalisedText {
            en: "Good evening!",
            ja: "こんばんは。",
            de: "Guten Abend!",
            fr: "Bonsoir !",
        },
        (2, 205) => LocalisedText {
            en: "Good night!",
            ja: "おやすみなさい。",
            de: "Gute Nacht!",
            fr: "Bonne nuit !",
        },
        (2, 206) => LocalisedText {
            en: "Good-bye.",
            ja: "さようなら。",
            de: "Tschüs.",
            fr: "Au revoir !",
        },
        (2, 207) => LocalisedText {
            en: "I had fun today!",
            ja: "今日は楽しかったです。",
            de: "Ich hatte heute viel Spaß!",
            fr: "Je me suis bien amusé(e).",
        },
        (2, 208) => LocalisedText {
            en: "See you again!",
            ja: "また会いましょう！",
            de: "Bis bald!",
            fr: "À la prochaine !",
        },
        (2, 209) => LocalisedText {
            en: "Let's play together again sometime!",
            ja: "また遊びましょう。",
            de: "Lass uns mal wieder gemeinsam spielen!",
            fr: "Rejouons ensemble une prochaine fois.",
        },
        (2, 210) => LocalisedText {
            en: "I'm back!",
            ja: "ただいま。",
            de: "Ich bin zurück.",
            fr: "Je suis de retour.",
        },
        (2, 211) => LocalisedText {
            en: "Welcome back.",
            ja: "おかえり。",
            de: "Willkommen zurück.",
            fr: "Ça fait plaisir de vous revoir.",
        },
        (2, 212) => LocalisedText {
            en: "Congratulations!",
            ja: "おめでとう！",
            de: "Gratulation!",
            fr: "Félicitations !",
        },
        (2, 213) => LocalisedText {
            en: "Good job!",
            ja: "よくやった！",
            de: "Gut gemacht!",
            fr: "Bien joué !",
        },
        (2, 214) => LocalisedText {
            en: "Good luck!",
            ja: "がんばって！",
            de: "Viel Glück!",
            fr: "Bon courage !",
        },
        (2, 215) => LocalisedText {
            en: "All right!",
            ja: "やったー！",
            de: "Super!",
            fr: "Super !",
        },
        (2, 216) => LocalisedText {
            en: "Thank you.",
            ja: "ありがとう。",
            de: "Danke.",
            fr: "Merci.",
        },
        (2, 217) => LocalisedText {
            en: "You're welcome.",
            ja: "どういたしまして。",
            de: "Gern geschehen.",
            fr: "De rien.",
        },
        (2, 218) => LocalisedText {
            en: "Take care.",
            ja: "気を付けてください。",
            de: "Mach's gut!",
            fr: "Prenez soin de vous.",
        },
        (2, 219) => LocalisedText {
            en: "I'm sorry.",
            ja: "ごめんなさい。",
            de: "Es tut mir leid.",
            fr: "Je suis désolé(e).",
        },
        (2, 220) => LocalisedText {
            en: "Please forgive me.",
            ja: "許してください。",
            de: "Verzeihung.",
            fr: "Pardonnez-moi.",
        },
        (2, 221) => LocalisedText {
            en: "That's too bad.",
            ja: "残念です。",
            de: "Schade.",
            fr: "C'est dommage.",
        },
        (2, 222) => LocalisedText {
            en: "Excuse me...",
            ja: "ちょっといいですか？",
            de: "Entschuldigung ...",
            fr: "Excusez-moi, vous avez un instant ?",
        },
        (2, 223) => LocalisedText {
            en: "Have a safe journey.",
            ja: "それではよい旅を。",
            de: "Gute Reise!",
            fr: "Bonne continuation !",
        },
        (2, 224) => LocalisedText {
            en: "This is my first time here.",
            ja: "ここに来るのは初めてです。",
            de: "Ich bin zum ersten Mal hier.",
            fr: "C'est la première fois que je viens ici.",
        },
        (2, 225) => LocalisedText {
            en: "I'm looking forward to it!",
            ja: "楽しみです！",
            de: "Ich freu mich schon darauf!",
            fr: "J'ai hâte !",
        },
        (2, 226) => LocalisedText {
            en: "Good game!",
            ja: "お疲れ様でした。",
            de: "Das war klasse!",
            fr: "C'était génial !",
        },
        (2, 227) => LocalisedText {
            en: "Let's do it!",
            ja: "よろしくお願いします！",
            de: "Auf geht's!",
            fr: "Faisons de notre mieux !",
        },
        (2, 228) => LocalisedText {
            en: "Thank you. I must now take my leave.",
            ja: "ありがとうございました。これで失礼します。",
            de: "Danke und vielleicht bis bald!",
            fr: "Merci pour tout, et à bientôt peut-être.",
        },
        (2, 229) => LocalisedText {
            en: "Let's have some fun!",
            ja: "気楽にやりましょう！",
            de: "Lass es uns entspannt angehen!",
            fr: "Allons nous amuser !",
        },
        (2, 230) => LocalisedText {
            en: "I'll do my best!",
            ja: "がんばります！",
            de: "Ich werd mein Bestes geben!",
            fr: "Je ferai de mon mieux !",
        },
        (2, 231) => LocalisedText {
            en: "Let's give it our best shot!",
            ja: "一緒にがんばりましょう！",
            de: "Lasst uns alle unser Bestes geben!",
            fr: "On réussira ensemble !",
        },
        (2, 232) => LocalisedText {
            en: "This should be fun!",
            ja: "楽しみましょう！",
            de: "Hauptsache, es macht Spaß!",
            fr: "On va bien s'amuser !",
        },
        (2, 233) => LocalisedText {
            en: "If we stay cool, everything will be fine!",
            ja: "落ち着いていきましょう！",
            de: "Kühlen Kopf behalten und nicht aufgeben!",
            fr: "Allons-y tranquillement, ça va aller !",
        },
        (3, 300) => LocalisedText {
            en: "【Questions】",
            ja: "【質問】",
            de: "【Fragen】",
            fr: "【Questions】",
        },
        (3, 301) => LocalisedText {
            en: "Who?",
            ja: "誰ですか？",
            de: "Wer?",
            fr: "Qui ?",
        },
        (3, 302) => LocalisedText {
            en: "Which?",
            ja: "どちらですか？",
            de: "Welche(r/s)?",
            fr: "Lequel ?",
        },
        (3, 303) => LocalisedText {
            en: "How?",
            ja: "どうすればいいですか？",
            de: "Wie?",
            fr: "Comment faire ?",
        },
        (3, 304) => LocalisedText {
            en: "What?",
            ja: "何ですか？",
            de: "Was?",
            fr: "Qu'est-ce que c'est ?",
        },
        (3, 305) => LocalisedText {
            en: "When?",
            ja: "いつですか？",
            de: "Wann?",
            fr: "Quand ?",
        },
        (3, 306) => LocalisedText {
            en: "How many?",
            ja: "いくつですか？",
            de: "Wie viele?",
            fr: "Combien ?",
        },
        (3, 307) => LocalisedText {
            en: "Where?",
            ja: "どこですか？",
            de: "Wo?",
            fr: "Où ?",
        },
        (3, 308) => LocalisedText {
            en: "Why?",
            ja: "なぜですか？",
            de: "Warum?",
            fr: "Pourquoi ?",
        },
        (3, 309) => LocalisedText {
            en: "How come?",
            ja: "どうしてですか？",
            de: "Wieso?",
            fr: "Pour quelle raison ?",
        },
        (3, 310) => LocalisedText {
            en: "Where shall we go?",
            ja: "どこに行きますか？",
            de: "Wohin gehst du?",
            fr: "Où allez-vous ?",
        },
        (3, 311) => LocalisedText {
            en: "Can you do it?",
            ja: "できますか？",
            de: "Ist dir das möglich?",
            fr: "Pouvez-vous le faire ?",
        },
        (3, 312) => LocalisedText {
            en: "Do you need any help?",
            ja: "助けはいりますか？",
            de: "Brauchst du Hilfe?",
            fr: "Avez-vous besoin d'aide ?",
        },
        (3, 313) => LocalisedText {
            en: "Which levequest shall we do?",
            ja: "どのコンテンツをやりますか？",
            de: "Welchen Gildenfreibrief nehmen wir?",
            fr: "Quelle mission voulez-vous faire ?",
        },
        (3, 314) => LocalisedText {
            en: "Which duties can you do?",
            ja: "どのコンテンツならできますか？",
            de: "Welche Inhalte kannst du schon machen?",
            fr: "Quelles missions pouvez-vous faire ?",
        },
        (3, 315) => LocalisedText {
            en: "Do you have it?",
            ja: "持っていますか？",
            de: "Hast du's?",
            fr: "L'avez-vous ?",
        },
        (3, 316) => LocalisedText {
            en: "What weapons can you use?",
            ja: "今どんな武器を持っていますか？",
            de: "Was für Waffen hast du?",
            fr: "Quelles armes possédez-vous ?",
        },
        (3, 317) => LocalisedText {
            en: "What other classes can you use?",
            ja: "他にどのクラスができますか？",
            de: "Zu welchen Charakterklassen kannst du wechseln?",
            fr: "Quelles autres classes pouvez-vous jouer ?",
        },
        (3, 318) => LocalisedText {
            en: "What other roles can you cover?",
            ja: "他にどのロールができますか？",
            de: "Welche Rollen kannst du sonst noch übernehmen?",
            fr: "Quels autres rôles pouvez-vous jouer ?",
        },
        (3, 319) => LocalisedText {
            en: "Do you have it set?",
            ja: "セットしていますか？",
            de: "Hast du es angelegt?",
            fr: "Avez-vous enregistré toutes vos actions ?",
        },
        (3, 320) => LocalisedText {
            en: "What's the battle plan?",
            ja: "どんな戦術でいきますか？",
            de: "Wie lautet der Schlachtplan?",
            fr: "Quel est le plan de bataille ?",
        },
        (3, 321) => LocalisedText {
            en: "Can I add you to my friend list?",
            ja: "フレンドになってくれませんか？",
            de: "Kann ich dich auf meine Freundesliste setzen?",
            fr: "Puis-je t'ajouter à ma liste d'amis ?",
        },
        (3, 322) => LocalisedText {
            en: "Shall we take a break?",
            ja: "休憩しませんか？",
            de: "Sollen wir eine Pause einlegen?",
            fr: "Pouvons-nous faire une pause ?",
        },
        (3, 323) => LocalisedText {
            en: "Do you want me to repair it?",
            ja: "修理しましょうか？",
            de: "Soll ich es reparieren?",
            fr: "Voulez-vous que je répare ?",
        },
        (3, 324) => LocalisedText {
            en: "Can you repair it for me?",
            ja: "修理してくれませんか？",
            de: "Kannst du es für mich reparieren?",
            fr: "Pouvez-vous réparer quelque chose pour moi ?",
        },
        (3, 325) => LocalisedText {
            en: "Do you want me to meld materia to it?",
            ja: "マテリア装着しましょうか？",
            de: "Soll ich für dich Materia einsetzen?",
            fr: "Voulez-vous que je sertisse une matéria ?",
        },
        (3, 326) => LocalisedText {
            en: "Can you meld materia for me?",
            ja: "マテリア装着してくれませんか？",
            de: "Kannst du für mich Materia einsetzen?",
            fr: "Pouvez-vous sertir une matéria pour moi ?",
        },
        (3, 327) => LocalisedText {
            en: "Would you join my party?",
            ja: "パーティに入りませんか？",
            de: "Willst du in meine Gruppe?",
            fr: "Voulez-vous rejoindre mon équipe ?",
        },
        (3, 328) => LocalisedText {
            en: "Can I join your party?",
            ja: "パーティに入れてくれませんか？",
            de: "Kann ich in deine Gruppe eintreten?",
            fr: "Pourriez-vous m'ajouter dans votre équipe ?",
        },
        (3, 329) => LocalisedText {
            en: "Should we disband?",
            ja: "パーティを解散しますか？",
            de: "Sollen wir die Gruppe auflösen?",
            fr: "Est-ce qu'on dissout l'équipe ?",
        },
        (3, 330) => LocalisedText {
            en: "Do you want to join my linkshell?",
            ja: "リンクシェルに入りませんか？",
            de: "Möchtest du meinem Kontaktkreis beitreten?",
            fr: "Voulez-vous rejoindre ma linkshell ?",
        },
        (3, 331) => LocalisedText {
            en: "Can I join your linkshell?",
            ja: "リンクシェルに入れてくれませんか？",
            de: "Kannst du mich in deinen Kontaktkreis aufnehmen?",
            fr: "Puis-je rejoindre votre linkshell ?",
        },
        (3, 332) => LocalisedText {
            en: "Do you want to join my free company?",
            ja: "フリーカンパニーに入りませんか？",
            de: "Möchtest du meiner Freien Gesellschaft beitreten?",
            fr: "Voulez-vous rejoindre ma compagnie libre ?",
        },
        (3, 333) => LocalisedText {
            en: "Can I join your free company?",
            ja: "フリーカンパニーに入れてくれませんか？",
            de: "Darf ich deiner Freien Gesellschaft beitreten?",
            fr: "Puis-je rejoindre votre compagnie libre ?",
        },
        (3, 334) => LocalisedText {
            en: "What are you doing?",
            ja: "何をしていますか？",
            de: "Was machst du?",
            fr: "Que faites-vous ?",
        },
        (3, 335) => LocalisedText {
            en: "How about a game of Triple Triad?",
            ja: "カード対戦しませんか？",
            de: "Wie wär's mit einer Partie Triple Triad?",
            fr: "Voulez-vous jouer à Triple Triade ?",
        },
        (3, 336) => LocalisedText {
            en: "Can we change the rules?",
            ja: "ルールを変えませんか？",
            de: "Können wir die Regeln ändern?",
            fr: "Peut-on changer les règles ?",
        },
        (3, 337) => LocalisedText {
            en: "How about joining a Triple Triad tournament?",
            ja: "大会の対戦をしませんか？",
            de: "Sollen wir an einem Triple Triad-Turnier teilnehmen?",
            fr: "Voulez-vous faire un match de tournoi ?",
        },
        (3, 338) => LocalisedText {
            en: "Do you have enough MP?",
            ja: "MPは大丈夫ですか？",
            de: "Hast du genug MP?",
            fr: "Avez-vous assez de PM ?",
        },
        (3, 339) => LocalisedText {
            en: "Is there anything you don't understand?",
            ja: "わからないことはありますか？",
            de: "Hast du irgendwelche Fragen?",
            fr: "Y a-t-il quelque chose que vous voulez savoir ?",
        },
        (3, 340) => LocalisedText {
            en: "Did you see it?",
            ja: "見ましたか？",
            de: "Hast du es gesehen?",
            fr: "Vous l'avez vu(e) ?",
        },
        (3, 341) => LocalisedText {
            en: "How should we handle this?",
            ja: "攻略方法はわかりますか？",
            de: "Welche Strategie benutzen wir?",
            fr: "Quelle est la stratégie ?",
        },
        (3, 342) => LocalisedText {
            en: "What's our strategy?",
            ja: "作戦を立てませんか？",
            de: "Lasst uns erst eine Strategie zurechtlegen!",
            fr: "Que penseriez-vous de mettre au point une stratégie ?",
        },
        (3, 343) => LocalisedText {
            en: "Do you want to ride together?",
            ja: "相乗りしませんか？",
            de: "Sollen wir zusammen reiten?",
            fr: "Vous pourriez monter avec moi, qu'en dites-vous ?",
        },
        (3, 344) => LocalisedText {
            en: "Can I ride with you?",
            ja: "相乗りさせてくれませんか？",
            de: "Kann ich mit dir reiten?",
            fr: "Pourrais-je monter avec vous ?",
        },
        (4, 400) => LocalisedText {
            en: "【Answers】",
            ja: "【答え】",
            de: "【Antworten】",
            fr: "【Réponses】",
        },
        (4, 401) => LocalisedText {
            en: "I don't understand.",
            ja: "わかりません。",
            de: "Ich verstehe nicht.",
            fr: "Je ne comprends pas.",
        },
        (4, 402) => LocalisedText {
            en: "No thanks.",
            ja: "いりません。",
            de: "Nein, danke.",
            fr: "Non, merci.",
        },
        (4, 403) => LocalisedText {
            en: "Yes, please.",
            ja: "はい。お願いします。",
            de: "Ja, bitte.",
            fr: "Oui, s'il vous plaît.",
        },
        (4, 404) => LocalisedText {
            en: "If you would be so kind.",
            ja: "お願いします。",
            de: "Ich bitte darum.",
            fr: "Je vous en prie, s'il vous plaît.",
        },
        (4, 405) => LocalisedText {
            en: "Understood.",
            ja: "わかりました。",
            de: "Verstanden.",
            fr: "Compris.",
        },
        (4, 406) => LocalisedText {
            en: "I'm sorry. I'm busy now.",
            ja: "今忙しいので、後にしてください。",
            de: "Es tut mir leid. Ich bin gerade beschäftigt.",
            fr: "Je regrette. Je suis occupé(e) pour l'instant.",
        },
        (4, 407) => LocalisedText {
            en: "I'm playing solo right now.",
            ja: "今は独りで行動したいんです。",
            de: "Ich möchte im Moment lieber allein spielen.",
            fr: "Je veux jouer en solo pour l'instant.",
        },
        (4, 408) => LocalisedText {
            en: "I don't know how to answer that question.",
            ja: "答えたいけど表現がわかりません。",
            de: "Ich weiß nicht, wie ich diese Frage beantworten soll.",
            fr: "Je ne sais pas répondre à cette question.",
        },
        (4, 409) => LocalisedText {
            en: "I see.",
            ja: "なるほど。",
            de: "Ach so.",
            fr: "Je vois.",
        },
        (4, 410) => LocalisedText {
            en: "Thanks for the offer, but I'll have to pass.",
            ja: "せっかくだけど遠慮します。",
            de: "Danke für das Angebot, aber ich muss ablehnen.",
            fr: "Merci de l'offre, mais je dois refuser.",
        },
        (4, 411) => LocalisedText {
            en: "That's interesting.",
            ja: "へぇー。",
            de: "Interessant.",
            fr: "C'est intéressant.",
        },
        (4, 412) => LocalisedText {
            en: "Um...",
            ja: "えーっと…",
            de: "Ähmmm ...",
            fr: "Euh...",
        },
        (4, 413) => LocalisedText {
            en: "Huh!?",
            ja: "えっ！？",
            de: "Häh!?",
            fr: "Hein !?",
        },
        (4, 414) => LocalisedText {
            en: "Really?",
            ja: "本当に？",
            de: "Wirklich?",
            fr: "Vraiment ?",
        },
        (4, 415) => LocalisedText {
            en: "Hmmm.",
            ja: "むむむ。",
            de: "Hmmm.",
            fr: "Hmmm.",
        },
        (4, 416) => LocalisedText {
            en: "I have to go soon.",
            ja: "もうすぐ別の予定があるんです。",
            de: "Ich muss bald weg.",
            fr: "J'ai quelque chose à faire bientôt.",
        },
        (4, 417) => LocalisedText {
            en: "I'd like another match.",
            ja: "再戦を希望します。",
            de: "Ich möchte noch eine Runde spielen.",
            fr: "J'aimerais refaire une partie.",
        },
        (4, 418) => LocalisedText {
            en: "Let's call it a day.",
            ja: "対戦を終わりにしましょう。",
            de: "Lass uns für heute Schluss machen.",
            fr: "Arrêtons-nous là.",
        },
        (4, 419) => LocalisedText {
            en: "I'd like to take a break.",
            ja: "一時中断させてください。",
            de: "Ich möchte eine Pause einlegen.",
            fr: "J'aimerais faire une pause.",
        },
        (4, 420) => LocalisedText {
            en: "Probably.",
            ja: "そうかもしれません。",
            de: "Kann sein.",
            fr: "Peut-être.",
        },
        (5, 500) => LocalisedText {
            en: "【Reasons】",
            ja: "【理由】",
            de: "【Gründe】",
            fr: "【Raisons】",
        },
        (5, 501) => LocalisedText {
            en: "Casting spell.",
            ja: "詠唱中です。",
            de: "Ich bin dabei, Magie anzuwenden.",
            fr: "Je lance un sort.",
        },
        (5, 502) => LocalisedText {
            en: "Time for work!",
            ja: "仕事の時間です。",
            de: "Zeit zu arbeiten!",
            fr: "C'est l'heure d'aller au travail.",
        },
        (5, 503) => LocalisedText {
            en: "I have plans.",
            ja: "約束があるんです。",
            de: "Ich habe bereits Pläne.",
            fr: "J'ai déjà quelque chose de prévu.",
        },
        (5, 504) => LocalisedText {
            en: "I'm sleepy.",
            ja: "眠くなりました。",
            de: "Ich bin schläfrig.",
            fr: "J'ai sommeil.",
        },
        (5, 505) => LocalisedText {
            en: "I'm tired.",
            ja: "疲れました。",
            de: "Ich bin müde.",
            fr: "Je suis fatigué(e).",
        },
        (5, 506) => LocalisedText {
            en: "Have stuff to do, gotta go!",
            ja: "用事があるので、これで。",
            de: "Ich hab was zu tun, muss gehen.",
            fr: "Désolé(e), mais je dois y aller.",
        },
        (5, 507) => LocalisedText {
            en: "I don't feel well.",
            ja: "体調が悪いんです。",
            de: "Ich fühle mich nicht gut.",
            fr: "Je ne me sens pas bien.",
        },
        (5, 508) => LocalisedText {
            en: "I'm not up for it.",
            ja: "気が向きません。",
            de: "Dafür bin ich nicht zu haben.",
            fr: "Je n'ai pas envie.",
        },
        (5, 509) => LocalisedText {
            en: "I'm interested.",
            ja: "興味があります。",
            de: "Ich bin interessiert.",
            fr: "Ça m'intéresse.",
        },
        (5, 510) => LocalisedText {
            en: "I'm bound by duty.",
            ja: "コンテンツ中なんです。",
            de: "Ich bin gerade in einem Spielinhalt beschäftigt.",
            fr: "Je suis en pleine mission.",
        },
        (5, 511) => LocalisedText {
            en: "Fighting right now!",
            ja: "戦闘中なんです。",
            de: "Kämpfe gerade!",
            fr: "Je suis en plein combat.",
        },
        (5, 512) => LocalisedText {
            en: "I want to make money.",
            ja: "お金を稼ぎたいです。",
            de: "Ich will Geld verdienen.",
            fr: "Je veux gagner de l'argent.",
        },
        (5, 513) => LocalisedText {
            en: "I don't remember.",
            ja: "覚えてないんです。",
            de: "Ich erinnere mich nicht.",
            fr: "Je ne me souviens pas.",
        },
        (5, 514) => LocalisedText {
            en: "I don't know.",
            ja: "知らないんです。",
            de: "Ich weiß nicht.",
            fr: "Je ne sais pas.",
        },
        (5, 515) => LocalisedText {
            en: "Just used it.",
            ja: "使ったばかりです。",
            de: "Habe es gerade benutzt.",
            fr: "Je viens de l'utiliser.",
        },
        (5, 516) => LocalisedText {
            en: "I want experience points.",
            ja: "経験値が欲しいです。",
            de: "Ich will Routine.",
            fr: "Je veux des points d'expérience.",
        },
        (5, 517) => LocalisedText {
            en: "Oops!",
            ja: "間違えました。",
            de: "Ups! Mein Fehler.",
            fr: "Oups ! J'ai fait une erreur.",
        },
        (5, 518) => LocalisedText {
            en: "I already have an invite.",
            ja: "すでに誘われています。",
            de: "Ich wurde schon eingeladen.",
            fr: "On m'a déjà invité(e).",
        },
        (5, 519) => LocalisedText {
            en: "I already have one.",
            ja: "すでに持っています。",
            de: "Das hab ich schon.",
            fr: "J'en ai déjà un(e).",
        },
        (5, 520) => LocalisedText {
            en: "No longer recruiting.",
            ja: "募集が終わりました。",
            de: "Die Rekrutierung wurde beendet.",
            fr: "Le recrutement est terminé.",
        },
        (5, 521) => LocalisedText {
            en: "I'm getting ready now.",
            ja: "準備をしています。",
            de: "Ich mache mich gerade bereit.",
            fr: "Je suis en train de me préparer.",
        },
        (5, 522) => LocalisedText {
            en: "My hands are full.",
            ja: "手が離せません。",
            de: "Ich kann hier gerade nicht weg.",
            fr: "Je suis occupé avec autre chose.",
        },
        (5, 523) => LocalisedText {
            en: "We're chewing the fat.",
            ja: "雑談をしています。",
            de: "Wir unterhalten uns.",
            fr: "Je bavarde.",
        },
        (5, 524) => LocalisedText {
            en: "I seem to have misplaced my keyboard.",
            ja: "キーボードを使っていません。",
            de: "Ich habe keine Tastatur.",
            fr: "Je n'utilise pas de clavier.",
        },
        (5, 525) => LocalisedText {
            en: "I'm checking things out.",
            ja: "探索中です。",
            de: "Ich seh mich hier um.",
            fr: "Je vérifie quelque chose.",
        },
        (6, 600) => LocalisedText {
            en: "【Trade】",
            ja: "【トレード】",
            de: "【Handel】",
            fr: "【Échange】",
        },
        (6, 601) => LocalisedText {
            en: "Can I have it?",
            ja: "くれませんか？",
            de: "Kann ich es haben?",
            fr: "Puis-je l'avoir ?",
        },
        (6, 602) => LocalisedText {
            en: "Can you do it for me?",
            ja: "してくれませんか？",
            de: "Kannst du das für mich erledigen?",
            fr: "Pouvez-vous le faire pour moi ?",
        },
        (6, 603) => LocalisedText {
            en: "Lower the price?",
            ja: "安くしてくれませんか？",
            de: "Kannst du mir einen besseren Preis machen?",
            fr: "Pourriez-vous baisser le prix ?",
        },
        (6, 604) => LocalisedText {
            en: "Buy?",
            ja: "買ってくれませんか？",
            de: "Kaufen?",
            fr: "Pourriez-vous me l'acheter ?",
        },
        (6, 605) => LocalisedText {
            en: "Sell?",
            ja: "売ってくれませんか？",
            de: "Verkaufen?",
            fr: "Pourriez-vous me le vendre ?",
        },
        (6, 606) => LocalisedText {
            en: "Trade?",
            ja: "トレードしませんか？",
            de: "Handeln?",
            fr: "Pourriez-vous faire un échange ?",
        },
        (6, 607) => LocalisedText {
            en: "Do you need it?",
            ja: "いりませんか？",
            de: "Brauchst du es?",
            fr: "En avez-vous besoin ?",
        },
        (6, 608) => LocalisedText {
            en: "Can you make it?",
            ja: "製作できませんか？",
            de: "Kannst du es herstellen?",
            fr: "Pouvez-vous fabriquer ?",
        },
        (6, 609) => LocalisedText {
            en: "Do you have it?",
            ja: "持っていませんか？",
            de: "Besitzt du es?",
            fr: "En avez-vous ?",
        },
        (6, 610) => LocalisedText {
            en: "What materials are needed?",
            ja: "必要な素材は何ですか？",
            de: "Welche Materialien benötigst du?",
            fr: "Quels sont les matériaux ?",
        },
        (6, 611) => LocalisedText {
            en: "I don't have any money.",
            ja: "お金がありません。",
            de: "Ich habe kein Geld.",
            fr: "Je n'ai pas d'argent.",
        },
        (6, 612) => LocalisedText {
            en: "I don't have the item.",
            ja: "アイテムがありません。",
            de: "Ich habe keine Gegenstände.",
            fr: "Je n'ai pas cet objet.",
        },
        (6, 613) => LocalisedText {
            en: "I don't have anything to give you.",
            ja: "あなたにあげられる物はなさそうです。",
            de: "Ich habe nichts, was ich dir geben könnte.",
            fr: "Je n'ai rien à vous donner.",
        },
        (6, 614) => LocalisedText {
            en: "You can have this.",
            ja: "これをあなたにあげましょう。",
            de: "Du kannst es haben.",
            fr: "Je vous donne ceci.",
        },
        (6, 615) => LocalisedText {
            en: "Please.",
            ja: "ください。",
            de: "Bitte.",
            fr: "S'il vous plaît.",
        },
        (6, 616) => LocalisedText {
            en: "Reward:",
            ja: "報酬：",
            de: "Belohnung:",
            fr: "Récompense :",
        },
        (6, 617) => LocalisedText {
            en: "Price:",
            ja: "値段：",
            de: "Preis:",
            fr: "Prix :",
        },
        (6, 618) => LocalisedText {
            en: "delivery",
            ja: "お届け物",
            de: "Warensendung",
            fr: "Colis",
        },
        (7, 700) => LocalisedText {
            en: "【Organize】",
            ja: "【編成】",
            de: "【Organisierung】",
            fr: "【Organisation】",
        },
        (7, 701) => LocalisedText {
            en: "Looking for members.",
            ja: "募集中です。",
            de: "Suche Mitglieder.",
            fr: "Je cherche des équipiers.",
        },
        (7, 702) => LocalisedText {
            en: "Gather together.",
            ja: "集まってください。",
            de: "Lasst uns versammeln.",
            fr: "Rassemblez-vous.",
        },
        (7, 703) => LocalisedText {
            en: "Team up?",
            ja: "一緒にやりませんか？",
            de: "Sollen wir uns verbünden?",
            fr: "Voulez-vous faire équipe ?",
        },
        (7, 704) => LocalisedText {
            en: "Are you alone?",
            ja: "1人ですか？",
            de: "Bist du allein?",
            fr: "Êtes-vous seul(e) ?",
        },
        (7, 705) => LocalisedText {
            en: "Any vacancies?",
            ja: "空きはありますか？",
            de: "Noch Platz in der Gruppe?",
            fr: "Reste-t-il de la place ?",
        },
        (7, 706) => LocalisedText {
            en: "Please invite me.",
            ja: "誘ってください。",
            de: "Lade mich bitte ein.",
            fr: "Invitez-moi, s'il vous plaît.",
        },
        (7, 707) => LocalisedText {
            en: "Please let me join.",
            ja: "入れてください。",
            de: "Lass mich beitreten.",
            fr: "Laissez-moi rejoindre l'équipe.",
        },
        (7, 708) => LocalisedText {
            en: "Who is the leader?",
            ja: "リーダーは誰ですか？",
            de: "Wer ist der Anführer?",
            fr: "Qui est le chef d'équipe ?",
        },
        (7, 709) => LocalisedText {
            en: "Just for a short time is fine.",
            ja: "短時間でもいいんですが。",
            de: "Für kurze Zeit ist auch okay!",
            fr: "Même une courte durée me va.",
        },
        (7, 710) => LocalisedText {
            en: "Our party's full.",
            ja: "フルパーティなんです。",
            de: "Unsere Gruppe ist voll.",
            fr: "Notre équipe est complète.",
        },
        (7, 711) => LocalisedText {
            en: "Please assist.",
            ja: "手伝ってください。",
            de: "Bitte assistieren.",
            fr: "Aidez-moi, s'il vous plaît.",
        },
        (7, 712) => LocalisedText {
            en: "Disbanding party.",
            ja: "パーティを解散します。",
            de: "Gruppe auflösen.",
            fr: "Je dissous l'équipe.",
        },
        (7, 713) => LocalisedText {
            en: "Taking a break.",
            ja: "席を外します。",
            de: "Mache eine Pause.",
            fr: "Je fais une pause.",
        },
        (7, 714) => LocalisedText {
            en: "Looking for party.",
            ja: "パーティ参加を希望します。",
            de: "Ich möchte teilnehmen.",
            fr: "Je souhaite participer.",
        },
        (7, 715) => LocalisedText {
            en: "light party",
            ja: "ライトパーティ",
            de: "Leichter Trupp",
            fr: "Petite équipe",
        },
        (7, 716) => LocalisedText {
            en: "full party",
            ja: "フルパーティ",
            de: "Voller Trupp",
            fr: "Équipe complète",
        },
        (7, 717) => LocalisedText {
            en: "I'm inexperienced.",
            ja: "不慣れです。",
            de: "Ich bin unerfahren.",
            fr: "Je suis inexpérimenté(e).",
        },
        (7, 718) => LocalisedText {
            en: "Should we find some replacements?",
            ja: "パーティメンバーの補充をしませんか？",
            de: "Sollen wir Ersatz aufnehmen?",
            fr: "Que diriez-vous d'ajouter des membres à l'équipe ?",
        },
        (8, 800) => LocalisedText {
            en: "【Tactics】",
            ja: "【戦術】",
            de: "【Taktiken】",
            fr: "【Tactiques】",
        },
        (8, 801) => LocalisedText {
            en: "Please follow.",
            ja: "ついてきてください。",
            de: "Bitte folgen.",
            fr: "Suivez-moi, s'il vous plaît.",
        },
        (8, 802) => LocalisedText {
            en: "I'll follow you.",
            ja: "ついていきます。",
            de: "Ich folge dir.",
            fr: "Je vous suis.",
        },
        (8, 803) => LocalisedText {
            en: "Please check it.",
            ja: "調べてみてください。",
            de: "Bitte untersuche es.",
            fr: "Veuillez l'examiner.",
        },
        (8, 804) => LocalisedText {
            en: "Found it!",
            ja: "見つけました！",
            de: "Gefunden!",
            fr: "Je l'ai trouvé(e) !",
        },
        (8, 805) => LocalisedText {
            en: "Full attack!",
            ja: "全力で攻撃してください！",
            de: "Volle Attacke!",
            fr: "Attaque maximum !",
        },
        (8, 806) => LocalisedText {
            en: "Pull back.",
            ja: "少し下がって。",
            de: "Komm ein Stück zurück!",
            fr: "Reculez un peu.",
        },
        (8, 807) => LocalisedText {
            en: "Watch your aggro!",
            ja: "敵視に注意してください。",
            de: "Achte auf die Feindseligkeit!",
            fr: "Attention à l'inimitié.",
        },
        (8, 808) => LocalisedText {
            en: "Defeat this one first!",
            ja: "これを先にやっつけて！",
            de: "Diesen Gegner zuerst besiegen!",
            fr: "Éliminez d'abord celui-ci !",
        },
        (8, 809) => LocalisedText {
            en: "Please don't attack.",
            ja: "攻撃しないでください。",
            de: "Nicht angreifen!",
            fr: "N'attaquez pas.",
        },
        (8, 810) => LocalisedText {
            en: "Hold the target!",
            ja: "足止めしてください！",
            de: "Bitte festhalten!",
            fr: "Entravez la cible !",
        },
        (8, 811) => LocalisedText {
            en: "Please deactivate it.",
            ja: "無効化してください！",
            de: "Bitte deaktivieren!",
            fr: "Empêchez-le d'agir !",
        },
        (8, 812) => LocalisedText {
            en: "Heal!",
            ja: "回復してください！",
            de: "Bitte heilen!",
            fr: "Soignez !",
        },
        (8, 813) => LocalisedText {
            en: "Cast it!",
            ja: "掛けてください！",
            de: "Bitte ausführen!",
            fr: "Lancez le sort !",
        },
        (8, 814) => LocalisedText {
            en: "Run away!",
            ja: "逃げて！",
            de: "Lauf weg!",
            fr: "Fuyez !",
        },
        (8, 815) => LocalisedText {
            en: "Help me out!",
            ja: "助けて！",
            de: "Hilfe!",
            fr: "À l'aide !",
        },
        (8, 816) => LocalisedText {
            en: "Stop!",
            ja: "止まって！",
            de: "Stopp!",
            fr: "Stop !",
        },
        (8, 817) => LocalisedText {
            en: "Standing by.",
            ja: "待機します。",
            de: "Bin bereit.",
            fr: "Je me tiens prêt(e).",
        },
        (8, 818) => LocalisedText {
            en: "None left.",
            ja: "なくなりました。",
            de: "Nicht mehr vorhanden.",
            fr: "Il n'y en a plus.",
        },
        (8, 819) => LocalisedText {
            en: "Don't have it.",
            ja: "ありません。",
            de: "Negativ.",
            fr: "Il n'y en a pas.",
        },
        (8, 820) => LocalisedText {
            en: "Please use it sparingly.",
            ja: "節約してください。",
            de: "Bitte sparsam sein.",
            fr: "Ne gaspillez pas.",
        },
        (8, 821) => LocalisedText {
            en: "I'll use it sparingly.",
            ja: "節約します。",
            de: "Ich werde sparsam sein. ",
            fr: "J'économise.",
        },
        (8, 822) => LocalisedText {
            en: "My gear is in poor condition.",
            ja: "装備品の耐久度が下がってきました。",
            de: "Ausrüstung ist in schlechtem Zustand.",
            fr: "Mon équipement est usé.",
        },
        (8, 823) => LocalisedText {
            en: "Ready!",
            ja: "準備完了！",
            de: "Fertig!",
            fr: "Prêt !",
        },
        (8, 824) => LocalisedText {
            en: "Please set enemy marks.",
            ja: "マーカーを設定してください。",
            de: "Bitte markieren.",
            fr: "Placez des signes.",
        },
        (8, 825) => LocalisedText {
            en: "Please use it.",
            ja: "使ってください。",
            de: "Bitte benutzen.",
            fr: "Utilisez ça.",
        },
        (8, 826) => LocalisedText {
            en: "Let's rest for a while.",
            ja: "休憩しましょう。",
            de: "Lasst uns eine Pause machen.",
            fr: "Reposons-nous un instant.",
        },
        (8, 827) => LocalisedText {
            en: "Pulling enemy over.",
            ja: "敵を連れてきます。",
            de: "Ich bringe Gegner mit.",
            fr: "Je vais conduire l'ennemi ici.",
        },
        (8, 828) => LocalisedText {
            en: "Drawing enemy over.",
            ja: "敵を引きつけます。",
            de: "Ich locke Gegner an.",
            fr: "Je vais attirer l'ennemi.",
        },
        (8, 829) => LocalisedText {
            en: "Wait, please.",
            ja: "待ってください。",
            de: "Bitte warten.",
            fr: "Attendez, s'il vous plaît.",
        },
        (8, 830) => LocalisedText {
            en: "Let's meet there.",
            ja: "現地に集合しましょう。",
            de: "Wir treffen uns dort.",
            fr: "Retrouvons-nous sur place.",
        },
        (8, 831) => LocalisedText {
            en: "Let's move.",
            ja: "移動しましょう。",
            de: "Gehen wir.",
            fr: "Déplaçons-nous.",
        },
        (8, 832) => LocalisedText {
            en: "Stay there.",
            ja: "そこにいてください。",
            de: "Bleib dort.",
            fr: "Restez où vous êtes.",
        },
        (8, 833) => LocalisedText {
            en: "On my way.",
            ja: "これから向かいます。",
            de: "Bin unterwegs.",
            fr: "J'arrive.",
        },
        (8, 834) => LocalisedText {
            en: "Will be right back.",
            ja: "すぐ戻ります。",
            de: "Bin gleich wieder da.",
            fr: "Je reviens tout de suite.",
        },
        (8, 835) => LocalisedText {
            en: "Roll for loot, please.",
            ja: "ロットしてください。",
            de: "Würfeln, bitte.",
            fr: "Jetez les dés.",
        },
        (8, 836) => LocalisedText {
            en: "Weakness:",
            ja: "弱点：",
            de: "Schwachstelle:",
            fr: "Point faible :",
        },
        (8, 837) => LocalisedText {
            en: "Beware of",
            ja: "注意：",
            de: "Achtung:",
            fr: "Attention :",
        },
        (8, 838) => LocalisedText {
            en: "Recommended:",
            ja: "おすすめ：",
            de: "Empfehlung:",
            fr: "Recommandation :",
        },
        (8, 839) => LocalisedText {
            en: "Kill Order:",
            ja: "討伐順：",
            de: "In dieser Reihenfolge niederstrecken:",
            fr: "Ordre d'élimination :",
        },
        (8, 840) => LocalisedText {
            en: "I want",
            ja: "欲しいもの：",
            de: "Ich möchte:",
            fr: "Je voudrais :",
        },
        (8, 841) => LocalisedText {
            en: "Please draw enmity.",
            ja: "敵視を稼いでください。",
            de: "Bitte zieh Feindseligkeit auf dich.",
            fr: "Accumulez de l'inimitié.",
        },
        (8, 842) => LocalisedText {
            en: "The enemy is near!",
            ja: "敵が近づいてます！",
            de: "Der Gegner ist nah!",
            fr: "L'ennemi approche !",
        },
        (8, 843) => LocalisedText {
            en: "Don't worry about it.",
            ja: "気にしないでください。",
            de: "Macht doch nichts.",
            fr: "Ne vous en faites pas pour ça.",
        },
        (8, 844) => LocalisedText {
            en: "Please ignore that.",
            ja: "無視してください。",
            de: "Bitte ignorier das.",
            fr: "Ignorez ça.",
        },
        (8, 845) => LocalisedText {
            en: "Switch.",
            ja: "スイッチ",
            de: "Tauschen!",
            fr: "On change.",
        },
        (8, 846) => LocalisedText {
            en: "Change with me.",
            ja: "交代してください！",
            de: "Spring für mich ein!",
            fr: "Remplacez-moi !",
        },
        (8, 847) => LocalisedText {
            en: "I'll deal the first blow.",
            ja: "私が最初に攻撃します！",
            de: "Ich greife zuerst an.",
            fr: "J'attaque en premier !",
        },
        (8, 848) => LocalisedText {
            en: "Let's take it slow.",
            ja: "ゆっくり進みましょう。",
            de: "Gehen wir es langsam an.",
            fr: "Allons-y doucement.",
        },
        (8, 849) => LocalisedText {
            en: "Do it!",
            ja: "実行してください！",
            de: "Tu es!",
            fr: "Allez-y !",
        },
        (8, 850) => LocalisedText {
            en: "Direction:",
            ja: "方向：",
            de: "Richtung:",
            fr: "Direction :",
        },
        (8, 851) => LocalisedText {
            en: "Avoid the attack!",
            ja: "回避してください！",
            de: "Weich aus.",
            fr: "Évitez l'attaque !",
        },
        (8, 852) => LocalisedText {
            en: "Turn away.",
            ja: "後ろを向いてください。",
            de: "Dreh dich weg.",
            fr: "Tournez le dos.",
        },
        (8, 853) => LocalisedText {
            en: "Get hit.",
            ja: "ダメージを受けてください。",
            de: "Lass dich treffen.",
            fr: "Encaissez des dégâts.",
        },
        (8, 854) => LocalisedText {
            en: "Pull the enemy away.",
            ja: "敵を離してください。",
            de: "Lock den Feind weg.",
            fr: "Éloignez les ennemis.",
        },
        (8, 855) => LocalisedText {
            en: "Move away.",
            ja: "離れてください。",
            de: "Geh weiter weg.",
            fr: "Éloignez-vous.",
        },
        (8, 856) => LocalisedText {
            en: "Come closer.",
            ja: "近づいてください。",
            de: "Komm näher.",
            fr: "Approchez-vous.",
        },
        (8, 857) => LocalisedText {
            en: "Carry it.",
            ja: "運んでください。",
            de: "Transportiere es.",
            fr: "Transportez ça.",
        },
        (8, 859) => LocalisedText {
            en: "Cut the link!",
            ja: "切断してください！",
            de: "Trenne es.",
            fr: "Brisez le lien !",
        },
        (8, 860) => LocalisedText {
            en: "Keep it busy.",
            ja: "維持してください！",
            de: "Halt es aufrecht!",
            fr: "Gardez-le comme ça !",
        },
        (8, 861) => LocalisedText {
            en: "Take them down at the same time!",
            ja: "同時に倒しましょう！",
            de: "Besiegen wir es zugleich!",
            fr: "Vainquons-les en même temps !",
        },
        (8, 862) => LocalisedText {
            en: "Hide!",
            ja: "隠れて！",
            de: "Versteck dich!",
            fr: "Cachez-vous !",
        },
        (8, 863) => LocalisedText {
            en: "Mounting machina!",
            ja: "搭乗します！",
            de: "Ich steuere eine Kriegsmaschine.",
            fr: "Je vais manœuvrer une machine de guerre !",
        },
        (8, 864) => LocalisedText {
            en: "Mount the machina!",
            ja: "搭乗してください！",
            de: "Steig in eine Kriegsmaschine!",
            fr: "Manœuvrez une machine de guerre !",
        },
        (9, 900) => LocalisedText {
            en: "【Roles】",
            ja: "【役割】",
            de: "【Rollen】",
            fr: "【Rôles】",
        },
        (9, 901) => LocalisedText {
            en: "leader",
            ja: "リーダー",
            de: "Anführer",
            fr: "Chef",
        },
        (9, 902) => LocalisedText {
            en: "master",
            ja: "マスター",
            de: "Meister",
            fr: "Maître",
        },
        (9, 903) => LocalisedText {
            en: "Disciples of War",
            ja: "ファイター",
            de: "Disziplinen der Krieger",
            fr: "Disciples de la guerre",
        },
        (9, 904) => LocalisedText {
            en: "Disciples of Magic",
            ja: "ソーサラー",
            de: "Disziplinen der Magier",
            fr: "Disciples de la magie",
        },
        (9, 905) => LocalisedText {
            en: "Disciples of the Land",
            ja: "ギャザラー",
            de: "Disziplinen der Sammler",
            fr: "Disciples de la terre",
        },
        (9, 906) => LocalisedText {
            en: "Disciples of the Hand",
            ja: "クラフター",
            de: "Disziplinen der Handwerker",
            fr: "Disciples de la main",
        },
        (9, 907) => LocalisedText {
            en: "tank",
            ja: "タンク",
            de: "Verteidiger (Rolle)",
            fr: "Tank",
        },
        (9, 908) => LocalisedText {
            en: "melee",
            ja: "近接攻撃役",
            de: "Nahkämpfer",
            fr: "Attaquant de mêlée",
        },
        (9, 909) => LocalisedText {
            en: "ranged",
            ja: "遠隔攻撃役",
            de: "Fernkämpfer",
            fr: "Attaquant à distance",
        },
        (9, 910) => LocalisedText {
            en: "healer",
            ja: "ヒーラー",
            de: "Heiler",
            fr: "Soigneur",
        },
        (9, 911) => LocalisedText {
            en: "buffer",
            ja: "強化役",
            de: "Unterstützer",
            fr: "Soutien",
        },
        (9, 912) => LocalisedText {
            en: "caster",
            ja: "魔法攻撃役",
            de: "Magischer Angreifer",
            fr: "Attaquant magique",
        },
        (9, 913) => LocalisedText {
            en: "DPS",
            ja: "DPS",
            de: "Angreifer",
            fr: "DPS",
        },
        (10, 950) => LocalisedText {
            en: "【Locations】",
            ja: "【位置】",
            de: "【Richtungen】",
            fr: "【Positions】",
        },
        (10, 951) => LocalisedText {
            en: "position",
            ja: "位置",
            de: "Position",
            fr: "Position",
        },
        (10, 952) => LocalisedText {
            en: "north",
            ja: "北",
            de: "Nord",
            fr: "Nord",
        },
        (10, 953) => LocalisedText {
            en: "south",
            ja: "南",
            de: "Süd",
            fr: "Sud",
        },
        (10, 954) => LocalisedText {
            en: "east",
            ja: "東",
            de: "Ost",
            fr: "Est",
        },
        (10, 955) => LocalisedText {
            en: "west",
            ja: "西",
            de: "West",
            fr: "Ouest",
        },
        (10, 956) => LocalisedText {
            en: "up",
            ja: "上",
            de: "Oben",
            fr: "Haut",
        },
        (10, 957) => LocalisedText {
            en: "down",
            ja: "下",
            de: "Unten",
            fr: "Bas",
        },
        (10, 958) => LocalisedText {
            en: "right",
            ja: "右",
            de: "Rechts",
            fr: "Droite",
        },
        (10, 959) => LocalisedText {
            en: "left",
            ja: "左",
            de: "Links",
            fr: "Gauche",
        },
        (10, 960) => LocalisedText {
            en: "front side",
            ja: "表",
            de: "Oberfläche",
            fr: "Avant",
        },
        (10, 961) => LocalisedText {
            en: "back",
            ja: "裏",
            de: "Hinten",
            fr: "Arrière",
        },
        (10, 962) => LocalisedText {
            en: "side",
            ja: "横",
            de: "Seite",
            fr: "Côté",
        },
        (10, 963) => LocalisedText {
            en: "front",
            ja: "正面",
            de: "Vorn",
            fr: "Devant",
        },
        (10, 964) => LocalisedText {
            en: "middle",
            ja: "中央",
            de: "Mitte",
            fr: "Milieu",
        },
        (10, 965) => LocalisedText {
            en: "flank",
            ja: "側面",
            de: "Flanke",
            fr: "Flanc",
        },
        (10, 966) => LocalisedText {
            en: "inside",
            ja: "内側",
            de: "Innen",
            fr: "À l'intérieur",
        },
        (10, 967) => LocalisedText {
            en: "outside",
            ja: "外側",
            de: "Draußen",
            fr: "Dehors",
        },
        (10, 968) => LocalisedText {
            en: "this way",
            ja: "こっち",
            de: "Hier lang",
            fr: "Par ici",
        },
        (10, 969) => LocalisedText {
            en: "over there",
            ja: "あっち",
            de: "Dort hinten",
            fr: "Là-bas",
        },
        (10, 970) => LocalisedText {
            en: "that way",
            ja: "そっち",
            de: "Dort lang",
            fr: "Par là",
        },
        (10, 971) => LocalisedText {
            en: "closer",
            ja: "もっと近く",
            de: "Näher ran",
            fr: "Plus près",
        },
        (10, 972) => LocalisedText {
            en: "farther",
            ja: "もっと遠く",
            de: "Weiter weg",
            fr: "Plus loin",
        },
        (10, 973) => LocalisedText {
            en: "entrance",
            ja: "入口",
            de: "Eingang",
            fr: "Entrée",
        },
        (10, 974) => LocalisedText {
            en: "exit",
            ja: "出口",
            de: "Ausgang",
            fr: "Sortie",
        },
        (10, 975) => LocalisedText {
            en: "rear",
            ja: "背面",
            de: "Rückseite",
            fr: "Par derrière",
        },
        (10, 976) => LocalisedText {
            en: "one o'clock",
            ja: "1時",
            de: "1 Uhr",
            fr: "1h",
        },
        (10, 977) => LocalisedText {
            en: "two o'clock",
            ja: "2時",
            de: "2 Uhr",
            fr: "2h",
        },
        (10, 978) => LocalisedText {
            en: "three o'clock",
            ja: "3時",
            de: "3 Uhr",
            fr: "3h",
        },
        (10, 979) => LocalisedText {
            en: "four o'clock",
            ja: "4時",
            de: "4 Uhr",
            fr: "4h",
        },
        (10, 980) => LocalisedText {
            en: "five o'clock",
            ja: "5時",
            de: "5 Uhr",
            fr: "5h",
        },
        (10, 981) => LocalisedText {
            en: "six o'clock",
            ja: "6時",
            de: "6 Uhr",
            fr: "6h",
        },
        (10, 982) => LocalisedText {
            en: "seven o'clock",
            ja: "7時",
            de: "7 Uhr",
            fr: "7h",
        },
        (10, 983) => LocalisedText {
            en: "eight o'clock",
            ja: "8時",
            de: "8 Uhr",
            fr: "8h",
        },
        (10, 984) => LocalisedText {
            en: "nine o'clock",
            ja: "9時",
            de: "9 Uhr",
            fr: "9h",
        },
        (10, 985) => LocalisedText {
            en: "ten o'clock",
            ja: "10時",
            de: "10 Uhr",
            fr: "10h",
        },
        (10, 986) => LocalisedText {
            en: "eleven o'clock",
            ja: "11時",
            de: "11 Uhr",
            fr: "11h",
        },
        (10, 987) => LocalisedText {
            en: "twelve o'clock",
            ja: "12時",
            de: "12 Uhr",
            fr: "12h",
        },
        (11, 1000) => LocalisedText {
            en: "【Time】",
            ja: "【時間】",
            de: "【Zeit】",
            fr: "【Repères temporels】",
        },
        (11, 1001) => LocalisedText {
            en: "day before yesterday",
            ja: "おととい",
            de: "Vorgestern",
            fr: "Avant-hier",
        },
        (11, 1002) => LocalisedText {
            en: "yesterday",
            ja: "昨日",
            de: "Gestern",
            fr: "Hier",
        },
        (11, 1003) => LocalisedText {
            en: "today",
            ja: "今日",
            de: "Heute",
            fr: "Aujourd'hui",
        },
        (11, 1004) => LocalisedText {
            en: "tomorrow",
            ja: "明日",
            de: "morgen",
            fr: "Demain",
        },
        (11, 1005) => LocalisedText {
            en: "day after tomorrow",
            ja: "あさって",
            de: "Übermorgen",
            fr: "Après-demain",
        },
        (11, 1006) => LocalisedText {
            en: "last week",
            ja: "先週",
            de: "Letzte Woche",
            fr: "La semaine dernière",
        },
        (11, 1007) => LocalisedText {
            en: "this week",
            ja: "今週",
            de: "Diese Woche",
            fr: "Cette semaine",
        },
        (11, 1008) => LocalisedText {
            en: "next week",
            ja: "来週",
            de: "Nächste Woche",
            fr: "La semaine prochaine",
        },
        (11, 1009) => LocalisedText {
            en: "a.m.",
            ja: "午前",
            de: "am Vormittag",
            fr: "am",
        },
        (11, 1010) => LocalisedText {
            en: "p.m.",
            ja: "午後",
            de: "am Nachmittag",
            fr: "pm",
        },
        (11, 1011) => LocalisedText {
            en: "morning",
            ja: "朝",
            de: "Vormittag",
            fr: "Matin",
        },
        (11, 1012) => LocalisedText {
            en: "afternoon",
            ja: "昼",
            de: "Nachmittag",
            fr: "Après-midi",
        },
        (11, 1013) => LocalisedText {
            en: "night",
            ja: "夜",
            de: "Nacht",
            fr: "Nuit",
        },
        (11, 1014) => LocalisedText {
            en: "day of the week",
            ja: "曜日",
            de: "Wochentag",
            fr: "Jour de la semaine",
        },
        (11, 1015) => LocalisedText {
            en: "Sunday",
            ja: "日曜日",
            de: "Sonntag",
            fr: "Dimanche",
        },
        (11, 1016) => LocalisedText {
            en: "Monday",
            ja: "月曜日",
            de: "Montag",
            fr: "Lundi",
        },
        (11, 1017) => LocalisedText {
            en: "Tuesday",
            ja: "火曜日",
            de: "Dienstag",
            fr: "Mardi",
        },
        (11, 1018) => LocalisedText {
            en: "Wednesday",
            ja: "水曜日",
            de: "Mittwoch",
            fr: "Mercredi",
        },
        (11, 1019) => LocalisedText {
            en: "Thursday",
            ja: "木曜日",
            de: "Donnerstag",
            fr: "Jeudi",
        },
        (11, 1020) => LocalisedText {
            en: "Friday",
            ja: "金曜日",
            de: "Freitag",
            fr: "Vendredi",
        },
        (11, 1021) => LocalisedText {
            en: "Saturday",
            ja: "土曜日",
            de: "Samstag",
            fr: "Samedi",
        },
        (11, 1022) => LocalisedText {
            en: "holiday",
            ja: "休日",
            de: "Ferien",
            fr: "Jour férié",
        },
        (11, 1023) => LocalisedText {
            en: "break",
            ja: "休み",
            de: "Pause",
            fr: "Jour de repos",
        },
        (11, 1024) => LocalisedText {
            en: "long time",
            ja: "長期",
            de: "Lange Zeit",
            fr: "Longtemps",
        },
        (11, 1025) => LocalisedText {
            en: "short time",
            ja: "短期",
            de: "Kurz",
            fr: "Pas longtemps",
        },
        (11, 1026) => LocalisedText {
            en: "date",
            ja: "日付",
            de: "Datum",
            fr: "Date",
        },
        (11, 1027) => LocalisedText {
            en: "second",
            ja: "秒",
            de: "Sekunde",
            fr: "Seconde",
        },
        (11, 1028) => LocalisedText {
            en: "minute",
            ja: "分",
            de: "Minute",
            fr: "Minute",
        },
        (11, 1029) => LocalisedText {
            en: "hour",
            ja: "時間",
            de: "Stunde",
            fr: "Heure",
        },
        (11, 1030) => LocalisedText {
            en: "month",
            ja: "月",
            de: "Monat",
            fr: "Mois",
        },
        (11, 1031) => LocalisedText {
            en: "year",
            ja: "年",
            de: "Jahr",
            fr: "Année",
        },
        (11, 1032) => LocalisedText {
            en: "January",
            ja: "1月",
            de: "Januar",
            fr: "Janvier",
        },
        (11, 1033) => LocalisedText {
            en: "February",
            ja: "2月",
            de: "Februar",
            fr: "Février",
        },
        (11, 1034) => LocalisedText {
            en: "March",
            ja: "3月",
            de: "März",
            fr: "Mars",
        },
        (11, 1035) => LocalisedText {
            en: "April",
            ja: "4月",
            de: "April",
            fr: "Avril",
        },
        (11, 1036) => LocalisedText {
            en: "May",
            ja: "5月",
            de: "Mai",
            fr: "Mai",
        },
        (11, 1037) => LocalisedText {
            en: "June",
            ja: "6月",
            de: "Juni",
            fr: "Juin",
        },
        (11, 1038) => LocalisedText {
            en: "July",
            ja: "7月",
            de: "Juli",
            fr: "Juillet",
        },
        (11, 1039) => LocalisedText {
            en: "August",
            ja: "8月",
            de: "August",
            fr: "Août",
        },
        (11, 1040) => LocalisedText {
            en: "September",
            ja: "9月",
            de: "September",
            fr: "Septembre",
        },
        (11, 1041) => LocalisedText {
            en: "October",
            ja: "10月",
            de: "Oktober",
            fr: "Octobre",
        },
        (11, 1042) => LocalisedText {
            en: "November",
            ja: "11月",
            de: "November",
            fr: "Novembre",
        },
        (11, 1043) => LocalisedText {
            en: "December",
            ja: "12月",
            de: "Dezember",
            fr: "Décembre",
        },
        (11, 1044) => LocalisedText {
            en: "last month",
            ja: "先月",
            de: "Letzen Monat",
            fr: "Le mois dernier",
        },
        (11, 1045) => LocalisedText {
            en: "this month",
            ja: "今月",
            de: "Diesen Monat",
            fr: "Ce mois",
        },
        (11, 1046) => LocalisedText {
            en: "next month",
            ja: "来月",
            de: "Nächsten Monat",
            fr: "Le mois prochain",
        },
        (11, 1047) => LocalisedText {
            en: "last year",
            ja: "去年",
            de: "Letztes Jahr",
            fr: "L'année dernière",
        },
        (11, 1048) => LocalisedText {
            en: "this year",
            ja: "今年",
            de: "Dieses Jahr",
            fr: "Cette année",
        },
        (11, 1049) => LocalisedText {
            en: "next year",
            ja: "来年",
            de: "Nächstes Jahr",
            fr: "L'année prochaine",
        },
        (11, 1050) => LocalisedText {
            en: "now",
            ja: "今",
            de: "Jetzt",
            fr: "Maintenant",
        },
        (11, 1051) => LocalisedText {
            en: "soon",
            ja: "もうすぐ",
            de: "Gleich",
            fr: "Bientôt",
        },
        (11, 1052) => LocalisedText {
            en: "any time",
            ja: "いつでも",
            de: "Jederzeit",
            fr: "N'importe quand",
        },
        (11, 1053) => LocalisedText {
            en: "first",
            ja: "最初",
            de: "erste(r/s)",
            fr: "Premier",
        },
        (11, 1054) => LocalisedText {
            en: "last",
            ja: "最後",
            de: "letzte(r/s)",
            fr: "Dernier",
        },
        (11, 1055) => LocalisedText {
            en: "waiting time",
            ja: "待ち時間",
            de: "Wartezeit",
            fr: "Temps d'attente",
        },
        (11, 1056) => LocalisedText {
            en: "time remaining",
            ja: "残り時間",
            de: "Verbleibende Zeit",
            fr: "Temps restant",
        },
        (11, 1057) => LocalisedText {
            en: "end time",
            ja: "終了時間",
            de: "Ende",
            fr: "Heure de fin",
        },
        (11, 1058) => LocalisedText {
            en: "Sixth Astral Era",
            ja: "第六星暦",
            de: "Sechste Ära des Lichts",
            fr: "6e ère astrale",
        },
        (11, 1059) => LocalisedText {
            en: "Seventh Umbral Era",
            ja: "第七霊災",
            de: "Siebte Katastrophe",
            fr: "7e ère ombrale",
        },
        (11, 1060) => LocalisedText {
            en: "Seventh Astral Era",
            ja: "第七星暦",
            de: "Siebte Ära des Lichts",
            fr: "7e ère astrale",
        },
        (11, 1061) => LocalisedText {
            en: "before",
            ja: "直前",
            de: "genau vorher",
            fr: "Juste avant",
        },
        (11, 1062) => LocalisedText {
            en: "after",
            ja: "直後",
            de: "genau nachher",
            fr: "Juste après",
        },
        (13, 1150) => LocalisedText {
            en: "【Transportation】",
            ja: "【移動方法】",
            de: "【Transport】",
            fr: "【Déplacements】",
        },
        (13, 1151) => LocalisedText {
            en: "walk",
            ja: "歩く",
            de: "Gehen",
            fr: "Marcher",
        },
        (13, 1152) => LocalisedText {
            en: "run",
            ja: "走る",
            de: "Laufen",
            fr: "Courir",
        },
        (13, 1153) => LocalisedText {
            en: "follow",
            ja: "追従",
            de: "Folgen",
            fr: "Poursuite",
        },
        (13, 1154) => LocalisedText {
            en: "chocobo",
            ja: "チョコボ",
            de: "Chocobo",
            fr: "Chocobo",
        },
        (13, 1155) => LocalisedText {
            en: "personal chocobo",
            ja: "マイチョコボ",
            de: "Persönlicher Chocobo",
            fr: "Chocobo personnel",
        },
        (13, 1156) => LocalisedText {
            en: "rental chocobo",
            ja: "レンタルチョコボ",
            de: "Miet-Chocobo",
            fr: "Chocobo de location",
        },
        (13, 1157) => LocalisedText {
            en: "chocobo porter",
            ja: "チョコボポーター",
            de: "Linien-Chocobo",
            fr: "Chocobo de relais",
        },
        (13, 1158) => LocalisedText {
            en: "mount",
            ja: "マウント",
            de: "Reittier",
            fr: "Monture",
        },
        (13, 1159) => LocalisedText {
            en: "Teleport",
            ja: "テレポ",
            de: "Teleportieren",
            fr: "Téléportation",
        },
        (13, 1160) => LocalisedText {
            en: "Return",
            ja: "デジョン",
            de: "Rückführung",
            fr: "Rapatriement",
        },
        (13, 1161) => LocalisedText {
            en: "Aethernet",
            ja: "都市転送網",
            de: "Ätherytennetz",
            fr: "Réseau de transport urbain éthéré",
        },
        (13, 1162) => LocalisedText {
            en: "airship",
            ja: "飛空艇",
            de: "Luftschiff",
            fr: "Aéronef",
        },
        (13, 1163) => LocalisedText {
            en: "Home Point",
            ja: "ホームポイント",
            de: "Heimatpunkt",
            fr: "Point de retour",
        },
        (13, 1164) => LocalisedText {
            en: "ferry",
            ja: "定期船",
            de: "Fähre",
            fr: "Transbordeur",
        },
        (13, 1165) => LocalisedText {
            en: "flying mount",
            ja: "フライングマウント",
            de: "fliegendes Reittier",
            fr: "Monture volante",
        },
        (14, 1200) => LocalisedText {
            en: "【Facilities】",
            ja: "【施設】",
            de: "【Einrichtungen】",
            fr: "【Installations】",
        },
        (14, 1201) => LocalisedText {
            en: "aetheryte",
            ja: "エーテライト",
            de: "Ätheryt",
            fr: "Éthérite",
        },
        (14, 1202) => LocalisedText {
            en: "city-state aetheryte",
            ja: "都市転送網",
            de: "Stadt-Ätheryt",
            fr: "Éthérite urbaine",
        },
        (14, 1203) => LocalisedText {
            en: "Adventurers' Guild",
            ja: "冒険者ギルド",
            de: "Abenteurergilde",
            fr: "Guilde des aventuriers",
        },
        (14, 1204) => LocalisedText {
            en: "guild",
            ja: "ギルド",
            de: "Gilde",
            fr: "Guilde",
        },
        (14, 1205) => LocalisedText {
            en: "shop",
            ja: "ショップ",
            de: "Händler",
            fr: "Échoppe",
        },
        (14, 1206) => LocalisedText {
            en: "chocobo stop",
            ja: "チョコボ留",
            de: "Chocobo-Stall",
            fr: "Relais à chocobos",
        },
        (14, 1207) => LocalisedText {
            en: "inn",
            ja: "宿屋",
            de: "Herberge",
            fr: "Auberge",
        },
        (14, 1208) => LocalisedText {
            en: "summoning bell",
            ja: "呼び鈴",
            de: "Krämerklingel",
            fr: "Sonnette",
        },
        (14, 1209) => LocalisedText {
            en: "sanctuary",
            ja: "レストエリア",
            de: "Ruhebereich",
            fr: "Lieu de repos",
        },
        (14, 1210) => LocalisedText {
            en: "levemete",
            ja: "ギルドリーヴ発行窓口",
            de: "Freibrief-Ausgabe",
            fr: "Mandant",
        },
        (14, 1211) => LocalisedText {
            en: "linkshell distributor",
            ja: "リンクシェル発行窓口",
            de: "Kontaktperlen-Ausgabe",
            fr: "Responsable des linkshells",
        },
        (14, 1212) => LocalisedText {
            en: "market board",
            ja: "マーケットボード",
            de: "Marktbrett",
            fr: "Tableau des ventes",
        },
        (14, 1213) => LocalisedText {
            en: "retainer vocate",
            ja: "リテイナー雇用窓口",
            de: "Schalter für Gehilfen",
            fr: "Comptoir d'embauche des servants",
        },
        (14, 1214) => LocalisedText {
            en: "armor mender",
            ja: "修理屋",
            de: "Reparateur",
            fr: "Réparateur",
        },
        (14, 1215) => LocalisedText {
            en: "landing",
            ja: "ランディング",
            de: "Luftschiff-Landeplatz",
            fr: "Aérodrome",
        },
        (14, 1216) => LocalisedText {
            en: "delivery moogle",
            ja: "レターモーグリ",
            de: "Mogry-Post",
            fr: "Mog postier",
        },
        (14, 1217) => LocalisedText {
            en: "docks",
            ja: "船着場",
            de: "Fähranleger",
            fr: "Quai",
        },
        (14, 1218) => LocalisedText {
            en: "ferry docks",
            ja: "フェリードック",
            de: "Fähre (Limsa-Abendstern)",
            fr: "Embarcadère",
        },
        (14, 1219) => LocalisedText {
            en: "landing",
            ja: "発着場",
            de: "Landeplatz",
            fr: "Voyages aériens",
        },
        (14, 1220) => LocalisedText {
            en: "Maelstrom Command",
            ja: "黒渦団：軍令部",
            de: "Mahlstrom-Flottenkommando",
            fr: "Quartier général du Maelstrom",
        },
        (14, 1221) => LocalisedText {
            en: "The Adders' Nest",
            ja: "双蛇党：統合司令部",
            de: "Abaton der Bruderschaft",
            fr: "Quartier général des Deux Vipères",
        },
        (14, 1222) => LocalisedText {
            en: "The Hall of Flames",
            ja: "不滅隊：作戦本部",
            de: "Ewige Halle",
            fr: "Quartier général des Immortels",
        },
        (14, 1223) => LocalisedText {
            en: "residential district",
            ja: "冒険者居住区",
            de: "Wohngebiet",
            fr: "Quartier résidentiel",
        },
        (14, 1224) => LocalisedText {
            en: "entry counter",
            ja: "コロセウム対戦受付",
            de: "Teilnahmeschalter",
            fr: "Comptoir d'enregistrement",
        },
        (14, 1225) => LocalisedText {
            en: "Hunt board",
            ja: "モブハントボード",
            de: "Jagdbrett",
            fr: "Tableau des contrats de chasse",
        },
        (14, 1226) => LocalisedText {
            en: "skywatcher",
            ja: "天気予報士",
            de: "Meteorologin",
            fr: "Météorologue",
        },
        (14, 1227) => LocalisedText {
            en: "crystal bell",
            ja: "美容師の呼び鈴",
            de: "Kosmetikerklingel",
            fr: "Sonnette de l'esthéticien",
        },
        (14, 1228) => LocalisedText {
            en: "subdivision",
            ja: "拡張街",
            de: "Erweiterter Bezirk",
            fr: "Annexe",
        },
        (14, 1229) => LocalisedText {
            en: "orchestrion",
            ja: "オーケストリオン",
            de: "Orchestrion",
            fr: "Orchestrion",
        },
        (14, 1230) => LocalisedText {
            en: "striking dummy",
            ja: "木人",
            de: "Trainingspuppe",
            fr: "Mannequin d'entraînement",
        },
        (14, 1231) => LocalisedText {
            en: "materia melder",
            ja: "マテリア師",
            de: "Materia-Einsetzer",
            fr: "Spécialiste ès matérias",
        },
        (14, 1232) => LocalisedText {
            en: "apartment",
            ja: "アパルトメント",
            de: "Wohnung",
            fr: "Appartement",
        },
        (14, 1233) => LocalisedText {
            en: "glamour dresser",
            ja: "ミラージュドレッサー",
            de: "Projektionskommode",
            fr: "Coiffeuse mirage",
        },
        (14, 1234) => LocalisedText {
            en: "message book",
            ja: "交流帳",
            de: "Diarium",
            fr: "Livre de correspondance",
        },
        (14, 1235) => LocalisedText {
            en: "armoire",
            ja: "愛蔵品キャビネット",
            de: "Kostbarkeitenkabinett",
            fr: "Bahut personnel",
        },
        (14, 1236) => LocalisedText {
            en: "The Unending Journey",
            ja: "愛用の紀行録",
            de: "Reisetagebuch",
            fr: "Carnet de voyage",
        },
        (15, 1250) => LocalisedText {
            en: "【Grand Company】",
            ja: "【グランドカンパニー】",
            de: "【Staatliche Gesellschaft】",
            fr: "【Grandes compagnies】",
        },
        (15, 1251) => LocalisedText {
            en: "Grand Company",
            ja: "グランドカンパニー",
            de: "Staatliche Gesellschaft",
            fr: "Grande compagnie",
        },
        (15, 1252) => LocalisedText {
            en: "Grand Company rank",
            ja: "階級",
            de: "Rang in Staatlicher Gesellschaft",
            fr: "Rang de grande compagnie",
        },
        (15, 1253) => LocalisedText {
            en: "supplies",
            ja: "軍需品",
            de: "Kriegsmaterialbeschaffung",
            fr: "Matériel",
        },
        (15, 1254) => LocalisedText {
            en: "provisions",
            ja: "補給品",
            de: "Nachschublieferung",
            fr: "Ressources",
        },
        (15, 1255) => LocalisedText {
            en: "high-quality supplies",
            ja: "稀少品",
            de: "Experteneinsätze",
            fr: "Prises de guerre",
        },
        (15, 1256) => LocalisedText {
            en: "Maelstrom",
            ja: "黒渦団",
            de: "Der Mahlstrom",
            fr: "Maelstrom",
        },
        (15, 1257) => LocalisedText {
            en: "Order of the Twin Adder",
            ja: "双蛇党",
            de: "Bruderschaft der Morgenviper",
            fr: "Ordre des Deux Vipères",
        },
        (15, 1258) => LocalisedText {
            en: "Immortal Flames",
            ja: "不滅隊",
            de: "Legion der Unsterblichen",
            fr: "Immortels",
        },
        (16, 1270) => LocalisedText {
            en: "【Free Company】",
            ja: "【フリーカンパニー】",
            de: "【Freie Gesellschaft】",
            fr: "【Compagnie libre】",
        },
        (16, 1271) => LocalisedText {
            en: "free company",
            ja: "フリーカンパニー",
            de: "Freie Gesellschaft",
            fr: "Compagnie libre",
        },
        (16, 1272) => LocalisedText {
            en: "signature",
            ja: "署名",
            de: "Unterschrift",
            fr: "Signature",
        },
        (16, 1273) => LocalisedText {
            en: "company board",
            ja: "カンパニーボード",
            de: "Anschlagbrett",
            fr: "Annonce de la compagnie",
        },
        (16, 1274) => LocalisedText {
            en: "company chest",
            ja: "カンパニーチェスト",
            de: "Gesellschaftstruhe",
            fr: "Coffre de la compagnie",
        },
        (16, 1275) => LocalisedText {
            en: "company crest",
            ja: "カンパニークレスト",
            de: "Gesellschaftswappen",
            fr: "Blason de la compagnie",
        },
        (16, 1276) => LocalisedText {
            en: "company action",
            ja: "カンパニーアクション",
            de: "Gesellschaftskommando",
            fr: "Bienfait de compagnie",
        },
        (16, 1277) => LocalisedText {
            en: "company crafting",
            ja: "カンパニークラフト",
            de: "Gesellschaftsprojekt",
            fr: "projet de construction",
        },
        (16, 1278) => LocalisedText {
            en: "company workshop",
            ja: "地下工房",
            de: "Gesellschaftswerkstätte",
            fr: "atelier de compagnie",
        },
        (16, 1279) => LocalisedText {
            en: "exploratory voyage",
            ja: "エアシップボイジャー",
            de: "Luftschifferkundung",
            fr: "expédition en aéronef",
        },
        (16, 1280) => LocalisedText {
            en: "subaquatic voyage",
            ja: "サブマリンボイジャー",
            de: "Tauchbooterkundung",
            fr: "expédition en sous-marin",
        },
        (17, 1300) => LocalisedText {
            en: "【System】",
            ja: "【ゲームシステム】",
            de: "【System】",
            fr: "【Systèmes】",
        },
        (17, 1301) => LocalisedText {
            en: "class",
            ja: "クラス",
            de: "Klasse",
            fr: "Classe",
        },
        (17, 1302) => LocalisedText {
            en: "class change",
            ja: "クラスチェンジ",
            de: "Klassenwechsel",
            fr: "Changement de classe",
        },
        (17, 1303) => LocalisedText {
            en: "job",
            ja: "ジョブ",
            de: "Job",
            fr: "Job",
        },
        (17, 1304) => LocalisedText {
            en: "job change",
            ja: "ジョブチェンジ",
            de: "Jobwechsel",
            fr: "Changement de job",
        },
        (17, 1305) => LocalisedText {
            en: "soul crystal",
            ja: "ソウルクリスタル",
            de: "Jobkristall",
            fr: "Cristal d'âme",
        },
        (17, 1306) => LocalisedText {
            en: "level",
            ja: "レベル",
            de: "Stufe",
            fr: "Niveau",
        },
        (17, 1307) => LocalisedText {
            en: "experience points",
            ja: "経験値",
            de: "Routine",
            fr: "Points d'expérience",
        },
        (17, 1308) => LocalisedText {
            en: "guardian",
            ja: "守護神",
            de: "Schutzgott",
            fr: "Divinité protectrice",
        },
        (17, 1309) => LocalisedText {
            en: "race",
            ja: "民族",
            de: "Volk",
            fr: "Race",
        },
        (17, 1310) => LocalisedText {
            en: "clan",
            ja: "部族",
            de: "Volksstamm",
            fr: "Ethnie",
        },
        (17, 1311) => LocalisedText {
            en: "area",
            ja: "エリア",
            de: "Areal",
            fr: "Zone",
        },
        (17, 1312) => LocalisedText {
            en: "Armoury",
            ja: "アーマリー",
            de: "Arsenal",
            fr: "Arsenal",
        },
        (17, 1313) => LocalisedText {
            en: "item",
            ja: "アイテム",
            de: "Gegenstand",
            fr: "Objet",
        },
        (17, 1314) => LocalisedText {
            en: "gear set",
            ja: "ギアセット",
            de: "Ausrüstungsset",
            fr: "Tenue",
        },
        (17, 1315) => LocalisedText {
            en: "rested bonus",
            ja: "レストボーナス",
            de: "Erholungsbonus",
            fr: "Bonus de repos",
        },
        (17, 1316) => LocalisedText {
            en: "solo",
            ja: "ソロ",
            de: "Solo",
            fr: "Solo",
        },
        (17, 1317) => LocalisedText {
            en: "party",
            ja: "パーティ",
            de: "Gruppe",
            fr: "Équipe",
        },
        (17, 1318) => LocalisedText {
            en: "mount",
            ja: "マウント",
            de: "Reittier",
            fr: "Monture",
        },
        (17, 1319) => LocalisedText {
            en: "minion",
            ja: "ミニオン",
            de: "Begleiter",
            fr: "Mascotte",
        },
        (17, 1320) => LocalisedText {
            en: "retainer",
            ja: "リテイナー",
            de: "Gehilfe",
            fr: "Servant",
        },
        (17, 1321) => LocalisedText {
            en: "battle",
            ja: "バトル",
            de: "Kampf",
            fr: "Combat",
        },
        (17, 1322) => LocalisedText {
            en: "crafting",
            ja: "製作",
            de: "Synthese",
            fr: "Synthèse",
        },
        (17, 1323) => LocalisedText {
            en: "gathering",
            ja: "採集",
            de: "Sammeln",
            fr: "Récolte",
        },
        (17, 1324) => LocalisedText {
            en: "durability",
            ja: "耐久度",
            de: "Zustand (von Ausrüstung)",
            fr: "Condition",
        },
        (17, 1325) => LocalisedText {
            en: "spiritbond",
            ja: "錬精度",
            de: "Emotionale Bindung",
            fr: "Symbiose",
        },
        (17, 1326) => LocalisedText {
            en: "gear affinity",
            ja: "装備適性",
            de: "Ausrüstungsaffinität",
            fr: "Affinité d'équipement",
        },
        (17, 1327) => LocalisedText {
            en: "treasure coffer",
            ja: "宝箱",
            de: "Schatztruhe",
            fr: "Coffre au trésor",
        },
        (17, 1328) => LocalisedText {
            en: "target",
            ja: "ターゲット",
            de: "Ziel",
            fr: "Cible",
        },
        (17, 1329) => LocalisedText {
            en: "lock on",
            ja: "ロックオン",
            de: "Ziel feststellen",
            fr: "Verrouiller",
        },
        (17, 1330) => LocalisedText {
            en: "focus target",
            ja: "フォーカスターゲット",
            de: "Fokusziel",
            fr: "Cible focalisée",
        },
        (17, 1331) => LocalisedText {
            en: "enemy sign",
            ja: "ターゲットマーカー",
            de: "Feindmarkierung",
            fr: "Signe",
        },
        (17, 1332) => LocalisedText {
            en: "menu",
            ja: "メニュー",
            de: "Menü",
            fr: "Menu",
        },
        (17, 1333) => LocalisedText {
            en: "map",
            ja: "マップ",
            de: "Karte",
            fr: "Carte",
        },
        (17, 1334) => LocalisedText {
            en: "support desk",
            ja: "サポートデスク",
            de: "Kundendienst",
            fr: "Aide (menu)",
        },
        (17, 1335) => LocalisedText {
            en: "screenshot",
            ja: "スクリーンショット",
            de: "Screenshot",
            fr: "Capture d'écran",
        },
        (17, 1336) => LocalisedText {
            en: "log out",
            ja: "ログアウト",
            de: "Ausloggen",
            fr: "Déconnexion",
        },
        (17, 1337) => LocalisedText {
            en: "exit game",
            ja: "ゲーム終了",
            de: "Spiel beenden",
            fr: "Sortir du jeu",
        },
        (17, 1338) => LocalisedText {
            en: "shutdown",
            ja: "シャットダウン",
            de: "Herunterfahren",
            fr: "Fermer le jeu",
        },
        (17, 1339) => LocalisedText {
            en: "auto-translation dictionary",
            ja: "定型文辞書",
            de: "Auto-Übersetzung",
            fr: "Traduction automatique",
        },
        (17, 1340) => LocalisedText {
            en: "emote",
            ja: "エモート",
            de: "Emote",
            fr: "Emote",
        },
        (17, 1341) => LocalisedText {
            en: "achievements",
            ja: "アチーブメント",
            de: "Errungenschaften",
            fr: "Hauts faits",
        },
        (17, 1342) => LocalisedText {
            en: "title",
            ja: "称号",
            de: "Titel",
            fr: "Titre",
        },
        (17, 1343) => LocalisedText {
            en: "currency",
            ja: "通貨",
            de: "Vermögen",
            fr: "Devises",
        },
        (17, 1344) => LocalisedText {
            en: "gil",
            ja: "ギル",
            de: "Gil",
            fr: "Gils",
        },
        (17, 1345) => LocalisedText {
            en: "trade",
            ja: "トレード",
            de: "Handel",
            fr: "Échanger",
        },
        (17, 1346) => LocalisedText {
            en: "examine",
            ja: "調べる",
            de: "Untersuchen",
            fr: "Examiner",
        },
        (17, 1347) => LocalisedText {
            en: "auto-follow",
            ja: "自動追尾",
            de: "Automatisches Folgen",
            fr: "Poursuite automatique",
        },
        (17, 1348) => LocalisedText {
            en: "alliance",
            ja: "アライアンス",
            de: "Allianz",
            fr: "Alliance",
        },
        (17, 1349) => LocalisedText {
            en: "housing",
            ja: "ハウジング",
            de: "Unterkunft",
            fr: "Logement",
        },
        (17, 1350) => LocalisedText {
            en: "reputation",
            ja: "友好度",
            de: "Verbundenheit",
            fr: "Réputation",
        },
        (17, 1351) => LocalisedText {
            en: "reputation rank",
            ja: "友好ランク",
            de: "Verbundenheitsrang",
            fr: "Rang de réputation",
        },
        (17, 1352) => LocalisedText {
            en: "commendation",
            ja: "MIP",
            de: "WS-Ehrung",
            fr: "Honneurs",
        },
        (17, 1353) => LocalisedText {
            en: "Allagan tomestone",
            ja: "アラガントームストーン",
            de: "Allagische Steine",
            fr: "mémoquartz",
        },
        (17, 1356) => LocalisedText {
            en: "vote dismiss",
            ja: "投票除名",
            de: "Für Ausschluss stimmen",
            fr: "Vote d'expulsion",
        },
        (17, 1357) => LocalisedText {
            en: "vote abandon",
            ja: "ギブアップ投票",
            de: "Für Abbruch stimmen",
            fr: "Vote d'abandon",
        },
        (17, 1358) => LocalisedText {
            en: "waymark",
            ja: "フィールドマーカー",
            de: "Geländemarkierung",
            fr: "marque au sol",
        },
        (17, 1359) => LocalisedText {
            en: "ground target",
            ja: "グラウンドターゲット",
            de: "Bodenziel",
            fr: "cible au sol",
        },
        (17, 1360) => LocalisedText {
            en: "glamours",
            ja: "武具投影",
            de: "Projizieren",
            fr: "mirages",
        },
        (17, 1362) => LocalisedText {
            en: "aesthetician",
            ja: "美容師",
            de: "Kosmetiker",
            fr: "Esthéticien",
        },
        (17, 1363) => LocalisedText {
            en: "retainer ventures",
            ja: "リテイナーベンチャー",
            de: "Gehilfenunternehmungen",
            fr: "Tâches de servant",
        },
        (17, 1364) => LocalisedText {
            en: "atma",
            ja: "アートマ",
            de: "Atma",
            fr: "Âtma",
        },
        (17, 1365) => LocalisedText {
            en: "Zodiac Weapon",
            ja: "ゾディアックウェポン",
            de: "Zodiak-Waffe",
            fr: "armes du zodiaque",
        },
        (17, 1366) => LocalisedText {
            en: "alexandrite",
            ja: "アレキサンドライト",
            de: "Alexandrit",
            fr: "alexandrite",
        },
        (17, 1367) => LocalisedText {
            en: "animus",
            ja: "アニムス",
            de: "Animus",
            fr: "animus",
        },
        (17, 1368) => LocalisedText {
            en: "novus",
            ja: "ノウス",
            de: "Novus",
            fr: "novus",
        },
        (17, 1369) => LocalisedText {
            en: "Allied Seals",
            ja: "同盟記章",
            de: "Jagdabzeichen",
            fr: "insignes alliés",
        },
        (17, 1370) => LocalisedText {
            en: "mark bill",
            ja: "モブ手配書",
            de: "Jagdlizenz",
            fr: "contrat de chasse",
        },
        (17, 1371) => LocalisedText {
            en: "Frontline",
            ja: "アウトロー戦区",
            de: "Carteneauer Front",
            fr: "Front de Carteneau",
        },
        (17, 1372) => LocalisedText {
            en: "The Echo",
            ja: "超える力",
            de: "Kraft des Transzendierens",
            fr: "l'Écho",
        },
        (17, 1373) => LocalisedText {
            en: "chocobo raising",
            ja: "チョコボ育成",
            de: "Chocobo-Aufzucht",
            fr: "entraînement de chocobos",
        },
        (17, 1374) => LocalisedText {
            en: "weather report",
            ja: "天気予報",
            de: "Wetterbericht",
            fr: "Météorologie",
        },
        (17, 1375) => LocalisedText {
            en: "Allagan tomestone of poetics",
            ja: "アラガントームストーン:詩学",
            de: "Allagische Steine der Poesie",
            fr: "mémoquartz allagois poétiques",
        },
        (17, 1376) => LocalisedText {
            en: "quota points",
            ja: "貢献値",
            de: "Sollpunkte",
            fr: "Contribution",
        },
        (17, 1377) => LocalisedText {
            en: "company seals",
            ja: "軍票",
            de: "Staatstaler",
            fr: "sceaux de grande compagnie",
        },
        (17, 1378) => LocalisedText {
            en: "Wolf Marks",
            ja: "対人戦績",
            de: "Wolfsmarken",
            fr: "marques de loup",
        },
        (17, 1379) => LocalisedText {
            en: "Zenith",
            ja: "ゼニス",
            de: "Zenit",
            fr: "zénith",
        },
        (17, 1380) => LocalisedText {
            en: "Nexus",
            ja: "ネクサス",
            de: "Nexus",
            fr: "nexus",
        },
        (17, 1381) => LocalisedText {
            en: "Centurio Seal",
            ja: "セントリオ記章",
            de: "Centurio-Abzeichen",
            fr: "insignes Centurio",
        },
        (17, 1384) => LocalisedText {
            en: "Loot Rule: Lootmaster",
            ja: "マスタールート",
            de: "Beuteregel: Anführer",
            fr: "Règle du butin : maître du butin",
        },
        (17, 1385) => LocalisedText {
            en: "Loot Rule: Greed Only",
            ja: "GREED限定",
            de: "Beuteregel: Gier",
            fr: "Règle du butin : juste cupidité",
        },
        (17, 1386) => LocalisedText {
            en: "Loot Rule: Normal",
            ja: "通常ロット",
            de: "Beuteregel: Standard",
            fr: "Règle du butin : normale",
        },
        (17, 1387) => LocalisedText {
            en: "Anima Weapon",
            ja: "アニマウェポン",
            de: "Anima-Waffe",
            fr: "armes anima",
        },
        (17, 1389) => LocalisedText {
            en: "Ceremony of Eternal Bonding",
            ja: "エターナルバンド",
            de: "Zeremonie des Ewigen Bundes",
            fr: "lien éternel",
        },
        (17, 1390) => LocalisedText {
            en: "item level",
            ja: "アイテムレベル",
            de: "Gegenstandsstufe",
            fr: "niveau d'objet",
        },
        (17, 1391) => LocalisedText {
            en: "quest sync",
            ja: "レベルアジャスト",
            de: "Schwierigkeitsanpassung",
            fr: "synchronisation de quête",
        },
        (17, 1392) => LocalisedText {
            en: "Wondrous Tails",
            ja: "クロの空想帳",
            de: "Khloes Abenteueralbum",
            fr: "Aventures imaginaires de Khloe",
        },
        (17, 1396) => LocalisedText {
            en: "unrestricted party",
            ja: "制限解除",
            de: "Keine Beschränkungen",
            fr: "Sans restriction",
        },
        (17, 1397) => LocalisedText {
            en: "performance",
            ja: "楽器演奏",
            de: "Bardenkompositionen",
            fr: "Actions d'interprétation",
        },
        (17, 1398) => LocalisedText {
            en: "Fashion Report",
            ja: "ファッションチェック",
            de: "Kleiderkritik",
            fr: "Revue de mode",
        },
        (18, 1400) => LocalisedText {
            en: "【Battle】",
            ja: "【バトル】",
            de: "【Kampf】",
            fr: "【Combat】",
        },
        (18, 1401) => LocalisedText {
            en: "attack",
            ja: "攻撃",
            de: "Attacke",
            fr: "Attaque",
        },
        (18, 1402) => LocalisedText {
            en: "action",
            ja: "アクション",
            de: "Kommando",
            fr: "Action",
        },
        (18, 1403) => LocalisedText {
            en: "general action",
            ja: "ジェネラルアクション",
            de: "Allgemeines Kommando",
            fr: "Action générale",
        },
        (18, 1404) => LocalisedText {
            en: "trait",
            ja: "特性",
            de: "Eigenschaft",
            fr: "Trait",
        },
        (18, 1405) => LocalisedText {
            en: "job action",
            ja: "ジョブアクション",
            de: "Job-Kommando",
            fr: "Action de job",
        },
        (18, 1406) => LocalisedText {
            en: "additional action",
            ja: "アディショナルアクション",
            de: "Ergänzendes Kommando",
            fr: "Talent",
        },
        (18, 1407) => LocalisedText {
            en: "magic",
            ja: "魔法",
            de: "Magie",
            fr: "Magie",
        },
        (18, 1408) => LocalisedText {
            en: "weaponskill",
            ja: "ウェポンスキル",
            de: "Waffenfertigkeit",
            fr: "Technique d'arme",
        },
        (18, 1409) => LocalisedText {
            en: "ability",
            ja: "アビリティ",
            de: "Talent",
            fr: "Aptitude",
        },
        (18, 1410) => LocalisedText {
            en: "cast time",
            ja: "キャストタイム",
            de: "Aktivierungszeit",
            fr: "Temps de lancement",
        },
        (18, 1411) => LocalisedText {
            en: "recast time",
            ja: "リキャストタイム",
            de: "Reaktivierungszeit",
            fr: "Temps de recharge",
        },
        (18, 1412) => LocalisedText {
            en: "affinity",
            ja: "行動適性",
            de: "Affinität",
            fr: "Affinité",
        },
        (18, 1413) => LocalisedText {
            en: "link",
            ja: "リンク",
            de: "Verbundene Monster",
            fr: "Lien",
        },
        (18, 1414) => LocalisedText {
            en: "pet",
            ja: "ペット",
            de: "Familiar",
            fr: "Familier",
        },
        (18, 1415) => LocalisedText {
            en: "companion",
            ja: "バディ",
            de: "Mitstreiter",
            fr: "Compagnon",
        },
        (18, 1416) => LocalisedText {
            en: "enmity",
            ja: "敵視",
            de: "Feindseligkeit",
            fr: "Inimitié",
        },
        (18, 1417) => LocalisedText {
            en: "active",
            ja: "好戦的",
            de: "Aggressiv",
            fr: "Agressif",
        },
        (18, 1418) => LocalisedText {
            en: "passive",
            ja: "非好戦的",
            de: "Nicht aggressiv",
            fr: "Non agressif",
        },
        (18, 1419) => LocalisedText {
            en: "KO'd",
            ja: "戦闘不能",
            de: "K. o.",
            fr: "Inconscient",
        },
        (18, 1420) => LocalisedText {
            en: "loot",
            ja: "戦利品",
            de: "Beutegut",
            fr: "Butin",
        },
        (18, 1421) => LocalisedText {
            en: "divvy loot",
            ja: "分配",
            de: "Beutegut verteilen",
            fr: "Répartition",
        },
        (18, 1422) => LocalisedText {
            en: "lot",
            ja: "ロット",
            de: "Würfeln",
            fr: "Dés",
        },
        (18, 1423) => LocalisedText {
            en: "resist",
            ja: "レジスト",
            de: "Widerstehen",
            fr: "Résistance",
        },
        (18, 1424) => LocalisedText {
            en: "acquire",
            ja: "修得",
            de: "Erlernen",
            fr: "Obtention",
        },
        (18, 1425) => LocalisedText {
            en: "aggro",
            ja: "感知",
            de: "Aggro",
            fr: "Détection",
        },
        (18, 1426) => LocalisedText {
            en: "area of effect",
            ja: "範囲",
            de: "Wirkungsbereich",
            fr: "Aire d'effet",
        },
        (18, 1427) => LocalisedText {
            en: "limit break",
            ja: "リミットブレイク",
            de: "Limitrausch",
            fr: "Transcendance",
        },
        (18, 1428) => LocalisedText {
            en: "enemy",
            ja: "敵",
            de: "Feind",
            fr: "Ennemi",
        },
        (18, 1429) => LocalisedText {
            en: "monster",
            ja: "モンスター",
            de: "Monster",
            fr: "Monstre",
        },
        (18, 1430) => LocalisedText {
            en: "notorious monster",
            ja: "ノートリアスモンスター",
            de: "Notorisches Monster",
            fr: "Monstre célèbre",
        },
        (18, 1432) => LocalisedText {
            en: "EXP chain",
            ja: "チェーンボーナス",
            de: "Erfolgssträhne",
            fr: "Chaîne d'expérience",
        },
        (18, 1433) => LocalisedText {
            en: "Armoury bonus",
            ja: "アーマリーボーナス",
            de: "Arsenal-Bonus",
            fr: "Bonus d'arsenal",
        },
        (18, 1434) => LocalisedText {
            en: "level sync",
            ja: "レベルシンク",
            de: "Stufenanpassung",
            fr: "Synchronisation de niveau",
        },
        (18, 1435) => LocalisedText {
            en: "PvP action",
            ja: "PvPアクション",
            de: "PvP-Kommando",
            fr: "Action JcJ",
        },
        (18, 1436) => LocalisedText {
            en: "area of effect attack",
            ja: "範囲攻撃",
            de: "Flächenangriff",
            fr: "Attaque à aire d'effet",
        },
        (18, 1437) => LocalisedText {
            en: "frontal area of effect attack",
            ja: "前方範囲攻撃",
            de: "Flächenangriff (vorne)",
            fr: "Attaque à aire d'effet frontale",
        },
        (18, 1438) => LocalisedText {
            en: "rear area of effect attack",
            ja: "後方範囲攻撃",
            de: "Flächenangriff (hinten)",
            fr: "Attaque à aire d'effet arrière",
        },
        (18, 1439) => LocalisedText {
            en: "whole area of effect attack",
            ja: "全体攻撃",
            de: "Flächenangriff (rundum)",
            fr: "Attaque à aire d'effet globale",
        },
        (18, 1440) => LocalisedText {
            en: "outpost",
            ja: "アウトポスト",
            de: "Posten",
            fr: "Avant-poste",
        },
        (18, 1441) => LocalisedText {
            en: "Allagan Manors",
            ja: "居住区跡",
            de: "Allagischer Wohnbezirk",
            fr: "Habitations allagoises",
        },
        (18, 1442) => LocalisedText {
            en: "stronghold",
            ja: "拠点",
            de: "Schlüsselstellung",
            fr: "Poste",
        },
        (18, 1443) => LocalisedText {
            en: "standard",
            ja: "フラッグ",
            de: "Flagge",
            fr: "Drapeau",
        },
        (18, 1444) => LocalisedText {
            en: "occupied",
            ja: "占拠",
            de: "Erobert",
            fr: "Occupé",
        },
        (18, 1445) => LocalisedText {
            en: "unoccupied",
            ja: "中立化",
            de: "Neutral",
            fr: "Inoccupé",
        },
        (18, 1446) => LocalisedText {
            en: "Tactical Rating",
            ja: "戦術値",
            de: "Punkte",
            fr: "Score stratégique",
        },
        (18, 1447) => LocalisedText {
            en: "Allagan Markets",
            ja: "市場跡",
            de: "Allagischer Marktplatz",
            fr: "Marchés allagois",
        },
        (18, 1448) => LocalisedText {
            en: "Allagan Sun Temple",
            ja: "太陽神殿",
            de: "Allagischer Sonnentempel",
            fr: "Temple allagois du soleil",
        },
        (18, 1449) => LocalisedText {
            en: "combo",
            ja: "コンボ",
            de: "Kombo",
            fr: "Combo",
        },
        (18, 1450) => LocalisedText {
            en: "adrenaline rush",
            ja: "アドレナリンラッシュ",
            de: "Adrenalinrausch",
            fr: "Décharge d'adrénaline",
        },
        (18, 1451) => LocalisedText {
            en: "single target attack",
            ja: "単体攻撃",
            de: "Attacke auf Einzelziel",
            fr: "Attaque sur cible unique",
        },
        (18, 1452) => LocalisedText {
            en: "ballista",
            ja: "バリスタ",
            de: "Balliste",
            fr: "baliste",
        },
        (18, 1453) => LocalisedText {
            en: "Allagan tomelith",
            ja: "アラガントームリス",
            de: "Allagischer Echolith",
            fr: "mémolithe allagois",
        },
        (18, 1454) => LocalisedText {
            en: "The Claws",
            ja: "チーム・クロウ",
            de: "Klauen",
            fr: "les Griffes",
        },
        (18, 1455) => LocalisedText {
            en: "The Fangs",
            ja: "チーム・ファング",
            de: "Fänge",
            fr: "les Crocs",
        },
        (18, 1456) => LocalisedText {
            en: "medal",
            ja: "メダル",
            de: "Medaillen",
            fr: "médaille",
        },
        (18, 1458) => LocalisedText {
            en: "Culling Time",
            ja: "デンジャータイム",
            de: "Blutrausch",
            fr: "Phase de carnage",
        },
        (18, 1462) => LocalisedText {
            en: "adrenaline",
            ja: "アドレナリンボックス",
            de: "Adrenalin-Depot",
            fr: "ravitaillement adrénergétique",
        },
        (18, 1466) => LocalisedText {
            en: "adrenaline kit",
            ja: "アドレナリンキット",
            de: "Adrenalin-Set",
            fr: "stimulant adrénergétique",
        },
        (18, 1467) => LocalisedText {
            en: "supply box",
            ja: "サプライボックス",
            de: "Depot",
            fr: "caisse de ravitaillement",
        },
        (18, 1468) => LocalisedText {
            en: "K - Knockout",
            ja: "ノックアウト",
            de: "besiegte Gegner",
            fr: "K.O",
        },
        (18, 1469) => LocalisedText {
            en: "D - Down",
            ja: "ダウン",
            de: "eigene K. o.s",
            fr: "assommés",
        },
        (18, 1470) => LocalisedText {
            en: "A - Assist",
            ja: "アシスト",
            de: "Unterstützungen",
            fr: "soutiens",
        },
        (18, 1471) => LocalisedText {
            en: "chain",
            ja: "連続攻撃",
            de: "Dauerangriff",
            fr: "attaque continue",
        },
        (18, 1472) => LocalisedText {
            en: "charge",
            ja: "突進攻撃",
            de: "Frontalangriff",
            fr: "attaque frontale",
        },
        (18, 1473) => LocalisedText {
            en: "knockback",
            ja: "ノックバック",
            de: "Zurückstoßen",
            fr: "projection",
        },
        (18, 1474) => LocalisedText {
            en: "draw in",
            ja: "引き寄せ",
            de: "Heranziehen",
            fr: "attraction",
        },
        (18, 1475) => LocalisedText {
            en: "icebound tomelith",
            ja: "アイスドトームリス",
            de: "vereister Echolith",
            fr: "mémolithe congelé",
        },
        (18, 1476) => LocalisedText {
            en: "data retrieved",
            ja: "情報値",
            de: "Daten",
            fr: "données obtenues",
        },
        (18, 1477) => LocalisedText {
            en: "wolf's heart",
            ja: "ウルフハートボックス",
            de: "Wolfsherz-Depot",
            fr: "ravitaillement cœur-de-loup",
        },
        (18, 1478) => LocalisedText {
            en: "wolf's heart kit",
            ja: "ウルフハートキット",
            de: "Wolfsherz-Set",
            fr: "stimulant cœur-de-loup",
        },
        (18, 1479) => LocalisedText {
            en: "core",
            ja: "コア",
            de: "Kern",
            fr: "cœur",
        },
        (18, 1480) => LocalisedText {
            en: "magitek field",
            ja: "魔導フィールド",
            de: "Magitek-Kraftfeld",
            fr: "champ de force magitek",
        },
        (18, 1481) => LocalisedText {
            en: "tower",
            ja: "タワー",
            de: "Turm",
            fr: "tour",
        },
        (18, 1482) => LocalisedText {
            en: "hangar",
            ja: "ハンガー",
            de: "Hangar",
            fr: "hangar",
        },
        (18, 1483) => LocalisedText {
            en: "ceruleum",
            ja: "青燐水",
            de: "Erdseim",
            fr: "céruleum",
        },
        (18, 1484) => LocalisedText {
            en: "power generator",
            ja: "パワージェネレーター",
            de: "Generator",
            fr: "générateur d'énergie",
        },
        (18, 1485) => LocalisedText {
            en: "steam cannon",
            ja: "スチームカノン",
            de: "Dampfkanone",
            fr: "canon à vapeur",
        },
        (18, 1486) => LocalisedText {
            en: "wind-up viking",
            ja: "オートマトン・バイキング",
            de: "Wikinger-Marionette",
            fr: "automate viking",
        },
        (18, 1487) => LocalisedText {
            en: "wind-up magus",
            ja: "オートマトン・マグス",
            de: "Magier-Marionette",
            fr: "automate mage",
        },
        (18, 1488) => LocalisedText {
            en: "machina",
            ja: "機工兵器",
            de: "Kriegsmaschine",
            fr: "machine de guerre",
        },
        (18, 1489) => LocalisedText {
            en: "Cruise Chaser",
            ja: "クルーズチェイサー",
            de: "Chaser",
            fr: "croiseur",
        },
        (18, 1490) => LocalisedText {
            en: "Oppressor",
            ja: "オプレッサー",
            de: "Unterdrücker",
            fr: "oppresseur",
        },
        (18, 1491) => LocalisedText {
            en: "Brute Justice",
            ja: "ブルートジャスティス",
            de: "Brutalus",
            fr: "justicier",
        },
        (18, 1492) => LocalisedText {
            en: "role action",
            ja: "ロールアクション",
            de: "Rollenkommando",
            fr: "Action de rôle",
        },
        (18, 1494) => LocalisedText {
            en: "quick chat",
            ja: "クイックチャット",
            de: "Schnellchat",
            fr: "Expressions figées",
        },
        (18, 1496) => LocalisedText {
            en: "ceruleum engine",
            ja: "青燐機関車",
            de: "Erdseim-Zug",
            fr: "locomotive à céruleum",
        },
        (18, 1497) => LocalisedText {
            en: "gobtank",
            ja: "ゴブリタンク",
            de: "Gobmaschine",
            fr: "gobtank",
        },
        (18, 1498) => LocalisedText {
            en: "goblin mercenary",
            ja: "ゴブリン・マーシナリー",
            de: "Goblin-Söldner",
            fr: "mercenaire gobelin",
        },
        (18, 1499) => LocalisedText {
            en: "tactical rating",
            ja: "戦略値",
            de: "Siegpunkte",
            fr: "Score tactique",
        },
        (19, 1500) => LocalisedText {
            en: "【Online Status】",
            ja: "【オンラインステータス】",
            de: "【Online-Status】",
            fr: "【État en ligne】",
        },
        (19, 1501) => LocalisedText {
            en: "Online",
            ja: "オンライン",
            de: "Online",
            fr: "En ligne",
        },
        (19, 1502) => LocalisedText {
            en: "Away",
            ja: "離席",
            de: "Abwesend",
            fr: "Absent(e)",
        },
        (19, 1503) => LocalisedText {
            en: "Busy",
            ja: "取り込み中",
            de: "Beschäftigt",
            fr: "Occupé(e)",
        },
        (19, 1504) => LocalisedText {
            en: "Looking for Party",
            ja: "パーティ希望",
            de: "Sucht Gruppe",
            fr: "Cherche une équipe",
        },
        (19, 1505) => LocalisedText {
            en: "Looking to Meld Materia",
            ja: "マテリア装着請負中",
            de: "Setzt Materia ein",
            fr: "Offre de sertir des matérias",
        },
        (19, 1506) => LocalisedText {
            en: "Bound by Duty",
            ja: "コンテンツ中",
            de: "In einem Inhalt",
            fr: "En mission",
        },
        (19, 1507) => LocalisedText {
            en: "Offline",
            ja: "オフライン",
            de: "Offline",
            fr: "Hors ligne",
        },
        (19, 1508) => LocalisedText {
            en: "Recruiting Party Members",
            ja: "パーティ募集中",
            de: "In Gruppensuche",
            fr: "Recrute des équipiers",
        },
        (19, 1509) => LocalisedText {
            en: "New Adventurer",
            ja: "ビギナー",
            de: "Neuling",
            fr: "Novice",
        },
        (19, 1510) => LocalisedText {
            en: "Mentor",
            ja: "メンター",
            de: "Mentor",
            fr: "Mentor",
        },
        (19, 1511) => LocalisedText {
            en: "PvE Mentor",
            ja: "バトルメンター",
            de: "Kampf-Mentor",
            fr: "Mentor (combat)",
        },
        (19, 1512) => LocalisedText {
            en: "PvP Mentor",
            ja: "PvPメンター",
            de: "PvP-Mentor",
            fr: "Mentor (JcJ)",
        },
        (19, 1513) => LocalisedText {
            en: "Trade Mentor",
            ja: "製作採集メンター",
            de: "Handwerker-/Sammler-Mentor",
            fr: "Mentor (artisanat/récolte)",
        },
        (19, 1514) => LocalisedText {
            en: "Role-playing",
            ja: "ロールプレイ中",
            de: "Im Rollenspiel",
            fr: "Jeu de rôle",
        },
        (19, 1515) => LocalisedText {
            en: "Returner",
            ja: "リターナー",
            de: "Rückkehrer",
            fr: "Rappelé(e)",
        },
        (19, 1516) => LocalisedText {
            en: "Participating in Event",
            ja: "イベント中",
            de: "Nimmt an Ereignis teil",
            fr: "Dans un événement",
        },
        (18, 1521) => LocalisedText {
            en: "ovoo",
            ja: "無垢の土地",
            de: "Unschuldsfleck",
            fr: "terre immaculée",
        },
        (18, 1522) => LocalisedText {
            en: "claim",
            ja: "契約",
            de: "Vereinigen",
            fr: "pacte",
        },
        (18, 1523) => LocalisedText {
            en: "tactical crystal",
            ja: "タクティカルクリスタル",
            de: "Taktischer Kristall",
            fr: "cristal tactique",
        },
        (18, 1524) => LocalisedText {
            en: "Crystal Credit",
            ja: "クリスタルポイント",
            de: "Kristallpunkt",
            fr: "points Crystal",
        },
        (18, 1525) => LocalisedText {
            en: "Rising Stars",
            ja: "勝ち星",
            de: "Siegessterne",
            fr: "étoiles de victoire",
        },
        (18, 1526) => LocalisedText {
            en: "Team Astra",
            ja: "チーム・アストラ",
            de: "Team Astra",
            fr: "équipe Astra",
        },
        (18, 1527) => LocalisedText {
            en: "Team Umbra",
            ja: "チーム・アンブラ",
            de: "Team Umbra",
            fr: "équipe Umbra",
        },
        (20, 1550) => LocalisedText {
            en: "【Communication】",
            ja: "【コミュニケーション】",
            de: "【Soziales】",
            fr: "【Communication】",
        },
        (20, 1551) => LocalisedText {
            en: "chat",
            ja: "チャット",
            de: "Chat",
            fr: "Discussion",
        },
        (20, 1552) => LocalisedText {
            en: "Tell",
            ja: "Tell",
            de: "Flüstern",
            fr: "Murmurer",
        },
        (20, 1553) => LocalisedText {
            en: "Say",
            ja: "Say",
            de: "Sagen",
            fr: "Dire",
        },
        (20, 1554) => LocalisedText {
            en: "Shout",
            ja: "Shout",
            de: "Schreien",
            fr: "Hurler",
        },
        (20, 1555) => LocalisedText {
            en: "Yell",
            ja: "Yell",
            de: "Rufen",
            fr: "Crier",
        },
        (20, 1556) => LocalisedText {
            en: "friend list",
            ja: "フレンドリスト",
            de: "Freundesliste",
            fr: "Liste d'amis",
        },
        (20, 1557) => LocalisedText {
            en: "blacklist",
            ja: "ブラックリスト",
            de: "Schwarze Liste",
            fr: "Liste noire",
        },
        (20, 1558) => LocalisedText {
            en: "Player Search",
            ja: "PCサーチ",
            de: "Spielersuche",
            fr: "Recherche de joueurs",
        },
        (20, 1559) => LocalisedText {
            en: "linkshell",
            ja: "リンクシェル",
            de: "Kontaktkreis",
            fr: "Linkshell",
        },
        (20, 1560) => LocalisedText {
            en: "moogle letter",
            ja: "モグレター",
            de: "Mogry-Post",
            fr: "Lettre mog",
        },
        (20, 1561) => LocalisedText {
            en: "Lodestone",
            ja: "ロードストーン",
            de: "Lodestone",
            fr: "Lodestone",
        },
        (20, 1562) => LocalisedText {
            en: "blog",
            ja: "日記",
            de: "Blog",
            fr: "Blog",
        },
        (20, 1563) => LocalisedText {
            en: "follow",
            ja: "フォロー",
            de: "Folgen",
            fr: "Suivre",
        },
        (20, 1564) => LocalisedText {
            en: "forums",
            ja: "フォーラム",
            de: "Forum",
            fr: "Forums",
        },
        (20, 1565) => LocalisedText {
            en: "ranking",
            ja: "ランキング",
            de: "Ranglisten",
            fr: "Classements",
        },
        (20, 1566) => LocalisedText {
            en: "Grand Company ranking",
            ja: "グランドカンパニーランキング",
            de: "Rangliste der Staatlichen Gesellschaften",
            fr: "Classement des grandes compagnies",
        },
        (20, 1567) => LocalisedText {
            en: "free company ranking",
            ja: "フリーカンパニーランキング",
            de: "Rangliste der Freien Gesellschaften",
            fr: "Classement des compagnies libres",
        },
        (20, 1568) => LocalisedText {
            en: "party",
            ja: "Party",
            de: "Gruppe",
            fr: "Équipe",
        },
        (20, 1569) => LocalisedText {
            en: "alliance",
            ja: "Alliance",
            de: "Allianz",
            fr: "Alliance",
        },
        (20, 1570) => LocalisedText {
            en: "linkshell",
            ja: "Linkshell",
            de: "Kontaktkreis",
            fr: "Linkshell",
        },
        (20, 1571) => LocalisedText {
            en: "free company",
            ja: "FreeCompany",
            de: "Freie Gesellschaft",
            fr: "Compagnie libre",
        },
        (20, 1572) => LocalisedText {
            en: "Novice Network",
            ja: "Beginner",
            de: "Neulings-Chat",
            fr: "Réseau des novices",
        },
        (20, 1573) => LocalisedText {
            en: "PvP Team",
            ja: "PvPTeam",
            de: "PvP-Team",
            fr: "Équipe JcJ",
        },
        (20, 1574) => LocalisedText {
            en: "cross-world linkshell",
            ja: "CrossWorldLinkshell",
            de: "Welten-Kontaktkreis",
            fr: "Linkshell inter-Monde",
        },
        (20, 1575) => LocalisedText {
            en: "cross-world linkshell",
            ja: "クロスワールドリンクシェル",
            de: "Welten-Kontaktkreis",
            fr: "Linkshell inter-Monde",
        },
        (20, 1576) => LocalisedText {
            en: "contacts",
            ja: "コンタクトリスト",
            de: "Gruppenverlauf",
            fr: "Liste de contacts",
        },
        (20, 1577) => LocalisedText {
            en: "Community Finder",
            ja: "コミュニティファインダー",
            de: "Community-Suche",
            fr: "Recherche d'aventuriers",
        },
        (22, 1600) => LocalisedText {
            en: "【Crafting & Gathering】",
            ja: "【製作&採集】",
            de: "【Sammeln & Synthese】",
            fr: "【Artisanat et récolte】",
        },
        (22, 1601) => LocalisedText {
            en: "shard",
            ja: "シャード",
            de: "Scherbe",
            fr: "Éclat",
        },
        (22, 1602) => LocalisedText {
            en: "crystal",
            ja: "クリスタル",
            de: "Kristall",
            fr: "Cristal",
        },
        (22, 1603) => LocalisedText {
            en: "cluster",
            ja: "クラスター",
            de: "Polykristall",
            fr: "Agrégat",
        },
        (22, 1604) => LocalisedText {
            en: "materia",
            ja: "マテリア",
            de: "Materia",
            fr: "Matéria",
        },
        (22, 1605) => LocalisedText {
            en: "materia extraction",
            ja: "マテリア精製",
            de: "Materia extrahieren",
            fr: "Matérialisation",
        },
        (22, 1606) => LocalisedText {
            en: "materia craft",
            ja: "マテリアクラフト",
            de: "Materia-Handwerk",
            fr: "Artisanat de matéria",
        },
        (22, 1607) => LocalisedText {
            en: "markets",
            ja: "マーケット",
            de: "Markt",
            fr: "Marché",
        },
        (22, 1608) => LocalisedText {
            en: "market search",
            ja: "マーケットサーチ",
            de: "Gegenstandssuche",
            fr: "Recherche d'objet",
        },
        (22, 1609) => LocalisedText {
            en: "HQ",
            ja: "HQ",
            de: "HQ",
            fr: "HQ",
        },
        (22, 1610) => LocalisedText {
            en: "Unique",
            ja: "Rare",
            de: "Selten",
            fr: "Unique",
        },
        (22, 1611) => LocalisedText {
            en: "Untradable",
            ja: "Ex",
            de: "Persönlich gebunden",
            fr: "Personnel",
        },
        (22, 1612) => LocalisedText {
            en: "material",
            ja: "素材",
            de: "Material",
            fr: "Matériau",
        },
        (22, 1613) => LocalisedText {
            en: "quick synthesis",
            ja: "簡易製作",
            de: "Eilsynthese",
            fr: "Synthèse rapide",
        },
        (22, 1614) => LocalisedText {
            en: "quality",
            ja: "品質",
            de: "Qualität",
            fr: "Qualité",
        },
        (22, 1615) => LocalisedText {
            en: "high-quality",
            ja: "高品質",
            de: "Hohe Qualität",
            fr: "Haute qualité",
        },
        (22, 1616) => LocalisedText {
            en: "highest-quality",
            ja: "最高品質",
            de: "Höchste Qualität",
            fr: "Excellente qualité",
        },
        (22, 1617) => LocalisedText {
            en: "repair",
            ja: "修理",
            de: "Reparieren",
            fr: "Réparer",
        },
        (22, 1618) => LocalisedText {
            en: "repair materials",
            ja: "修理材",
            de: "Material für Reparatur",
            fr: "À réparer",
        },
        (22, 1619) => LocalisedText {
            en: "gear dyeing",
            ja: "染色",
            de: "Färben von Gegenständen",
            fr: "Teinture",
        },
        (22, 1620) => LocalisedText {
            en: "mature tree",
            ja: "良木",
            de: "Nutzbaum",
            fr: "Arbre mature",
        },
        (22, 1621) => LocalisedText {
            en: "logging",
            ja: "伐採",
            de: "Abholzen",
            fr: "Coupe",
        },
        (22, 1622) => LocalisedText {
            en: "harvesting",
            ja: "草刈",
            de: "Abernten",
            fr: "Fauche",
        },
        (22, 1623) => LocalisedText {
            en: "mineral deposit",
            ja: "採掘場",
            de: "Lagerstätte",
            fr: "Filon de minerai",
        },
        (22, 1624) => LocalisedText {
            en: "mining",
            ja: "採掘",
            de: "Abbauen",
            fr: "Extraction de minerai",
        },
        (22, 1625) => LocalisedText {
            en: "quarrying",
            ja: "砕岩",
            de: "Herausbrechen",
            fr: "Extraction de pierre",
        },
        (22, 1626) => LocalisedText {
            en: "fishing",
            ja: "釣り",
            de: "Fischen",
            fr: "Pêche",
        },
        (22, 1627) => LocalisedText {
            en: "materia melding",
            ja: "マテリア装着",
            de: "Materia einsetzen",
            fr: "Sertissage",
        },
        (22, 1628) => LocalisedText {
            en: "advanced materia melding",
            ja: "禁断のマテリア装着",
            de: "Mehr als eine Materia einsetzen",
            fr: "Sertissage avancé",
        },
        (22, 1629) => LocalisedText {
            en: "gardening",
            ja: "栽培",
            de: "Gartenarbeit",
            fr: "Jardinage",
        },
        (22, 1630) => LocalisedText {
            en: "raising",
            ja: "育成",
            de: "Gartenpflege",
            fr: "Culture (jardinage)",
        },
        (22, 1631) => LocalisedText {
            en: "harvesting",
            ja: "収穫",
            de: "Ernte",
            fr: "Récolte (jardinage)",
        },
        (22, 1632) => LocalisedText {
            en: "master recipe book",
            ja: "秘伝書",
            de: "Meisterrezepte",
            fr: "Recettes expertes",
        },
        (22, 1633) => LocalisedText {
            en: "desynthesis",
            ja: "分解",
            de: "Materialverwertung",
            fr: "recyclage",
        },
        (22, 1634) => LocalisedText {
            en: "big fishing",
            ja: "ヌシ釣り",
            de: "Kapitalfische",
            fr: "Défi de pêche",
        },
        (22, 1635) => LocalisedText {
            en: "release",
            ja: "リリース",
            de: "Freilassen",
            fr: "Remise à l'eau",
        },
        (22, 1636) => LocalisedText {
            en: "unspoiled locations",
            ja: "未知の採集場所",
            de: "Unbekannter Ort",
            fr: "Point de récolte intact",
        },
        (22, 1637) => LocalisedText {
            en: "collectable",
            ja: "収集品",
            de: "Sammlerstück",
            fr: "Objet de collection",
        },
        (22, 1638) => LocalisedText {
            en: "specialist",
            ja: "マイスター",
            de: "Spezialist",
            fr: "spécialiste",
        },
        (22, 1639) => LocalisedText {
            en: "DoL favors",
            ja: "護符",
            de: "Gunst-Talismane",
            fr: "talisman",
        },
        (22, 1640) => LocalisedText {
            en: "tome of regional folklore",
            ja: "伝承録",
            de: "Folklore-Almanach",
            fr: "Légendes locales",
        },
        (22, 1642) => LocalisedText {
            en: "gatherers' scrip",
            ja: "ギャザラースクリップ",
            de: "Sammlerschein",
            fr: "assignats de récolteur",
        },
        (22, 1647) => LocalisedText {
            en: "yellow crafters' scrip",
            ja: "クラフタースクリップ:黄貨",
            de: "Gelber Handwerkerschein",
            fr: "assignats jaunes d'artisan",
        },
        (22, 1648) => LocalisedText {
            en: "yellow gatherers' scrip",
            ja: "ギャザラースクリップ:黄貨",
            de: "Gelber Sammlerschein",
            fr: "assignats jaunes de récolteur",
        },
        (22, 1649) => LocalisedText {
            en: "white crafters' scrip",
            ja: "クラフタースクリップ:白貨",
            de: "Weißer Handwerkerschein",
            fr: "assignat blanc d'artisan",
        },
        (22, 1650) => LocalisedText {
            en: "white gatherers' scrip",
            ja: "ギャザラースクリップ:白貨",
            de: "Weißer Sammlerschein",
            fr: "assignat blanc de récolteur",
        },
        (22, 1651) => LocalisedText {
            en: "quick gathering",
            ja: "簡易採集",
            de: "Vereinfachtes Aufnehmen",
            fr: "récolte automatique",
        },
        (22, 1652) => LocalisedText {
            en: "trial synthesis",
            ja: "製作練習",
            de: "Testsynthese",
            fr: "synthèse d'essai",
        },
        (22, 1653) => LocalisedText {
            en: "skybuilders' scrip",
            ja: "蒼天街振興券",
            de: "Aufbauschein",
            fr: "assignats d'Azurée",
        },
        (22, 1654) => LocalisedText {
            en: "collectability",
            ja: "収集価値",
            de: "Sammlerwert",
            fr: "valeur de collection",
        },
        (22, 1655) => LocalisedText {
            en: "purple crafters' scrip",
            ja: "クラフタースクリップ:紫貨",
            de: "Violetter Handwerkerschein",
            fr: "assignats mauves d'artisan",
        },
        (22, 1656) => LocalisedText {
            en: "purple gatherers' scrip",
            ja: "ギャザラースクリップ:紫貨",
            de: "Violetter Sammlerschein",
            fr: "assignats mauves de récolteur",
        },
        (22, 1657) => LocalisedText {
            en: "spearfishing",
            ja: "刺突漁",
            de: "Speerfischen",
            fr: "pêche au harpon",
        },
        (23, 1900) => LocalisedText {
            en: "【Primals】",
            ja: "【蛮神&蛮族】",
            de: "【Primae & Stämme】",
            fr: "【Primordiaux】",
        },
        (23, 1901) => LocalisedText {
            en: "primal",
            ja: "蛮神",
            de: "Primae",
            fr: "Primordial",
        },
        (23, 1902) => LocalisedText {
            en: "Ifrit",
            ja: "イフリート",
            de: "Ifrit",
            fr: "Ifrit",
        },
        (23, 1903) => LocalisedText {
            en: "Titan",
            ja: "タイタン",
            de: "Titan",
            fr: "Titan",
        },
        (23, 1904) => LocalisedText {
            en: "Garuda",
            ja: "ガルーダ",
            de: "Garuda",
            fr: "Garuda",
        },
        (23, 1905) => LocalisedText {
            en: "Bahamut",
            ja: "バハムート",
            de: "Bahamut",
            fr: "Bahamut",
        },
        (23, 1906) => LocalisedText {
            en: "Moggle Mog XII",
            ja: "モグル・モグXII世",
            de: "Mogul Mog",
            fr: "Moggle Mog XII",
        },
        (23, 1907) => LocalisedText {
            en: "beastmen",
            ja: "蛮族",
            de: "Wilde Stämme",
            fr: "Hommes-bêtes",
        },
        (23, 1908) => LocalisedText {
            en: "Amalj'aa",
            ja: "アマルジャ",
            de: "Amalj'aa",
            fr: "Amalj'aa",
        },
        (23, 1909) => LocalisedText {
            en: "sylphs",
            ja: "シルフ",
            de: "Sylphen",
            fr: "Sylphes",
        },
        (23, 1910) => LocalisedText {
            en: "Ixal",
            ja: "イクサル",
            de: "Ixal",
            fr: "Ixali",
        },
        (23, 1911) => LocalisedText {
            en: "kobolds",
            ja: "コボルド",
            de: "Kobolde",
            fr: "Kobolds",
        },
        (23, 1912) => LocalisedText {
            en: "Sahagin",
            ja: "サハギン",
            de: "Sahagin",
            fr: "Sahuagins",
        },
        (23, 1913) => LocalisedText {
            en: "Leviathan",
            ja: "リヴァイアサン",
            de: "Leviathan",
            fr: "Léviathan",
        },
        (23, 1914) => LocalisedText {
            en: "Ramuh",
            ja: "ラムウ",
            de: "Ramuh",
            fr: "Ramuh",
        },
        (23, 1915) => LocalisedText {
            en: "Shiva",
            ja: "シヴァ",
            de: "Shiva",
            fr: "Shiva",
        },
        (23, 1916) => LocalisedText {
            en: "Alexander (primal)",
            ja: "アレキサンダー",
            de: "Alexander",
            fr: "Alexander",
        },
        (23, 1917) => LocalisedText {
            en: "Ravana",
            ja: "ラーヴァナ",
            de: "Ravana",
            fr: "Ravana",
        },
        (23, 1918) => LocalisedText {
            en: "Bismarck",
            ja: "ビスマルク",
            de: "Bismarck",
            fr: "Bismarck",
        },
        (23, 1919) => LocalisedText {
            en: "Gnath",
            ja: "グナース",
            de: "Gnath",
            fr: "Gnathes",
        },
        (23, 1920) => LocalisedText {
            en: "Vanu Vanu",
            ja: "バヌバヌ",
            de: "Vanu Vanu",
            fr: "Vanu Vanu",
        },
        (23, 1921) => LocalisedText {
            en: "goblins",
            ja: "ゴブリン",
            de: "Goblins",
            fr: "Gobelins",
        },
        (23, 1922) => LocalisedText {
            en: "dragons",
            ja: "ドラゴン",
            de: "Drachen",
            fr: "Dragons",
        },
        (23, 1923) => LocalisedText {
            en: "Nidhogg",
            ja: "ニーズヘッグ",
            de: "Nidhogg",
            fr: "Nidhogg",
        },
        (23, 1924) => LocalisedText {
            en: "Sephirot",
            ja: "セフィロト",
            de: "Sephirot",
            fr: "Sephirot",
        },
        (23, 1925) => LocalisedText {
            en: "Sophia",
            ja: "ソフィア",
            de: "Sophia",
            fr: "Sophia",
        },
        (23, 1926) => LocalisedText {
            en: "Zurvan",
            ja: "ズルワーン",
            de: "Zurvan",
            fr: "Zurvan",
        },
        (23, 1927) => LocalisedText {
            en: "Kojin",
            ja: "コウジン",
            de: "Kojin",
            fr: "Kojin",
        },
        (23, 1928) => LocalisedText {
            en: "Ananta",
            ja: "アナンタ",
            de: "Ananta",
            fr: "Ananta",
        },
        (23, 1929) => LocalisedText {
            en: "Susano",
            ja: "スサノオ",
            de: "Susano",
            fr: "Susano",
        },
        (23, 1930) => LocalisedText {
            en: "Lakshmi",
            ja: "ラクシュミ",
            de: "Lakshmi",
            fr: "Lakshmi",
        },
        (23, 1931) => LocalisedText {
            en: "Shinryu",
            ja: "神龍",
            de: "Shinryu",
            fr: "Shinryu",
        },
        (23, 1932) => LocalisedText {
            en: "Omega",
            ja: "オメガ",
            de: "Omega",
            fr: "Oméga",
        },
        (23, 1933) => LocalisedText {
            en: "Byakko",
            ja: "白虎",
            de: "Byakko",
            fr: "Byakko",
        },
        (23, 1934) => LocalisedText {
            en: "Tsukuyomi",
            ja: "ツクヨミ",
            de: "Tsukuyomi",
            fr: "Tsukuyomi",
        },
        (23, 1935) => LocalisedText {
            en: "moogle",
            ja: "モーグリ",
            de: "Mogry",
            fr: "Mogs",
        },
        (23, 1936) => LocalisedText {
            en: "Namazu",
            ja: "ナマズオ",
            de: "Namazuo",
            fr: "Namazu",
        },
        (23, 1937) => LocalisedText {
            en: "Phoenix",
            ja: "フェニックス",
            de: "Phönix",
            fr: "Phénix",
        },
        (23, 1938) => LocalisedText {
            en: "Odin",
            ja: "オーディン",
            de: "Odin",
            fr: "Odin",
        },
        (23, 1939) => LocalisedText {
            en: "Suzaku",
            ja: "朱雀",
            de: "Suzaku",
            fr: "Suzaku",
        },
        (23, 1940) => LocalisedText {
            en: "Seiryu",
            ja: "青龍",
            de: "Seiryu",
            fr: "Seiryû",
        },
        (23, 1941) => LocalisedText {
            en: "Titania",
            ja: "ティターニア",
            de: "Titania",
            fr: "Titania",
        },
        (23, 1942) => LocalisedText {
            en: "Innocence",
            ja: "イノセンス",
            de: "Innozenz",
            fr: "Innocence",
        },
        (23, 1943) => LocalisedText {
            en: "Hades",
            ja: "ハーデス",
            de: "Hades",
            fr: "Hadès",
        },
        (23, 1944) => LocalisedText {
            en: "Dwarf",
            ja: "ドワーフ",
            de: "Zwerg",
            fr: "Nain",
        },
        (23, 1945) => LocalisedText {
            en: "Pixie",
            ja: "ピクシー",
            de: "Pixie",
            fr: "Pixie",
        },
        (23, 1946) => LocalisedText {
            en: "Nu Mou",
            ja: "ン・モゥ",
            de: "Nu Mou",
            fr: "Nu Mou",
        },
        (23, 1947) => LocalisedText {
            en: "Qitari",
            ja: "キタリ",
            de: "Qitari",
            fr: "Qitari",
        },
        (23, 1948) => LocalisedText {
            en: "Ruby Weapon",
            ja: "ルビーウェポン",
            de: "Rubin-Waffe",
            fr: "Arme Rubis",
        },
        (23, 1949) => LocalisedText {
            en: "Warrior of Light",
            ja: "ウォーリア・オブ・ライト",
            de: "Krieger des Lichts (Charakter)",
            fr: "Guerrier de la Lumière primordial",
        },
        (23, 1950) => LocalisedText {
            en: "Emerald Weapon",
            ja: "エメラルドウェポン",
            de: "Smaragd-Waffe",
            fr: "Arme Émeraude",
        },
        (23, 1951) => LocalisedText {
            en: "Diamond Weapon",
            ja: "ダイヤウェポン",
            de: "Diamant-Waffe",
            fr: "Arme Diamant",
        },
        (23, 1952) => LocalisedText {
            en: "Matanga",
            ja: "マタンガ",
            de: "Matanga",
            fr: "Matanga",
        },
        (23, 1953) => LocalisedText {
            en: "Arkasodara",
            ja: "アルカソーダラ",
            de: "Arkasodara",
            fr: "Arkasodara",
        },
        (23, 1954) => LocalisedText {
            en: "Loporrit",
            ja: "レポリット",
            de: "Loporrit",
            fr: "Loporrite",
        },
        (23, 1955) => LocalisedText {
            en: "Zodiark",
            ja: "ゾディアーク",
            de: "Zodiark",
            fr: "Zordiarche",
        },
        (23, 1956) => LocalisedText {
            en: "Hydaelyn",
            ja: "ハイデリン",
            de: "Hydaelyn",
            fr: "Hydaelyn",
        },
        (24, 2000) => LocalisedText {
            en: "【Duty】",
            ja: "【コンテンツ】",
            de: "【Inhalte】",
            fr: "【Missions】",
        },
        (24, 2001) => LocalisedText {
            en: "duty",
            ja: "コンテンツ",
            de: "Spielinhalt",
            fr: "Mission",
        },
        (24, 2002) => LocalisedText {
            en: "quest",
            ja: "クエスト",
            de: "Auftrag",
            fr: "Quête",
        },
        (24, 2003) => LocalisedText {
            en: "main scenario quest",
            ja: "メインクエスト",
            de: "Hauptszenario-Auftrag",
            fr: "Quête de l'épopée",
        },
        (24, 2004) => LocalisedText {
            en: "sidequest",
            ja: "サブクエスト",
            de: "Miniauftrag",
            fr: "Quête annexe",
        },
        (24, 2005) => LocalisedText {
            en: "class quest",
            ja: "クラスクエスト",
            de: "Gildenauftrag",
            fr: "Quête de classe",
        },
        (24, 2006) => LocalisedText {
            en: "job quest",
            ja: "ジョブクエスト",
            de: "Jobauftrag",
            fr: "Quête de job",
        },
        (24, 2007) => LocalisedText {
            en: "Grand Company quest",
            ja: "グランドカンパニークエスト",
            de: "Staatlicher Auftrag",
            fr: "Quête de grande compagnie",
        },
        (24, 2008) => LocalisedText {
            en: "FATE",
            ja: "FATE",
            de: "FATE",
            fr: "ALÉA",
        },
        (24, 2009) => LocalisedText {
            en: "guildleve",
            ja: "ギルドリーヴ",
            de: "Gildenfreibrief",
            fr: "Mandat de guilde",
        },
        (24, 2010) => LocalisedText {
            en: "levequest",
            ja: "リーヴ",
            de: "Freibrief",
            fr: "Mandat",
        },
        (24, 2011) => LocalisedText {
            en: "battlecraft leve",
            ja: "傭兵稼業",
            de: "Gefechtserlass",
            fr: "Mandat de mercenariat",
        },
        (24, 2012) => LocalisedText {
            en: "fieldcraft leve",
            ja: "採集稼業",
            de: "Sammelerlass",
            fr: "Mandat de récolte",
        },
        (24, 2013) => LocalisedText {
            en: "tradecraft leve",
            ja: "製作稼業",
            de: "Fertigungserlass",
            fr: "Mandat d'artisanat",
        },
        (24, 2015) => LocalisedText {
            en: "Grand Company leve",
            ja: "グランドカンパニーリーヴ",
            de: "Staatserlass",
            fr: "Mandat de grande compagnie",
        },
        (24, 2017) => LocalisedText {
            en: "leve link",
            ja: "リーヴリンク",
            de: "Verbundschein",
            fr: "Lien de mandat",
        },
        (24, 2018) => LocalisedText {
            en: "leve allowance",
            ja: "リーヴの受注権",
            de: "Freibrief-Vollmachten",
            fr: "Autorisation",
        },
        (24, 2019) => LocalisedText {
            en: "levequest difficulty",
            ja: "リーヴの難易度",
            de: "Freibrief-Schwierigkeit",
            fr: "Difficulté du mandat",
        },
        (24, 2020) => LocalisedText {
            en: "guildhest",
            ja: "ギルドオーダー",
            de: "Gildengeheiß",
            fr: "Opération de guilde",
        },
        (24, 2021) => LocalisedText {
            en: "dungeon",
            ja: "ダンジョン",
            de: "Dungeon",
            fr: "Donjon",
        },
        (24, 2022) => LocalisedText {
            en: "primal battle",
            ja: "討伐戦",
            de: "Primae-Kampf/Prüfung",
            fr: "Combat contre un Primordial",
        },
        (24, 2023) => LocalisedText {
            en: "hard primal battle",
            ja: "討滅戦",
            de: "Primae-Kampf (Götterdämmerung/Zenit)",
            fr: "Combat contre un Primordial (brutal)",
        },
        (24, 2024) => LocalisedText {
            en: "legatus battle",
            ja: "強襲戦",
            de: "Kampf gegen Legatus",
            fr: "Combat contre un légat",
        },
        (24, 2025) => LocalisedText {
            en: "raid",
            ja: "レイド",
            de: "Raid",
            fr: "Raid",
        },
        (24, 2026) => LocalisedText {
            en: "The Coliseum",
            ja: "コロセウム",
            de: "Kolosseum",
            fr: "Colisée",
        },
        (24, 2027) => LocalisedText {
            en: "The Thousand Maws of Toto-Rak",
            ja: "トトラクの千獄",
            de: "Tausend Löcher von Toto-Rak",
            fr: "Les Mille Gueules de Toto-Rak",
        },
        (24, 2028) => LocalisedText {
            en: "The Tam-Tara Deepcroft",
            ja: "タムタラの墓所",
            de: "Totenacker Tam-Tara",
            fr: "L'Hypogée de Tam-Tara",
        },
        (24, 2029) => LocalisedText {
            en: "Copperbell Mines",
            ja: "カッパーベル銅山",
            de: "Kupferglocken-Mine",
            fr: "Les Mines de Clochecuivre",
        },
        (24, 2030) => LocalisedText {
            en: "Sastasha",
            ja: "サスタシャ浸食洞",
            de: "Sastasha",
            fr: "Sastasha",
        },
        (24, 2031) => LocalisedText {
            en: "The Aurum Vale",
            ja: "オーラムヴェイル",
            de: "Goldklamm",
            fr: "Le Val d'Aurum",
        },
        (24, 2032) => LocalisedText {
            en: "Haukke Manor",
            ja: "ハウケタ御用邸",
            de: "Haukke-Herrenhaus",
            fr: "Le Manoir des Haukke",
        },
        (24, 2033) => LocalisedText {
            en: "Halatali",
            ja: "ハラタリ修練所",
            de: "Halatali",
            fr: "Halatali",
        },
        (24, 2034) => LocalisedText {
            en: "Brayflox's Longstop",
            ja: "ブレイフロクスの野営地",
            de: "Brüllvolx' Langrast",
            fr: "Le Bivouac de Brayflox",
        },
        (24, 2035) => LocalisedText {
            en: "The Sunken Temple of Qarn",
            ja: "カルン埋没寺院",
            de: "Versunkener Tempel von Qarn",
            fr: "Le Temple enseveli de Qarn",
        },
        (24, 2036) => LocalisedText {
            en: "The Wanderer's Palace",
            ja: "ワンダラーパレス",
            de: "Palast des Wanderers",
            fr: "Le Palais du Vagabond",
        },
        (24, 2037) => LocalisedText {
            en: "The Stone Vigil",
            ja: "ストーンヴィジル",
            de: "Steinerne Wacht",
            fr: "Le Vigile de Pierre",
        },
        (24, 2038) => LocalisedText {
            en: "Cutter's Cry",
            ja: "カッターズクライ",
            de: "Sägerschrei",
            fr: "Le Gouffre hurlant",
        },
        (24, 2039) => LocalisedText {
            en: "Dzemael Darkhold",
            ja: "ゼーメル要塞",
            de: "Feste Dzemael",
            fr: "La Forteresse de Dzemael",
        },
        (24, 2040) => LocalisedText {
            en: "Amdapor Keep",
            ja: "古城アムダプール",
            de: "Die Ruinen von Amdapor",
            fr: "Le Château d'Amdapor",
        },
        (24, 2041) => LocalisedText {
            en: "Castrum Meridianum",
            ja: "カストルム・メリディアヌム",
            de: "Castrum Meridianum - Außenbereich",
            fr: "Castrum Meridianum",
        },
        (24, 2042) => LocalisedText {
            en: "The Praetorium",
            ja: "魔導城プラエトリウム",
            de: "Castrum Meridianum - Praetorium",
            fr: "Le Praetorium",
        },
        (24, 2043) => LocalisedText {
            en: "Basic Training: Enemy Parties",
            ja: "集団戦訓練をくぐり抜けろ！",
            de: "Einer für alle, alle für einen",
            fr: "Entraînement : groupes d'ennemis",
        },
        (24, 2044) => LocalisedText {
            en: "Under the Armor",
            ja: "彷徨う死霊を討て！",
            de: "Bockmanns Gefolge",
            fr: "Chasse au fantôme fantoche",
        },
        (24, 2045) => LocalisedText {
            en: "Basic Training: Enemy Strongholds",
            ja: "全関門を突破し、最深部の敵を討て！",
            de: "Sturmkommando",
            fr: "Entraînement : infiltration en base ennemie",
        },
        (24, 2046) => LocalisedText {
            en: "Hero on the Half Shell",
            ja: "ギルガメを捕獲せよ！",
            de: "Gil oder Leben",
            fr: "Reconquête d'une carapace escamotée",
        },
        (24, 2047) => LocalisedText {
            en: "Pulling Poison Posies",
            ja: "有毒妖花を駆除せよ！",
            de: "Unkraut jäten",
            fr: "Opération fleurs du mal",
        },
        (24, 2048) => LocalisedText {
            en: "Stinging Back",
            ja: "無法者「似我蜂団」を撃滅せよ！",
            de: "Ins Wespennest stechen",
            fr: "Expédition punitive contre les Ventrerouge",
        },
        (24, 2049) => LocalisedText {
            en: "All's Well that Ends in the Well",
            ja: "夢幻のブラキシオを討て！",
            de: "Briaxio ausschalten",
            fr: "Briaxio à bras raccourcis",
        },
        (24, 2050) => LocalisedText {
            en: "Flicking Sticks and Taking Names",
            ja: "爆弾魔ゴブリン軍団を撃滅せよ！",
            de: "Bombige Goblins",
            fr: "Les Gobelins bombardiers",
        },
        (24, 2051) => LocalisedText {
            en: "More than a Feeler",
            ja: "汚染源モルボルを討て！",
            de: "Tödliches Rankenspiel",
            fr: "Sus au morbol pollueur",
        },
        (24, 2052) => LocalisedText {
            en: "Annoy the Void",
            ja: "坑道に現れた妖異ブソを討て！",
            de: "Gefahr aus dem Nichts",
            fr: "Buso l'immolateur",
        },
        (24, 2053) => LocalisedText {
            en: "Shadow and Claw",
            ja: "無敵の眷属を従えし、大型妖異を討て！",
            de: "Kampf gegen Schatten",
            fr: "Ombres et griffes",
        },
        (24, 2054) => LocalisedText {
            en: "Long Live the Queen",
            ja: "ボムを率いる「ボムクイーン」を討て！",
            de: "Miss Bombastic",
            fr: "Longue vie à la Reine",
        },
        (24, 2055) => LocalisedText {
            en: "Ward Up",
            ja: "不気味な陣形を組む妖異をせん滅せよ！",
            de: "Unzertrennlich",
            fr: "Quintettes infernaux",
        },
        (24, 2056) => LocalisedText {
            en: "Solemn Trinity",
            ja: "三つ巴の巨人族を制し、遺物を守れ！",
            de: "Reliquienstreit",
            fr: "Trinité sinistre",
        },
        (24, 2057) => LocalisedText {
            en: "The Bowl of Embers (Normal)",
            ja: "イフリート討伐戦",
            de: "Grab der Lohe (normal)",
            fr: "Le Cratère des tisons (normal)",
        },
        (24, 2058) => LocalisedText {
            en: "The Navel (Normal)",
            ja: "タイタン討伐戦",
            de: "Der Nabel (normal)",
            fr: "Le Nombril (normal)",
        },
        (24, 2059) => LocalisedText {
            en: "The Howling Eye (Normal)",
            ja: "ガルーダ討伐戦",
            de: "Das Tosende Auge (normal)",
            fr: "Hurlœil (normal)",
        },
        (24, 2060) => LocalisedText {
            en: "The Bowl of Embers (Hard)",
            ja: "真イフリート討滅戦",
            de: "Götterdämmerung - Ifrit",
            fr: "Le Cratère des tisons (brutal)",
        },
        (24, 2061) => LocalisedText {
            en: "The Navel (Hard)",
            ja: "真タイタン討滅戦",
            de: "Götterdämmerung - Titan",
            fr: "Le Nombril (brutal)",
        },
        (24, 2062) => LocalisedText {
            en: "The Howling Eye (Hard)",
            ja: "真ガルーダ討滅戦",
            de: "Götterdämmerung - Garuda",
            fr: "Hurlœil (brutal)",
        },
        (24, 2063) => LocalisedText {
            en: "Cape Westwind",
            ja: "リットアティン強襲戦",
            de: "Kap Westwind",
            fr: "Cap Vendouest",
        },
        (24, 2064) => LocalisedText {
            en: "The Binding Coil of Bahamut",
            ja: "大迷宮バハムート",
            de: "Bahamuts Verschlungene Schatten",
            fr: "Le Labyrinthe de Bahamut",
        },
        (24, 2065) => LocalisedText {
            en: "The Wolves' Den",
            ja: "ウルヴズジェイル",
            de: "Wolfshöhle (PvP)",
            fr: "L'Antre des loups",
        },
        (24, 2066) => LocalisedText {
            en: "The Bowl of Embers (Extreme)",
            ja: "極イフリート討滅戦",
            de: "Zenit der Götter - Ifrit",
            fr: "Le Cratère des tisons (extrême)",
        },
        (24, 2067) => LocalisedText {
            en: "The Navel (Extreme)",
            ja: "極タイタン討滅戦",
            de: "Zenit der Götter - Titan",
            fr: "Le Nombril (extrême)",
        },
        (24, 2068) => LocalisedText {
            en: "The Howling Eye (Extreme)",
            ja: "極ガルーダ討滅戦",
            de: "Zenit der Götter - Garuda",
            fr: "Hurlœil (extrême)",
        },
        (24, 2069) => LocalisedText {
            en: "Thornmarch (Hard)",
            ja: "善王モグル・モグXII世討滅戦",
            de: "Königliche Konfrontation (schwer)",
            fr: "La Lisière de ronces (brutal)",
        },
        (24, 2070) => LocalisedText {
            en: "The Minstrel's Ballad: Ultima's Bane",
            ja: "アルテマウェポン破壊作戦",
            de: "Heldenlied von Ultima",
            fr: "Le fléau d'Ultima",
        },
        (24, 2071) => LocalisedText {
            en: "Crystal Tower",
            ja: "クリスタルタワー",
            de: "Kristallturm",
            fr: "La Tour de Cristal",
        },
        (24, 2072) => LocalisedText {
            en: "The Binding Coil of Bahamut",
            ja: "邂逅編",
            de: "Bahamut 1 - Begegnung",
            fr: "Le Labyrinthe de Bahamut I-V",
        },
        (24, 2073) => LocalisedText {
            en: "Labyrinth of the Ancients",
            ja: "古代の民の迷宮",
            de: "Labyrinth der Alten",
            fr: "Le Dédale antique",
        },
        (24, 2074) => LocalisedText {
            en: "Decipher",
            ja: "ディサイファー",
            de: "Entziffern",
            fr: "Déchiffrage",
        },
        (24, 2075) => LocalisedText {
            en: "hard mode",
            ja: "ハードモード",
            de: "Schwere Version",
            fr: "mode difficile",
        },
        (24, 2076) => LocalisedText {
            en: "Duty Roulette",
            ja: "コンテンツルーレット",
            de: "Zufallsinhalt",
            fr: "Mission aléatoire",
        },
        (24, 2077) => LocalisedText {
            en: "The Second Coil of Bahamut",
            ja: "侵攻編",
            de: "Bahamut 2 - Vorstoß",
            fr: "Les Méandres de Bahamut",
        },
        (24, 2078) => LocalisedText {
            en: "Pharos Sirius",
            ja: "シリウス大灯台",
            de: "Pharos Sirius",
            fr: "Le Phare de Sirius",
        },
        (24, 2079) => LocalisedText {
            en: "Thornmarch (Extreme)",
            ja: "極王モグル・モグXII世討滅戦",
            de: "Königliche Konfrontation (extrem)",
            fr: "La Lisière de ronces (extrême)",
        },
        (24, 2080) => LocalisedText {
            en: "The Whorleater (Hard)",
            ja: "真リヴァイアサン討滅戦",
            de: "Götterdämmerung - Leviathan",
            fr: "Le Briseur de marées (brutal)",
        },
        (24, 2081) => LocalisedText {
            en: "The Whorleater (Extreme)",
            ja: "極リヴァイアサン討滅戦",
            de: "Zenit der Götter - Leviathan",
            fr: "Le Briseur de marées (extrême)",
        },
        (24, 2082) => LocalisedText {
            en: "A Relic Reborn: The Chimera",
            ja: "ドルムキマイラ討伐戦",
            de: "Kampf gegen die Chimära",
            fr: "La chimère dhorme du Coerthas",
        },
        (24, 2083) => LocalisedText {
            en: "A Relic Reborn: The Hydra",
            ja: "ハイドラ討伐戦",
            de: "Kampf gegen die Hydra",
            fr: "L'hydre d'Halatali",
        },
        (24, 2084) => LocalisedText {
            en: "Battle on the Big Bridge",
            ja: "ギルガメッシュ討伐戦",
            de: "Duell auf der großen Brücke",
            fr: "Affrontement sur le grand pont",
        },
        (24, 2085) => LocalisedText {
            en: "PvP",
            ja: "PvP",
            de: "PvP",
            fr: "JcJ",
        },
        (24, 2086) => LocalisedText {
            en: "beast tribe quest",
            ja: "蛮族クエスト",
            de: "Auftrag der Wilden Stämme",
            fr: "Quêtes tribales",
        },
        (24, 2087) => LocalisedText {
            en: "beast tribe sidequest",
            ja: "蛮族デイリークエスト",
            de: "Nebenauftrag der Wilden Stämme",
            fr: "Quêtes tribales quotidiennes",
        },
        (24, 2088) => LocalisedText {
            en: "beast tribe main quest",
            ja: "蛮族ストーリークエスト",
            de: "Hauptauftrag der Wilden Stämme",
            fr: "Quêtes tribales (scénario)",
        },
        (24, 2089) => LocalisedText {
            en: "side story quest",
            ja: "サブストーリークエスト",
            de: "Szenario-Nebenaufträge",
            fr: "Histoires annexes",
        },
        (24, 2090) => LocalisedText {
            en: "treasure hunt",
            ja: "トレジャーハント",
            de: "Schatzsuche",
            fr: "Chasse aux trésors",
        },
        (24, 2091) => LocalisedText {
            en: "chronicles of a new era quest",
            ja: "クロニクルクエスト",
            de: "Chroniken der neuen Ära",
            fr: "Chroniques d'une nouvelle ère",
        },
        (24, 2092) => LocalisedText {
            en: "seasonal event",
            ja: "シーズナルイベント",
            de: "Saisonales Ereignis",
            fr: "Événements saisonniers",
        },
        (24, 2093) => LocalisedText {
            en: "The Lost City of Amdapor",
            ja: "古アムダプール市街",
            de: "Historisches Amdapor",
            fr: "Les Vestiges de la cité d'Amdapor",
        },
        (24, 2094) => LocalisedText {
            en: "Trials of the Braves",
            ja: "黄道十二文書",
            de: "Ephemeriden",
            fr: "Chroniques du zodiaque",
        },
        (24, 2095) => LocalisedText {
            en: "sphere scroll",
            ja: "スフィアスクロール",
            de: "Sphärentafeln",
            fr: "parchemin stellaire",
        },
        (24, 2096) => LocalisedText {
            en: "The Hunt",
            ja: "モブハント",
            de: "Hohe Jagd",
            fr: "Contrats de chasse",
        },
        (24, 2097) => LocalisedText {
            en: "elite marks",
            ja: "リスキーモブ",
            de: "Hochwild",
            fr: "Contrat d'élite",
        },
        (24, 2098) => LocalisedText {
            en: "The Striking Tree (Hard)",
            ja: "真ラムウ討滅戦",
            de: "Götterdämmerung - Ramuh",
            fr: "L'Arbre du jugement (brutal)",
        },
        (24, 2099) => LocalisedText {
            en: "The Striking Tree (Extreme)",
            ja: "極ラムウ討滅戦",
            de: "Zenit der Götter - Ramuh",
            fr: "L'Arbre du jugement (extrême)",
        },
        (24, 2100) => LocalisedText {
            en: "Syrcus Tower",
            ja: "シルクスの塔",
            de: "Syrcus-Turm",
            fr: "La Tour de Syrcus",
        },
        (24, 2101) => LocalisedText {
            en: "The Second Coil of Bahamut (Savage)",
            ja: "大迷宮バハムート零式",
            de: "Bahamut 2 - Epischer Vorstoß",
            fr: "Les Méandres de Bahamut (sadique)",
        },
        (24, 2102) => LocalisedText {
            en: "The Carteneau Flats",
            ja: "カルテノー平原",
            de: "Carteneauer Front",
            fr: "La Plaine de Carteneau",
        },
        (24, 2103) => LocalisedText {
            en: "Hullbreaker Isle",
            ja: "ハルブレーカー・アイル",
            de: "Schiffbrecher-Insel",
            fr: "L'Île de Crèvecarène",
        },
        (24, 2104) => LocalisedText {
            en: "Frontline",
            ja: "フロントライン",
            de: "Front (PvP)",
            fr: "Front",
        },
        (24, 2106) => LocalisedText {
            en: "expert",
            ja: "エキスパート",
            de: "Experte",
            fr: "expert",
        },
        (24, 2108) => LocalisedText {
            en: "The Final Coil of Bahamut",
            ja: "真成編",
            de: "Verschlungene Schatten - Finale",
            fr: "L'Abîme de Bahamut",
        },
        (24, 2109) => LocalisedText {
            en: "The Akh Afah Amphitheatre (Hard)",
            ja: "真シヴァ討滅戦",
            de: "Götterdämmerung - Shiva",
            fr: "L'Amphithéâtre d'Akh Afah (brutal)",
        },
        (24, 2110) => LocalisedText {
            en: "The Akh Afah Amphitheatre (Extreme)",
            ja: "極シヴァ討滅戦",
            de: "Zenit der Götter - Shiva",
            fr: "L'Amphithéâtre d'Akh Afah (extrême)",
        },
        (24, 2111) => LocalisedText {
            en: "Snowcloak",
            ja: "スノークローク大氷壁",
            de: "Schneekleid",
            fr: "Manteneige",
        },
        (24, 2112) => LocalisedText {
            en: "The Dragon's Neck",
            ja: "アマジナ杯闘技会決勝戦",
            de: "Drachenhals-Kolosseum",
            fr: "Le Col du dragon",
        },
        (24, 2113) => LocalisedText {
            en: "delivery quest",
            ja: "納品クエスト",
            de: "Lieferauftrag",
            fr: "Quêtes de livraison",
        },
        (24, 2114) => LocalisedText {
            en: "Urth's Fount (Hard)",
            ja: "闘神オーディン討滅戦",
            de: "Jenseits Urths Quelle",
            fr: "La Fontaine d'Urth",
        },
        (24, 2115) => LocalisedText {
            en: "The World of Darkness",
            ja: "闇の世界",
            de: "Die Welt der Dunkelheit",
            fr: "Le Monde des Ténèbres",
        },
        (24, 2117) => LocalisedText {
            en: "The Borderland Ruins (Secure)",
            ja: "外縁遺跡群 (制圧戦)",
            de: "Äußere Ruinen (Sicherung)",
            fr: "Ruines frontalières (annexion)",
        },
        (24, 2119) => LocalisedText {
            en: "The Keeper of the Lake",
            ja: "黙約の塔",
            de: "Hüter des Sees",
            fr: "Gardien du lac",
        },
        (24, 2120) => LocalisedText {
            en: "Battle in the Big Keep",
            ja: "真ギルガメッシュ討滅戦",
            de: "Revanche in den Ruinen",
            fr: "Revanche au vieux château",
        },
        (24, 2121) => LocalisedText {
            en: "Ascian",
            ja: "アシエン",
            de: "Ascian",
            fr: "Ascien",
        },
        (24, 2122) => LocalisedText {
            en: "the void",
            ja: "ヴォイド",
            de: "Das Nichts",
            fr: "néant",
        },
        (24, 2123) => LocalisedText {
            en: "The Great Gubal Library",
            ja: "グブラ幻想図書館",
            de: "Große Gubal-Bibliothek",
            fr: "Grande bibliothèque de Gubal",
        },
        (24, 2124) => LocalisedText {
            en: "Neverreap",
            ja: "ネバーリープ",
            de: "Nimmerreich",
            fr: "Nalloncques",
        },
        (24, 2125) => LocalisedText {
            en: "The Vault",
            ja: "イシュガルド教皇庁",
            de: "Erzbasilika",
            fr: "La Voûte",
        },
        (24, 2126) => LocalisedText {
            en: "The Fractal Continuum",
            ja: "フラクタル・コンティニアム",
            de: "Fraktal-Kontinuum",
            fr: "Le Continuum fractal",
        },
        (24, 2127) => LocalisedText {
            en: "Dusk Vigil",
            ja: "ダスクヴィジル",
            de: "Abendrot-Wacht",
            fr: "Vigile du Crépuscule",
        },
        (24, 2128) => LocalisedText {
            en: "Sohm Al",
            ja: "ソーム・アル",
            de: "Sohm Al",
            fr: "Sohm Al",
        },
        (24, 2129) => LocalisedText {
            en: "Aetherochemical Research Facility",
            ja: "魔科学研究所",
            de: "Ätherochemisches Forschungslabor",
            fr: "Laboratoire de magismologie",
        },
        (24, 2130) => LocalisedText {
            en: "The Aery",
            ja: "ドラゴンズエアリー",
            de: "Nest des Drachen",
            fr: "L'Aire",
        },
        (24, 2131) => LocalisedText {
            en: "Thok ast Thok (Hard)",
            ja: "真ラーヴァナ討滅戦",
            de: "Götterdämmerung - Ravana",
            fr: "Thok ast Thok (brutal)",
        },
        (24, 2132) => LocalisedText {
            en: "Thok ast Thok (Extreme)",
            ja: "極ラーヴァナ討滅戦",
            de: "Zenit der Götter - Ravana",
            fr: "Thok ast Thok (extrême)",
        },
        (24, 2133) => LocalisedText {
            en: "Limitless Blue (Hard)",
            ja: "真ビスマルク討滅戦",
            de: "Götterdämmerung - Bismarck",
            fr: "L'Immensité bleue (brutal)",
        },
        (24, 2134) => LocalisedText {
            en: "Limitless Blue (Extreme)",
            ja: "極ビスマルク討滅戦",
            de: "Zenit der Götter - Bismarck",
            fr: "L'Immensité bleue (extrême)",
        },
        (24, 2135) => LocalisedText {
            en: "Seal Rock (Seize)",
            ja: "シールロック (争奪戦)",
            de: "Robbenholm (Eroberung)",
            fr: "Rocher des tréfonds (invasion)",
        },
        (24, 2137) => LocalisedText {
            en: "Alexander",
            ja: "機工城アレキサンダー",
            de: "Alexander",
            fr: "Alexander",
        },
        (24, 2138) => LocalisedText {
            en: "Alexander (Savage)",
            ja: "機工城アレキサンダー零式",
            de: "Alexander (episch)",
            fr: "Alexander (sadique)",
        },
        (24, 2139) => LocalisedText {
            en: "Gordias",
            ja: "起動編",
            de: "Gordios",
            fr: "Gordias : le Père",
        },
        (24, 2140) => LocalisedText {
            en: "The Chrysalis (Normal)",
            ja: "アシエン・ナプリアレス討伐戦",
            de: "Chrysalis (normal)",
            fr: "La Chrysalide (normal)",
        },
        (24, 2141) => LocalisedText {
            en: "The Steps of Faith (Normal)",
            ja: "皇都イシュガルド防衛戦",
            de: "Der Schicksalsweg (normal)",
            fr: "Le Siège de la sainte Cité d'Ishgard (normal)",
        },
        (24, 2142) => LocalisedText {
            en: "The Singularity Reactor (Normal)",
            ja: "ナイツ・オブ・ラウンド討滅戦",
            de: "Singularitäts-Reaktor (normal)",
            fr: "Le Réacteur de singularité (normal)",
        },
        (24, 2143) => LocalisedText {
            en: "Temple leve",
            ja: "神殿騎士団リーヴ",
            de: "Ishgarder Freibriefe",
            fr: "Mandats ishgardais",
        },
        (24, 2144) => LocalisedText {
            en: "Saint Mocianne's Arboretum",
            ja: "聖モシャーヌ植物園",
            de: "Sankt Mocianne-Arboretum",
            fr: "Arboretum Sainte-Mocianne",
        },
        (24, 2145) => LocalisedText {
            en: "The Shadow of Mhach",
            ja: "シャドウ・オブ・マハ",
            de: "Schatten von Mhach",
            fr: "Ombre de Mhach",
        },
        (24, 2146) => LocalisedText {
            en: "The Void Ark",
            ja: "魔航船ヴォイドアーク",
            de: "Nichts-Arche",
            fr: "Arche du néant",
        },
        (24, 2147) => LocalisedText {
            en: "Hall of the Novice",
            ja: "初心者の館",
            de: "Anfänger-Arena",
            fr: "Salle des novices",
        },
        (24, 2148) => LocalisedText {
            en: "The Antitower",
            ja: "逆さの塔",
            de: "Antiturm",
            fr: "L'Antitour",
        },
        (24, 2149) => LocalisedText {
            en: "Midas",
            ja: "律動編",
            de: "Midas",
            fr: "Midas : le Fils",
        },
        (24, 2150) => LocalisedText {
            en: "Containment Bay S1T7 (Normal)",
            ja: "魔神セフィロト討滅戦",
            de: "Götterdämmerung - Sephirot",
            fr: "Unité de contention S1P7 (normal)",
        },
        (24, 2151) => LocalisedText {
            en: "Containment Bay S1T7 (Extreme)",
            ja: "極魔神セフィロト討滅戦",
            de: "Zenit der Götter - Sephirot",
            fr: "Unité de contention S1P7 (extrême)",
        },
        (24, 2153) => LocalisedText {
            en: "The Feast",
            ja: "ザ・フィースト",
            de: "The Feast",
            fr: "The Feast",
        },
        (24, 2154) => LocalisedText {
            en: "Stone, Sky, Sea",
            ja: "木人討滅戦",
            de: "Steinpuppen-Training",
            fr: "Entraînement sur mannequin",
        },
        (24, 2155) => LocalisedText {
            en: "Sohr Khai",
            ja: "ソール・カイ",
            de: "Sohr Khai",
            fr: "Sohr Khai",
        },
        (24, 2156) => LocalisedText {
            en: "The Weeping City of Mhach",
            ja: "禁忌都市マハ",
            de: "Stadt der Tränen",
            fr: "la Cité défendue de Mhach",
        },
        (24, 2157) => LocalisedText {
            en: "Deep Dungeon",
            ja: "ディープダンジョン",
            de: "Tiefes Gewölbe",
            fr: "Donjon sans fond",
        },
        (24, 2158) => LocalisedText {
            en: "Palace of the Dead",
            ja: "死者の宮殿",
            de: "Palast der Toten",
            fr: "Palais des morts",
        },
        (24, 2159) => LocalisedText {
            en: "The Final Steps of Faith",
            ja: "ニーズヘッグ征竜戦",
            de: "Der letzte Schicksalsweg",
            fr: "La Dernière avancée de la Foi",
        },
        (24, 2160) => LocalisedText {
            en: "The Minstrel's Ballad: Nidhogg's Rage",
            ja: "極ニーズヘッグ征竜戦",
            de: "Das Lied von Nidhoggs letztem Ruf",
            fr: "L'ire de Nidhogg",
        },
        (24, 2161) => LocalisedText {
            en: "The Fields of Glory (Shatter)",
            ja: "フィールド・オブ・グローリー (砕氷戦)",
            de: "Feld der Ehre (Zersplitterung)",
            fr: "Champs de la gloire (brise-glace)",
        },
        (24, 2163) => LocalisedText {
            en: "duel",
            ja: "デュエル",
            de: "Duell",
            fr: "Combat singulier",
        },
        (24, 2164) => LocalisedText {
            en: "Xelphatol",
            ja: "ゼルファトル",
            de: "Xelphatol",
            fr: "Xelphatol",
        },
        (24, 2165) => LocalisedText {
            en: "Containment Bay P1T6",
            ja: "女神ソフィア討滅戦",
            de: "Götterdämmerung - Sophia",
            fr: "Unité de contention P1P6 (normal)",
        },
        (24, 2166) => LocalisedText {
            en: "Containment Bay P1T6 (Extreme)",
            ja: "極女神ソフィア討滅戦",
            de: "Zenit der Götter - Sophia",
            fr: "Unité de contention P1P6 (extrême)",
        },
        (24, 2167) => LocalisedText {
            en: "The Creator",
            ja: "天動編",
            de: "Augen des Schöpfers",
            fr: "Alexander : le Créateur",
        },
        (24, 2168) => LocalisedText {
            en: "Baelsar's Wall",
            ja: "バエサルの長城",
            de: "Baelsar-Wall",
            fr: "Muraille de Baelsar",
        },
        (24, 2169) => LocalisedText {
            en: "Dun Scaith",
            ja: "影の国ダン・スカー",
            de: "Dun Scaith",
            fr: "Dun Scaith",
        },
        (24, 2170) => LocalisedText {
            en: "Containment Bay Z1T9",
            ja: "鬼神ズルワーン討滅戦",
            de: "Götterdämmerung - Zurvan",
            fr: "Unité de contention Z1P9 (normal)",
        },
        (24, 2171) => LocalisedText {
            en: "Containment Bay Z1T9 (Extreme)",
            ja: "極鬼神ズルワーン討滅戦",
            de: "Zenit der Götter - Zurvan",
            fr: "Unité de contention Z1P9 (extrême)",
        },
        (24, 2172) => LocalisedText {
            en: "Shisui of the Violet Tides",
            ja: "紫水宮",
            de: "Shisui",
            fr: "Palais aux Marées violettes",
        },
        (24, 2173) => LocalisedText {
            en: "Bardam's Mettle",
            ja: "バルダム覇道",
            de: "Bardams Probe",
            fr: "La Force de Bardam",
        },
        (24, 2174) => LocalisedText {
            en: "Doma Castle",
            ja: "ドマ城",
            de: "Burg Doma",
            fr: "Château de Doma",
        },
        (24, 2175) => LocalisedText {
            en: "Castrum Abania",
            ja: "カストルム・アバニア",
            de: "Castrum Abania",
            fr: "Castrum Abania",
        },
        (24, 2176) => LocalisedText {
            en: "The Pool of Tribute",
            ja: "スサノオ討滅戦",
            de: "Götterdämmerung - Susano",
            fr: "Crique aux tributs",
        },
        (24, 2177) => LocalisedText {
            en: "The Pool of Tribute (Extreme)",
            ja: "極スサノオ討滅戦",
            de: "Zenit der Götter - Susano",
            fr: "Crique aux tributs (extrême)",
        },
        (24, 2178) => LocalisedText {
            en: "Emanation",
            ja: "ラクシュミ討滅戦",
            de: "Götterdämmerung - Lakshmi",
            fr: "Émanation",
        },
        (24, 2179) => LocalisedText {
            en: "Emanation (Extreme)",
            ja: "極ラクシュミ討滅戦",
            de: "Zenit der Götter - Lakshmi",
            fr: "Émanation (extrême)",
        },
        (24, 2180) => LocalisedText {
            en: "The Royal Menagerie",
            ja: "神龍討滅戦",
            de: "Königliche Menagerie",
            fr: "Ménagerie royale",
        },
        (24, 2181) => LocalisedText {
            en: "Interdimensional Rift",
            ja: "次元の狭間オメガ",
            de: "Interdimensionaler Riss",
            fr: "Oméga : Deltastice",
        },
        (24, 2182) => LocalisedText {
            en: "Interdimensional Rift (Savage)",
            ja: "次元の狭間オメガ零式",
            de: "Interdimensionaler Riss (episch)",
            fr: "Oméga : Deltastice (sadique)",
        },
        (24, 2183) => LocalisedText {
            en: "Deltascape",
            ja: "デルタ編",
            de: "Deltametrie",
            fr: "Deltastice",
        },
        (24, 2184) => LocalisedText {
            en: "The Temple of the Fist",
            ja: "星導山寺院",
            de: "Tempel der Faust",
            fr: "Temple du Poing",
        },
        (24, 2185) => LocalisedText {
            en: "The Drowned City of Skalla",
            ja: "スカラ",
            de: "Skalla",
            fr: "Cité engloutie de Skalla",
        },
        (24, 2186) => LocalisedText {
            en: "The Minstrel's Ballad: Shinryu's Domain",
            ja: "極神龍討滅戦",
            de: "Heldenlied von Shinryu",
            fr: "Le domaine de Shinryu",
        },
        (24, 2187) => LocalisedText {
            en: "alliance raid",
            ja: "アライアンスレイド",
            de: "Allianz-Raid",
            fr: "raid en alliance",
        },
        (24, 2188) => LocalisedText {
            en: "Return to Ivalice",
            ja: "リターン・トゥ・イヴァリース",
            de: "Rückkehr nach Ivalice",
            fr: "Retour à Ivalice",
        },
        (24, 2189) => LocalisedText {
            en: "The Royal City of Rabanastre",
            ja: "失われた都 ラバナスタ",
            de: "Die königliche Stadt Rabanastre",
            fr: "Cité royale de Rabanastre",
        },
        (24, 2190) => LocalisedText {
            en: "The Unending Coil of Bahamut (Ultimate)",
            ja: "絶バハムート討滅戦",
            de: "Endlose Schatten von Bahamut (fatal)",
            fr: "Abîme infini de Bahamut (fatal)",
        },
        (24, 2191) => LocalisedText {
            en: "Rival Wings",
            ja: "ライバルウィングズ",
            de: "Stahlschwingen",
            fr: "Ailes rivales",
        },
        (24, 2192) => LocalisedText {
            en: "Astragalos",
            ja: "アストラガロス (機工戦)",
            de: "Astragalos",
            fr: "Astragalos (machinerie)",
        },
        (24, 2193) => LocalisedText {
            en: "Hells' Lid",
            ja: "獄之蓋",
            de: "Höllenspund",
            fr: "Couvercle des enfers",
        },
        (24, 2194) => LocalisedText {
            en: "The Jade Stoa",
            ja: "白虎征魂戦",
            de: "Seelentanz - Byakko",
            fr: "Clairière de Jade",
        },
        (24, 2195) => LocalisedText {
            en: "The Jade Stoa (Extreme)",
            ja: "極白虎征魂戦",
            de: "Seelensturm - Byakko",
            fr: "Clairière de Jade (extrême)",
        },
        (24, 2196) => LocalisedText {
            en: "Sigmascape",
            ja: "シグマ編",
            de: "Sigmametrie",
            fr: "Sigmastice",
        },
        (24, 2197) => LocalisedText {
            en: "The Forbidden Land, Eureka",
            ja: "禁断の地 エウレカ",
            de: "Das Verbotene Land Eureka",
            fr: "terre interdite d'Eurêka",
        },
        (24, 2198) => LocalisedText {
            en: "Anemos",
            ja: "アネモス編",
            de: "Anemos",
            fr: "Eurêka Anemos",
        },
        (49, 1) => LocalisedText {
            en: "company chocobo",
            ja: "マイチョコボ",
            de: "Gesellschafts-Chocobo",
            fr: "chocobo destrier",
        },
        (49, 4) => LocalisedText {
            en: "goobbue",
            ja: "グゥーブー",
            de: "Goobbue",
            fr: "goobbue",
        },
        (49, 5) => LocalisedText {
            en: "legacy chocobo",
            ja: "レガシーチョコボ",
            de: "Legacy-Chocobo",
            fr: "chocobo Legacy",
        },
        (49, 6) => LocalisedText {
            en: "magitek armor",
            ja: "魔導アーマー",
            de: "Magitek-Kampfmaschine",
            fr: "armure magitek",
        },
        (49, 8) => LocalisedText {
            en: "coeurl",
            ja: "クァール",
            de: "Coeurl",
            fr: "coeurl",
        },
        (49, 9) => LocalisedText {
            en: "ahriman",
            ja: "アーリマン",
            de: "Ahriman",
            fr: "ahriman",
        },
        (49, 15) => LocalisedText {
            en: "unicorn",
            ja: "ユニコーン",
            de: "Einhorn",
            fr: "licorne",
        },
        (49, 18) => LocalisedText {
            en: "behemoth",
            ja: "ベヒーモス",
            de: "Behemoth",
            fr: "béhémoth",
        },
        (49, 19) => LocalisedText {
            en: "cavalry drake",
            ja: "キャバルリー・ドレイク",
            de: "Reit-Drakon",
            fr: "draconide de cavalerie",
        },
        (49, 20) => LocalisedText {
            en: "laurel goobbue",
            ja: "ローレル・グゥーブー",
            de: "Lorbeer-Goobbue",
            fr: "goobbue couronné",
        },
        (49, 21) => LocalisedText {
            en: "gilded magitek armor",
            ja: "C式魔導アーマー",
            de: "Typ C-Magitek-Kampfmaschine",
            fr: "armure magitek type C",
        },
        (49, 22) => LocalisedText {
            en: "nightmare",
            ja: "ナイトメア",
            de: "Albtraum",
            fr: "alicorne de jais",
        },
        (49, 25) => LocalisedText {
            en: "fat chocobo",
            ja: "でぶチョコボ",
            de: "Chocomoppel",
            fr: "gros chocobo",
        },
        (49, 26) => LocalisedText {
            en: "cavalry elbst",
            ja: "キャバルリー・エルブスト",
            de: "Kavallerie-Elbst",
            fr: "elbst de cavalerie",
        },
        (49, 27) => LocalisedText {
            en: "bomb palanquin",
            ja: "ボムバルーン",
            de: "Bomber-Sänfte",
            fr: "balancelle bombo",
        },
        (49, 28) => LocalisedText {
            en: "Aithon",
            ja: "アイトーン",
            de: "Aithon",
            fr: "Aithon",
        },
        (49, 29) => LocalisedText {
            en: "Xanthos",
            ja: "クサントス",
            de: "Xanthos",
            fr: "Xanthos",
        },
        (49, 30) => LocalisedText {
            en: "Gullfaxi",
            ja: "グルファクシ",
            de: "Gullfaxi",
            fr: "Gullfaxi",
        },
        (49, 31) => LocalisedText {
            en: "Enbarr",
            ja: "エンバール",
            de: "Enbarr",
            fr: "Enbarr",
        },
        (49, 32) => LocalisedText {
            en: "warlion",
            ja: "ウォーライオン",
            de: "Kriegslöwe",
            fr: "lion de guerre",
        },
        (49, 33) => LocalisedText {
            en: "warbear",
            ja: "ウォーベアー",
            de: "Kriegsbär",
            fr: "ours de guerre",
        },
        (49, 34) => LocalisedText {
            en: "draught chocobo",
            ja: "グランチョコボ",
            de: "Tandem-Chocobo",
            fr: "chocobo de trait",
        },
        (49, 35) => LocalisedText {
            en: "direwolf",
            ja: "ダイアウルフ",
            de: "Düsterwolf",
            fr: "loup sinistre",
        },
        (49, 36) => LocalisedText {
            en: "storm warsteed",
            ja: "ロミンサン・スティード",
            de: "Mahlstrom-Schlachtross",
            fr: "cheval des tempêtes",
        },
        (49, 37) => LocalisedText {
            en: "serpent warsteed",
            ja: "グリダニアン・スティード",
            de: "Bruderschaftsschlachtross",
            fr: "cheval du serpent",
        },
        (49, 38) => LocalisedText {
            en: "flame warsteed",
            ja: "ウルダハン・スティード",
            de: "Legionsschlachtross",
            fr: "cheval des sables",
        },
        (49, 40) => LocalisedText {
            en: "Markab",
            ja: "マルカブ",
            de: "Markab",
            fr: "Markab",
        },
        (49, 41) => LocalisedText {
            en: "ceremony chocobo",
            ja: "エターナルチョコボ",
            de: "Zeremonial-Chocobo",
            fr: "chocobo de cérémonie",
        },
        (49, 42) => LocalisedText {
            en: "Sleipnir",
            ja: "スレイプニル",
            de: "Sleipnir",
            fr: "Sleipnir",
        },
        (49, 43) => LocalisedText {
            en: "Boreas",
            ja: "ボレアス",
            de: "Boreas",
            fr: "Boréas",
        },
        (49, 44) => LocalisedText {
            en: "parade chocobo",
            ja: "パレードチョコボ",
            de: "Parade-Chocobo",
            fr: "chocobo de parade",
        },
        (49, 45) => LocalisedText {
            en: "black chocobo",
            ja: "黒チョコボ",
            de: "Flug-Chocobo",
            fr: "chocobo noir",
        },
        (49, 46) => LocalisedText {
            en: "adamantoise",
            ja: "アダマンタス",
            de: "Adaman-Taimai",
            fr: "adamankhélone",
        },
        (49, 47) => LocalisedText {
            en: "Kirin",
            ja: "麒麟",
            de: "Kirin",
            fr: "Kirin",
        },
        (49, 48) => LocalisedText {
            en: "logistics system",
            ja: "騎乗システム",
            de: "Logistiksystem",
            fr: "module de déplacement sphérique",
        },
        (49, 49) => LocalisedText {
            en: "Fenrir",
            ja: "フェンリル",
            de: "Fenrir",
            fr: "Fenrir",
        },
        (49, 50) => LocalisedText {
            en: "Midgardsormr",
            ja: "ミドガルズオルム",
            de: "Midgardsormr",
            fr: "Midgardsormr",
        },
        (49, 52) => LocalisedText {
            en: "amber draught chocobo",
            ja: "赤グランチョコボ",
            de: "Bernstein-Chocobo",
            fr: "chocobo de trait rouge",
        },
        (49, 53) => LocalisedText {
            en: "sanuwa",
            ja: "サヌワ",
            de: "Sanuwa",
            fr: "sanuwa",
        },
        (49, 54) => LocalisedText {
            en: "griffin",
            ja: "グリフィン",
            de: "Greif",
            fr: "griffon",
        },
        (49, 55) => LocalisedText {
            en: "manacutter",
            ja: "マナカッター",
            de: "Manaschnitter",
            fr: "Manacutter",
        },
        (49, 56) => LocalisedText {
            en: "war panther",
            ja: "ウォーパンサー",
            de: "Kriegspanther",
            fr: "panthère de guerre",
        },
        (49, 58) => LocalisedText {
            en: "gobwalker",
            ja: "ゴブリウォーカー",
            de: "Gob-Stampfer",
            fr: "gobblindé",
        },
        (49, 59) => LocalisedText {
            en: "Twintania",
            ja: "ツインタニア",
            de: "Twintania",
            fr: "Gémellia",
        },
        (49, 62) => LocalisedText {
            en: "witch's broom",
            ja: "ウィッチズブルーム",
            de: "Hexenbesen",
            fr: "balai de sorcière",
        },
        (49, 67) => LocalisedText {
            en: "pegasus",
            ja: "ペガサス",
            de: "Pegasus",
            fr: "Pégase",
        },
        (49, 68) => LocalisedText {
            en: "White Devil",
            ja: "リウィア専用魔導アーマー",
            de: "Weißer Teufel",
            fr: "armure magitek de Livia",
        },
        (49, 69) => LocalisedText {
            en: "Red Baron",
            ja: "ネロ専用魔導アーマー",
            de: "Roter Baron",
            fr: "armure magitek de Nero",
        },
        (49, 70) => LocalisedText {
            en: "wyvern",
            ja: "ワイバーン",
            de: "Wyvern",
            fr: "wyverne",
        },
        (49, 71) => LocalisedText {
            en: "SDS Fenrir",
            ja: "SDSフェンリル",
            de: "SKD-Fenrir",
            fr: "SLE Fenrir",
        },
        (49, 72) => LocalisedText {
            en: "kongamato",
            ja: "コンガマトー",
            de: "Kongamato",
            fr: "kongamato",
        },
        (49, 73) => LocalisedText {
            en: "zu",
            ja: "ズー",
            de: "Montisaurus",
            fr: "zu",
        },
        (49, 74) => LocalisedText {
            en: "bennu",
            ja: "ベンヌ",
            de: "Bennu",
            fr: "bénou",
        },
        (49, 75) => LocalisedText {
            en: "white lanner",
            ja: "ホワイトラナー",
            de: "Weißlanner",
            fr: "faucon des brumes",
        },
        (49, 76) => LocalisedText {
            en: "rose lanner",
            ja: "ラースラナー",
            de: "Rotlanner",
            fr: "faucon des lames",
        },
        (49, 77) => LocalisedText {
            en: "round lanner",
            ja: "ラナー・オブ・ラウンド",
            de: "Rundlanner",
            fr: "faucon de la Table ronde",
        },
        (49, 78) => LocalisedText {
            en: "warring lanner",
            ja: "ウォーリングラナー",
            de: "Kriegslanner",
            fr: "faucon séphirotique",
        },
        (49, 80) => LocalisedText {
            en: "Gloria-class airship",
            ja: "グローリア号",
            de: "Gloria-Luftschiff",
            fr: "Gloria",
        },
        (49, 81) => LocalisedText {
            en: "flying cumulus",
            ja: "筋斗雲",
            de: "Kumulus",
            fr: "nuage magique",
        },
        (49, 82) => LocalisedText {
            en: "original fat chocobo",
            ja: "初代でぶチョコボ",
            de: "Original-Chocomoppel",
            fr: "gros chocobo original",
        },
        (49, 83) => LocalisedText {
            en: "Astrope",
            ja: "アストロペ",
            de: "Astrope",
            fr: "Astérope",
        },
        (49, 84) => LocalisedText {
            en: "fat moogle",
            ja: "でぶモーグリ",
            de: "Moppel-Mogry",
            fr: "gros Mog",
        },
        (49, 86) => LocalisedText {
            en: "cloud mallow",
            ja: "クラウドマロウ",
            de: "Bommelin",
            fr: "mogrette",
        },
        (49, 87) => LocalisedText {
            en: "Whisper-go",
            ja: "ウィスパー号",
            de: "Whisper-Gondel",
            fr: "Whisper",
        },
        (49, 90) => LocalisedText {
            en: "dark lanner",
            ja: "ダークラナー",
            de: "Düsterlanner",
            fr: "faucon des ombres",
        },
        (49, 91) => LocalisedText {
            en: "aerodynamics system",
            ja: "浮行システム",
            de: "Aerodynamisches System",
            fr: "sphère de lévitation",
        },
        (49, 92) => LocalisedText {
            en: "disembodied head",
            ja: "ヘッドマウント",
            de: "Nichtskopf",
            fr: "tête ailée",
        },
        (49, 93) => LocalisedText {
            en: "managarm",
            ja: "マーナガルム",
            de: "Managarm",
            fr: "managarm",
        },
        (49, 94) => LocalisedText {
            en: "Whisper A-go-go",
            ja: "ウィスパー号改",
            de: "Whisper-Gondel Spezial",
            fr: "Whisper II",
        },
        (49, 95) => LocalisedText {
            en: "Goten",
            ja: "轟天",
            de: "Goten",
            fr: "Gôten",
        },
        (49, 96) => LocalisedText {
            en: "Ginga",
            ja: "銀牙",
            de: "Ginga",
            fr: "Ginga",
        },
        (49, 97) => LocalisedText {
            en: "mystic panda",
            ja: "仙術熊猫",
            de: "Mystischer Panda",
            fr: "panda mystique",
        },
        (49, 98) => LocalisedText {
            en: "Sophic lanner",
            ja: "ゴッデスラナー",
            de: "Sophia-Lanner",
            fr: "faucon sophique",
        },
        (49, 99) => LocalisedText {
            en: "Starlight bear",
            ja: "スターライト・ベアー",
            de: "Sternenlicht-Bär",
            fr: "ours de la fête des étoiles",
        },
        (49, 100) => LocalisedText {
            en: "black pegasus",
            ja: "ブラックペガサス",
            de: "Schwarzer Pegasus",
            fr: "pégase des ténèbres",
        },
        (49, 101) => LocalisedText {
            en: "Arrhidaeus",
            ja: "アリダイオス",
            de: "Arrhidaios",
            fr: "messager d'Arrhabée",
        },
        (49, 102) => LocalisedText {
            en: "Raigo",
            ja: "雷剛",
            de: "Raigo",
            fr: "Raigo",
        },
        (49, 104) => LocalisedText {
            en: "Demonic lanner",
            ja: "デモンラナー",
            de: "Dämonen-Lanner",
            fr: "faucon zurvanique",
        },
        (49, 105) => LocalisedText {
            en: "Firebird",
            ja: "鳳凰",
            de: "Feuervogel",
            fr: "Hôô",
        },
        (49, 106) => LocalisedText {
            en: "eggshilaration system",
            ja: "エッグシステム",
            de: "Wundereisystem",
            fr: "Lévitœuf",
        },
        (49, 108) => LocalisedText {
            en: "lone hellhound",
            ja: "ローン・ヘルハウンド",
            de: "Höllenhund-Einzelgänger",
            fr: "limier des enfers solitaire",
        },
        (49, 109) => LocalisedText {
            en: "pack hellhound",
            ja: "パック・ヘルハウンド",
            de: "Höllenhund-Alphatier",
            fr: "limier des enfers de meute",
        },
        (49, 110) => LocalisedText {
            en: "Archon throne",
            ja: "アルコンズ・チェアー",
            de: "Exegeten-Thron",
            fr: "chaise des Preux",
        },
        (49, 111) => LocalisedText {
            en: "Syldra",
            ja: "シルドラ",
            de: "Syldra",
            fr: "Syldra",
        },
        (49, 112) => LocalisedText {
            en: "Falcon",
            ja: "ファルコン号",
            de: "Falke",
            fr: "mini aéronef Faucon",
        },
        (49, 113) => LocalisedText {
            en: "lone faehound",
            ja: "ローン・フェイハウンド",
            de: "Feenhund-Einzelgänger",
            fr: "limier fantasmagorique solitaire",
        },
        (49, 114) => LocalisedText {
            en: "pack faehound",
            ja: "パック・フェイハウンド",
            de: "Feenhund-Alphatier",
            fr: "limier fantasmagorique de meute",
        },
        (49, 115) => LocalisedText {
            en: "blissful kamuy",
            ja: "美のカムイ",
            de: "Gepflegter Kamuy",
            fr: "kamuy de la grâce",
        },
        (49, 116) => LocalisedText {
            en: "reveling kamuy",
            ja: "豪のカムイ",
            de: "Verspielter Kamuy",
            fr: "kamuy de la splendeur",
        },
        (49, 121) => LocalisedText {
            en: "magitek predator",
            ja: "魔導プレデター",
            de: "Magitek-Prädator",
            fr: "prédateur magitek",
        },
        (49, 122) => LocalisedText {
            en: "battle lion",
            ja: "バトルライオン",
            de: "Kampflöwe",
            fr: "lion de combat",
        },
        (49, 123) => LocalisedText {
            en: "battle bear",
            ja: "バトルベアー",
            de: "Kampfbär",
            fr: "ours de combat",
        },
        (49, 124) => LocalisedText {
            en: "battle panther",
            ja: "バトルパンサー",
            de: "Kampfpanther",
            fr: "panthère de combat",
        },
        (49, 125) => LocalisedText {
            en: "yol",
            ja: "ヨル",
            de: "Yol",
            fr: "yol",
        },
        (49, 126) => LocalisedText {
            en: "Alte Roite",
            ja: "アルテ・ロイテ",
            de: "Alte Roite",
            fr: "Alte Roite",
        },
        (49, 127) => LocalisedText {
            en: "Centurio tiger",
            ja: "セントリオタイガー",
            de: "Centurio-Tiger",
            fr: "tigre centurio",
        },
        (49, 130) => LocalisedText {
            en: "Ixion",
            ja: "イクシオン",
            de: "Ixion",
            fr: "Ixion",
        },
        (49, 133) => LocalisedText {
            en: "legendary kamuy",
            ja: "龍のカムイ",
            de: "Legendärer Kamuy",
            fr: "kamuy de la divinité",
        },
        (49, 135) => LocalisedText {
            en: "Nezha chariot",
            ja: "風火二輪",
            de: "Feuerwind",
            fr: "roues Feu-et-Vent",
        },
        (49, 136) => LocalisedText {
            en: "striped ray",
            ja: "シマエイ",
            de: "Streifenrochen",
            fr: "raie rayée",
        },
        (49, 138) => LocalisedText {
            en: "Aquamarine Carbuncle",
            ja: "カーバンクル・アクアマリン",
            de: "Aquamarin-Karfunkel",
            fr: "Carbuncle aigue-marine",
        },
        (49, 139) => LocalisedText {
            en: "Citrine Carbuncle",
            ja: "カーバンクル・シトリン",
            de: "Citrin-Karfunkel",
            fr: "Carbuncle citrine",
        },
        (49, 140) => LocalisedText {
            en: "flying chair",
            ja: "フライング・チェアー",
            de: "Fliegender Sessel",
            fr: "fauteuil volant",
        },
        (49, 141) => LocalisedText {
            en: "magitek avenger",
            ja: "魔導アヴェンジャー騎乗型",
            de: "Magitek-Rächer",
            fr: "vengeur magitek",
        },
        (49, 142) => LocalisedText {
            en: "korpokkur kolossus",
            ja: "ハコボックル",
            de: "Koloss-Korpokkur",
            fr: "poro-korpokkur",
        },
        (49, 143) => LocalisedText {
            en: "Red Hare",
            ja: "赤兎馬",
            de: "Feuerross",
            fr: "Lièvre rouge",
        },
        (49, 144) => LocalisedText {
            en: "auspicious kamuy",
            ja: "白のカムイ",
            de: "Glücklicher Kamuy",
            fr: "kamuy de l'ouest",
        },
        (49, 145) => LocalisedText {
            en: "magitek death claw",
            ja: "魔導デスクロー拘束型",
            de: "Magitek-Todesklaue",
            fr: "griffe d'arrestation magitek",
        },
        (49, 146) => LocalisedText {
            en: "marid",
            ja: "マーリド",
            de: "Maride",
            fr: "marid",
        },
        (49, 148) => LocalisedText {
            en: "true griffin",
            ja: "トゥルーグリフィン",
            de: "Urgreif",
            fr: "vrai griffon",
        },
        (49, 150) => LocalisedText {
            en: "tyrannosaur",
            ja: "ティラノサウルス",
            de: "Tyrannosaurus",
            fr: "tyrannosaure",
        },
        (49, 151) => LocalisedText {
            en: "Regalia Type-G",
            ja: "レガリア TYPE-G",
            de: "Regalia Typ-G",
            fr: "Regalia Type-G",
        },
        (49, 152) => LocalisedText {
            en: "broken heart (right)",
            ja: "フライング・ライトハート",
            de: "Gebrochenes Herz (rechts)",
            fr: "fauteuil demi-cœur droit volant",
        },
        (49, 153) => LocalisedText {
            en: "broken heart (left)",
            ja: "フライング・レフトハート",
            de: "Gebrochenes Herz (links)",
            fr: "fauteuil demi-cœur gauche volant",
        },
        (49, 154) => LocalisedText {
            en: "Typhon",
            ja: "テュポーン先生",
            de: "Meister Typhon",
            fr: "Maître Typhon",
        },
        (49, 156) => LocalisedText {
            en: "Air Force",
            ja: "エアフォース",
            de: "Luftwaffe",
            fr: "force aérienne",
        },
        (49, 158) => LocalisedText {
            en: "lunar kamuy",
            ja: "夜のカムイ",
            de: "Lunarischer Kamuy",
            fr: "kamuy de la nuit",
        },
        (49, 159) => LocalisedText {
            en: "dodo",
            ja: "ドードー",
            de: "Dodo",
            fr: "dodo",
        },
        (49, 160) => LocalisedText {
            en: "Indigo whale",
            ja: "ホエール",
            de: "Indigo-Wal",
            fr: "baleine",
        },
        (49, 161) => LocalisedText {
            en: "Rathalos",
            ja: "リオレウス",
            de: "Rathalos",
            fr: "Rathalos",
        },
        (49, 162) => LocalisedText {
            en: "magitek conveyor",
            ja: "魔導キャリアー",
            de: "Magitek-Transporter",
            fr: "transporteur magitek",
        },
        (49, 163) => LocalisedText {
            en: "prototype conveyor",
            ja: "プロトキャリアー",
            de: "Proto-Transporter",
            fr: "transporteur prototype",
        },
        (49, 164) => LocalisedText {
            en: "mikoshi",
            ja: "オオナマズ御輿",
            de: "Namazuo-Mikoshi",
            fr: "mikoshi",
        },
        (49, 166) => LocalisedText {
            en: "Juedi",
            ja: "絶地",
            de: "Juedi",
            fr: "Juedi",
        },
        (49, 168) => LocalisedText {
            en: "safeguard system",
            ja: "護送システム",
            de: "Sicherheitssystem",
            fr: "module d'escorte sphérique",
        },
        (49, 170) => LocalisedText {
            en: "Grani",
            ja: "グラニ",
            de: "Grani",
            fr: "Grani",
        },
        (49, 171) => LocalisedText {
            en: "fatter cat",
            ja: "ビッグ・ファットキャット",
            de: "Fette Katze",
            fr: "très gros minou",
        },
        (49, 172) => LocalisedText {
            en: "euphonious kamuy",
            ja: "朱のカムイ",
            de: "Feuriger Kamuy",
            fr: "kamuy du sud",
        },
        (49, 173) => LocalisedText {
            en: "Model O",
            ja: "オメガ",
            de: "Modell O",
            fr: "Oméga prime",
        },
        (49, 174) => LocalisedText {
            en: "magitek sky armor",
            ja: "魔導スカイアーマー",
            de: "Magitek-Himmelspanzer",
            fr: "armure aérienne magitek",
        },
        (49, 175) => LocalisedText {
            en: "magicked carpet",
            ja: "マジックカーペット",
            de: "Fliegender Teppich",
            fr: "tapis magique",
        },
        (49, 176) => LocalisedText {
            en: "magna roader",
            ja: "魔導マグナローダー",
            de: "Magna Rotula",
            fr: "magna rouleur magitek",
        },
        (49, 177) => LocalisedText {
            en: "fat black chocobo",
            ja: "でぶ黒チョコボ",
            de: "Schwarzer Chocomoppel",
            fr: "gros chocobo noir",
        },
        (49, 178) => LocalisedText {
            en: "Eldthurs",
            ja: "エルドスルス",
            de: "Eldthurs",
            fr: "Eldthurs",
        },
        (49, 179) => LocalisedText {
            en: "maxima roader",
            ja: "魔導マグナローダー改",
            de: "Magna Rotula X",
            fr: "magna rouleur magitek plus",
        },
        (49, 180) => LocalisedText {
            en: "Sabotender Emperador",
            ja: "サボテンダー・エンペラドール",
            de: "Imperator Kaktor",
            fr: "pampa emperador",
        },
        (49, 181) => LocalisedText {
            en: "kamuy of the Nine Tails",
            ja: "九尾のカムイ",
            de: "Neunschwänziger Kamuy",
            fr: "kamuy à neuf queues",
        },
        (49, 182) => LocalisedText {
            en: "hallowed kamuy",
            ja: "青のカムイ",
            de: "Blauer Kamuy",
            fr: "kamuy de l'est",
        },
        (49, 183) => LocalisedText {
            en: "war tiger",
            ja: "ウォータイガー",
            de: "Säbelzahntiger",
            fr: "tigre de guerre",
        },
        (49, 184) => LocalisedText {
            en: "Eurekan petrel",
            ja: "エウレカスワロー",
            de: "Eureka-Schwalbe",
            fr: "hirondelle d'Eurêka",
        },
        (49, 185) => LocalisedText {
            en: "magitek avenger A-1",
            ja: "魔導アヴェンジャー寒冷地仕様",
            de: "Kälteresistenter Magitek-Rächer",
            fr: "vengeur magitek anti-froid",
        },
        (49, 186) => LocalisedText {
            en: "Demi-Ozma",
            ja: "デミオズマ",
            de: "Demi-Yadis",
            fr: "demi-Ozma",
        },
        (49, 187) => LocalisedText {
            en: "amaro",
            ja: "アマロ",
            de: "Amaro",
            fr: "amaro",
        },
        (49, 188) => LocalisedText {
            en: "skyslipper",
            ja: "ホバー船",
            de: "Wüstengleiter",
            fr: "aéroglisseur",
        },
        (49, 189) => LocalisedText {
            en: "fae gwiber",
            ja: "ピクシーグイベル",
            de: "Pixie-Gwiber",
            fr: "dragon pixie",
        },
        (49, 190) => LocalisedText {
            en: "triceratops",
            ja: "トリケラトプス",
            de: "Triceratops",
            fr: "tricératops",
        },
        (49, 191) => LocalisedText {
            en: "ironfrog mover",
            ja: "フロッガーM",
            de: "Eisenfrosch",
            fr: "grenouilleur M",
        },
        (49, 192) => LocalisedText {
            en: "innocent gwiber",
            ja: "イノセントグイベル",
            de: "Innozenz-Gwiber",
            fr: "dragon immaculé",
        },
        (49, 193) => LocalisedText {
            en: "magicked bed",
            ja: "マジックベッド",
            de: "Zauberbett",
            fr: "lit magique",
        },
        (49, 194) => LocalisedText {
            en: "Circus ahriman",
            ja: "コンチネンタル・アーリマン",
            de: "Circus-Ahriman",
            fr: "ahriman du Cirque",
        },
        (49, 195) => LocalisedText {
            en: "sunspun cumulus",
            ja: "金斗雲",
            de: "Goldener Kumulus",
            fr: "nuage magique doré",
        },
        (49, 197) => LocalisedText {
            en: "battle tiger",
            ja: "バトルタイガー",
            de: "Kampftiger",
            fr: "tigre de combat",
        },
        (49, 200) => LocalisedText {
            en: "morbol",
            ja: "モルボル",
            de: "Morbol",
            fr: "morbol",
        },
        (49, 201) => LocalisedText {
            en: "portly porxie",
            ja: "ファットポークシー",
            de: "Stattliche Quiexie",
            fr: "porxie potelé",
        },
        (49, 202) => LocalisedText {
            en: "Epimetheus",
            ja: "エピメテウス",
            de: "Epimetheus",
            fr: "Epimetheus",
        },
        (49, 203) => LocalisedText {
            en: "Menoetius",
            ja: "メノイティオス",
            de: "Menoetius",
            fr: "Menoetius",
        },
        (49, 204) => LocalisedText {
            en: "Construct VII",
            ja: "コンストラクト07",
            de: "Konstrukt7",
            fr: "constructeur 07",
        },
        (49, 205) => LocalisedText {
            en: "shadow gwiber",
            ja: "シャドウグイベル",
            de: "Schatten-Gwiber",
            fr: "dragon ombragé",
        },
        (49, 206) => LocalisedText {
            en: "spriggan stonecarrier",
            ja: "スプリガン・キャリアー",
            de: "Träger-Spriggan",
            fr: "spriggan porte-pierre",
        },
        (49, 207) => LocalisedText {
            en: "forgiven reticence",
            ja: "フォーギヴン・レティセンス",
            de: "Geläuterte Sprachlosigkeit",
            fr: "Réticence pardonnée",
        },
        (49, 208) => LocalisedText {
            en: "dhalmel",
            ja: "ダルメル",
            de: "Dhalmel",
            fr: "dhalmel",
        },
        (49, 209) => LocalisedText {
            en: "albino karakul",
            ja: "アルビノカラクール",
            de: "Albino-Karakul",
            fr: "karakul albinos",
        },
        (49, 211) => LocalisedText {
            en: "ufiti",
            ja: "ウフィティ",
            de: "Ufiti",
            fr: "ufiti",
        },
        (49, 212) => LocalisedText {
            en: "deinonychus",
            ja: "ディノニクス",
            de: "Deinonychus",
            fr: "deinonychus",
        },
        (49, 213) => LocalisedText {
            en: "Construct 14",
            ja: "労働十四号",
            de: "Automat Nr. 14",
            fr: "bâtisseur n°14",
        },
        (49, 214) => LocalisedText {
            en: "kingly peacock",
            ja: "王孔雀",
            de: "Königspfau",
            fr: "paon roi",
        },
        (49, 215) => LocalisedText {
            en: "Great Vessel of Ronka",
            ja: "水蛇様の壺",
            de: "Urne der Wasserschlange",
            fr: "jarre du Serpent marin",
        },
        (49, 216) => LocalisedText {
            en: "pteranodon",
            ja: "プテラノドン",
            de: "Pteranodon",
            fr: "ptéranodon",
        },
        (49, 217) => LocalisedText {
            en: "ruby gwiber",
            ja: "ルビーグイベル",
            de: "Rubin-Gwiber",
            fr: "dragon rubis",
        },
        (49, 218) => LocalisedText {
            en: "hybodus",
            ja: "ヒボドゥス",
            de: "Hybodus",
            fr: "hybodus",
        },
        (49, 219) => LocalisedText {
            en: "Ramuh",
            ja: "ラムウ",
            de: "Ramuh",
            fr: "Ramuh",
        },
        (49, 220) => LocalisedText {
            en: "Rubellite Carbuncle",
            ja: "カーバンクル・ルベライト",
            de: "Rubellit-Karfunkel",
            fr: "Carbuncle rubellite",
        },
        (49, 222) => LocalisedText {
            en: "chocobo carriage",
            ja: "チョコボキャリッジ",
            de: "Chocobo-Kutsche",
            fr: "chococaravane",
        },
        (49, 223) => LocalisedText {
            en: "Rolling Tankard",
            ja: "ドワーフ戦車",
            de: "Zwergenpanzer",
            fr: "automatank",
        },
        (49, 224) => LocalisedText {
            en: "Gabriel α",
            ja: "ガブリエルα",
            de: "Gabriel α",
            fr: "Gabriel α",
        },
        (49, 225) => LocalisedText {
            en: "megalotragus",
            ja: "メガロトラグス",
            de: "Megalotragus",
            fr: "mégalotragus",
        },
        (49, 226) => LocalisedText {
            en: "gwiber of light",
            ja: "グイベル・オブ・ライト",
            de: "Gwiber des Lichts",
            fr: "dragon de la Lumière",
        },
        (49, 228) => LocalisedText {
            en: "Jibanyan couch",
            ja: "ジバニャンソファ",
            de: "Jibanyan-Sofa",
            fr: "fauteuil Jibanyan",
        },
        (49, 229) => LocalisedText {
            en: "magitek hyperconveyor",
            ja: "魔導キャリアー改",
            de: "Magitek-Transporter NEO",
            fr: "transporteur magitek amélioré",
        },
        (49, 230) => LocalisedText {
            en: "Ehll Tou",
            ja: "エル・トゥ",
            de: "Ehll Tou",
            fr: "Ehll Tou",
        },
        (49, 231) => LocalisedText {
            en: "Incitatus",
            ja: "インキタトゥス",
            de: "Incitatus",
            fr: "Incitatus",
        },
        (49, 232) => LocalisedText {
            en: "snowman",
            ja: "スノーマン",
            de: "Schneemann",
            fr: "bonhomme de neige",
        },
        (49, 233) => LocalisedText {
            en: "Lunar Whale",
            ja: "ルナホエール",
            de: "Mondwal",
            fr: "baleine lunaire",
        },
        (49, 234) => LocalisedText {
            en: "Eden",
            ja: "エデン",
            de: "Eden",
            fr: "Éden",
        },
        (49, 235) => LocalisedText {
            en: "Cerberus",
            ja: "ケルベロス",
            de: "Cerberus",
            fr: "Cerbère",
        },
        (49, 236) => LocalisedText {
            en: "big shell",
            ja: "ビッグシェル",
            de: "Riesenkrabbe",
            fr: "coquénorme",
        },
        (49, 237) => LocalisedText {
            en: "Arion",
            ja: "アレイオン",
            de: "Areion",
            fr: "Arion",
        },
        (49, 238) => LocalisedText {
            en: "emerald gwiber",
            ja: "エメラルドグイベル",
            de: "Smaragd-Gwiber",
            fr: "dragon émeraude",
        },
        (49, 239) => LocalisedText {
            en: "prototype roader",
            ja: "プロトマグナローダー",
            de: "Proto-Magna Rotula",
            fr: "proto-magna rouleur magitek",
        },
        (49, 240) => LocalisedText {
            en: "chocorpokkur",
            ja: "チョコロポックル",
            de: "Chokorpokkur",
            fr: "choco-korpokkur",
        },
        (49, 241) => LocalisedText {
            en: "Gabriel Mark III",
            ja: "ガブリエルIII号機",
            de: "Gabriel III",
            fr: "Gabriel v3",
        },
        (49, 242) => LocalisedText {
            en: "antelope doe",
            ja: "アンテロープ・ドゥ",
            de: "Antilopenkuh",
            fr: "antilope biche",
        },
        (49, 243) => LocalisedText {
            en: "antelope stag",
            ja: "アンテロープ・スタッグ",
            de: "Antilopenbock",
            fr: "antilope cerf",
        },
        (49, 245) => LocalisedText {
            en: "Landerwaffe",
            ja: "レンダーバッフェ",
            de: "Mechamander",
            fr: "Landerwaffe",
        },
        (49, 246) => LocalisedText {
            en: "Al-iklil",
            ja: "イクリール",
            de: "Ikril",
            fr: "Iklil",
        },
        (49, 247) => LocalisedText {
            en: "Cruise Chaser",
            ja: "クルーズチェイサー",
            de: "Chaser",
            fr: "croiseur-chasseur",
        },
        (49, 248) => LocalisedText {
            en: "Construct VI-S",
            ja: "コンストラクト06S",
            de: "Konstrukt6S",
            fr: "constructeur 06S",
        },
        (49, 249) => LocalisedText {
            en: "diamond gwiber",
            ja: "ダイヤグイベル",
            de: "Diamant-Gwiber",
            fr: "dragon diamant",
        },
        (49, 250) => LocalisedText {
            en: "polar bear",
            ja: "ポーラーベアー",
            de: "Eisbär",
            fr: "ours polaire",
        },
        (49, 252) => LocalisedText {
            en: "gilded mikoshi",
            ja: "オオナマキン御輿",
            de: "Gold-Mikoshi",
            fr: "mikoshi doré",
        },
        (49, 253) => LocalisedText {
            en: "Resplendent Vessel of Ronka",
            ja: "金蛇様の壺",
            de: "Urne der goldenen Wasserschlange",
            fr: "jarre du Serpent doré",
        },
        (49, 254) => LocalisedText {
            en: "magicked card",
            ja: "マジックカード",
            de: "Magische Karte",
            fr: "carte magique",
        },
        (49, 261) => LocalisedText {
            en: "lynx of eternal darkness",
            ja: "エターナルダークネス・リンクス",
            de: "Umbral-Luchs",
            fr: "lynx des Ténèbres éternelles",
        },
        (49, 262) => LocalisedText {
            en: "lynx of divine light",
            ja: "ディヴァインライト・リンクス",
            de: "Astral-Luchs",
            fr: "lynx de la Lumière divine",
        },
        (49, 263) => LocalisedText {
            en: "Argos",
            ja: "アルゴス",
            de: "Argos",
            fr: "Argos",
        },
        (49, 264) => LocalisedText {
            en: "dreadnaught",
            ja: "ドレッドノート",
            de: "Brummonaut",
            fr: "cuirassé Dreadnaught",
        },
        (49, 265) => LocalisedText {
            en: "Demi-Phoinix",
            ja: "デミフェネクス",
            de: "Demi-Phoinix",
            fr: "demi-protophénix",
        },
        (49, 266) => LocalisedText {
            en: "Calydontis",
            ja: "カリュドンティス",
            de: "Calydontis",
            fr: "Calydontis",
        },
        (49, 267) => LocalisedText {
            en: "Victor",
            ja: "ヴィクトル",
            de: "Victor",
            fr: "Victor",
        },
        (49, 268) => LocalisedText {
            en: "level checker",
            ja: "レベルチェッカー",
            de: "Monitor",
            fr: "vérifiniveau",
        },
        (49, 273) => LocalisedText {
            en: "wivre",
            ja: "ウィヴル",
            de: "Wivre",
            fr: "wivre",
        },
        (49, 274) => LocalisedText {
            en: "porxie king",
            ja: "キングポークシー",
            de: "Quiexie-König",
            fr: "porxie royal",
        },
        (49, 281) => LocalisedText {
            en: "alkonost",
            ja: "アルコノスト",
            de: "Alkonost",
            fr: "alkonost",
        },
        (49, 284) => LocalisedText {
            en: "pod 602",
            ja: "ポッド602",
            de: "Pod 602",
            fr: "pod 602",
        },
        (49, 288) => LocalisedText {
            en: "troll",
            ja: "トロル",
            de: "Troll",
            fr: "troll",
        },
        (49, 291) => LocalisedText {
            en: "vinegaroon",
            ja: "ビネガロン",
            de: "Geißelskorpion",
            fr: "vinaigrier",
        },
        (49, 293) => LocalisedText {
            en: "bluefeather lynx",
            ja: "ブルーフェザー・リンクス",
            de: "Blaufeder-Luchs",
            fr: "lynx au pelage bleu",
        },
        (50, 1) => LocalisedText {
            en: "gladiator",
            ja: "剣術士",
            de: "Gladiator",
            fr: "gladiateur",
        },
        (50, 2) => LocalisedText {
            en: "pugilist",
            ja: "格闘士",
            de: "Faustkämpfer",
            fr: "pugiliste",
        },
        (50, 3) => LocalisedText {
            en: "marauder",
            ja: "斧術士",
            de: "Marodeur",
            fr: "maraudeur",
        },
        (50, 4) => LocalisedText {
            en: "lancer",
            ja: "槍術士",
            de: "Pikenier",
            fr: "maître d'hast",
        },
        (50, 5) => LocalisedText {
            en: "archer",
            ja: "弓術士",
            de: "Waldläufer",
            fr: "archer",
        },
        (50, 6) => LocalisedText {
            en: "conjurer",
            ja: "幻術士",
            de: "Druide",
            fr: "élémentaliste",
        },
        (50, 7) => LocalisedText {
            en: "thaumaturge",
            ja: "呪術士",
            de: "Thaumaturg",
            fr: "occultiste",
        },
        (50, 8) => LocalisedText {
            en: "carpenter",
            ja: "木工師",
            de: "Zimmerer",
            fr: "menuisier",
        },
        (50, 9) => LocalisedText {
            en: "blacksmith",
            ja: "鍛冶師",
            de: "Grobschmied",
            fr: "forgeron",
        },
        (50, 10) => LocalisedText {
            en: "armorer",
            ja: "甲冑師",
            de: "Plattner",
            fr: "armurier",
        },
        (50, 11) => LocalisedText {
            en: "goldsmith",
            ja: "彫金師",
            de: "Goldschmied",
            fr: "orfèvre",
        },
        (50, 12) => LocalisedText {
            en: "leatherworker",
            ja: "革細工師",
            de: "Gerber",
            fr: "tanneur",
        },
        (50, 13) => LocalisedText {
            en: "weaver",
            ja: "裁縫師",
            de: "Weber",
            fr: "couturier",
        },
        (50, 14) => LocalisedText {
            en: "alchemist",
            ja: "錬金術師",
            de: "Alchemist",
            fr: "alchimiste",
        },
        (50, 15) => LocalisedText {
            en: "culinarian",
            ja: "調理師",
            de: "Gourmet",
            fr: "cuisinier",
        },
        (50, 16) => LocalisedText {
            en: "miner",
            ja: "採掘師",
            de: "Minenarbeiter",
            fr: "mineur",
        },
        (50, 17) => LocalisedText {
            en: "botanist",
            ja: "園芸師",
            de: "Gärtner",
            fr: "botaniste",
        },
        (50, 18) => LocalisedText {
            en: "fisher",
            ja: "漁師",
            de: "Fischer",
            fr: "pêcheur",
        },
        (50, 19) => LocalisedText {
            en: "paladin",
            ja: "ナイト",
            de: "Paladin",
            fr: "paladin",
        },
        (50, 20) => LocalisedText {
            en: "monk",
            ja: "モンク",
            de: "Mönch",
            fr: "moine",
        },
        (50, 21) => LocalisedText {
            en: "warrior",
            ja: "戦士",
            de: "Krieger",
            fr: "guerrier",
        },
        (50, 22) => LocalisedText {
            en: "dragoon",
            ja: "竜騎士",
            de: "Dragoon",
            fr: "chevalier dragon",
        },
        (50, 23) => LocalisedText {
            en: "bard",
            ja: "吟遊詩人",
            de: "Barde",
            fr: "barde",
        },
        (50, 24) => LocalisedText {
            en: "white mage",
            ja: "白魔道士",
            de: "Weißmagier",
            fr: "mage blanc",
        },
        (50, 25) => LocalisedText {
            en: "black mage",
            ja: "黒魔道士",
            de: "Schwarzmagier",
            fr: "mage noir",
        },
        (50, 26) => LocalisedText {
            en: "arcanist",
            ja: "巴術士",
            de: "Hermetiker",
            fr: "arcaniste",
        },
        (50, 27) => LocalisedText {
            en: "summoner",
            ja: "召喚士",
            de: "Beschwörer",
            fr: "invocateur",
        },
        (50, 28) => LocalisedText {
            en: "scholar",
            ja: "学者",
            de: "Gelehrter",
            fr: "érudit",
        },
        (50, 29) => LocalisedText {
            en: "rogue",
            ja: "双剣士",
            de: "Schurke",
            fr: "surineur",
        },
        (50, 30) => LocalisedText {
            en: "ninja",
            ja: "忍者",
            de: "Ninja",
            fr: "ninja",
        },
        (50, 31) => LocalisedText {
            en: "machinist",
            ja: "機工士",
            de: "Maschinist",
            fr: "machiniste",
        },
        (50, 32) => LocalisedText {
            en: "dark knight",
            ja: "暗黒騎士",
            de: "Dunkelritter",
            fr: "chevalier noir",
        },
        (50, 33) => LocalisedText {
            en: "astrologian",
            ja: "占星術師",
            de: "Astrologe",
            fr: "astromancien",
        },
        (50, 34) => LocalisedText {
            en: "samurai",
            ja: "侍",
            de: "Samurai",
            fr: "samouraï",
        },
        (50, 35) => LocalisedText {
            en: "red mage",
            ja: "赤魔道士",
            de: "Rotmagier",
            fr: "mage rouge",
        },
        (50, 36) => LocalisedText {
            en: "blue mage",
            ja: "青魔道士",
            de: "Blaumagier",
            fr: "mage bleu",
        },
        (50, 37) => LocalisedText {
            en: "gunbreaker",
            ja: "ガンブレイカー",
            de: "Revolverklinge",
            fr: "pistosabreur",
        },
        (50, 38) => LocalisedText {
            en: "dancer",
            ja: "踊り子",
            de: "Tänzer",
            fr: "danseur",
        },
        (50, 39) => LocalisedText {
            en: "reaper",
            ja: "リーパー",
            de: "Schnitter",
            fr: "faucheur",
        },
        (50, 40) => LocalisedText {
            en: "sage",
            ja: "賢者",
            de: "Weiser",
            fr: "sage",
        },
        (51, 20) => LocalisedText {
            en: "Hydaelyn",
            ja: "ハイデリン",
            de: "Hydaelyn",
            fr: "Hydaelyn",
        },
        (51, 21) => LocalisedText {
            en: "Eorzea",
            ja: "エオルゼア",
            de: "Eorzea",
            fr: "Éorzéa",
        },
        (51, 22) => LocalisedText {
            en: "La Noscea",
            ja: "ラノシア",
            de: "La Noscea",
            fr: "Noscea",
        },
        (51, 23) => LocalisedText {
            en: "The Black Shroud",
            ja: "黒衣森",
            de: "Finsterwald",
            fr: "Forêt de Sombrelinceul",
        },
        (51, 24) => LocalisedText {
            en: "Thanalan",
            ja: "ザナラーン",
            de: "Thanalan",
            fr: "Thanalan",
        },
        (51, 25) => LocalisedText {
            en: "Coerthas",
            ja: "クルザス",
            de: "Coerthas",
            fr: "Coerthas",
        },
        (51, 26) => LocalisedText {
            en: "Mor Dhona",
            ja: "モードゥナ",
            de: "Mor Dhona",
            fr: "Mor Dhona",
        },
        (51, 27) => LocalisedText {
            en: "Limsa Lominsa",
            ja: "リムサ・ロミンサ",
            de: "Limsa Lominsa",
            fr: "Limsa Lominsa",
        },
        (51, 28) => LocalisedText {
            en: "Limsa Lominsa Upper Decks",
            ja: "リムサ・ロミンサ：上甲板層",
            de: "Obere Decks",
            fr: "Limsa Lominsa - Le Tillac",
        },
        (51, 29) => LocalisedText {
            en: "Limsa Lominsa Lower Decks",
            ja: "リムサ・ロミンサ：下甲板層",
            de: "Untere Decks",
            fr: "Limsa Lominsa - L'Entrepont",
        },
        (51, 30) => LocalisedText {
            en: "Middle La Noscea",
            ja: "中央ラノシア",
            de: "Zentrales La Noscea",
            fr: "Noscea centrale",
        },
        (51, 31) => LocalisedText {
            en: "Lower La Noscea",
            ja: "低地ラノシア",
            de: "Unteres La Noscea",
            fr: "Basse-Noscea",
        },
        (51, 32) => LocalisedText {
            en: "Eastern La Noscea",
            ja: "東ラノシア",
            de: "Östliches La Noscea",
            fr: "Noscea orientale",
        },
        (51, 33) => LocalisedText {
            en: "Western La Noscea",
            ja: "西ラノシア",
            de: "Westliches La Noscea",
            fr: "Noscea occidentale",
        },
        (51, 34) => LocalisedText {
            en: "Upper La Noscea",
            ja: "高地ラノシア",
            de: "Oberes La Noscea",
            fr: "Haute-Noscea",
        },
        (51, 35) => LocalisedText {
            en: "Sastasha",
            ja: "サスタシャ浸食洞",
            de: "Sastasha-Höhle",
            fr: "Sastasha",
        },
        (51, 36) => LocalisedText {
            en: "Brayflox's Longstop",
            ja: "ブレイフロクスの野営地",
            de: "Brüllvolx' Langrast",
            fr: "Bivouac de Brayflox",
        },
        (51, 37) => LocalisedText {
            en: "The Wanderer's Palace",
            ja: "ワンダラーパレス",
            de: "Palast des Wanderers",
            fr: "Palais du Vagabond",
        },
        (51, 39) => LocalisedText {
            en: "Gridania",
            ja: "グリダニア",
            de: "Gridania",
            fr: "Gridania",
        },
        (51, 40) => LocalisedText {
            en: "Ul'dah - Steps of Nald",
            ja: "ウルダハ：ナル回廊",
            de: "Nald-Kreuzgang",
            fr: "Ul'dah - Faubourg de Nald",
        },
        (51, 41) => LocalisedText {
            en: "Ul'dah - Steps of Thal",
            ja: "ウルダハ：ザル回廊",
            de: "Thal-Kreuzgang",
            fr: "Ul'dah - Faubourg de Thal",
        },
        (51, 42) => LocalisedText {
            en: "Western Thanalan",
            ja: "西ザナラーン",
            de: "Westliches Thanalan",
            fr: "Thanalan occidental",
        },
        (51, 43) => LocalisedText {
            en: "Central Thanalan",
            ja: "中央ザナラーン",
            de: "Zentrales Thanalan",
            fr: "Thanalan central",
        },
        (51, 44) => LocalisedText {
            en: "Eastern Thanalan",
            ja: "東ザナラーン",
            de: "Östliches Thanalan",
            fr: "Thanalan oriental",
        },
        (51, 45) => LocalisedText {
            en: "Southern Thanalan",
            ja: "南ザナラーン",
            de: "Südliches Thanalan",
            fr: "Thanalan méridional",
        },
        (51, 46) => LocalisedText {
            en: "Northern Thanalan",
            ja: "北ザナラーン",
            de: "Nördliches Thanalan",
            fr: "Thanalan septentrional",
        },
        (51, 47) => LocalisedText {
            en: "Cutter's Cry",
            ja: "カッターズクライ",
            de: "Sägerschrei",
            fr: "Gouffre hurlant",
        },
        (51, 48) => LocalisedText {
            en: "Copperbell Mines",
            ja: "カッパーベル銅山",
            de: "Kupferglocken-Mine",
            fr: "Mines de Clochecuivre",
        },
        (51, 49) => LocalisedText {
            en: "Halatali",
            ja: "ハラタリ修練所",
            de: "Halatali",
            fr: "Halatali",
        },
        (51, 50) => LocalisedText {
            en: "The Sunken Temple of Qarn",
            ja: "カルン埋没寺院",
            de: "Versunkener Tempel von Qarn",
            fr: "Temple enseveli de Qarn",
        },
        (51, 51) => LocalisedText {
            en: "Ul'dah",
            ja: "ウルダハ",
            de: "Ul'dah",
            fr: "Ul'dah",
        },
        (51, 52) => LocalisedText {
            en: "New Gridania",
            ja: "グリダニア：新市街",
            de: "Neu-Gridania",
            fr: "Nouvelle Gridania",
        },
        (51, 53) => LocalisedText {
            en: "Old Gridania",
            ja: "グリダニア：旧市街",
            de: "Alt-Gridania",
            fr: "Vieille Gridania",
        },
        (51, 54) => LocalisedText {
            en: "Central Shroud",
            ja: "黒衣森：中央森林",
            de: "Tiefer Wald",
            fr: "Forêt centrale",
        },
        (51, 55) => LocalisedText {
            en: "East Shroud",
            ja: "黒衣森：東部森林",
            de: "Ostwald",
            fr: "Forêt de l'est",
        },
        (51, 56) => LocalisedText {
            en: "South Shroud",
            ja: "黒衣森：南部森林",
            de: "Südwald",
            fr: "Forêt du sud",
        },
        (51, 57) => LocalisedText {
            en: "North Shroud",
            ja: "黒衣森：北部森林",
            de: "Nordwald",
            fr: "Forêt du nord",
        },
        (51, 58) => LocalisedText {
            en: "The Tam-Tara Deepcroft",
            ja: "タムタラの墓所",
            de: "Totenacker Tam-Tara",
            fr: "Hypogée de Tam-Tara",
        },
        (51, 59) => LocalisedText {
            en: "Haukke Manor",
            ja: "ハウケタ御用邸",
            de: "Haukke-Herrenhaus",
            fr: "Manoir des Haukke",
        },
        (51, 61) => LocalisedText {
            en: "The Thousand Maws of Toto-Rak",
            ja: "トトラクの千獄",
            de: "Tausend Löcher von Toto-Rak",
            fr: "Mille Gueules de Toto-Rak",
        },
        (51, 62) => LocalisedText {
            en: "Ishgard",
            ja: "イシュガルド",
            de: "Ishgard",
            fr: "Ishgard",
        },
        (51, 63) => LocalisedText {
            en: "Coerthas Central Highlands",
            ja: "クルザス中央高地",
            de: "Zentrales Hochland von Coerthas",
            fr: "Hautes terres du Coerthas central",
        },
        (51, 64) => LocalisedText {
            en: "Dzemael Darkhold",
            ja: "ゼーメル要塞",
            de: "Feste Dzemael",
            fr: "Forteresse de Dzemael",
        },
        (51, 65) => LocalisedText {
            en: "Aurum Vale",
            ja: "オーラムヴェイル",
            de: "Goldklamm",
            fr: "Val d'Aurum",
        },
        (51, 94) => LocalisedText {
            en: "Bentbranch Meadows",
            ja: "ベントブランチ牧場",
            de: "Gut Zwieselgrund",
            fr: "Ranch de Brancharquée",
        },
        (51, 107) => LocalisedText {
            en: "The Hawthorne Hut",
            ja: "ホウソーン家の山塞",
            de: "Jagdhütte der Hawthornes",
            fr: "Hutte des Hawthorne",
        },
        (51, 112) => LocalisedText {
            en: "Sanctum of the Twelve",
            ja: "十二神大聖堂",
            de: "Dom der Einkehr",
            fr: "Sanctuaire des Douze",
        },
        (51, 123) => LocalisedText {
            en: "Camp Tranquil",
            ja: "キャンプ・トランキル",
            de: "Camp Seelenruhe",
            fr: "Camp des Sentes tranquilles",
        },
        (51, 125) => LocalisedText {
            en: "The Lost City of Amdapor",
            ja: "古アムダプール市街",
            de: "Historisches Amdapor",
            fr: "Vestiges de la cité d'Amdapor",
        },
        (51, 128) => LocalisedText {
            en: "Amdapor Keep",
            ja: "古城アムダプール",
            de: "Ruinen von Amdapor",
            fr: "Château d'Amdapor",
        },
        (51, 129) => LocalisedText {
            en: "Quarrymill",
            ja: "クォーリーミル",
            de: "Mühlenbruch",
            fr: "Moulin de la Carrière",
        },
        (51, 140) => LocalisedText {
            en: "Fallgourd Float",
            ja: "フォールゴウド",
            de: "Herbstkürbis-See",
            fr: "Radeau de la Calebasse",
        },
        (51, 154) => LocalisedText {
            en: "The Fold",
            ja: "ザ・フォールド",
            de: "Wolfspferch (The Fold)",
            fr: "La Louverie (The Fold)",
        },
        (51, 184) => LocalisedText {
            en: "Summerford Farms",
            ja: "サマーフォード庄",
            de: "Sommerfurt-Höfe",
            fr: "Vergers d'Estival",
        },
        (51, 206) => LocalisedText {
            en: "Costa del Sol",
            ja: "コスタ・デル・ソル",
            de: "Sonnenküste",
            fr: "Costa del Sol",
        },
        (51, 216) => LocalisedText {
            en: "Wineport",
            ja: "ワインポート",
            de: "Weinhafen",
            fr: "Port-aux-Vins",
        },
        (51, 218) => LocalisedText {
            en: "Swiftperch",
            ja: "スウィフトパーチ入植地",
            de: "Tölpelturm-Siedlung",
            fr: "Le Martinet",
        },
        (51, 223) => LocalisedText {
            en: "Aleport",
            ja: "エールポート",
            de: "Bierhafen",
            fr: "Port-aux-Ales",
        },
        (51, 230) => LocalisedText {
            en: "Pharos Sirius",
            ja: "シリウス大灯台",
            de: "Pharos Sirius",
            fr: "Phare de Sirius",
        },
        (51, 237) => LocalisedText {
            en: "Camp Overlook",
            ja: "キャンプ・オーバールック",
            de: "Camp Ausblick",
            fr: "Camp du Guet",
        },
        (51, 239) => LocalisedText {
            en: "Camp Bronze Lake",
            ja: "キャンプ・ブロンズレイク",
            de: "Camp Bronzesee",
            fr: "Camp du lac d'Airain",
        },
        (51, 260) => LocalisedText {
            en: "Castrum Meridianum",
            ja: "カストルム・メリディアヌム",
            de: "Castrum Meridianum",
            fr: "Castrum Meridianum",
        },
        (51, 271) => LocalisedText {
            en: "Horizon",
            ja: "ホライズン",
            de: "Horizont",
            fr: "Horizon",
        },
        (51, 290) => LocalisedText {
            en: "Black Brush Station",
            ja: "ブラックブラッシュ停留所",
            de: "Kohlenstaub-Bahnhof",
            fr: "Gare de Roncenoire",
        },
        (51, 300) => LocalisedText {
            en: "Camp Drybone",
            ja: "キャンプ・ドライボーン",
            de: "Camp Knochenbleich",
            fr: "Camp des Os desséchés",
        },
        (51, 313) => LocalisedText {
            en: "Little Ala Mhigo",
            ja: "リトルアラミゴ",
            de: "Klein-Ala Mhigo",
            fr: "Petite Ala Mhigo",
        },
        (51, 323) => LocalisedText {
            en: "Forgotten Springs",
            ja: "忘れられたオアシス",
            de: "Die Vergessene Oase",
            fr: "Oasis oubliée",
        },
        (51, 325) => LocalisedText {
            en: "Camp Bluefog",
            ja: "キャンプ・ブルーフォグ",
            de: "Camp Blauer Dunst",
            fr: "Camp de Brumebleue",
        },
        (51, 331) => LocalisedText {
            en: "Ceruleum Processing Plant",
            ja: "青燐精製所",
            de: "Erdseim-Fabrik",
            fr: "Usine de céruleum",
        },
        (51, 337) => LocalisedText {
            en: "Moraby Drydocks",
            ja: "モラビー造船廠",
            de: "Moraby-Trockendocks",
            fr: "Chantier naval de Moraby",
        },
        (51, 350) => LocalisedText {
            en: "Outer La Noscea",
            ja: "外地ラノシア",
            de: "Äußeres La Noscea",
            fr: "Noscea extérieure",
        },
        (51, 357) => LocalisedText {
            en: "Bowl of Embers",
            ja: "炎帝祭跡",
            de: "Das Grab der Lohe",
            fr: "Cratère des tisons",
        },
        (51, 358) => LocalisedText {
            en: "Wolves' Den Pier",
            ja: "ウルヴズジェイル係船場",
            de: "Wolfshöhlen-Pier",
            fr: "Jetée de l'Antre des loups",
        },
        (51, 359) => LocalisedText {
            en: "The Navel",
            ja: "オ・ゴモロ火口神殿",
            de: "Der Nabel",
            fr: "Le Nombril",
        },
        (51, 360) => LocalisedText {
            en: "Thornmarch",
            ja: "茨の園",
            de: "Dornmarsch",
            fr: "Lisière de ronces",
        },
        (51, 361) => LocalisedText {
            en: "The Howling Eye",
            ja: "ハウリングアイ石塔群",
            de: "Das Tosende Auge",
            fr: "Hurlœil",
        },
        (51, 386) => LocalisedText {
            en: "Griffin Crossing",
            ja: "グリフィン大橋",
            de: "Greifenbrücke",
            fr: "Traversée du Griffon",
        },
        (51, 388) => LocalisedText {
            en: "Camp Dragonhead",
            ja: "キャンプ・ドラゴンヘッド",
            de: "Camp Drachenkopf",
            fr: "Camp de la Tête du dragon",
        },
        (51, 392) => LocalisedText {
            en: "The Weeping Saint",
            ja: "聖ダナフェンの落涙",
            de: "Der Trauernde Heilige ",
            fr: "Le Saint affligé",
        },
        (51, 401) => LocalisedText {
            en: "Stone Vigil",
            ja: "ストーンヴィジル",
            de: "Steinerne Wacht",
            fr: "Vigile de Pierre",
        },
        (51, 404) => LocalisedText {
            en: "Snowcloak",
            ja: "スノークローク大氷壁",
            de: "Schneekleid",
            fr: "Manteneige",
        },
        (51, 406) => LocalisedText {
            en: "Steps of Faith",
            ja: "雲廊",
            de: "Schicksalsweg",
            fr: "Avancée de la Foi",
        },
        (51, 411) => LocalisedText {
            en: "Revenant's Toll",
            ja: "レヴナンツトール",
            de: "Geisterzoll",
            fr: "Glas des revenants",
        },
        (51, 418) => LocalisedText {
            en: "The Keeper of the Lake",
            ja: "黙約の塔",
            de: "Der Hüter des Sees",
            fr: "Gardien du lac",
        },
        (51, 425) => LocalisedText {
            en: "Mist",
            ja: "ミスト・ヴィレッジ",
            de: "Dorf des Nebels",
            fr: "Brumée",
        },
        (51, 426) => LocalisedText {
            en: "The Lavender Beds",
            ja: "ラベンダーベッド",
            de: "Lavendelbeete",
            fr: "Lavandière",
        },
        (51, 427) => LocalisedText {
            en: "The Goblet",
            ja: "ゴブレットビュート",
            de: "Kelchkuppe",
            fr: "La Coupe",
        },
        (51, 430) => LocalisedText {
            en: "The Praetorium",
            ja: "魔導城プラエトリウム",
            de: "Praetorium",
            fr: "Praetorium",
        },
        (51, 457) => LocalisedText {
            en: "Porta Decumana",
            ja: "ポルタ・デクマーナ",
            de: "Porta Decumana",
            fr: "Porta Decumana",
        },
        (51, 464) => LocalisedText {
            en: "Upper Aetheroacoustic Exploratory Site",
            ja: "メテオ探査坑浅部",
            de: "Obere ätheroakustische Grabung",
            fr: "Site impérial d'exploration supérieur",
        },
        (51, 465) => LocalisedText {
            en: "Lower Aetheroacoustic Exploratory Site",
            ja: "メテオ探査坑深部",
            de: "Untere ätheroakustische Grabung",
            fr: "Site impérial d'exploration inférieur",
        },
        (51, 466) => LocalisedText {
            en: "The Ragnarok",
            ja: "ラグナロク級拘束艦",
            de: "Die Ragnarök",
            fr: "Le Ragnarok",
        },
        (51, 467) => LocalisedText {
            en: "Ragnarok Drive Cylinder",
            ja: "稼働隔壁",
            de: "Antriebszylinder der Ragnarök",
            fr: "Cylindre propulseur du Ragnarok",
        },
        (51, 468) => LocalisedText {
            en: "Ragnarok Central Core",
            ja: "中枢区画",
            de: "Kernsektor der Ragnarök",
            fr: "Noyau central du Ragnarok",
        },
        (51, 478) => LocalisedText {
            en: "Labyrinth of the Ancients",
            ja: "古代の民の迷宮",
            de: "Labyrinth der Alten",
            fr: "Dédale antique",
        },
        (51, 493) => LocalisedText {
            en: "Syrcus Tower",
            ja: "シルクスの塔",
            de: "Syrcus-Turm",
            fr: "Tour de Syrcus",
        },
        (51, 496) => LocalisedText {
            en: "Seal Rock",
            ja: "シールロック",
            de: "Robbenholm",
            fr: "Rocher des tréfonds",
        },
        (51, 497) => LocalisedText {
            en: "Abalathia's Spine",
            ja: "アバラシア",
            de: "Abalathia",
            fr: "Abalathia",
        },
        (51, 498) => LocalisedText {
            en: "Dravania",
            ja: "ドラヴァニア",
            de: "Dravania",
            fr: "Dravania",
        },
        (51, 548) => LocalisedText {
            en: "The Roost",
            ja: "旅館「とまり木」",
            de: "Der Traumbaum",
            fr: "Le Perchoir",
        },
        (51, 617) => LocalisedText {
            en: "The Hourglass",
            ja: "宿屋「砂時計亭」",
            de: "Die Sanduhr",
            fr: "Le Sablier",
        },
        (51, 694) => LocalisedText {
            en: "Hall of the Bestiarii",
            ja: "獣闘の間",
            de: "Halle der Bestie",
            fr: "Hall des Bestiarii",
        },
        (51, 695) => LocalisedText {
            en: "Frondale's Home for Friendless Foundlings",
            ja: "フロンデール薬学院小児病棟",
            de: "Frondales Physiatrische Akademie",
            fr: "Clinique pédiatrique de l'Académie médicale de Frondale",
        },
        (51, 733) => LocalisedText {
            en: "Mizzenmast Inn",
            ja: "宿屋「ミズンマスト」",
            de: "Gasthaus Gaffelschoner",
            fr: "Auberge de L'Artimon",
        },
        (51, 1301) => LocalisedText {
            en: "Dalamud's Shadow",
            ja: "メテオの陰地",
            de: "Meteoritenkrater",
            fr: "Ombrage du Météore",
        },
        (51, 1302) => LocalisedText {
            en: "The Outer Coil",
            ja: "バインディングコイル",
            de: "Äußere Schächte",
            fr: "L'Écheveau",
        },
        (51, 1303) => LocalisedText {
            en: "Central Decks",
            ja: "ラグナロク級三番艦：艦体中央部",
            de: "Zentrale Decks",
            fr: "Secteur central RCI-03",
        },
        (51, 1304) => LocalisedText {
            en: "The Holocharts",
            ja: "ラグナロク級三番艦：作戦室",
            de: "Holosphäre",
            fr: "Salle de contrôle RCI-03",
        },
        (51, 1334) => LocalisedText {
            en: "The Whorleater",
            ja: "対リヴァイアサン双胴船",
            de: "Der Wogenbrecher",
            fr: "Briseur de marées",
        },
        (51, 1363) => LocalisedText {
            en: "The Striking Tree",
            ja: "古木の神判地",
            de: "Baum des Urteils",
            fr: "Arbre du jugement",
        },
        (51, 1374) => LocalisedText {
            en: "Carteneau Flats: Borderland Ruins",
            ja: "カルテノー平原：外縁遺跡群",
            de: "Äußere Ruinen",
            fr: "Ruines frontalières",
        },
        (51, 1377) => LocalisedText {
            en: "Hullbreaker Isle",
            ja: "ハルブレーカー・アイル",
            de: "Schiffbrecher-Insel",
            fr: "Île de Crèvecarène",
        },
        (51, 1390) => LocalisedText {
            en: "The Chrysalis",
            ja: "戒律の殻",
            de: "Chrysalis",
            fr: "La Chrysalide",
        },
        (51, 1399) => LocalisedText {
            en: "Akh Afah Amphitheatre",
            ja: "アク・アファー円形劇場",
            de: "Akh Afah",
            fr: "Amphithéâtre d'Akh Afah",
        },
        (51, 1406) => LocalisedText {
            en: "IC-06 Central Decks",
            ja: "ラグナロク級六番艦：艦体中央部",
            de: "Zentrale Decks RVH-06",
            fr: "Secteur central RCI-06",
        },
        (51, 1407) => LocalisedText {
            en: "IC-06 Regeneration Grid",
            ja: "ラグナロク級六番艦：再生制御区",
            de: "Regenerationssektor RVH-06",
            fr: "Réseau de régénération RCI-06",
        },
        (51, 1408) => LocalisedText {
            en: "IC-06 Main Bridge",
            ja: "ラグナロク級六番艦：第一艦橋",
            de: "Kommandobrücke RVH-06",
            fr: "Pont principal RCI-06",
        },
        (51, 1409) => LocalisedText {
            en: "The Burning Heart",
            ja: "バハムートコア",
            de: "Bahamuts Herz",
            fr: "Cœur de Bahamut",
        },
        (51, 1431) => LocalisedText {
            en: "The World of Darkness",
            ja: "闇の世界",
            de: "Welt der Dunkelheit",
            fr: "Monde des Ténèbres",
        },
        (51, 1484) => LocalisedText {
            en: "The Gold Saucer",
            ja: "ゴールドソーサー",
            de: "Gold Saucer",
            fr: "Gold Saucer",
        },
        (51, 1500) => LocalisedText {
            en: "Chocobo Square",
            ja: "チョコボスクウェア",
            de: "Chocobo-Platz",
            fr: "Square des chocobos",
        },
        (51, 1628) => LocalisedText {
            en: "The Fist of the Father",
            ja: "フィスト・オブ・ゴルディオス",
            de: "Faust des Vaters",
            fr: "Le Poing du Père",
        },
        (51, 1633) => LocalisedText {
            en: "The Cuff of the Father",
            ja: "カフ・オブ・ゴルディオス",
            de: "Elle des Vaters",
            fr: "Le Poignet du Père",
        },
        (51, 1638) => LocalisedText {
            en: "The Arm of the Father",
            ja: "アーム・オブ・ゴルディオス",
            de: "Arm des Vaters",
            fr: "Le Bras du Père",
        },
        (51, 1645) => LocalisedText {
            en: "The Burden of the Father",
            ja: "バーデン・オブ・ゴルディオス",
            de: "Last des Vaters",
            fr: "Le Fardeau du Père",
        },
        (51, 1647) => LocalisedText {
            en: "The Diadem",
            ja: "ディアデム諸島",
            de: "Das Diadem",
            fr: "Le Diadème",
        },
        (51, 1664) => LocalisedText {
            en: "The Feasting Grounds",
            ja: "ウルヴズジェイル演習場",
            de: "Kampfplatz",
            fr: "Festin des loups",
        },
        (51, 1665) => LocalisedText {
            en: "The Battlehall",
            ja: "カードバトルルーム",
            de: "Weltensalon",
            fr: "Arène Triple Triade",
        },
        (51, 1708) => LocalisedText {
            en: "The Fist of the Son",
            ja: "フィスト・オブ・ミダース",
            de: "Faust des Sohnes",
            fr: "Le Poing du Fils",
        },
        (51, 1714) => LocalisedText {
            en: "The Cuff of the Son",
            ja: "カフ・オブ・ミダース",
            de: "Elle des Sohnes",
            fr: "Le Poignet du Fils",
        },
        (51, 1723) => LocalisedText {
            en: "The Arm of the Son",
            ja: "アーム・オブ・ミダース",
            de: "Arm des Sohnes",
            fr: "Le Bras du Fils",
        },
        (51, 1731) => LocalisedText {
            en: "The Burden of the Son",
            ja: "バーデン・オブ・ミダース",
            de: "Last des Sohnes",
            fr: "Le Fardeau du Fils",
        },
        (51, 1740) => LocalisedText {
            en: "The Fields of Glory",
            ja: "フィールド・オブ・グローリー",
            de: "Feld der Ehre",
            fr: "Champs de la Gloire",
        },
        (51, 1742) => LocalisedText {
            en: "The Weeping City of Mhach",
            ja: "禁忌都市マハ",
            de: "Die Stadt der Tränen",
            fr: "Cité défendue de Mhach",
        },
        (51, 1759) => LocalisedText {
            en: "The Binding Coil of Bahamut",
            ja: "大迷宮バハムート",
            de: "Die Verschlungenen Schatten von Bahamut",
            fr: "Abîme infini de Bahamut",
        },
        (51, 1792) => LocalisedText {
            en: "Xelphatol",
            ja: "ゼルファトル",
            de: "Xelphatol",
            fr: "Xelphatol",
        },
        (51, 1793) => LocalisedText {
            en: "The Palace of the Dead",
            ja: "死者の宮殿",
            de: "Palast der Toten",
            fr: "Palais des morts",
        },
        (51, 1799) => LocalisedText {
            en: "The Aquapolis",
            ja: "アクアポリス",
            de: "Aquapolis",
            fr: "Aquapole",
        },
        (51, 1800) => LocalisedText {
            en: "Twin Adder Barracks",
            ja: "双蛇党兵舎",
            de: "Wachstube der Bruderschaft",
            fr: "Caserne de l'ordre des Deux Vipères",
        },
        (51, 1801) => LocalisedText {
            en: "Flame Barracks",
            ja: "不滅隊兵舎",
            de: "Wachstube der Legion",
            fr: "Caserne des Immortels",
        },
        (51, 1802) => LocalisedText {
            en: "Maelstrom Barracks",
            ja: "黒渦団兵舎",
            de: "Wachstube des Mahlstroms",
            fr: "Caserne du Maelstrom",
        },
        (51, 1805) => LocalisedText {
            en: "The Topmast",
            ja: "トップマスト",
            de: "Fockmast",
            fr: "Le Mât de hune",
        },
        (51, 1807) => LocalisedText {
            en: "Lily Hills",
            ja: "リリーヒルズ",
            de: "Lilienhügel",
            fr: "Plaine-aux-Lys",
        },
        (51, 1809) => LocalisedText {
            en: "The Sultana's Breath",
            ja: "ナナモ大風車",
            de: "Nanamo-Windrad",
            fr: "Le Souffle de la Sultane",
        },
        (51, 1835) => LocalisedText {
            en: "Eyes of the Creator",
            ja: "アイズ・オブ・アレキサンダー",
            de: "Augen des Schöpfers",
            fr: "Les Yeux du Créateur",
        },
        (51, 1841) => LocalisedText {
            en: "Breath of the Creator",
            ja: "ブレス・オブ・アレキサンダー",
            de: "Atem des Schöpfers",
            fr: "Le Souffle du Créateur",
        },
        (51, 1847) => LocalisedText {
            en: "Heart of the Creator",
            ja: "ハート・オブ・アレキサンダー",
            de: "Herz des Schöpfers",
            fr: "Le Cœur du Créateur",
        },
        (51, 1853) => LocalisedText {
            en: "Soul of the Creator",
            ja: "ソウル・オブ・アレキサンダー",
            de: "Seele des Schöpfers",
            fr: "L'Âme du Créateur",
        },
        (51, 1857) => LocalisedText {
            en: "Baelsar's Wall",
            ja: "バエサルの長城",
            de: "Baelsar-Wall",
            fr: "Muraille de Baelsar",
        },
        (51, 1868) => LocalisedText {
            en: "Dun Scaith",
            ja: "影の国ダン・スカー",
            de: "Dun Scaith",
            fr: "Dun Scaith",
        },
        (51, 1884) => LocalisedText {
            en: "Lichenweed",
            ja: "ライケンウィード演習場",
            de: "Flechtenhain",
            fr: "Pré-de-lichen",
        },
        (51, 2000) => LocalisedText {
            en: "The Dravanian Forelands",
            ja: "高地ドラヴァニア",
            de: "Dravanisches Vorland",
            fr: "Avant-pays dravanien",
        },
        (51, 2001) => LocalisedText {
            en: "The Dravanian Hinterlands",
            ja: "低地ドラヴァニア",
            de: "Dravanisches Hinterland",
            fr: "Arrière-pays dravanien",
        },
        (51, 2002) => LocalisedText {
            en: "The Churning Mists",
            ja: "ドラヴァニア雲海",
            de: "Wallende Nebel",
            fr: "L'Écume des cieux de Dravania",
        },
        (51, 2018) => LocalisedText {
            en: "Tailfeather",
            ja: "テイルフェザー",
            de: "Fiederhag",
            fr: "La Penne",
        },
        (51, 2025) => LocalisedText {
            en: "Anyx Trine",
            ja: "不浄の三塔",
            de: "Trinär-Annex",
            fr: "Annexe Trine",
        },
        (51, 2031) => LocalisedText {
            en: "Sohm Al",
            ja: "ソーム・アル",
            de: "Sohm Al",
            fr: "Sohm Al",
        },
        (51, 2034) => LocalisedText {
            en: "Saint Mocianne's Arboretum",
            ja: "聖モシャーヌ植物園",
            de: "Sankt Mocianne-Arboretum",
            fr: "Arboretum Sainte-Mocianne",
        },
        (51, 2038) => LocalisedText {
            en: "The Great Gubal Library",
            ja: "グブラ幻想図書館",
            de: "Große Gubal-Bibliothek",
            fr: "Grande bibliothèque de Gubal",
        },
        (51, 2042) => LocalisedText {
            en: "Moghome",
            ja: "モグモグホーム",
            de: "Moglingen",
            fr: "Nid-Mog",
        },
        (51, 2046) => LocalisedText {
            en: "Zenith",
            ja: "白亜の宮殿",
            de: "Kreidetempel",
            fr: "Zénith",
        },
        (51, 2050) => LocalisedText {
            en: "The Aery",
            ja: "ドラゴンズエアリー",
            de: "Nest des Drachen",
            fr: "L'Aire",
        },
        (51, 2081) => LocalisedText {
            en: "Thok ast Thok",
            ja: "武神の闘技場",
            de: "Thok ast Thok",
            fr: "Thok ast Thok",
        },
        (51, 2082) => LocalisedText {
            en: "Idyllshire",
            ja: "イディルシャイア",
            de: "Frohehalde",
            fr: "Idyllée",
        },
        (51, 2088) => LocalisedText {
            en: "The Antitower",
            ja: "逆さの塔",
            de: "Antiturm",
            fr: "L'Antitour",
        },
        (51, 2090) => LocalisedText {
            en: "Sohr Khai",
            ja: "ソール・カイ",
            de: "Sohr Khai",
            fr: "Sohr Khai",
        },
        (51, 2099) => LocalisedText {
            en: "Astragalos",
            ja: "アストラガロス",
            de: "Astragalos",
            fr: "Astragalos",
        },
        (51, 2100) => LocalisedText {
            en: "The Sea of Clouds",
            ja: "アバラシア雲海",
            de: "Abalathisches Wolkenmeer",
            fr: "L'Écume des cieux d'Abalathia",
        },
        (51, 2101) => LocalisedText {
            en: "Azys Lla",
            ja: "アジス・ラー",
            de: "Azys Lla",
            fr: "Azys Lla",
        },
        (51, 2116) => LocalisedText {
            en: "Camp Cloudtop",
            ja: "キャンプ・クラウドトップ",
            de: "Camp Wolkenwipfel",
            fr: "Camp de Cime-des-nuages",
        },
        (51, 2123) => LocalisedText {
            en: "Ok' Zundu",
            ja: "オク・ズンド",
            de: "Ok' Zundo",
            fr: "Ok' Zundu",
        },
        (51, 2130) => LocalisedText {
            en: "Neverreap",
            ja: "ネバーリープ",
            de: "Nimmerreich",
            fr: "Nalloncques",
        },
        (51, 2131) => LocalisedText {
            en: "Helix",
            ja: "ポート・ヘリックス",
            de: "Helix",
            fr: "Hélice",
        },
        (51, 2147) => LocalisedText {
            en: "Aetherochemical Research Facility",
            ja: "魔科学研究所",
            de: "Ätherochemisches Forschungslabor",
            fr: "Laboratoire de magismologie",
        },
        (51, 2148) => LocalisedText {
            en: "The Fractal Continuum",
            ja: "フラクタル・コンティニアム",
            de: "Die Fraktal-Kontinuum",
            fr: "Le Continuum fractal",
        },
        (51, 2151) => LocalisedText {
            en: "The Limitless Blue",
            ja: "群青空域",
            de: "Grenzenloses Blau",
            fr: "L'Immensité bleue",
        },
        (51, 2178) => LocalisedText {
            en: "Singularity Reactor",
            ja: "シンギュラリティ・リアクター",
            de: "Singularitäts-Reaktor",
            fr: "Réacteur de singularité",
        },
        (51, 2181) => LocalisedText {
            en: "Void Ark",
            ja: "魔航船ヴォイドアーク",
            de: "Nichts-Arche",
            fr: "Arche du néant",
        },
        (51, 2200) => LocalisedText {
            en: "Coerthas Western Highlands",
            ja: "クルザス西部高地",
            de: "Westliches Hochland von Coerthas",
            fr: "Hautes terres du Coerthas occidental",
        },
        (51, 2204) => LocalisedText {
            en: "Falcon's Nest",
            ja: "ファルコンネスト",
            de: "Falkenhorst",
            fr: "Nid du faucon",
        },
        (51, 2214) => LocalisedText {
            en: "Dusk Vigil",
            ja: "ダスクヴィジル",
            de: "Abendrot-Wacht",
            fr: "Vigile du Crépuscule",
        },
        (51, 2256) => LocalisedText {
            en: "Containment Bay S1T7",
            ja: "コンテイメントベイS1T7",
            de: "Arretur S1T7",
            fr: "Unité de contention S1P7",
        },
        (51, 2265) => LocalisedText {
            en: "Containment Bay P1T6",
            ja: "コンテイメントベイP1T6",
            de: "Arretur P1T6",
            fr: "Unité de contention P1P6",
        },
        (51, 2266) => LocalisedText {
            en: "Containment Bay Z1T9",
            ja: "コンテイメントベイZ1T9",
            de: "Arretur Z1T9",
            fr: "Unité de contention Z1P9",
        },
        (51, 2295) => LocalisedText {
            en: "The Blessed Treasury",
            ja: "宝物殿",
            de: "Schatzkammer",
            fr: "Crique aux tributs",
        },
        (51, 2297) => LocalisedText {
            en: "The Sirensong Sea",
            ja: "セイレーン海",
            de: "Sirenen-See",
            fr: "Mer du Chant des sirènes",
        },
        (51, 2298) => LocalisedText {
            en: "Kugane Castle",
            ja: "クガネ城",
            de: "Schloss Kugane",
            fr: "Château de Kugane",
        },
        (51, 2299) => LocalisedText {
            en: "Emanation",
            ja: "美神の地下神殿",
            de: "Heiligtum der Qalyana",
            fr: "Émanation",
        },
        (51, 2300) => LocalisedText {
            en: "Foundation",
            ja: "イシュガルド：下層",
            de: "Fundamente",
            fr: "Ishgard - L'Assise",
        },
        (51, 2301) => LocalisedText {
            en: "The Pillars",
            ja: "イシュガルド：上層",
            de: "Strebewerk",
            fr: "Ishgard - Les Contreforts",
        },
        (51, 2310) => LocalisedText {
            en: "Cloud Nine",
            ja: "旅籠「九つの雲」",
            de: "Zum Wolkenschäfchen",
            fr: "Les Neuf-Nuages",
        },
        (51, 2327) => LocalisedText {
            en: "The Vault",
            ja: "イシュガルド教皇庁",
            de: "Erzbasilika",
            fr: "La Voûte",
        },
        (51, 2340) => LocalisedText {
            en: "The Lost Canals of Uznair",
            ja: "ウズネアカナル",
            de: "Kanäle von Uznair",
            fr: "Canaux perdus d'Uznair",
        },
        (51, 2354) => LocalisedText {
            en: "The Jade Stoa",
            ja: "白帝の竹林",
            de: "Jadelichtung",
            fr: "Clairière de Jade",
        },
        (51, 2357) => LocalisedText {
            en: "Deltascape V1.0",
            ja: "被検世界「デルタ」V1.0",
            de: "Deltametrie 1.0",
            fr: "Deltastice v1.0",
        },
        (51, 2358) => LocalisedText {
            en: "Deltascape V2.0",
            ja: "被検世界「デルタ」V2.0",
            de: "Deltametrie 2.0",
            fr: "Deltastice v2.0",
        },
        (51, 2359) => LocalisedText {
            en: "Deltascape V3.0",
            ja: "被検世界「デルタ」V3.0",
            de: "Deltametrie 3.0",
            fr: "Deltastice v3.0",
        },
        (51, 2360) => LocalisedText {
            en: "Deltascape V4.0",
            ja: "被検世界「デルタ」V4.0",
            de: "Deltametrie 4.0",
            fr: "Deltastice v4.0",
        },
        (51, 2367) => LocalisedText {
            en: "The Drowned City of Skalla",
            ja: "スカラ",
            de: "Die versunkene Stadt Skalla",
            fr: "Cité engloutie de Skalla",
        },
        (51, 2372) => LocalisedText {
            en: "The Royal City of Rabanastre",
            ja: "失われた都 ラバナスタ",
            de: "Die königliche Stadt Rabanastre",
            fr: "Cité royale de Rabanastre",
        },
        (51, 2383) => LocalisedText {
            en: "Crystal Tower Training Grounds",
            ja: "クリスタルタワー演習場",
            de: "Kristallturm-Arena",
            fr: "Arène de la Tour de Cristal",
        },
        (51, 2400) => LocalisedText {
            en: "Gyr Abania",
            ja: "ギラバニア",
            de: "Gyr Abania",
            fr: "Gyr Abania",
        },
        (51, 2401) => LocalisedText {
            en: "Othard",
            ja: "オサード",
            de: "Othard",
            fr: "Othard",
        },
        (51, 2402) => LocalisedText {
            en: "Hingashi",
            ja: "ひんがしの国",
            de: "Hingashi",
            fr: "Hingashi",
        },
        (51, 2403) => LocalisedText {
            en: "Rhalgr's Reach",
            ja: "ラールガーズリーチ",
            de: "Rhalgrs Wacht",
            fr: "L'Étendue de Rhalgr",
        },
        (51, 2404) => LocalisedText {
            en: "Kugane",
            ja: "クガネ",
            de: "Kugane",
            fr: "Kugane",
        },
        (51, 2406) => LocalisedText {
            en: "The Fringes",
            ja: "ギラバニア辺境地帯",
            de: "Abanisches Grenzland",
            fr: "Les Marges",
        },
        (51, 2407) => LocalisedText {
            en: "The Peaks",
            ja: "ギラバニア山岳地帯",
            de: "Die Zinnen",
            fr: "Les Pics",
        },
        (51, 2408) => LocalisedText {
            en: "The Lochs",
            ja: "ギラバニア湖畔地帯",
            de: "Das Fenn",
            fr: "Les Lacs",
        },
        (51, 2409) => LocalisedText {
            en: "The Ruby Sea",
            ja: "紅玉海",
            de: "Rubinsee",
            fr: "Mer de Rubis",
        },
        (51, 2410) => LocalisedText {
            en: "Yanxia",
            ja: "ヤンサ",
            de: "Yanxia",
            fr: "Yanxia",
        },
        (51, 2411) => LocalisedText {
            en: "The Azim Steppe",
            ja: "アジムステップ",
            de: "Azim-Steppe",
            fr: "Steppe d'Azim",
        },
        (51, 2412) => LocalisedText {
            en: "Shirogane",
            ja: "シロガネ",
            de: "Shirogane",
            fr: "Shirogane",
        },
        (51, 2413) => LocalisedText {
            en: "Bokairo Inn",
            ja: "望海楼",
            de: "Badehaus Bokairo",
            fr: "Auberge du Bokairo",
        },
        (51, 2414) => LocalisedText {
            en: "Eureka Anemos",
            ja: "エウレカ：アネモス帯",
            de: "Eureka Anemos",
            fr: "Eurêka Anemos",
        },
        (51, 2448) => LocalisedText {
            en: "The Great Hunt",
            ja: "大草原の狩場",
            de: "Revier des Rathalos",
            fr: "Domaine du roi des cieux",
        },
        (51, 2449) => LocalisedText {
            en: "Ultimacy",
            ja: "禁絶幻想",
            de: "Endgültigkeit",
            fr: "Fantasmagorie d'Ultima",
        },
        (51, 2450) => LocalisedText {
            en: "Castrum Fluminis",
            ja: "カストルム・フルーミニス",
            de: "Castrum Fluminis",
            fr: "Castrum Fluminis",
        },
        (51, 2462) => LocalisedText {
            en: "Eureka Pagos",
            ja: "エウレカ：パゴス帯",
            de: "Eureka Pagos",
            fr: "Eurêka Pagos",
        },
        (51, 2483) => LocalisedText {
            en: "The Ridorana Lighthouse",
            ja: "封じられた聖塔 リドルアナ",
            de: "Richtfeuer von Ridorana",
            fr: "Le Phare de Ridorana",
        },
        (51, 2485) => LocalisedText {
            en: "The Shifting Altars of Uznair",
            ja: "ウズネアカナル祭殿",
            de: "Halle der Glücksaltäre",
            fr: "Temple sacré d'Uznair",
        },
        (51, 2496) => LocalisedText {
            en: "Hells' Kier",
            ja: "獄之底",
            de: "Höllengrund",
            fr: "Fond des enfers",
        },
        (51, 2499) => LocalisedText {
            en: "Kugane Ohashi",
            ja: "クガネ大橋",
            de: "Kugane-Brücke",
            fr: "Pont Ohashi",
        },
        (51, 2510) => LocalisedText {
            en: "The Wreath of Snakes",
            ja: "蛇神大社",
            de: "Heiligtum der Schlange",
            fr: "Sanctuaire du Grand Serpent",
        },
        (51, 2530) => LocalisedText {
            en: "Eureka Pyros",
            ja: "エウレカ：ピューロス帯",
            de: "Eureka Pyros",
            fr: "Eurêka Pyros",
        },
        (51, 2545) => LocalisedText {
            en: "Eureka Hydatos",
            ja: "エウレカ：ヒュダトス帯",
            de: "Eureka Hydatos",
            fr: "Eurêka Hydatos",
        },
        (51, 2549) => LocalisedText {
            en: "The Manderville Tables",
            ja: "魔導雀荘「マンダヴィル」",
            de: "Manderville-Tische",
            fr: "Tables des Manderville",
        },
        (51, 2586) => LocalisedText {
            en: "The Ghimlyt Dark",
            ja: "ギムリトダーク",
            de: "Ghimlyt-Finsternis",
            fr: "Les Ténèbres de Ghimlyt",
        },
        (51, 2589) => LocalisedText {
            en: "Hidden Gorge",
            ja: "ヒドゥンゴージ",
            de: "Verborgene Schlucht",
            fr: "Gorge dérobée",
        },
        (51, 2613) => LocalisedText {
            en: "Castrum Oriens",
            ja: "カストルム・オリエンス",
            de: "Castrum Oriens",
            fr: "Castrum Oriens",
        },
        (51, 2634) => LocalisedText {
            en: "The Peering Stones",
            ja: "ピーリングストーンズ",
            de: "Die Spähenden Steine",
            fr: "Rochers d'aguet",
        },
        (51, 2646) => LocalisedText {
            en: "Ala Gannha",
            ja: "アラガーナ",
            de: "Ala Gannha",
            fr: "Ala Gannha",
        },
        (51, 2660) => LocalisedText {
            en: "Ala Ghiri",
            ja: "アラギリ",
            de: "Ala Ghiri",
            fr: "Ala Ghiri",
        },
        (51, 2665) => LocalisedText {
            en: "Castrum Abania",
            ja: "カストルム・アバニア",
            de: "Castrum Abania",
            fr: "Castrum Abania",
        },
        (51, 2670) => LocalisedText {
            en: "Porta Praetoria",
            ja: "ポルタ・プレトリア",
            de: "Porta Praetoria",
            fr: "Porta Praetoria",
        },
        (51, 2691) => LocalisedText {
            en: "Ala Mhigo",
            ja: "アラミゴ",
            de: "Ala Mhigo",
            fr: "Ala Mhigo",
        },
        (51, 2693) => LocalisedText {
            en: "The Ala Mhigan Quarter",
            ja: "アラミガン・クォーター",
            de: "Mhigisches Wohnviertel",
            fr: "Faubourg mhigois",
        },
        (51, 2707) => LocalisedText {
            en: "The Temple of the Fist",
            ja: "星導山寺院",
            de: "Tempel der Faust",
            fr: "Temple du Poing",
        },
        (51, 2708) => LocalisedText {
            en: "The Royal Airship Landing",
            ja: "アラミゴ王立飛空艇発着場",
            de: "Königlicher Luftschiff-Landeplatz",
            fr: "Aérodrome royal",
        },
        (51, 2715) => LocalisedText {
            en: "Transparency",
            ja: "結晶化空間",
            de: "Raum der Transparenz",
            fr: "Transparence",
        },
        (51, 2717) => LocalisedText {
            en: "Sigmascape V1.0",
            ja: "被検世界「シグマ」V1.0",
            de: "Sigmametrie 1.0",
            fr: "Sigmastice v1.0",
        },
        (51, 2718) => LocalisedText {
            en: "Sigmascape V2.0",
            ja: "被検世界「シグマ」V2.0",
            de: "Sigmametrie 2.0",
            fr: "Sigmastice v2.0",
        },
        (51, 2719) => LocalisedText {
            en: "Sigmascape V3.0",
            ja: "被検世界「シグマ」V3.0",
            de: "Sigmametrie 3.0",
            fr: "Sigmastice v3.0",
        },
        (51, 2720) => LocalisedText {
            en: "Sigmascape V4.0",
            ja: "被検世界「シグマ」V4.0",
            de: "Sigmametrie 4.0",
            fr: "Sigmastice v4.0",
        },
        (51, 2725) => LocalisedText {
            en: "Psiscape V1.0",
            ja: "被検世界「プサイ」V1.0",
            de: "Psimetrie 1.0",
            fr: "Psistice v1.0",
        },
        (51, 2727) => LocalisedText {
            en: "The Interdimensional Rift",
            ja: "次元の狭間",
            de: "Interdimensionaler Riss",
            fr: "Fissure interdimensionnelle",
        },
        (51, 2736) => LocalisedText {
            en: "Psiscape V2.0",
            ja: "被検世界「プサイ」V2.0",
            de: "Psimetrie 2.0",
            fr: "Psistice v2.0",
        },
        (51, 2762) => LocalisedText {
            en: "Hells' Lid",
            ja: "獄之蓋",
            de: "Höllenspund",
            fr: "Couvercle des enfers",
        },
        (51, 2769) => LocalisedText {
            en: "Tamamizu",
            ja: "碧のタマミズ",
            de: "Tamamizu",
            fr: "Tamamizu",
        },
        (51, 2773) => LocalisedText {
            en: "Onokoro",
            ja: "オノコロ島",
            de: "Onokoro",
            fr: "Onokoro",
        },
        (51, 2775) => LocalisedText {
            en: "Heaven-on-High",
            ja: "アメノミハシラ",
            de: "Himmelssäule",
            fr: "Pilier des Cieux",
        },
        (51, 2779) => LocalisedText {
            en: "Shisui of the Violet Tides",
            ja: "紫水宮",
            de: "Shisui-Palast",
            fr: "Palais aux Marées violettes",
        },
        (51, 2793) => LocalisedText {
            en: "Namai",
            ja: "ナマイ村",
            de: "Namai",
            fr: "Namai",
        },
        (51, 2801) => LocalisedText {
            en: "The Swallow's Compass",
            ja: "ガンエン廟",
            de: "Kompass der Schwalbe",
            fr: "Compas de l'Hirondelle",
        },
        (51, 2805) => LocalisedText {
            en: "The House of the Fierce",
            ja: "烈士庵",
            de: "Klause der Grimmigen",
            fr: "Cercle des Intrépides",
        },
        (51, 2810) => LocalisedText {
            en: "Doma Castle",
            ja: "ドマ城",
            de: "Burg Doma",
            fr: "Château de Doma",
        },
        (51, 2813) => LocalisedText {
            en: "The Doman Enclave",
            ja: "ドマ町人地",
            de: "Domanische Enklave",
            fr: "Quartier enclavé de Doma",
        },
        (51, 2814) => LocalisedText {
            en: "Reunion",
            ja: "再会の市",
            de: "Reunion",
            fr: "Ralliement",
        },
        (51, 2822) => LocalisedText {
            en: "The Dawn Throne",
            ja: "明けの玉座",
            de: "Morgenthron",
            fr: "Trône de l'Aube",
        },
        (51, 2833) => LocalisedText {
            en: "Bardam's Mettle",
            ja: "バルダム覇道",
            de: "Bardams Probe",
            fr: "Force de Bardam",
        },
        (51, 2850) => LocalisedText {
            en: "Dhoro Iloh",
            ja: "ドーロ・イロー",
            de: "Dhoro Iloh",
            fr: "Dhoro Iloh",
        },
        (51, 2851) => LocalisedText {
            en: "The Burn",
            ja: "ザ・バーン",
            de: "Kargland",
            fr: "L'Escarre",
        },
        (51, 2864) => LocalisedText {
            en: "The Orbonne Monastery",
            ja: "楽欲の僧院 オーボンヌ",
            de: "Kloster von Orbonne",
            fr: "Monastère d'Orbonne",
        },
        (51, 2936) => LocalisedText {
            en: "Paglth'an",
            ja: "パガルザン",
            de: "Paglth'an",
            fr: "Paglth'an",
        },
        (51, 2950) => LocalisedText {
            en: "Norvrandt",
            ja: "ノルヴラント",
            de: "Norvrandt",
            fr: "Norvrandt",
        },
        (51, 2951) => LocalisedText {
            en: "The Crystarium",
            ja: "クリスタリウム",
            de: "Crystarium",
            fr: "Cristarium",
        },
        (51, 2952) => LocalisedText {
            en: "Eulmore",
            ja: "ユールモア",
            de: "Eulmore",
            fr: "Eulmore",
        },
        (51, 2953) => LocalisedText {
            en: "Lakeland",
            ja: "レイクランド",
            de: "Seenland",
            fr: "Grand-Lac",
        },
        (51, 2954) => LocalisedText {
            en: "Kholusia",
            ja: "コルシア島",
            de: "Kholusia",
            fr: "Kholusia",
        },
        (51, 2955) => LocalisedText {
            en: "Amh Araeng",
            ja: "アム・アレーン",
            de: "Amh Araeng",
            fr: "Amh Araeng",
        },
        (51, 2956) => LocalisedText {
            en: "Il Mheg",
            ja: "イル・メグ",
            de: "Il Mheg",
            fr: "Il Mheg",
        },
        (51, 2957) => LocalisedText {
            en: "The Rak'tika Greatwood",
            ja: "ラケティカ大森林",
            de: "Der Große Wald Rak'tika",
            fr: "Rak'tika",
        },
        (51, 2958) => LocalisedText {
            en: "The Tempest",
            ja: "テンペスト",
            de: "Tempest",
            fr: "La Tempête",
        },
        (51, 2979) => LocalisedText {
            en: "Dohn Mheg",
            ja: "ドォーヌ・メグ",
            de: "Dohn Mheg",
            fr: "Dohn Mheg",
        },
        (51, 2982) => LocalisedText {
            en: "The Twinning",
            ja: "シルクス・ツイニング",
            de: "Kristallzwilling",
            fr: "Macle de Syrcus",
        },
        (51, 2985) => LocalisedText {
            en: "Amaurot",
            ja: "アーモロート",
            de: "Amaurot",
            fr: "Amaurote",
        },
        (51, 2997) => LocalisedText {
            en: "Mt. Gulg",
            ja: "グルグ火山",
            de: "Gulg",
            fr: "Mont Gulg",
        },
        (51, 3007) => LocalisedText {
            en: "Akadaemia Anyder",
            ja: "アナイダアカデミア",
            de: "Akadaemia Anyder",
            fr: "Akadaemia Anydre",
        },
        (51, 3017) => LocalisedText {
            en: "Liminal Space",
            ja: "階差閉宇宙",
            de: "Geschlossene Raumzeit",
            fr: "Univers différentiel",
        },
        (51, 3018) => LocalisedText {
            en: "The Qitana Ravel",
            ja: "キタンナ神影洞",
            de: "Irrungen der Qitari",
            fr: "Enchevêtrement des Qitari",
        },
        (51, 3044) => LocalisedText {
            en: "Fort Jobb",
            ja: "ジョッブ砦",
            de: "Jobb-Feste",
            fr: "Fort Jobb",
        },
        (51, 3050) => LocalisedText {
            en: "Holminster Switch",
            ja: "ホルミンスター",
            de: "Holminster",
            fr: "Holminster",
        },
        (51, 3057) => LocalisedText {
            en: "The Ostall Imperative",
            ja: "オスタル厳命城",
            de: "Ostalls Befehl",
            fr: "L'Impératif d'Ostall",
        },
        (51, 3075) => LocalisedText {
            en: "Stilltide",
            ja: "スティルタイド",
            de: "Stillwasser",
            fr: "Douceonde",
        },
        (51, 3094) => LocalisedText {
            en: "Wright",
            ja: "ライト村",
            de: "Werfting",
            fr: "Cherpant",
        },
        (51, 3105) => LocalisedText {
            en: "Tomra",
            ja: "トメラの村",
            de: "Tomra",
            fr: "Tomra",
        },
        (51, 3122) => LocalisedText {
            en: "Mord Souq",
            ja: "モルド・スーク",
            de: "Mordh Souq",
            fr: "Mord Souq",
        },
        (51, 3129) => LocalisedText {
            en: "The Inn at Journey's Head",
            ja: "旅立ちの宿",
            de: "Ruhstatt vor der Langen Reise",
            fr: "Auberge du Grand Départ",
        },
        (51, 3135) => LocalisedText {
            en: "Twine",
            ja: "トゥワイン",
            de: "Pfotenschlag",
            fr: "Queues-liées",
        },
        (51, 3139) => LocalisedText {
            en: "Malikah's Well",
            ja: "マリカの大井戸",
            de: "Malikahs Brunnen",
            fr: "Puits de Malikah",
        },
        (51, 3147) => LocalisedText {
            en: "Lydha Lran",
            ja: "リダ・ラーン",
            de: "Lydha Lran",
            fr: "Lydha Lran",
        },
        (51, 3156) => LocalisedText {
            en: "Pla Enni",
            ja: "プラ・エンニ茸窟",
            de: "Pla Enni",
            fr: "Pla Enni",
        },
        (51, 3157) => LocalisedText {
            en: "Wolekdorf",
            ja: "ヴォレクドルフ",
            de: "Wolekdorf",
            fr: "Wolekdorf",
        },
        (51, 3170) => LocalisedText {
            en: "Slitherbough",
            ja: "スリザーバウ",
            de: "Schlangenzweig",
            fr: "Serpentige",
        },
        (51, 3179) => LocalisedText {
            en: "Fanow",
            ja: "ファノヴの里",
            de: "Fanow",
            fr: "Fanow",
        },
        (51, 3195) => LocalisedText {
            en: "The Ondo Cups",
            ja: "オンドの潮溜まり",
            de: "Ondo-Becken",
            fr: "Baïne",
        },
        (51, 3205) => LocalisedText {
            en: "The Macarenses Angle",
            ja: "マカレンサス広場",
            de: "Macarenses-Forum",
            fr: "Carré macarien",
        },
        (51, 3214) => LocalisedText {
            en: "The Core",
            ja: "エデンコア",
            de: "Das Herz",
            fr: "Cœur d'Éden",
        },
        (51, 3215) => LocalisedText {
            en: "The Halo",
            ja: "エデン甲板",
            de: "Das Deck",
            fr: "Halo pinaculaire",
        },
        (51, 3216) => LocalisedText {
            en: "The Nereus Trench",
            ja: "ネレウス海淵",
            de: "Nereus-Graben",
            fr: "Fosse de Nérée",
        },
        (51, 3217) => LocalisedText {
            en: "Atlas Peak",
            ja: "アトラス山頂",
            de: "Atlas-Gipfel",
            fr: "Mont Atlas",
        },
        (51, 3218) => LocalisedText {
            en: "The Dancing Plague",
            ja: "妖精王の舞踏場",
            de: "Königlicher Tanzgarten",
            fr: "Salle de bal de Titania",
        },
        (51, 3219) => LocalisedText {
            en: "The Crown of the Immaculate",
            ja: "イマキュレートクラウン",
            de: "Strahlenkranz der Makellosigkeit",
            fr: "Couronne de l'Immaculé",
        },
        (51, 3220) => LocalisedText {
            en: "The Dying Gasp",
            ja: "終末のアーモロート",
            de: "Die Letzten Tage von Amaurot",
            fr: "Râle de l'Agonie",
        },
        (51, 3222) => LocalisedText {
            en: "The Pendants Personal Suite",
            ja: "ペンダント居住館：居室",
            de: "Isomorph-Quartiere",
            fr: "Les Pendants",
        },
        (51, 3229) => LocalisedText {
            en: "The Dungeons of Lyhe Ghiah",
            ja: "リェー・ギア・ダンジョン",
            de: "Verliese von Lyhe Ghiah",
            fr: "Donjon hypogéen du Lyhe Ghiah",
        },
        (51, 3378) => LocalisedText {
            en: "Onsal Hakair",
            ja: "オンサル・ハカイル",
            de: "Onsal Hakair",
            fr: "Onsal Hakair",
        },
        (51, 3385) => LocalisedText {
            en: "The Grand Cosmos",
            ja: "グラン・コスモス",
            de: "Chateau Cosmea",
            fr: "Cosmos coruscant",
        },
        (51, 3425) => LocalisedText {
            en: "The Copied Factory",
            ja: "複製サレタ工場廃墟",
            de: "Die kopierte Fabrik",
            fr: "Réplique de l'usine désaffectée",
        },
        (51, 3435) => LocalisedText {
            en: "The Firmament",
            ja: "蒼天街",
            de: "Himmelsstadt",
            fr: "Azurée",
        },
        (51, 3442) => LocalisedText {
            en: "Cinder Drift",
            ja: "ギムリト防衛ライン",
            de: "Verteidigungslinie Ghimlyt",
            fr: "Les Nuées de Brandons",
        },
        (51, 3467) => LocalisedText {
            en: "Anamnesis Anyder",
            ja: "アニドラス・アナムネーシス",
            de: "Anamnesis Anyder",
            fr: "Anamnesis Anydre",
        },
        (51, 3468) => LocalisedText {
            en: "The Gandof Thunder Plains",
            ja: "ガンドフ雷平原",
            de: "Gandoph-Donnersteppe",
            fr: "Plaine foudroyée de Gandof",
        },
        (51, 3469) => LocalisedText {
            en: "Ashfall",
            ja: "灰の旅程",
            de: "Aschenfall",
            fr: "Bassin cendreux",
        },
        (51, 3477) => LocalisedText {
            en: "The Endeavor",
            ja: "エンデバー号",
            de: "Die Endeavor",
            fr: "L'Ambition",
        },
        (51, 3486) => LocalisedText {
            en: "Transmission Control",
            ja: "交信雷波塔管制室",
            de: "Signalkontrollstation",
            fr: "Salle de contrôle",
        },
        (51, 3487) => LocalisedText {
            en: "Great Glacier",
            ja: "大氷河",
            de: "Gletscher",
            fr: "Grand Glacier",
        },
        (51, 3492) => LocalisedText {
            en: "The Puppets' Bunker",
            ja: "人形タチノ軍事基地",
            de: "Die Puppenfestung",
            fr: "Base militaire des Pantins",
        },
        (51, 3511) => LocalisedText {
            en: "The Heroes' Gauntlet",
            ja: "漆黒決戦 ノルヴラント",
            de: "Schlacht um Norvrandt",
            fr: "Traversée de Norvrandt",
        },
        (51, 3534) => LocalisedText {
            en: "Bozjan Southern Front",
            ja: "南方ボズヤ戦線",
            de: "Bozja-Südfront",
            fr: "Front sud de Bozja",
        },
        (51, 3568) => LocalisedText {
            en: "The Seat of Sacrifice",
            ja: "始皇帝の玉座",
            de: "Der Kaiserliche Thron",
            fr: "Le Trône du Sacrifice",
        },
        (51, 3581) => LocalisedText {
            en: "Castrum Marinum Drydocks",
            ja: "カステッルム・マリヌム・ドライドック",
            de: "Trockendock von Castrum Marinum",
            fr: "Cale sèche de Castrum Marinum",
        },
        (51, 3590) => LocalisedText {
            en: "Matoya's Relict",
            ja: "マトーヤのアトリエ",
            de: "Matoyas Atelier",
            fr: "Atelier abandonné de Matoya",
        },
        (51, 3595) => LocalisedText {
            en: "Bygone Gaol",
            ja: "ユールモア監獄棟",
            de: "Verlies von Eulmore",
            fr: "Prison d'Eulmore",
        },
        (51, 3596) => LocalisedText {
            en: "Sphere of Naught",
            ja: "暗闇の領域",
            de: "Nichtssphäre",
            fr: "Sphère de l'absence",
        },
        (51, 3597) => LocalisedText {
            en: "Delubrum Reginae",
            ja: "グンヒルド・ディルーブラム",
            de: "Delubrum Reginae",
            fr: "Delubrum Reginae",
        },
        (51, 3620) => LocalisedText {
            en: "Laxan Loft",
            ja: "ラクサン城",
            de: "Burg Selard",
            fr: "Château de Ganlag",
        },
        (51, 3635) => LocalisedText {
            en: "The Garden of Nowhere",
            ja: "絶望の園",
            de: "Nimmerwo-Garten",
            fr: "Jardin du désespoir",
        },
        (51, 3644) => LocalisedText {
            en: "The Shifting Oubliettes of Lyhe Ghiah",
            ja: "リェー・ギア・ダンジョン祭殿",
            de: "Das Karussell von Lyhe Ghiah",
            fr: "Jardin secret du Lyhe Ghiah",
        },
        (51, 3647) => LocalisedText {
            en: "The Tower at Paradigm's Breach",
            ja: "希望ノ砲台：「塔」",
            de: "Der Turm, Paradigmenbrecher",
            fr: "Tour de la Contingence",
        },
        (51, 3662) => LocalisedText {
            en: "Zadnor",
            ja: "ザトゥノル高原",
            de: "Zadnor-Hochebene",
            fr: "Hauts plateaux de Zadnor",
        },
        (51, 3663) => LocalisedText {
            en: "G-Savior Deck",
            ja: "Gセイヴァー戦闘甲板",
            de: "Gefechtsplattform des G-Retters",
            fr: "Ponton du Gardien-G",
        },
        (51, 3684) => LocalisedText {
            en: "The Dark Inside",
            ja: "月の底",
            de: "Grund des Mondes",
            fr: "Prison lunaire",
        },
        (51, 3685) => LocalisedText {
            en: "The Mothercrystal",
            ja: "マザークリスタル",
            de: "Urkristall",
            fr: "Cristal-mère",
        },
        (51, 3686) => LocalisedText {
            en: "The Final Day",
            ja: "果ての終焉",
            de: "Am Rande des Endes",
            fr: "Orée de la fin",
        },
        (51, 3700) => LocalisedText {
            en: "The Source",
            ja: "原初世界",
            de: "Ursprungswelt",
            fr: "Monde primitif",
        },
        (51, 3701) => LocalisedText {
            en: "The First",
            ja: "第一世界",
            de: "Erste Splitterwelt",
            fr: "Premier reflet",
        },
        (51, 3702) => LocalisedText {
            en: "The Northern Empty",
            ja: "北洋地域",
            de: "Nördliche Meere",
            fr: "Mers du Nord",
        },
        (51, 3703) => LocalisedText {
            en: "Ilsabard",
            ja: "イルサバード",
            de: "Ilsabard",
            fr: "Ilsabard",
        },
        (51, 3704) => LocalisedText {
            en: "The Sea of Stars",
            ja: "星外宙域",
            de: "Kosmos",
            fr: "Espace intersidéral",
        },
        (51, 3705) => LocalisedText {
            en: "The World Unsundered",
            ja: "古代世界",
            de: "Unzersplitterte Welt",
            fr: "Ancien monde",
        },
        (51, 3706) => LocalisedText {
            en: "Old Sharlayan",
            ja: "オールド・シャーレアン",
            de: "Alt-Sharlayan",
            fr: "Vieille Sharlayan",
        },
        (51, 3707) => LocalisedText {
            en: "Radz-at-Han",
            ja: "ラザハン",
            de: "Radz-at-Han",
            fr: "Radz-at-Han",
        },
        (51, 3708) => LocalisedText {
            en: "Labyrinthos",
            ja: "ラヴィリンソス",
            de: "Labyrinthos",
            fr: "Le Labyrinthos",
        },
        (51, 3709) => LocalisedText {
            en: "Thavnair",
            ja: "サベネア島",
            de: "Thavnair",
            fr: "Thavnair",
        },
        (51, 3710) => LocalisedText {
            en: "Garlemald",
            ja: "ガレマルド",
            de: "Garlemald",
            fr: "Garlemald",
        },
        (51, 3711) => LocalisedText {
            en: "Mare Lamentorum",
            ja: "嘆きの海",
            de: "Mare Lamentorum",
            fr: "Mare Lamentorum",
        },
        (51, 3712) => LocalisedText {
            en: "Ultima Thule",
            ja: "ウルティマ・トゥーレ",
            de: "Ultima Thule",
            fr: "Ultima Thulé",
        },
        (51, 3713) => LocalisedText {
            en: "Elpis",
            ja: "エルピス",
            de: "Elpis",
            fr: "Elpis",
        },
        (51, 3736) => LocalisedText {
            en: "The Tower of Zot",
            ja: "ゾットの塔",
            de: "Turm von Zot",
            fr: "Tour de Zott",
        },
        (51, 3759) => LocalisedText {
            en: "Ktisis Hyperboreia",
            ja: "ヒュペルボレア造物院",
            de: "Ktisis Hyperboreia",
            fr: "Hyperborée",
        },
        (51, 3769) => LocalisedText {
            en: "The Gates of Pandæmonium",
            ja: "パンデモニウム正門",
            de: "Pforten des Pandæmoniums",
            fr: "Limbes du Pandæmonium - Parvis",
        },
        (51, 3770) => LocalisedText {
            en: "Smileton",
            ja: "スマイルトン",
            de: "Smileton",
            fr: "Risette-sur-lune",
        },
        (51, 3783) => LocalisedText {
            en: "The Stigma Dreamscape",
            ja: "スティグマ・フォー",
            de: "Stigma-4",
            fr: "Rêve électrique de Stigma-4",
        },
        (51, 3797) => LocalisedText {
            en: "The Fervid Limbo",
            ja: "パンデモニウム辺獄下層",
            de: "Glühender Limbus",
            fr: "Limbes du Pandæmonium - Fournaise",
        },
        (51, 3798) => LocalisedText {
            en: "The Stagnant Limbo",
            ja: "パンデモニウム辺獄水道",
            de: "Trister Limbus",
            fr: "Limbes du Pandæmonium - Cloaque",
        },
        (51, 3818) => LocalisedText {
            en: "Andron",
            ja: "ナップルーム",
            de: "Ruheräume",
            fr: "Chambre à coucher",
        },
        (51, 3833) => LocalisedText {
            en: "The Archeion",
            ja: "アルケイオン保管院",
            de: "Archeion",
            fr: "Archéion",
        },
        (51, 3850) => LocalisedText {
            en: "Sharlayan Hamlet",
            ja: "リトルシャーレアン",
            de: "Klein-Sharlayan",
            fr: "Petite Sharlayan",
        },
        (51, 3852) => LocalisedText {
            en: "Aporia",
            ja: "アポリア本部",
            de: "Aporia",
            fr: "L'Aporie",
        },
        (51, 3880) => LocalisedText {
            en: "Yedlihmad",
            ja: "イェドリマン",
            de: "Yedlihmad",
            fr: "Yedlihmad",
        },
        (51, 3886) => LocalisedText {
            en: "The Great Work",
            ja: "デミールの遺烈郷",
            de: "Daemirs Werk",
            fr: "Le Grand Œuvre",
        },
        (51, 3894) => LocalisedText {
            en: "Palaka's Stand",
            ja: "パーラカの里",
            de: "Palakas Ruh",
            fr: "Écoute des Palaka",
        },
        (51, 3903) => LocalisedText {
            en: "Camp Broken Glass",
            ja: "キャンプ・ブロークングラス",
            de: "Camp Splitterglas",
            fr: "Camp Bris-de-glace",
        },
        (51, 3915) => LocalisedText {
            en: "Tertium",
            ja: "テルティウム駅",
            de: "Tertium",
            fr: "Gare de Tertium",
        },
        (51, 3926) => LocalisedText {
            en: "Propylaion",
            ja: "プロピュライオン",
            de: "Propylaion",
            fr: "Le Propylée",
        },
        (51, 3928) => LocalisedText {
            en: "Anagnorisis",
            ja: "アナグノリシス天測園",
            de: "Anagnorisis",
            fr: "L'Anagnorisis",
        },
        (51, 3929) => LocalisedText {
            en: "The Twelve Wonders",
            ja: "十二節の園",
            de: "Die Zwölf Gärten",
            fr: "Les Douze Merveilles",
        },
        (51, 3938) => LocalisedText {
            en: "Poieten Oikos",
            ja: "ポイエテーン・オイコス",
            de: "Poieten Oikos",
            fr: "Poièten Oïkos",
        },
        (51, 3949) => LocalisedText {
            en: "Sinus Lacrimarum",
            ja: "涙の入江",
            de: "Sinus Lacrimarum",
            fr: "Sinus Lacrimarum",
        },
        (51, 3966) => LocalisedText {
            en: "Bestways Burrow",
            ja: "ベストウェイ・バロー",
            de: "Bestways Rast",
            fr: "Halot de Bestway",
        },
        (51, 3973) => LocalisedText {
            en: "Reah Tahra",
            ja: "リア・ターラ",
            de: "Reah Tahra",
            fr: "Reah Tahra",
        },
        (51, 3978) => LocalisedText {
            en: "Abode of the Ea",
            ja: "イーアの里",
            de: "Domizil der Ea",
            fr: "Demeure des Éas",
        },
        (51, 3983) => LocalisedText {
            en: "Base Omicron",
            ja: "オミクロンベース",
            de: "Omikron-Basis",
            fr: "Base des Omicrons",
        },
        (51, 4015) => LocalisedText {
            en: "Vanaspati",
            ja: "ヴァナスパティ樹海",
            de: "Vanaspati",
            fr: "Vanaspati",
        },
        (51, 4090) => LocalisedText {
            en: "The Palaistra",
            ja: "パライストラ",
            de: "Die Palästra",
            fr: "Le Palestre",
        },
        (51, 4091) => LocalisedText {
            en: "The Volcanic Heart",
            ja: "ヴォルカニック・ハート",
            de: "Das Herz des Vulkans",
            fr: "Le Cœur volcanique",
        },
        (51, 4092) => LocalisedText {
            en: "Cloud Nine",
            ja: "クラウドナイン",
            de: "Wolke Sieben",
            fr: "Le Petit Nuage",
        },
        (51, 4098) => LocalisedText {
            en: "Medias Res",
            ja: "詩想空間",
            de: "Medias Res",
            fr: "Interstice du chant onirique",
        },
        (51, 4100) => LocalisedText {
            en: "The Dead Ends",
            ja: "レムナント",
            de: "Das Sternengrab",
            fr: "L'Issue aux Impasses",
        },
        (51, 4111) => LocalisedText {
            en: "The Excitatron 6000",
            ja: "エキサイトロン",
            de: "Euphoratron",
            fr: "Ludodrome",
        },
        (51, 4118) => LocalisedText {
            en: "The Tower of Babil",
            ja: "バブイルの塔",
            de: "Turm von Babil",
            fr: "Tour de Babil",
        },
        (51, 4135) => LocalisedText {
            en: "The Sanguine Limbo",
            ja: "パンデモニウム辺獄深層",
            de: "Blutender Limbus",
            fr: "Limbes du Pandæmonium - Abîme",
        },
        (51, 4139) => LocalisedText {
            en: "Empyreum",
            ja: "エンピレアム",
            de: "Empyreum",
            fr: "Empyrée",
        },
        (51, 4146) => LocalisedText {
            en: "The Aitiascope",
            ja: "アイティオン星晶鏡",
            de: "Aitiaskop",
            fr: "Prisme de l'Aitia",
        },
        (51, 4154) => LocalisedText {
            en: "Alzadaal's Legacy",
            ja: "アルザダール海底遺跡群",
            de: "Alzadaals Vermächtnis",
            fr: "Legs d'Alzadaal",
        },
        (51, 4167) => LocalisedText {
            en: "Aglaia",
            ja: "輝ける神域 アグライア",
            de: "Aglaia",
            fr: "Domaine divin - Aglaé",
        },
        (54, 1) => LocalisedText {
            en: "Halone, the Fury",
            ja: "ハルオーネ",
            de: "Halone - Die Furie",
            fr: "Halone, la Conquérante",
        },
        (54, 2) => LocalisedText {
            en: "Menphina, the Lover",
            ja: "メネフィナ",
            de: "Menphina - Die Liebende",
            fr: "Menphina, la Bien-aimante",
        },
        (54, 3) => LocalisedText {
            en: "Thaliak, the Scholar",
            ja: "サリャク",
            de: "Thaliak - Der Forscher",
            fr: "Thaliak, l'Érudit",
        },
        (54, 4) => LocalisedText {
            en: "Nymeia, the Spinner",
            ja: "ニメーヤ",
            de: "Nymeia - Die Norne",
            fr: "Nymeia, la Fileuse",
        },
        (54, 5) => LocalisedText {
            en: "Llymlaen, the Navigator",
            ja: "リムレーン",
            de: "Llymlaen - Die Lotsin",
            fr: "Llymlaen, la Navigatrice",
        },
        (54, 6) => LocalisedText {
            en: "Oschon, the Wanderer",
            ja: "オシュオン",
            de: "Oschon - Der Wanderer",
            fr: "Oschon, le Vagabond",
        },
        (54, 7) => LocalisedText {
            en: "Byregot, the Builder",
            ja: "ビエルゴ",
            de: "Byregot - Der Erbauer",
            fr: "Byregot, l'Artisan",
        },
        (54, 8) => LocalisedText {
            en: "Rhalgr, the Destroyer",
            ja: "ラールガー",
            de: "Rhalgr - Der Zerstörer",
            fr: "Rhalgr, le Destructeur",
        },
        (54, 9) => LocalisedText {
            en: "Azeyma, the Warden",
            ja: "アーゼマ",
            de: "Azeyma - Die Aufseherin",
            fr: "Azeyma, la Gardienne",
        },
        (54, 10) => LocalisedText {
            en: "Nald'thal, the Traders",
            ja: "ナルザル",
            de: "Nald'thal - Die Kaufleute",
            fr: "Nald'thal, les Marchands",
        },
        (54, 11) => LocalisedText {
            en: "Nophica, the Matron",
            ja: "ノフィカ",
            de: "Nophica - Die Mutter",
            fr: "Nophica, la Mère",
        },
        (54, 12) => LocalisedText {
            en: "Althyk, the Keeper",
            ja: "アルジク",
            de: "Althyk - Der Hüter",
            fr: "Althyk, le Contemplateur",
        },
        (55, 1) => LocalisedText {
            en: "Auto-attack",
            ja: "オートアタック",
            de: "Auto-Attacke",
            fr: "Attaque automatique",
        },
        (55, 2) => LocalisedText {
            en: "Jump",
            ja: "ジャンプ",
            de: "Springen",
            fr: "Saut",
        },
        (55, 3) => LocalisedText {
            en: "Limit Break",
            ja: "リミットブレイク",
            de: "Limitrausch",
            fr: "Transcendance",
        },
        (55, 4) => LocalisedText {
            en: "Sprint",
            ja: "スプリント",
            de: "Sprint",
            fr: "Sprint",
        },
        (55, 5) => LocalisedText {
            en: "Desynthesis",
            ja: "分解",
            de: "Materialverwertung",
            fr: "Recyclage",
        },
        (55, 6) => LocalisedText {
            en: "Repair",
            ja: "修理",
            de: "Reparatur",
            fr: "Réparation",
        },
        (55, 7) => LocalisedText {
            en: "Teleport",
            ja: "テレポ",
            de: "Teleport",
            fr: "Téléportation",
        },
        (55, 8) => LocalisedText {
            en: "Return",
            ja: "デジョン",
            de: "Rückführung",
            fr: "Rapatriement",
        },
        (55, 9) => LocalisedText {
            en: "Mount Roulette",
            ja: "マウント・ルーレット",
            de: "Reittier-Roulette",
            fr: "Monture aléatoire",
        },
        (55, 10) => LocalisedText {
            en: "Minion Roulette",
            ja: "ミニオン・ルーレット",
            de: "Begleiter-Roulette",
            fr: "Mascotte aléatoire",
        },
        (55, 12) => LocalisedText {
            en: "Materia Melding",
            ja: "マテリア装着",
            de: "Materia einsetzen",
            fr: "Sertissage de matéria",
        },
        (55, 13) => LocalisedText {
            en: "Advanced Materia Melding",
            ja: "マテリア装着II",
            de: "Materia einsetzen II",
            fr: "Sertissage avancé de matéria",
        },
        (55, 14) => LocalisedText {
            en: "Materia Extraction",
            ja: "マテリア精製",
            de: "Materia-Extraktion",
            fr: "Matérialisation",
        },
        (55, 15) => LocalisedText {
            en: "Dye",
            ja: "染色",
            de: "Färben",
            fr: "Teinture",
        },
        (55, 16) => LocalisedText {
            en: "Target Forward",
            ja: "ターゲット・順送り",
            de: "Nächstes Ziel",
            fr: "Ciblage",
        },
        (55, 17) => LocalisedText {
            en: "Target Back",
            ja: "ターゲット・逆送り",
            de: "Voriges Ziel",
            fr: "Ciblage contraire",
        },
        (55, 18) => LocalisedText {
            en: "Set Down",
            ja: "設置",
            de: "Abstellen",
            fr: "Déposer",
        },
        (55, 19) => LocalisedText {
            en: "Decipher",
            ja: "ディサイファー",
            de: "Entziffern",
            fr: "Déchiffrage",
        },
        (55, 20) => LocalisedText {
            en: "Dig",
            ja: "ディグ",
            de: "Ausgraben",
            fr: "Excavation",
        },
        (55, 21) => LocalisedText {
            en: "Aetherial Reduction",
            ja: "精選",
            de: "Raffinieren",
            fr: "Éthérolyse",
        },
        (55, 22) => LocalisedText {
            en: "Cast Glamour",
            ja: "武具投影",
            de: "Projizieren",
            fr: "Mirage",
        },
        (55, 24) => LocalisedText {
            en: "Flying Mount Roulette",
            ja: "フライングマウント・ルーレット",
            de: "Flugreittier-Roulette",
            fr: "Monture volante aléatoire",
        },
        (55, 25) => LocalisedText {
            en: "Glamour Plate",
            ja: "ミラージュプレート",
            de: "Projektionsplatte",
            fr: "Planche mirage",
        },
        (55, 26) => LocalisedText {
            en: "Duty Action I",
            ja: "コンテンツアクション1",
            de: "Spezialkommando 1",
            fr: "Action de mission 1",
        },
        (55, 27) => LocalisedText {
            en: "Duty Action II",
            ja: "コンテンツアクション2",
            de: "Spezialkommando 2",
            fr: "Action de mission 2",
        },
        (56, 9) => LocalisedText {
            en: "Fast Blade",
            ja: "ファストブレード",
            de: "Vortexschnitt",
            fr: "Lame rapide",
        },
        (56, 15) => LocalisedText {
            en: "Riot Blade",
            ja: "ライオットソード",
            de: "Donnerklinge",
            fr: "Lame déchaînée",
        },
        (56, 16) => LocalisedText {
            en: "Shield Bash",
            ja: "シールドバッシュ",
            de: "Schildschlag",
            fr: "Coup de bouclier",
        },
        (56, 17) => LocalisedText {
            en: "Sentinel",
            ja: "センチネル",
            de: "Sentinel",
            fr: "Sentinelle",
        },
        (56, 20) => LocalisedText {
            en: "Fight or Flight",
            ja: "ファイト・オア・フライト",
            de: "Verwegenheit",
            fr: "Combat acharné",
        },
        (56, 21) => LocalisedText {
            en: "Rage of Halone",
            ja: "レイジ・オブ・ハルオーネ",
            de: "Halones Groll",
            fr: "Rage de Halone",
        },
        (56, 23) => LocalisedText {
            en: "Circle of Scorn",
            ja: "サークル・オブ・ドゥーム",
            de: "Kreis der Verachtung",
            fr: "Cercle du destin",
        },
        (56, 24) => LocalisedText {
            en: "Shield Lob",
            ja: "シールドロブ",
            de: "Schildwurf",
            fr: "Lancer de bouclier",
        },
        (56, 27) => LocalisedText {
            en: "Cover",
            ja: "かばう",
            de: "Deckung",
            fr: "Couverture",
        },
        (56, 28) => LocalisedText {
            en: "Iron Will",
            ja: "アイアンウィル",
            de: "Schild-Eid",
            fr: "Volonté de fer",
        },
        (56, 29) => LocalisedText {
            en: "Spirits Within",
            ja: "スピリッツウィズイン",
            de: "Selbsterhaltungstrieb",
            fr: "Esprits intérieurs",
        },
        (56, 30) => LocalisedText {
            en: "Hallowed Ground",
            ja: "インビンシブル",
            de: "Heiliger Boden",
            fr: "Invincible",
        },
        (56, 31) => LocalisedText {
            en: "Heavy Swing",
            ja: "ヘヴィスウィング",
            de: "Gewaltiger Hieb",
            fr: "Coup puissant",
        },
        (56, 37) => LocalisedText {
            en: "Maim",
            ja: "メイム",
            de: "Verstümmelung",
            fr: "Mutilation",
        },
        (56, 38) => LocalisedText {
            en: "Berserk",
            ja: "バーサク",
            de: "Tollwut",
            fr: "Berserk",
        },
        (56, 40) => LocalisedText {
            en: "Thrill of Battle",
            ja: "スリル・オブ・バトル",
            de: "Kampfrausch",
            fr: "Frisson de la bataille",
        },
        (56, 41) => LocalisedText {
            en: "Overpower",
            ja: "オーバーパワー",
            de: "Kahlrodung",
            fr: "Domination",
        },
        (56, 42) => LocalisedText {
            en: "Storm's Path",
            ja: "シュトルムヴィント",
            de: "Sturmkeil",
            fr: "Couperet de justice",
        },
        (56, 43) => LocalisedText {
            en: "Holmgang",
            ja: "ホルムギャング",
            de: "Holmgang",
            fr: "Holmgang",
        },
        (56, 44) => LocalisedText {
            en: "Vengeance",
            ja: "ヴェンジェンス",
            de: "Rachsucht",
            fr: "Représailles",
        },
        (56, 45) => LocalisedText {
            en: "Storm's Eye",
            ja: "シュトルムブレハ",
            de: "Sturmbrecher",
            fr: "Œil de la tempête",
        },
        (56, 46) => LocalisedText {
            en: "Tomahawk",
            ja: "トマホーク",
            de: "Tomahawk",
            fr: "Tomahawk",
        },
        (56, 48) => LocalisedText {
            en: "Defiance",
            ja: "ディフェンダー",
            de: "Verteidiger",
            fr: "Défi",
        },
        (56, 49) => LocalisedText {
            en: "Inner Beast",
            ja: "原初の魂",
            de: "Tier in dir",
            fr: "Bête intérieure",
        },
        (56, 51) => LocalisedText {
            en: "Steel Cyclone",
            ja: "スチールサイクロン",
            de: "Kreiselklinge",
            fr: "Cyclone de fer",
        },
        (56, 52) => LocalisedText {
            en: "Infuriate",
            ja: "ウォークライ",
            de: "Schlachtruf",
            fr: "Cri de guerre",
        },
        (56, 53) => LocalisedText {
            en: "Bootshine",
            ja: "連撃",
            de: "Schlag auf Schlag",
            fr: "Volée de coups",
        },
        (56, 54) => LocalisedText {
            en: "True Strike",
            ja: "正拳突き",
            de: "Wahrer Schlag",
            fr: "Coup de poing direct",
        },
        (56, 56) => LocalisedText {
            en: "Snap Punch",
            ja: "崩拳",
            de: "Peitschenhieb",
            fr: "Coup de poing brusque",
        },
        (56, 61) => LocalisedText {
            en: "Twin Snakes",
            ja: "双掌打",
            de: "Doppelviper",
            fr: "Serpents jumeaux",
        },
        (56, 62) => LocalisedText {
            en: "Arm of the Destroyer",
            ja: "壊神衝",
            de: "Arm des Zerstörers",
            fr: "Frappe du Destructeur",
        },
        (56, 65) => LocalisedText {
            en: "Mantra",
            ja: "マントラ",
            de: "Mantra",
            fr: "Mantra",
        },
        (56, 66) => LocalisedText {
            en: "Demolish",
            ja: "破砕拳",
            de: "Demolieren",
            fr: "Démolition",
        },
        (56, 69) => LocalisedText {
            en: "Perfect Balance",
            ja: "踏鳴",
            de: "Improvisation",
            fr: "Équilibre parfait",
        },
        (56, 70) => LocalisedText {
            en: "Rockbreaker",
            ja: "地烈斬",
            de: "Erdspaltung",
            fr: "Briseur de rocs",
        },
        (56, 74) => LocalisedText {
            en: "Dragon Kick",
            ja: "双竜脚",
            de: "Drachentritt",
            fr: "Tacle du dragon",
        },
        (56, 75) => LocalisedText {
            en: "True Thrust",
            ja: "トゥルースラスト",
            de: "Sauberer Stoß",
            fr: "Percée véritable",
        },
        (56, 78) => LocalisedText {
            en: "Vorpal Thrust",
            ja: "ボーパルスラスト",
            de: "Vorpalstoß",
            fr: "Percée vorpale",
        },
        (56, 83) => LocalisedText {
            en: "Life Surge",
            ja: "ライフサージ",
            de: "Vitalwallung",
            fr: "Souffle de vie",
        },
        (56, 84) => LocalisedText {
            en: "Full Thrust",
            ja: "フルスラスト",
            de: "Ungebremster Stoß",
            fr: "Percée totale",
        },
        (56, 85) => LocalisedText {
            en: "Lance Charge",
            ja: "ランスチャージ",
            de: "Lanzenschärfung",
            fr: "Lance acérée",
        },
        (56, 86) => LocalisedText {
            en: "Doom Spike",
            ja: "ドゥームスパイク",
            de: "Schicksalsdorn",
            fr: "Pointe du destin",
        },
        (56, 87) => LocalisedText {
            en: "Disembowel",
            ja: "ディセムボウル",
            de: "Drachengriff",
            fr: "Éventration",
        },
        (56, 88) => LocalisedText {
            en: "Chaos Thrust",
            ja: "桜華狂咲",
            de: "Chaotischer Tjost",
            fr: "Percée chaotique",
        },
        (56, 90) => LocalisedText {
            en: "Piercing Talon",
            ja: "ピアシングタロン",
            de: "Reißende Klauen",
            fr: "Griffe perçante",
        },
        (56, 92) => LocalisedText {
            en: "Jump",
            ja: "ジャンプ",
            de: "Sprung",
            fr: "Saut",
        },
        (56, 94) => LocalisedText {
            en: "Elusive Jump",
            ja: "イルーシブジャンプ",
            de: "Ausweichsprung",
            fr: "Saut insaisissable",
        },
        (56, 95) => LocalisedText {
            en: "Spineshatter Dive",
            ja: "スパインダイブ",
            de: "Wirbelsprenger",
            fr: "Piqué brise-échine",
        },
        (56, 96) => LocalisedText {
            en: "Dragonfire Dive",
            ja: "ドラゴンダイブ",
            de: "Wyrmodem",
            fr: "Piqué du dragon",
        },
        (56, 97) => LocalisedText {
            en: "Heavy Shot",
            ja: "ヘヴィショット",
            de: "Gewaltiger Schuss",
            fr: "Tir à l'arc puissant",
        },
        (56, 98) => LocalisedText {
            en: "Straight Shot",
            ja: "ストレートショット",
            de: "Direkter Schuss",
            fr: "Tir droit",
        },
        (56, 100) => LocalisedText {
            en: "Venomous Bite",
            ja: "ベノムバイト",
            de: "Giftbiss",
            fr: "Morsure venimeuse",
        },
        (56, 101) => LocalisedText {
            en: "Raging Strikes",
            ja: "猛者の撃",
            de: "Wütende Attacke",
            fr: "Tir furieux",
        },
        (56, 106) => LocalisedText {
            en: "Quick Nock",
            ja: "クイックノック",
            de: "Pfeilsalve",
            fr: "Salve fulgurante",
        },
        (56, 107) => LocalisedText {
            en: "Barrage",
            ja: "乱れ撃ち",
            de: "Sperrfeuer",
            fr: "Rafale de coups",
        },
        (56, 110) => LocalisedText {
            en: "Bloodletter",
            ja: "ブラッドレッター",
            de: "Aderlass",
            fr: "Hémorragie",
        },
        (56, 112) => LocalisedText {
            en: "Repelling Shot",
            ja: "リペリングショット",
            de: "Mannstopper",
            fr: "Tir repoussant",
        },
        (56, 113) => LocalisedText {
            en: "Windbite",
            ja: "ウィンドバイト",
            de: "Beißender Wind",
            fr: "Morsure du vent",
        },
        (56, 114) => LocalisedText {
            en: "Mage's Ballad",
            ja: "賢人のバラード",
            de: "Ballade des Weisen",
            fr: "Ballade du mage",
        },
        (56, 116) => LocalisedText {
            en: "Army's Paeon",
            ja: "軍神のパイオン",
            de: "Hymne der Krieger",
            fr: "Péan martial",
        },
        (56, 117) => LocalisedText {
            en: "Rain of Death",
            ja: "レイン・オブ・デス",
            de: "Tödlicher Regen",
            fr: "Pluie mortelle",
        },
        (56, 118) => LocalisedText {
            en: "Battle Voice",
            ja: "バトルボイス",
            de: "Ode an die Seele",
            fr: "Voix de combat",
        },
        (56, 119) => LocalisedText {
            en: "Stone",
            ja: "ストーン",
            de: "Stein",
            fr: "Terre",
        },
        (56, 120) => LocalisedText {
            en: "Cure",
            ja: "ケアル",
            de: "Vita",
            fr: "Soin",
        },
        (56, 121) => LocalisedText {
            en: "Aero",
            ja: "エアロ",
            de: "Wind",
            fr: "Vent",
        },
        (56, 124) => LocalisedText {
            en: "Medica",
            ja: "メディカ",
            de: "Reseda",
            fr: "Médica",
        },
        (56, 125) => LocalisedText {
            en: "Raise",
            ja: "レイズ",
            de: "Wiederbeleben",
            fr: "Vie",
        },
        (56, 127) => LocalisedText {
            en: "Stone II",
            ja: "ストンラ",
            de: "Steinra",
            fr: "Extra Terre",
        },
        (56, 131) => LocalisedText {
            en: "Cure III",
            ja: "ケアルガ",
            de: "Vitaga",
            fr: "Méga Soin",
        },
        (56, 132) => LocalisedText {
            en: "Aero II",
            ja: "エアロラ",
            de: "Windra",
            fr: "Extra Vent",
        },
        (56, 133) => LocalisedText {
            en: "Medica II",
            ja: "メディカラ",
            de: "Resedra",
            fr: "Extra Médica",
        },
        (56, 135) => LocalisedText {
            en: "Cure II",
            ja: "ケアルラ",
            de: "Vitra",
            fr: "Extra Soin",
        },
        (56, 136) => LocalisedText {
            en: "Presence of Mind",
            ja: "神速魔",
            de: "Geistesgegenwart",
            fr: "Présence d'esprit",
        },
        (56, 137) => LocalisedText {
            en: "Regen",
            ja: "リジェネ",
            de: "Regena",
            fr: "Récup",
        },
        (56, 139) => LocalisedText {
            en: "Holy",
            ja: "ホーリー",
            de: "Sanctus",
            fr: "Miracle",
        },
        (56, 140) => LocalisedText {
            en: "Benediction",
            ja: "ベネディクション",
            de: "Benediktion",
            fr: "Bénédiction",
        },
        (56, 141) => LocalisedText {
            en: "Fire",
            ja: "ファイア",
            de: "Feuer",
            fr: "Feu",
        },
        (56, 142) => LocalisedText {
            en: "Blizzard",
            ja: "ブリザド",
            de: "Eis",
            fr: "Glace",
        },
        (56, 144) => LocalisedText {
            en: "Thunder",
            ja: "サンダー",
            de: "Blitz",
            fr: "Foudre",
        },
        (56, 147) => LocalisedText {
            en: "Fire II",
            ja: "ファイラ",
            de: "Feura",
            fr: "Extra Feu",
        },
        (56, 149) => LocalisedText {
            en: "Transpose",
            ja: "トランス",
            de: "Transposition",
            fr: "Transposition",
        },
        (56, 152) => LocalisedText {
            en: "Fire III",
            ja: "ファイガ",
            de: "Feuga",
            fr: "Méga Feu",
        },
        (56, 153) => LocalisedText {
            en: "Thunder III",
            ja: "サンダガ",
            de: "Blitzga",
            fr: "Méga Foudre",
        },
        (56, 154) => LocalisedText {
            en: "Blizzard III",
            ja: "ブリザガ",
            de: "Eisga",
            fr: "Méga Glace",
        },
        (56, 155) => LocalisedText {
            en: "Aetherial Manipulation",
            ja: "エーテリアルステップ",
            de: "Äthermanipulation",
            fr: "Manipulation de l'éther",
        },
        (56, 156) => LocalisedText {
            en: "Scathe",
            ja: "コラプス",
            de: "Kollaps",
            fr: "Blessure",
        },
        (56, 157) => LocalisedText {
            en: "Manaward",
            ja: "マバリア",
            de: "Mana-Schild",
            fr: "Barrière de mana",
        },
        (56, 158) => LocalisedText {
            en: "Manafont",
            ja: "マナフォント",
            de: "Mana-Brunnen",
            fr: "Vasque de mana",
        },
        (56, 159) => LocalisedText {
            en: "Freeze",
            ja: "フリーズ",
            de: "Einfrieren",
            fr: "Gel",
        },
        (56, 162) => LocalisedText {
            en: "Flare",
            ja: "フレア",
            de: "Flare",
            fr: "Brasier",
        },
        (56, 163) => LocalisedText {
            en: "Ruin",
            ja: "ルイン",
            de: "Ruin",
            fr: "Ruine",
        },
        (56, 166) => LocalisedText {
            en: "Aetherflow",
            ja: "エーテルフロー",
            de: "Ätherfluss",
            fr: "Flux d'éther",
        },
        (56, 167) => LocalisedText {
            en: "Energy Drain",
            ja: "エナジードレイン",
            de: "Energieentzug",
            fr: "Aspiration d'énergie",
        },
        (56, 172) => LocalisedText {
            en: "Ruin II",
            ja: "ルインラ",
            de: "Ruinra",
            fr: "Extra Ruine",
        },
        (56, 173) => LocalisedText {
            en: "Resurrection",
            ja: "リザレク",
            de: "Resuszitation",
            fr: "Résurrection",
        },
        (56, 181) => LocalisedText {
            en: "Fester",
            ja: "ミアズマバースト",
            de: "Schwäre",
            fr: "Suppuration",
        },
        (56, 185) => LocalisedText {
            en: "Adloquium",
            ja: "鼓舞激励の策",
            de: "Adloquium",
            fr: "Traité du réconfort",
        },
        (56, 186) => LocalisedText {
            en: "Succor",
            ja: "士気高揚の策",
            de: "Kurieren",
            fr: "Traité du soulagement",
        },
        (56, 188) => LocalisedText {
            en: "Sacred Soil",
            ja: "野戦治療の陣",
            de: "Geweihte Erde",
            fr: "Dogme de survie",
        },
        (56, 189) => LocalisedText {
            en: "Lustrate",
            ja: "生命活性法",
            de: "Revitalisierung",
            fr: "Loi de revivification",
        },
        (56, 190) => LocalisedText {
            en: "Physick",
            ja: "フィジク",
            de: "Physick",
            fr: "Médecine",
        },
        (56, 197) => LocalisedText {
            en: "Shield Wall",
            ja: "シールドウォール",
            de: "Schutzschild",
            fr: "Mur protecteur",
        },
        (56, 198) => LocalisedText {
            en: "Stronghold",
            ja: "ストロングホールド",
            de: "Bastion",
            fr: "Citadelle",
        },
        (56, 199) => LocalisedText {
            en: "Last Bastion",
            ja: "ラストバスティオン",
            de: "Letzte Bastion",
            fr: "Dernier bastion",
        },
        (56, 200) => LocalisedText {
            en: "Braver",
            ja: "ブレイバー",
            de: "Mutangriff",
            fr: "Ardeur courageuse",
        },
        (56, 201) => LocalisedText {
            en: "Bladedance",
            ja: "ブレードダンス",
            de: "Schwertertanz",
            fr: "Danse de la lame",
        },
        (56, 202) => LocalisedText {
            en: "Final Heaven",
            ja: "ファイナルヘヴン",
            de: "Endgültiger Himmel",
            fr: "Paradis final",
        },
        (56, 203) => LocalisedText {
            en: "Skyshard",
            ja: "スカイシャード",
            de: "Himmelsscherbe",
            fr: "Éclat de ciel",
        },
        (56, 204) => LocalisedText {
            en: "Starstorm",
            ja: "プチメテオ",
            de: "Sternensturm",
            fr: "Tempête d'étoiles",
        },
        (56, 205) => LocalisedText {
            en: "Meteor",
            ja: "メテオ",
            de: "Meteor",
            fr: "Météore",
        },
        (56, 206) => LocalisedText {
            en: "Healing Wind",
            ja: "癒しの風",
            de: "Heilender Wind",
            fr: "Vent curateur",
        },
        (56, 207) => LocalisedText {
            en: "Breath of the Earth",
            ja: "大地の息吹",
            de: "Atem der Erde",
            fr: "Souffle de la Terre",
        },
        (56, 208) => LocalisedText {
            en: "Pulse of Life",
            ja: "生命の鼓動",
            de: "Lebenspuls",
            fr: "Pulsation vitale",
        },
        (56, 210) => LocalisedText {
            en: "Triangulate",
            ja: "トライアングレート",
            de: "Geodäsie",
            fr: "Forestier",
        },
        (56, 211) => LocalisedText {
            en: "Arbor Call",
            ja: "アーバーコール",
            de: "Ruf des Waldes",
            fr: "Dendrologie",
        },
        (56, 215) => LocalisedText {
            en: "Ageless Words",
            ja: "老農の知",
            de: "Bauernschläue",
            fr: "Sagesse du fermier",
        },
        (56, 218) => LocalisedText {
            en: "Field Mastery",
            ja: "フィールドマスタリー",
            de: "Meister der Felder",
            fr: "Maîtrise du terrain",
        },
        (56, 220) => LocalisedText {
            en: "Field Mastery II",
            ja: "フィールドマスタリーII",
            de: "Meister der Felder II",
            fr: "Maîtrise du terrain II",
        },
        (56, 221) => LocalisedText {
            en: "Truth of Forests",
            ja: "トゥルー・オブ・フォレスト",
            de: "Flurenthüllung",
            fr: "Science des végétaux",
        },
        (56, 222) => LocalisedText {
            en: "Blessed Harvest",
            ja: "ブレスドハーベスト",
            de: "Gesegnete Ernte",
            fr: "Récolte bénie",
        },
        (56, 224) => LocalisedText {
            en: "Blessed Harvest II",
            ja: "ブレスドハーベストII",
            de: "Gesegnete Ernte II",
            fr: "Récolte bénie II",
        },
        (56, 227) => LocalisedText {
            en: "Prospect",
            ja: "プロスペクト",
            de: "Prospektion",
            fr: "Prospecteur",
        },
        (56, 228) => LocalisedText {
            en: "Lay of the Land",
            ja: "ランドサーベイ",
            de: "Bodenbefund",
            fr: "Géologie",
        },
        (56, 232) => LocalisedText {
            en: "Solid Reason",
            ja: "石工の理",
            de: "Felsenfeste Überzeugung",
            fr: "Raison solide",
        },
        (56, 235) => LocalisedText {
            en: "Sharp Vision",
            ja: "シャープビジョン",
            de: "Scharfer Blick",
            fr: "Vue perçante",
        },
        (56, 237) => LocalisedText {
            en: "Sharp Vision II",
            ja: "シャープビジョンII",
            de: "Scharfer Blick II",
            fr: "Vue perçante II",
        },
        (56, 238) => LocalisedText {
            en: "Truth of Mountains",
            ja: "トゥルー・オブ・ミネラル",
            de: "Tellurische Enthüllung",
            fr: "Science des minéraux",
        },
        (56, 239) => LocalisedText {
            en: "King's Yield",
            ja: "キングスイールド",
            de: "Fürstlicher Ertrag",
            fr: "Récolte royale",
        },
        (56, 240) => LocalisedText {
            en: "Collect",
            ja: "収集品採集",
            de: "Sammeln",
            fr: "Récolte",
        },
        (56, 241) => LocalisedText {
            en: "King's Yield II",
            ja: "キングスイールドII",
            de: "Fürstlicher Ertrag II",
            fr: "Récolte royale II",
        },
        (56, 260) => LocalisedText {
            en: "Great Strides",
            ja: "グレートストライド",
            de: "Große Schritte",
            fr: "Grands progrès",
        },
        (56, 268) => LocalisedText {
            en: "Mooch II",
            ja: "泳がせ釣りII",
            de: "Naturköder II",
            fr: "Pêche au vif II",
        },
        (56, 269) => LocalisedText {
            en: "Double Hook",
            ja: "ダブルフッキング",
            de: "Doppelhaken",
            fr: "Ferrage double",
        },
        (56, 272) => LocalisedText {
            en: "Bountiful Yield II",
            ja: "バウンティフルイールドII",
            de: "Reiche Ausbeute II",
            fr: "Extraction abondante II",
        },
        (56, 273) => LocalisedText {
            en: "Bountiful Harvest II",
            ja: "バウンティフルハーベストII",
            de: "Reiche Ernte II",
            fr: "Récolte abondante II",
        },
        (56, 280) => LocalisedText {
            en: "the Twelve's Bounty",
            ja: "十二神の加護",
            de: "Gunst der Zwölf",
            fr: "Bénédiction des Douze",
        },
        (56, 288) => LocalisedText {
            en: "Bait",
            ja: "釣り餌",
            de: "Köder",
            fr: "Appât",
        },
        (56, 289) => LocalisedText {
            en: "Cast",
            ja: "キャスティング",
            de: "Angeln",
            fr: "Pêche",
        },
        (56, 290) => LocalisedText {
            en: "Arbor Call II",
            ja: "アーバーコールII",
            de: "Ruf des Waldes II",
            fr: "Dendrologie II",
        },
        (56, 291) => LocalisedText {
            en: "Lay of the Land II",
            ja: "ランドサーベイII",
            de: "Bodenbefund II",
            fr: "Géologie II",
        },
        (56, 294) => LocalisedText {
            en: "Field Mastery III",
            ja: "フィールドマスタリーIII",
            de: "Meister der Felder III",
            fr: "Maîtrise du terrain III",
        },
        (56, 295) => LocalisedText {
            en: "Sharp Vision III",
            ja: "シャープビジョンIII",
            de: "Scharfer Blick III",
            fr: "Vue perçante III",
        },
        (56, 296) => LocalisedText {
            en: "Hook",
            ja: "フッキング",
            de: "Anschlag",
            fr: "Ferrage",
        },
        (56, 297) => LocalisedText {
            en: "Mooch",
            ja: "泳がせ釣り",
            de: "Naturköder",
            fr: "Pêche au vif",
        },
        (56, 299) => LocalisedText {
            en: "Quit",
            ja: "中断",
            de: "Beenden",
            fr: "Abandon",
        },
        (56, 300) => LocalisedText {
            en: "Release",
            ja: "リリース",
            de: "Freilassen",
            fr: "Remise à l'eau",
        },
        (56, 303) => LocalisedText {
            en: "Sneak",
            ja: "スニーク",
            de: "Verstohlenheit",
            fr: "Discrétion",
        },
        (56, 802) => LocalisedText {
            en: "Embrace",
            ja: "光の癒し",
            de: "Sanfte Umarmung",
            fr: "Embrassement",
        },
        (56, 803) => LocalisedText {
            en: "Whispering Dawn",
            ja: "光の囁き",
            de: "Erhebendes Flüstern",
            fr: "Murmure de l'aurore",
        },
        (56, 805) => LocalisedText {
            en: "Fey Illumination",
            ja: "フェイイルミネーション",
            de: "Illumination",
            fr: "Illumination féerique",
        },
        (56, 2135) => LocalisedText {
            en: "Cast Light",
            ja: "キャストライト",
            de: "Leuchtpose",
            fr: "Éclairage",
        },
        (56, 2240) => LocalisedText {
            en: "Spinning Edge",
            ja: "双刃旋",
            de: "Wirbelklinge",
            fr: "Lame tournoyante",
        },
        (56, 2241) => LocalisedText {
            en: "Shade Shift",
            ja: "残影",
            de: "Superkniff",
            fr: "Décalage d'ombre",
        },
        (56, 2242) => LocalisedText {
            en: "Gust Slash",
            ja: "風断ち",
            de: "Windschlitzer",
            fr: "Tourbillon tailladant",
        },
        (56, 2245) => LocalisedText {
            en: "Hide",
            ja: "かくれる",
            de: "Verstecken",
            fr: "Dissimulation",
        },
        (56, 2246) => LocalisedText {
            en: "Assassinate",
            ja: "終撃",
            de: "Meuchelsprung",
            fr: "Assassinement",
        },
        (56, 2247) => LocalisedText {
            en: "Throwing Dagger",
            ja: "投刃",
            de: "Wurfmesser",
            fr: "Dagues de jet",
        },
        (56, 2248) => LocalisedText {
            en: "Mug",
            ja: "ぶんどる",
            de: "Ausrauben",
            fr: "Rapine",
        },
        (56, 2254) => LocalisedText {
            en: "Death Blossom",
            ja: "血花五月雨",
            de: "Todesblüte",
            fr: "Roseraie fatale",
        },
        (56, 2255) => LocalisedText {
            en: "Aeolian Edge",
            ja: "旋風刃",
            de: "Äolusklinge",
            fr: "Lame éolienne",
        },
        (56, 2258) => LocalisedText {
            en: "Trick Attack",
            ja: "だまし討ち",
            de: "Trickattacke",
            fr: "Attaque sournoise",
        },
        (56, 2259) => LocalisedText {
            en: "Ten",
            ja: "天の印",
            de: "Ten",
            fr: "Ten",
        },
        (56, 2260) => LocalisedText {
            en: "Ninjutsu",
            ja: "忍術",
            de: "Ninjutsu",
            fr: "Ninjutsu",
        },
        (56, 2261) => LocalisedText {
            en: "Chi",
            ja: "地の印",
            de: "Chi",
            fr: "Chi",
        },
        (56, 2262) => LocalisedText {
            en: "Shukuchi",
            ja: "縮地",
            de: "Shukuchi",
            fr: "Shukuchi",
        },
        (56, 2263) => LocalisedText {
            en: "Jin",
            ja: "人の印",
            de: "Jin",
            fr: "Jin",
        },
        (56, 2264) => LocalisedText {
            en: "Kassatsu",
            ja: "活殺自在",
            de: "Kassatsu",
            fr: "Kassatsu",
        },
        (56, 2265) => LocalisedText {
            en: "Fuma Shuriken",
            ja: "風魔手裏剣",
            de: "Fuma Shuriken",
            fr: "Fûma Shuriken",
        },
        (56, 2266) => LocalisedText {
            en: "Katon",
            ja: "火遁の術",
            de: "Katon",
            fr: "Katon",
        },
        (56, 2267) => LocalisedText {
            en: "Raiton",
            ja: "雷遁の術",
            de: "Raiton",
            fr: "Raiton",
        },
        (56, 2268) => LocalisedText {
            en: "Hyoton",
            ja: "氷遁の術",
            de: "Hyoton",
            fr: "Hyôton",
        },
        (56, 2269) => LocalisedText {
            en: "Huton",
            ja: "風遁の術",
            de: "Huton",
            fr: "Fûton",
        },
        (56, 2270) => LocalisedText {
            en: "Doton",
            ja: "土遁の術",
            de: "Doton",
            fr: "Doton",
        },
        (56, 2271) => LocalisedText {
            en: "Suiton",
            ja: "水遁の術",
            de: "Suiton",
            fr: "Suiton",
        },
        (56, 2272) => LocalisedText {
            en: "Rabbit Medium",
            ja: "口寄せの術",
            de: "Hasenmedium",
            fr: "Kuchiyose",
        },
        (56, 2864) => LocalisedText {
            en: "Rook Autoturret",
            ja: "オートタレット・ルーク",
            de: "Selbstschuss-Gyrocopter TURM",
            fr: "Auto-tourelle Tour",
        },
        (56, 2866) => LocalisedText {
            en: "Split Shot",
            ja: "スプリットショット",
            de: "Feuer frei",
            fr: "Tir scindé",
        },
        (56, 2868) => LocalisedText {
            en: "Slug Shot",
            ja: "スラッグショット",
            de: "Flintenlaufgeschoss",
            fr: "Tir de balle",
        },
        (56, 2870) => LocalisedText {
            en: "Spread Shot",
            ja: "スプレッドショット",
            de: "Splitterschuss",
            fr: "Tir dispersé",
        },
        (56, 2872) => LocalisedText {
            en: "Hot Shot",
            ja: "ホットショット",
            de: "Explosivgeschoss",
            fr: "Tir déchaîné",
        },
        (56, 2873) => LocalisedText {
            en: "Clean Shot",
            ja: "クリーンショット",
            de: "Sauberer Schuss",
            fr: "Tir net",
        },
        (56, 2874) => LocalisedText {
            en: "Gauss Round",
            ja: "ガウスラウンド",
            de: "Gauß-Geschoss",
            fr: "Décharge Gauss",
        },
        (56, 2876) => LocalisedText {
            en: "Reassemble",
            ja: "整備",
            de: "Justieren",
            fr: "Réassemblage",
        },
        (56, 2878) => LocalisedText {
            en: "Wildfire",
            ja: "ワイルドファイア",
            de: "Wildfeuer",
            fr: "Flambée",
        },
        (56, 2890) => LocalisedText {
            en: "Ricochet",
            ja: "リコシェット",
            de: "Rikoschettschuss",
            fr: "Ricochet",
        },
        (56, 3538) => LocalisedText {
            en: "Goring Blade",
            ja: "ゴアブレード",
            de: "Ausweiden",
            fr: "Lame étripante",
        },
        (56, 3539) => LocalisedText {
            en: "Royal Authority",
            ja: "ロイヤルアソリティ",
            de: "Königsklinge",
            fr: "Autorité royale",
        },
        (56, 3540) => LocalisedText {
            en: "Divine Veil",
            ja: "ディヴァインヴェール",
            de: "Heiliger Quell",
            fr: "Voile divin",
        },
        (56, 3541) => LocalisedText {
            en: "Clemency",
            ja: "クレメンシー",
            de: "Klemenz",
            fr: "Clémence",
        },
        (56, 3542) => LocalisedText {
            en: "Sheltron",
            ja: "シェルトロン",
            de: "Schiltron",
            fr: "Schiltron",
        },
        (56, 3543) => LocalisedText {
            en: "Tornado Kick",
            ja: "闘魂旋風脚",
            de: "Tornadotritt",
            fr: "Coup de pied tornade",
        },
        (56, 3545) => LocalisedText {
            en: "Elixir Field",
            ja: "蒼気砲",
            de: "Elixier-Feld",
            fr: "Champ-élixir",
        },
        (56, 3546) => LocalisedText {
            en: "Meditation",
            ja: "闘気",
            de: "Meditation",
            fr: "Méditation",
        },
        (56, 3547) => LocalisedText {
            en: "the Forbidden Chakra",
            ja: "陰陽闘気斬",
            de: "Verbotenes Chakra",
            fr: "Chakra interdit",
        },
        (56, 3549) => LocalisedText {
            en: "Fell Cleave",
            ja: "フェルクリーヴ",
            de: "Niedermähen",
            fr: "Sape-fendeur",
        },
        (56, 3550) => LocalisedText {
            en: "Decimate",
            ja: "デシメート",
            de: "Dezimieren",
            fr: "Décimation",
        },
        (56, 3551) => LocalisedText {
            en: "Raw Intuition",
            ja: "原初の直感",
            de: "Urinstinkt",
            fr: "Intuition pure",
        },
        (56, 3552) => LocalisedText {
            en: "Equilibrium",
            ja: "エクリブリウム",
            de: "Äquilibrium",
            fr: "Équilibre",
        },
        (56, 3554) => LocalisedText {
            en: "Fang and Claw",
            ja: "竜牙竜爪",
            de: "Fang und Klaue",
            fr: "Croc-et-griffe",
        },
        (56, 3555) => LocalisedText {
            en: "Geirskogul",
            ja: "ゲイルスコグル",
            de: "Geirskogul",
            fr: "Geirskögul",
        },
        (56, 3556) => LocalisedText {
            en: "Wheeling Thrust",
            ja: "竜尾大車輪",
            de: "Fächerstoß",
            fr: "Percée tournante",
        },
        (56, 3557) => LocalisedText {
            en: "Battle Litany",
            ja: "バトルリタニー",
            de: "Litanei der Schlacht",
            fr: "Litanie combattante",
        },
        (56, 3558) => LocalisedText {
            en: "Empyreal Arrow",
            ja: "エンピリアルアロー",
            de: "Empyreischer Pfeil",
            fr: "Flèche empyréenne",
        },
        (56, 3559) => LocalisedText {
            en: "the Wanderer's Minuet",
            ja: "旅神のメヌエット",
            de: "Menuett des Wanderers",
            fr: "Menuet du Vagabond",
        },
        (56, 3560) => LocalisedText {
            en: "Iron Jaws",
            ja: "アイアンジョー",
            de: "Eiserne Kiefer",
            fr: "Mâchoires de fer",
        },
        (56, 3561) => LocalisedText {
            en: "the Warden's Paean",
            ja: "時神のピーアン",
            de: "Päan des Hüters",
            fr: "Péon du Contemplateur",
        },
        (56, 3562) => LocalisedText {
            en: "Sidewinder",
            ja: "サイドワインダー",
            de: "Seitenschneider",
            fr: "Vent venimeux",
        },
        (56, 3563) => LocalisedText {
            en: "Armor Crush",
            ja: "強甲破点突",
            de: "Harnischbrecher",
            fr: "Perce-armure",
        },
        (56, 3566) => LocalisedText {
            en: "Dream Within a Dream",
            ja: "夢幻三段",
            de: "Ternäre Trance",
            fr: "Rêve dans un rêve",
        },
        (56, 3568) => LocalisedText {
            en: "Stone III",
            ja: "ストンガ",
            de: "Steinga",
            fr: "Méga Terre",
        },
        (56, 3569) => LocalisedText {
            en: "Asylum",
            ja: "アサイラム",
            de: "Refugium",
            fr: "Asile",
        },
        (56, 3570) => LocalisedText {
            en: "Tetragrammaton",
            ja: "テトラグラマトン",
            de: "Tetragrammaton",
            fr: "Tétragramme",
        },
        (56, 3571) => LocalisedText {
            en: "Assize",
            ja: "アサイズ",
            de: "Assise",
            fr: "Assises",
        },
        (56, 3573) => LocalisedText {
            en: "Ley Lines",
            ja: "黒魔紋",
            de: "Ley-Linien",
            fr: "Manalignements",
        },
        (56, 3574) => LocalisedText {
            en: "Sharpcast",
            ja: "激成魔",
            de: "Augmentierung ",
            fr: "Dynamisation",
        },
        (56, 3576) => LocalisedText {
            en: "Blizzard IV",
            ja: "ブリザジャ",
            de: "Eiska",
            fr: "Giga Glace",
        },
        (56, 3577) => LocalisedText {
            en: "Fire IV",
            ja: "ファイジャ",
            de: "Feuka",
            fr: "Giga Feu",
        },
        (56, 3578) => LocalisedText {
            en: "Painflare",
            ja: "ペインフレア",
            de: "Schmerzensflare",
            fr: "Brasier de peine",
        },
        (56, 3579) => LocalisedText {
            en: "Ruin III",
            ja: "ルインガ",
            de: "Ruinga",
            fr: "Méga Ruine",
        },
        (56, 3581) => LocalisedText {
            en: "Dreadwyrm Trance",
            ja: "トランス・バハムート",
            de: "Bahamut-Trance",
            fr: "Transe-Bahamut",
        },
        (56, 3582) => LocalisedText {
            en: "Deathflare",
            ja: "デスフレア",
            de: "Todesflamme",
            fr: "Brasier de mort",
        },
        (56, 3583) => LocalisedText {
            en: "Indomitability",
            ja: "不撓不屈の策",
            de: "Unbezwingbarkeit",
            fr: "Traité de la persévérance",
        },
        (56, 3584) => LocalisedText {
            en: "Broil",
            ja: "気炎法",
            de: "Bravade",
            fr: "Loi de l'ébullition",
        },
        (56, 3585) => LocalisedText {
            en: "Deployment Tactics",
            ja: "展開戦術",
            de: "Dislokation",
            fr: "Stratagème du déploiement",
        },
        (56, 3586) => LocalisedText {
            en: "Emergency Tactics",
            ja: "応急戦術",
            de: "Apotropaion",
            fr: "Stratagème de l'urgence",
        },
        (56, 3587) => LocalisedText {
            en: "Dissipation",
            ja: "転化",
            de: "Dissipation",
            fr: "Dissipation",
        },
        (56, 3590) => LocalisedText {
            en: "Draw",
            ja: "ドロー",
            de: "Ziehen",
            fr: "Tirage",
        },
        (56, 3593) => LocalisedText {
            en: "Redraw",
            ja: "リドロー",
            de: "Mischen",
            fr: "Retirage",
        },
        (56, 3594) => LocalisedText {
            en: "Benefic",
            ja: "ベネフィク",
            de: "Harmonieaspekt",
            fr: "Bienfaisance",
        },
        (56, 3595) => LocalisedText {
            en: "Aspected Benefic",
            ja: "アスペクト・ベネフィク",
            de: "Harmonischer Orbis",
            fr: "Bienfaisance aspectée",
        },
        (56, 3596) => LocalisedText {
            en: "Malefic",
            ja: "マレフィク",
            de: "Spannungsaspekt",
            fr: "Malfaisance",
        },
        (56, 3598) => LocalisedText {
            en: "Malefic II",
            ja: "マレフィラ",
            de: "Spannungsaspekt II",
            fr: "Malfaisance II",
        },
        (56, 3599) => LocalisedText {
            en: "Combust",
            ja: "コンバス",
            de: "Konjunktion",
            fr: "Conjonction supérieure",
        },
        (56, 3600) => LocalisedText {
            en: "Helios",
            ja: "ヘリオス",
            de: "Helios",
            fr: "Hélios",
        },
        (56, 3601) => LocalisedText {
            en: "Aspected Helios",
            ja: "アスペクト・ヘリオス",
            de: "Aspektierter Helios",
            fr: "Hélios aspecté",
        },
        (56, 3603) => LocalisedText {
            en: "Ascend",
            ja: "アセンド",
            de: "Aszension",
            fr: "Ascendant",
        },
        (56, 3606) => LocalisedText {
            en: "Lightspeed",
            ja: "ライトスピード",
            de: "Lichtgeschwindigkeit",
            fr: "Vitesse de la lumière",
        },
        (56, 3608) => LocalisedText {
            en: "Combust II",
            ja: "コンバラ",
            de: "Konjunktion II",
            fr: "Conjonction supérieure II",
        },
        (56, 3610) => LocalisedText {
            en: "Benefic II",
            ja: "ベネフィラ",
            de: "Harmonieaspekt II",
            fr: "Bienfaisance II",
        },
        (56, 3612) => LocalisedText {
            en: "Synastry",
            ja: "シナストリー",
            de: "Synastrie",
            fr: "Synastrie",
        },
        (56, 3613) => LocalisedText {
            en: "Collective Unconscious",
            ja: "運命の輪",
            de: "Numinosum",
            fr: "Inconscient collectif",
        },
        (56, 3614) => LocalisedText {
            en: "Essential Dignity",
            ja: "ディグニティ",
            de: "Essenzielle Würde",
            fr: "Dignité essentielle",
        },
        (56, 3615) => LocalisedText {
            en: "Gravity",
            ja: "グラビデ",
            de: "Gravitas",
            fr: "Gravité",
        },
        (56, 3617) => LocalisedText {
            en: "Hard Slash",
            ja: "ハードスラッシュ",
            de: "Harter Schlitzer",
            fr: "Taillade violente",
        },
        (56, 3621) => LocalisedText {
            en: "Unleash",
            ja: "アンリーシュ",
            de: "Entfesselung",
            fr: "Déchaînement",
        },
        (56, 3623) => LocalisedText {
            en: "Syphon Strike",
            ja: "サイフォンストライク",
            de: "Osmose",
            fr: "Frappe siphon",
        },
        (56, 3624) => LocalisedText {
            en: "Unmend",
            ja: "アンメンド",
            de: "Zerfall",
            fr: "Détérioration",
        },
        (56, 3625) => LocalisedText {
            en: "Blood Weapon",
            ja: "ブラッドウェポン",
            de: "Blutwaffe",
            fr: "Arme de sang",
        },
        (56, 3629) => LocalisedText {
            en: "Grit",
            ja: "グリットスタンス",
            de: "Zähigkeit",
            fr: "Férocité",
        },
        (56, 3632) => LocalisedText {
            en: "Souleater",
            ja: "ソウルイーター",
            de: "Seelenfresser",
            fr: "Dévoreur d'âme",
        },
        (56, 3634) => LocalisedText {
            en: "Dark Mind",
            ja: "ダークマインド",
            de: "Dunkler Geist",
            fr: "Esprit ténébreux",
        },
        (56, 3636) => LocalisedText {
            en: "Shadow Wall",
            ja: "シャドウウォール",
            de: "Schattenwand",
            fr: "Mur d'ombre",
        },
        (56, 3638) => LocalisedText {
            en: "Living Dead",
            ja: "リビングデッド",
            de: "Totenerweckung",
            fr: "Mort-vivant",
        },
        (56, 3639) => LocalisedText {
            en: "Salted Earth",
            ja: "ソルトアース",
            de: "Salzige Erde",
            fr: "Terre salée",
        },
        (56, 3640) => LocalisedText {
            en: "Plunge",
            ja: "プランジカット",
            de: "Hiebsprung",
            fr: "Coupe plongeante",
        },
        (56, 3641) => LocalisedText {
            en: "Abyssal Drain",
            ja: "アビサルドレイン",
            de: "Abyssale Blutung",
            fr: "Drainage abyssal",
        },
        (56, 3643) => LocalisedText {
            en: "Carve and Spit",
            ja: "カーヴ・アンド・スピット",
            de: "Zweischnitt",
            fr: "Tranchage-habillage",
        },
        (56, 4072) => LocalisedText {
            en: "Clear Vision",
            ja: "クリアビジョン",
            de: "Klare Sicht",
            fr: "Vision claire",
        },
        (56, 4073) => LocalisedText {
            en: "Bountiful Yield",
            ja: "バウンティフルイールド",
            de: "Reiche Ausbeute",
            fr: "Extraction abondante",
        },
        (56, 4081) => LocalisedText {
            en: "Luck of the Mountaineer",
            ja: "山師の眼力",
            de: "Glück des Bergmanns",
            fr: "Chance du montagnard",
        },
        (56, 4086) => LocalisedText {
            en: "Flora Mastery",
            ja: "フローラマスタリー",
            de: "Florale Meisterschaft",
            fr: "Maîtrise florale",
        },
        (56, 4087) => LocalisedText {
            en: "Bountiful Harvest",
            ja: "バウンティフルハーベスト",
            de: "Reiche Ernte",
            fr: "Récolte abondante",
        },
        (56, 4095) => LocalisedText {
            en: "Luck of the Pioneer",
            ja: "開拓者の眼力",
            de: "Glück des Pioniers",
            fr: "Chance du pionnier",
        },
        (56, 4100) => LocalisedText {
            en: "Snagging",
            ja: "引掛釣り",
            de: "Reißen",
            fr: "Casaquage",
        },
        (56, 4102) => LocalisedText {
            en: "Patience",
            ja: "ペーシェンス",
            de: "Geduld",
            fr: "Patience",
        },
        (56, 4103) => LocalisedText {
            en: "Powerful Hookset",
            ja: "ストロングフッキング",
            de: "Fester Anschlag",
            fr: "Ferrage puissant",
        },
        (56, 4104) => LocalisedText {
            en: "Chum",
            ja: "撒き餌",
            de: "Streuköder",
            fr: "Amorçage",
        },
        (56, 4105) => LocalisedText {
            en: "Fish Eyes",
            ja: "フィッシュアイ",
            de: "Fischaugen",
            fr: "Yeux de poisson",
        },
        (56, 4106) => LocalisedText {
            en: "Patience II",
            ja: "ペーシェンスII",
            de: "Geduld II",
            fr: "Patience II",
        },
        (56, 4179) => LocalisedText {
            en: "Precision Hookset",
            ja: "プレシジョンフッキング",
            de: "Präziser Anschlag",
            fr: "Ferrage précis",
        },
        (56, 4238) => LocalisedText {
            en: "Big Shot",
            ja: "ビッグショット",
            de: "Volles Rohr",
            fr: "Gros calibre",
        },
        (56, 4239) => LocalisedText {
            en: "Desperado",
            ja: "デスペラード",
            de: "Desperado",
            fr: "Desperado",
        },
        (56, 4240) => LocalisedText {
            en: "Land Waker",
            ja: "原初の大地",
            de: "Erdbrecher",
            fr: "Terre vierge",
        },
        (56, 4241) => LocalisedText {
            en: "Dark Force",
            ja: "ダークフォース",
            de: "Dunkle Macht",
            fr: "Force des ténèbres",
        },
        (56, 4242) => LocalisedText {
            en: "Dragonsong Dive",
            ja: "蒼天のドラゴンダイブ",
            de: "Azurner Drachensturz",
            fr: "Plongeon des dragons",
        },
        (56, 4243) => LocalisedText {
            en: "Chimatsuri",
            ja: "月遁血祭",
            de: "Chimatsuri",
            fr: "Chimatsuri",
        },
        (56, 4244) => LocalisedText {
            en: "Sagittarius Arrow",
            ja: "サジタリウスアロー",
            de: "Choral der Pfeile",
            fr: "Flèche du sagittaire",
        },
        (56, 4245) => LocalisedText {
            en: "Satellite Beam",
            ja: "サテライトビーム",
            de: "Satellit",
            fr: "Rayon satellite",
        },
        (56, 4246) => LocalisedText {
            en: "Teraflare",
            ja: "テラフレア",
            de: "Teraflare",
            fr: "TéraBrasier",
        },
        (56, 4247) => LocalisedText {
            en: "Angel Feathers",
            ja: "エンジェルフェザー",
            de: "Engelsfedern",
            fr: "Plumes d'ange",
        },
        (56, 4248) => LocalisedText {
            en: "Astral Stasis",
            ja: "星天開門",
            de: "Astrale Stasis",
            fr: "Équilibre astral",
        },
        (56, 4262) => LocalisedText {
            en: "Form Shift",
            ja: "演武",
            de: "Formwechsel",
            fr: "Glissement de posture",
        },
        (56, 4401) => LocalisedText {
            en: "the Balance",
            ja: "アーゼマの均衡",
            de: "Waage",
            fr: "La Balance",
        },
        (56, 4402) => LocalisedText {
            en: "the Arrow",
            ja: "オシュオンの矢",
            de: "Pfeil",
            fr: "La Flèche",
        },
        (56, 4403) => LocalisedText {
            en: "the Spear",
            ja: "ハルオーネの槍",
            de: "Speer",
            fr: "L'Épieu",
        },
        (56, 4404) => LocalisedText {
            en: "the Bole",
            ja: "世界樹の幹",
            de: "Eiche",
            fr: "Le Tronc",
        },
        (56, 4405) => LocalisedText {
            en: "the Ewer",
            ja: "サリャクの水瓶",
            de: "Krug",
            fr: "L'Aiguière",
        },
        (56, 4406) => LocalisedText {
            en: "the Spire",
            ja: "ビエルゴの塔",
            de: "Turm",
            fr: "La Tour",
        },
        (56, 4574) => LocalisedText {
            en: "Manipulation",
            ja: "マニピュレーション",
            de: "Manipulation",
            fr: "Manipulation",
        },
        (56, 4589) => LocalisedText {
            en: "The Giving Land",
            ja: "大地の恵み",
            de: "Geschenk der Erde",
            fr: "Bénédiction de la nature",
        },
        (56, 4595) => LocalisedText {
            en: "Surface Slap",
            ja: "トレードリリース",
            de: "Bunte Gewässer",
            fr: "Pêche variée",
        },
        (56, 4596) => LocalisedText {
            en: "Identical Cast",
            ja: "セイムキャスト",
            de: "Stetes Angeln",
            fr: "Pêche ciblée",
        },
        (56, 4631) => LocalisedText {
            en: "Waste Not",
            ja: "倹約",
            de: "Nachhaltigkeit",
            fr: "Parcimonie",
        },
        (56, 4639) => LocalisedText {
            en: "Waste Not II",
            ja: "長期倹約",
            de: "Nachhaltigkeit II",
            fr: "Parcimonie pérenne",
        },
        (56, 7381) => LocalisedText {
            en: "Total Eclipse",
            ja: "トータルエクリプス",
            de: "Eklipse",
            fr: "Éclipse totale",
        },
        (56, 7382) => LocalisedText {
            en: "Intervention",
            ja: "インターベンション",
            de: "Intervention",
            fr: "Intervention",
        },
        (56, 7383) => LocalisedText {
            en: "Requiescat",
            ja: "レクイエスカット",
            de: "Requiescat",
            fr: "Requiescat",
        },
        (56, 7384) => LocalisedText {
            en: "Holy Spirit",
            ja: "ホーリースピリット",
            de: "Heiliger Geist",
            fr: "Esprit saint",
        },
        (56, 7385) => LocalisedText {
            en: "Passage of Arms",
            ja: "パッセージ・オブ・アームズ",
            de: "Waffengang",
            fr: "Passe d'armes",
        },
        (56, 7386) => LocalisedText {
            en: "Onslaught",
            ja: "オンスロート",
            de: "Sturmlauf",
            fr: "Assaut violent",
        },
        (56, 7387) => LocalisedText {
            en: "Upheaval",
            ja: "アップヒーバル",
            de: "Umsturz",
            fr: "Révolte",
        },
        (56, 7388) => LocalisedText {
            en: "Shake It Off",
            ja: "シェイクオフ",
            de: "Abschütteln",
            fr: "Débarrassage",
        },
        (56, 7389) => LocalisedText {
            en: "Inner Release",
            ja: "原初の解放",
            de: "Urbefreiung",
            fr: "Relâchement bestial",
        },
        (56, 7390) => LocalisedText {
            en: "Delirium",
            ja: "ブラッドデリリアム",
            de: "Blutdelirium",
            fr: "Delirium de sang",
        },
        (56, 7391) => LocalisedText {
            en: "Quietus",
            ja: "クワイタス",
            de: "Quietus",
            fr: "Quietus",
        },
        (56, 7392) => LocalisedText {
            en: "Bloodspiller",
            ja: "ブラッドスピラー",
            de: "Blutvergießen",
            fr: "Marée de sang",
        },
        (56, 7393) => LocalisedText {
            en: "The Blackest Night",
            ja: "ブラックナイト",
            de: "Schwärzeste Nacht",
            fr: "Nuit noirissime",
        },
        (56, 7394) => LocalisedText {
            en: "Riddle of Earth",
            ja: "金剛の極意",
            de: "Steinernes Enigma",
            fr: "Énigme de la terre",
        },
        (56, 7395) => LocalisedText {
            en: "Riddle of Fire",
            ja: "紅蓮の極意",
            de: "Flammendes Enigma",
            fr: "Énigme du feu",
        },
        (56, 7396) => LocalisedText {
            en: "Brotherhood",
            ja: "桃園結義",
            de: "Bruderschaft",
            fr: "Fraternité",
        },
        (56, 7397) => LocalisedText {
            en: "Sonic Thrust",
            ja: "ソニックスラスト",
            de: "Schallspitze",
            fr: "Percée sonique",
        },
        (56, 7398) => LocalisedText {
            en: "Dragon Sight",
            ja: "ドラゴンサイト",
            de: "Drachenblick",
            fr: "Vue de dragon",
        },
        (56, 7399) => LocalisedText {
            en: "Mirage Dive",
            ja: "ミラージュダイブ",
            de: "Illusionssprung",
            fr: "Piqué mirage",
        },
        (56, 7400) => LocalisedText {
            en: "Nastrond",
            ja: "ナーストレンド",
            de: "Nastrond",
            fr: "Naströnd",
        },
        (56, 7401) => LocalisedText {
            en: "Hellfrog Medium",
            ja: "口寄せの術・大蝦蟇",
            de: "Krötenmedium",
            fr: "Kuchiyose Ogama",
        },
        (56, 7402) => LocalisedText {
            en: "Bhavacakra",
            ja: "六道輪廻",
            de: "Rad des Werdens",
            fr: "Bhavacakra",
        },
        (56, 7403) => LocalisedText {
            en: "Ten Chi Jin",
            ja: "天地人",
            de: "Ten Chi Jin",
            fr: "Ten Chi Jin",
        },
        (56, 7404) => LocalisedText {
            en: "Pitch Perfect",
            ja: "ピッチパーフェクト",
            de: "Absolutes Gehör",
            fr: "Ton parfait",
        },
        (56, 7405) => LocalisedText {
            en: "Troubadour",
            ja: "トルバドゥール",
            de: "Troubadour",
            fr: "Troubadour",
        },
        (56, 7406) => LocalisedText {
            en: "Caustic Bite",
            ja: "コースティックバイト",
            de: "Kaustischer Biss",
            fr: "Morsure caustique",
        },
        (56, 7407) => LocalisedText {
            en: "Stormbite",
            ja: "ストームバイト",
            de: "Sturmbiss",
            fr: "Morsure de la tempête",
        },
        (56, 7408) => LocalisedText {
            en: "Nature's Minne",
            ja: "地神のミンネ",
            de: "Nophicas Minne",
            fr: "Minne de la nature",
        },
        (56, 7409) => LocalisedText {
            en: "Refulgent Arrow",
            ja: "リフルジェントアロー",
            de: "Glänzender Pfeil",
            fr: "Flèche rayonnante",
        },
        (56, 7410) => LocalisedText {
            en: "Heat Blast",
            ja: "ヒートブラスト",
            de: "Hitzeschuss",
            fr: "Pyro-explosion",
        },
        (56, 7411) => LocalisedText {
            en: "Heated Split Shot",
            ja: "ヒートスプリットショット",
            de: "Höllenfeuer frei",
            fr: "Tir scindé chauffé",
        },
        (56, 7412) => LocalisedText {
            en: "Heated Slug Shot",
            ja: "ヒートスラッグショット",
            de: "Höllen-Flintenlaufgeschoss",
            fr: "Tir de balle chauffé",
        },
        (56, 7413) => LocalisedText {
            en: "Heated Clean Shot",
            ja: "ヒートクリーンショット",
            de: "Sauberer Höllenschuss",
            fr: "Tir net chauffé",
        },
        (56, 7414) => LocalisedText {
            en: "Barrel Stabilizer",
            ja: "バレルヒーター",
            de: "Laufstabilisator",
            fr: "Stabilisateur de canon",
        },
        (56, 7415) => LocalisedText {
            en: "Rook Overdrive",
            ja: "オーバードライブ・ルーク",
            de: "TURM-Überhitzung",
            fr: "Surmultiplication de Tour",
        },
        (56, 7416) => LocalisedText {
            en: "Rook Overload",
            ja: "オーバーロード・ルーク",
            de: "TURM-Überladung",
            fr: "Surcharge de Tour",
        },
        (56, 7418) => LocalisedText {
            en: "Flamethrower",
            ja: "フレイムスロアー",
            de: "Flammenwerfer",
            fr: "Lance-flammes",
        },
        (56, 7419) => LocalisedText {
            en: "Between the Lines",
            ja: "ラインズステップ",
            de: "Leitlinie",
            fr: "Entre les lignes",
        },
        (56, 7420) => LocalisedText {
            en: "Thunder IV",
            ja: "サンダジャ",
            de: "Blitzka",
            fr: "Giga Foudre",
        },
        (56, 7421) => LocalisedText {
            en: "Triplecast",
            ja: "三連魔",
            de: "Tripelzauber",
            fr: "Triple sort",
        },
        (56, 7422) => LocalisedText {
            en: "Foul",
            ja: "ファウル",
            de: "Moder",
            fr: "Infect",
        },
        (56, 7426) => LocalisedText {
            en: "Ruin IV",
            ja: "ルインジャ",
            de: "Ruinka",
            fr: "Giga Ruine",
        },
        (56, 7427) => LocalisedText {
            en: "Summon Bahamut",
            ja: "サモン・バハムート",
            de: "Bahamut-Beschwörung",
            fr: "Invocation Bahamut",
        },
        (56, 7428) => LocalisedText {
            en: "Wyrmwave",
            ja: "ウィルムウェーブ",
            de: "Drachenwelle",
            fr: "Onde draconique",
        },
        (56, 7429) => LocalisedText {
            en: "Enkindle Bahamut",
            ja: "エンキンドル・バハムート",
            de: "Bahamut-Entflammung",
            fr: "Galvanisation Bahamut",
        },
        (56, 7430) => LocalisedText {
            en: "Thin Air",
            ja: "シンエアー",
            de: "Ex Machina",
            fr: "Sponte",
        },
        (56, 7431) => LocalisedText {
            en: "Stone IV",
            ja: "ストンジャ",
            de: "Steinka",
            fr: "Giga Terre",
        },
        (56, 7432) => LocalisedText {
            en: "Divine Benison",
            ja: "ディヴァインベニゾン",
            de: "Göttlicher Segen",
            fr: "Faveur divine",
        },
        (56, 7433) => LocalisedText {
            en: "Plenary Indulgence",
            ja: "インドゥルゲンティア",
            de: "Vollkommener Ablass",
            fr: "Indulgence plénière",
        },
        (56, 7434) => LocalisedText {
            en: "Excogitation",
            ja: "深謀遠慮の策",
            de: "Weise Voraussicht",
            fr: "Traité de l'excogitation",
        },
        (56, 7435) => LocalisedText {
            en: "Broil II",
            ja: "魔炎法",
            de: "Hitzige Bravade",
            fr: "Loi de l'ébullition II",
        },
        (56, 7436) => LocalisedText {
            en: "Chain Stratagem",
            ja: "連環計",
            de: "Kritische Strategie",
            fr: "Stratagèmes entrelacés",
        },
        (56, 7437) => LocalisedText {
            en: "Aetherpact",
            ja: "エーテルパクト",
            de: "Ätherpakt",
            fr: "Pacte d'éther",
        },
        (56, 7438) => LocalisedText {
            en: "Fey Union",
            ja: "フェイユニオン",
            de: "Feenbund",
            fr: "Union féerique",
        },
        (56, 7439) => LocalisedText {
            en: "Earthly Star",
            ja: "アーサリースター",
            de: "Irdischer Stern",
            fr: "Étoile terrestre",
        },
        (56, 7440) => LocalisedText {
            en: "Stellar Burst",
            ja: "ステラバースト",
            de: "Stellarer Knall",
            fr: "Éclatement stellaire",
        },
        (56, 7441) => LocalisedText {
            en: "Stellar Explosion",
            ja: "ステラエクスプロージョン",
            de: "Stellare Explosion",
            fr: "Explosion stellaire",
        },
        (56, 7442) => LocalisedText {
            en: "Malefic III",
            ja: "マレフィガ",
            de: "Spannungsaspekt III",
            fr: "Malfaisance III",
        },
        (56, 7443) => LocalisedText {
            en: "Minor Arcana",
            ja: "マイナーアルカナ",
            de: "Kleine Arkana",
            fr: "Arcane mineur",
        },
        (56, 7444) => LocalisedText {
            en: "Lord of Crowns",
            ja: "クラウンロード",
            de: "König der Kronen",
            fr: "Roi des couronnes",
        },
        (56, 7445) => LocalisedText {
            en: "Lady of Crowns",
            ja: "クラウンレディ",
            de: "Königin der Kronen",
            fr: "Reine des couronnes",
        },
        (56, 7447) => LocalisedText {
            en: "Thunder II",
            ja: "サンダラ",
            de: "Blitzra",
            fr: "Extra Foudre",
        },
        (56, 7449) => LocalisedText {
            en: "Akh Morn",
            ja: "アク・モーン",
            de: "Akh Morn",
            fr: "Akh Morn",
        },
        (56, 7477) => LocalisedText {
            en: "Hakaze",
            ja: "刃風",
            de: "Hakaze",
            fr: "Hakaze",
        },
        (56, 7478) => LocalisedText {
            en: "Jinpu",
            ja: "陣風",
            de: "Jinpu",
            fr: "Jinpû",
        },
        (56, 7479) => LocalisedText {
            en: "Shifu",
            ja: "士風",
            de: "Shifu",
            fr: "Shifû",
        },
        (56, 7480) => LocalisedText {
            en: "Yukikaze",
            ja: "雪風",
            de: "Yukikaze",
            fr: "Yukikaze",
        },
        (56, 7481) => LocalisedText {
            en: "Gekko",
            ja: "月光",
            de: "Gekko",
            fr: "Gekkô",
        },
        (56, 7482) => LocalisedText {
            en: "Kasha",
            ja: "花車",
            de: "Kasha",
            fr: "Kasha",
        },
        (56, 7483) => LocalisedText {
            en: "Fuga",
            ja: "風雅",
            de: "Fuga",
            fr: "Fûga",
        },
        (56, 7484) => LocalisedText {
            en: "Mangetsu",
            ja: "満月",
            de: "Mangetsu",
            fr: "Mangetsu",
        },
        (56, 7485) => LocalisedText {
            en: "Oka",
            ja: "桜花",
            de: "Oka",
            fr: "Ôka",
        },
        (56, 7486) => LocalisedText {
            en: "Enpi",
            ja: "燕飛",
            de: "Enpi",
            fr: "Enpi",
        },
        (56, 7487) => LocalisedText {
            en: "Midare Setsugekka",
            ja: "乱れ雪月花",
            de: "Midare Setsugekka",
            fr: "Midare Setsugekka",
        },
        (56, 7488) => LocalisedText {
            en: "Tenka Goken",
            ja: "天下五剣",
            de: "Tenka Goken",
            fr: "Tenka Goken",
        },
        (56, 7489) => LocalisedText {
            en: "Higanbana",
            ja: "彼岸花",
            de: "Higanbana",
            fr: "Higanbana",
        },
        (56, 7490) => LocalisedText {
            en: "Hissatsu: Shinten",
            ja: "必殺剣・震天",
            de: "Hissatsu: Shinten",
            fr: "Hissatsu : Shinten",
        },
        (56, 7491) => LocalisedText {
            en: "Hissatsu: Kyuten",
            ja: "必殺剣・九天",
            de: "Hissatsu: Kyuten",
            fr: "Hissatsu : Kyûten",
        },
        (56, 7492) => LocalisedText {
            en: "Hissatsu: Gyoten",
            ja: "必殺剣・暁天",
            de: "Hissatsu: Gyoten",
            fr: "Hissatsu : Gyôten",
        },
        (56, 7493) => LocalisedText {
            en: "Hissatsu: Yaten",
            ja: "必殺剣・夜天",
            de: "Hissatsu: Yaten",
            fr: "Hissatsu : Yaten",
        },
        (56, 7495) => LocalisedText {
            en: "Hagakure",
            ja: "葉隠",
            de: "Hagakure",
            fr: "Hagakure",
        },
        (56, 7496) => LocalisedText {
            en: "Hissatsu: Guren",
            ja: "必殺剣・紅蓮",
            de: "Hissatsu: Guren",
            fr: "Hissatsu : Guren",
        },
        (56, 7497) => LocalisedText {
            en: "Meditate",
            ja: "黙想",
            de: "Meditieren",
            fr: "Mokusô",
        },
        (56, 7498) => LocalisedText {
            en: "Third Eye",
            ja: "心眼",
            de: "Drittes Auge",
            fr: "Troisième œil",
        },
        (56, 7499) => LocalisedText {
            en: "Meikyo Shisui",
            ja: "明鏡止水",
            de: "Meikyo Shisui",
            fr: "Meikyô Shisui",
        },
        (56, 7503) => LocalisedText {
            en: "Jolt",
            ja: "ジョルト",
            de: "Stoß",
            fr: "À-coup",
        },
        (56, 7504) => LocalisedText {
            en: "Riposte",
            ja: "リポスト",
            de: "Riposte",
            fr: "Riposte",
        },
        (56, 7505) => LocalisedText {
            en: "Verthunder",
            ja: "ヴァルサンダー",
            de: "Erzblitz",
            fr: "VerFoudre",
        },
        (56, 7506) => LocalisedText {
            en: "Corps-a-corps",
            ja: "コル・ア・コル",
            de: "Corps-a-corps",
            fr: "Corps-à-corps",
        },
        (56, 7507) => LocalisedText {
            en: "Veraero",
            ja: "ヴァルエアロ",
            de: "Erzwind",
            fr: "VerVent",
        },
        (56, 7509) => LocalisedText {
            en: "Scatter",
            ja: "スキャッター",
            de: "Zerstreuen",
            fr: "Éparpillement",
        },
        (56, 7510) => LocalisedText {
            en: "Verfire",
            ja: "ヴァルファイア",
            de: "Erzfeuer",
            fr: "VerFeu",
        },
        (56, 7511) => LocalisedText {
            en: "Verstone",
            ja: "ヴァルストーン",
            de: "Erzstein",
            fr: "VerTerre",
        },
        (56, 7512) => LocalisedText {
            en: "Zwerchhau",
            ja: "ツヴェルクハウ",
            de: "Zwerchhau",
            fr: "Zwerchhau",
        },
        (56, 7513) => LocalisedText {
            en: "Moulinet",
            ja: "ムーリネ",
            de: "Moulinet",
            fr: "Moulinet",
        },
        (56, 7514) => LocalisedText {
            en: "Vercure",
            ja: "ヴァルケアル",
            de: "Erzvita",
            fr: "VerSoin",
        },
        (56, 7515) => LocalisedText {
            en: "Displacement",
            ja: "デプラスマン",
            de: "Körperparade",
            fr: "Déplacement",
        },
        (56, 7516) => LocalisedText {
            en: "Redoublement",
            ja: "ルドゥブルマン",
            de: "Redoublement",
            fr: "Redoublement",
        },
        (56, 7517) => LocalisedText {
            en: "Fleche",
            ja: "フレッシュ",
            de: "Fleche",
            fr: "Flèche",
        },
        (56, 7518) => LocalisedText {
            en: "Acceleration",
            ja: "アクセラレーション",
            de: "Präparation",
            fr: "Accélération",
        },
        (56, 7519) => LocalisedText {
            en: "Contre Sixte",
            ja: "コントルシクスト",
            de: "Contre Sixte",
            fr: "Contre de sixte",
        },
        (56, 7520) => LocalisedText {
            en: "Embolden",
            ja: "エンボルデン",
            de: "Ermutigen",
            fr: "Enhardissement",
        },
        (56, 7521) => LocalisedText {
            en: "Manafication",
            ja: "マナフィケーション",
            de: "Manafizierung",
            fr: "Manafication",
        },
        (56, 7523) => LocalisedText {
            en: "Verraise",
            ja: "ヴァルレイズ",
            de: "Erz-Wiederbeleben",
            fr: "VerVie",
        },
        (56, 7524) => LocalisedText {
            en: "Jolt II",
            ja: "ジョルラ",
            de: "Stoß II",
            fr: "Extra À-coup",
        },
        (56, 7525) => LocalisedText {
            en: "Verflare",
            ja: "ヴァルフレア",
            de: "Erzflare",
            fr: "VerBrasier",
        },
        (56, 7526) => LocalisedText {
            en: "Verholy",
            ja: "ヴァルホーリー",
            de: "Erz-Sanctus",
            fr: "VerMiracle",
        },
        (56, 7527) => LocalisedText {
            en: "Enchanted Riposte",
            ja: "エンリポスト",
            de: "Magische Riposte",
            fr: "Riposte enchantée",
        },
        (56, 7528) => LocalisedText {
            en: "Enchanted Zwerchhau",
            ja: "エンツヴェルクハウ",
            de: "Magischer Zwerchhau",
            fr: "Zwerchhau enchanté",
        },
        (56, 7529) => LocalisedText {
            en: "Enchanted Redoublement",
            ja: "エンルドゥブルマン",
            de: "Magisches Redoublement",
            fr: "Redoublement enchanté",
        },
        (56, 7530) => LocalisedText {
            en: "Enchanted Moulinet",
            ja: "エンムーリネ",
            de: "Magischer Moulinet",
            fr: "Moulinet enchanté",
        },
        (56, 7531) => LocalisedText {
            en: "Rampart",
            ja: "ランパート",
            de: "Schutzwall",
            fr: "Rempart",
        },
        (56, 7533) => LocalisedText {
            en: "Provoke",
            ja: "挑発",
            de: "Herausforderung",
            fr: "Provocation",
        },
        (56, 7535) => LocalisedText {
            en: "Reprisal",
            ja: "リプライザル",
            de: "Reflexion",
            fr: "Rétorsion",
        },
        (56, 7537) => LocalisedText {
            en: "Shirk",
            ja: "シャーク",
            de: "Geteiltes Leid",
            fr: "Dérobade",
        },
        (56, 7538) => LocalisedText {
            en: "Interject",
            ja: "インタージェクト",
            de: "Zwischenruf",
            fr: "Interjection",
        },
        (56, 7540) => LocalisedText {
            en: "Low Blow",
            ja: "ロウブロウ",
            de: "Tiefschlag",
            fr: "Coup bas",
        },
        (56, 7541) => LocalisedText {
            en: "Second Wind",
            ja: "内丹",
            de: "Chakra",
            fr: "Second souffle",
        },
        (56, 7542) => LocalisedText {
            en: "Bloodbath",
            ja: "ブラッドバス",
            de: "Blutbad",
            fr: "Bain de sang",
        },
        (56, 7546) => LocalisedText {
            en: "True North",
            ja: "トゥルーノース",
            de: "Neuausrichtung",
            fr: "Vrai Nord",
        },
        (56, 7548) => LocalisedText {
            en: "Arm's Length",
            ja: "アームズレングス",
            de: "Abtausch",
            fr: "Portée de bras",
        },
        (56, 7549) => LocalisedText {
            en: "Feint",
            ja: "牽制",
            de: "Zermürben",
            fr: "Restreinte",
        },
        (56, 7551) => LocalisedText {
            en: "Head Graze",
            ja: "ヘッドグレイズ",
            de: "Haarscharfer Schuss",
            fr: "Écorche-tête",
        },
        (56, 7553) => LocalisedText {
            en: "Foot Graze",
            ja: "フットグレイズ",
            de: "Stolperschuss",
            fr: "Écorche-pied",
        },
        (56, 7554) => LocalisedText {
            en: "Leg Graze",
            ja: "レッググレイズ",
            de: "Beinschuss",
            fr: "Écorche-jambe",
        },
        (56, 7557) => LocalisedText {
            en: "Peloton",
            ja: "プロトン",
            de: "Peloton",
            fr: "Peloton",
        },
        (56, 7559) => LocalisedText {
            en: "Surecast",
            ja: "堅実魔",
            de: "Unbeirrbarkeit",
            fr: "Stoïcisme",
        },
        (56, 7560) => LocalisedText {
            en: "Addle",
            ja: "アドル",
            de: "Stumpfsinn",
            fr: "Embrouillement",
        },
        (56, 7561) => LocalisedText {
            en: "Swiftcast",
            ja: "迅速魔",
            de: "Spontaneität",
            fr: "Magie prompte",
        },
        (56, 7562) => LocalisedText {
            en: "Lucid Dreaming",
            ja: "ルーシッドドリーム",
            de: "Klartraum",
            fr: "Rêve lucide",
        },
        (56, 7568) => LocalisedText {
            en: "Esuna",
            ja: "エスナ",
            de: "Medica",
            fr: "Guérison",
        },
        (56, 7571) => LocalisedText {
            en: "Rescue",
            ja: "救出",
            de: "Rettung",
            fr: "Secours",
        },
        (56, 7632) => LocalisedText {
            en: "Gig",
            ja: "ギギング",
            de: "Harpunieren",
            fr: "Harponnage",
        },
        (56, 7861) => LocalisedText {
            en: "Doom of the Living",
            ja: "生者必滅",
            de: "Hissatsu: Metsu",
            fr: "Destin des vivants",
        },
        (56, 7862) => LocalisedText {
            en: "Vermilion Scourge",
            ja: "ヴァーミリオンスカージ",
            de: "Zinnobergeißel",
            fr: "Fléau vermeil",
        },
        (56, 7863) => LocalisedText {
            en: "Leg Sweep",
            ja: "レッグスウィープ",
            de: "Stolperfalle",
            fr: "Fauchage de jambe",
        },
        (56, 7867) => LocalisedText {
            en: "Iaijutsu",
            ja: "居合術",
            de: "Iaijutsu",
            fr: "Iaijutsu",
        },
        (56, 7869) => LocalisedText {
            en: "Dissolve Union",
            ja: "フェイユニオン解除",
            de: "Bundauflösung",
            fr: "Défait union",
        },
        (56, 7903) => LocalisedText {
            en: "Fathom",
            ja: "ファゾム",
            de: "Ausloten",
            fr: "Sondage",
        },
        (56, 7904) => LocalisedText {
            en: "Shark Eye",
            ja: "シャークアイ",
            de: "Hai-Auge",
            fr: "Œil de requin",
        },
        (56, 7905) => LocalisedText {
            en: "Shark Eye II",
            ja: "シャークアイII",
            de: "Hai-Auge II",
            fr: "Œil de requin II",
        },
        (56, 7906) => LocalisedText {
            en: "Veteran Trade",
            ja: "ベテラントレード",
            de: "Artenvielfalt",
            fr: "Métier de vétéran",
        },
        (56, 7909) => LocalisedText {
            en: "Nature's Bounty",
            ja: "ネイチャーバウンティ",
            de: "Großes Füllhorn",
            fr: "Trésor de la nature",
        },
        (56, 7910) => LocalisedText {
            en: "Salvage",
            ja: "サルベージ",
            de: "Flaschenpost",
            fr: "Ramassage sous-marin",
        },
        (56, 7911) => LocalisedText {
            en: "Truth of Oceans",
            ja: "トゥルー・オブ・オーシャン",
            de: "Jungfräuliches Meer",
            fr: "Science des océans",
        },
        (56, 8324) => LocalisedText {
            en: "Stellar Detonation",
            ja: "ステラデトネーション",
            de: "Stellare Detonation",
            fr: "Détonation stellaire",
        },
        (56, 9629) => LocalisedText {
            en: "Undraw",
            ja: "アルカナ破棄",
            de: "Verwerfen",
            fr: "Défausse",
        },
        (56, 15989) => LocalisedText {
            en: "Cascade",
            ja: "カスケード",
            de: "Cascade",
            fr: "Cascade",
        },
        (56, 15990) => LocalisedText {
            en: "Fountain",
            ja: "ファウンテン",
            de: "Fontäne",
            fr: "Fontaine",
        },
        (56, 15991) => LocalisedText {
            en: "Reverse Cascade",
            ja: "リバースカスケード",
            de: "Invertierte Cascade",
            fr: "Cascade inversée",
        },
        (56, 15992) => LocalisedText {
            en: "Fountainfall",
            ja: "ファウンテンフォール",
            de: "Fallende Fontäne",
            fr: "Fontaine en cascade",
        },
        (56, 15993) => LocalisedText {
            en: "Windmill",
            ja: "ウィンドミル",
            de: "Windmühle",
            fr: "Moulin à vent",
        },
        (56, 15994) => LocalisedText {
            en: "Bladeshower",
            ja: "ブレードシャワー",
            de: "Klingenflut",
            fr: "Averse de lames",
        },
        (56, 15995) => LocalisedText {
            en: "Rising Windmill",
            ja: "ライジングウィンドミル",
            de: "Steigende Windmühle",
            fr: "Moulin à vent ascendant",
        },
        (56, 15996) => LocalisedText {
            en: "Bloodshower",
            ja: "ブラッドシャワー",
            de: "Blutklingen",
            fr: "Averse de sang",
        },
        (56, 15997) => LocalisedText {
            en: "Standard Step",
            ja: "スタンダードステップ",
            de: "Einfache Choreographie",
            fr: "Pas classique",
        },
        (56, 15998) => LocalisedText {
            en: "Technical Step",
            ja: "テクニカルステップ",
            de: "Komplexe Choreographie",
            fr: "Pas technique",
        },
        (56, 15999) => LocalisedText {
            en: "Emboite",
            ja: "薔薇のアンボワテ",
            de: "Pas Emboite der Rose",
            fr: "Emboîté carmin",
        },
        (56, 16000) => LocalisedText {
            en: "Entrechat",
            ja: "小鳥のアントルシャ",
            de: "Entrechat der Schwalbe",
            fr: "Entrechat outremer",
        },
        (56, 16001) => LocalisedText {
            en: "Jete",
            ja: "緑葉のジュテ",
            de: "Jete des Laubs",
            fr: "Jeté verdoyant",
        },
        (56, 16002) => LocalisedText {
            en: "Pirouette",
            ja: "金冠のピルエッテ",
            de: "Pirouette der Krone",
            fr: "Pirouette dorée",
        },
        (56, 16003) => LocalisedText {
            en: "Standard Finish",
            ja: "スタンダードフィニッシュ",
            de: "Einfaches Finale",
            fr: "Final classique",
        },
        (56, 16004) => LocalisedText {
            en: "Technical Finish",
            ja: "テクニカルフィニッシュ",
            de: "Komplexes Finale",
            fr: "Final technique",
        },
        (56, 16005) => LocalisedText {
            en: "Saber Dance",
            ja: "剣の舞い",
            de: "Schwerttanz",
            fr: "Danse du sabre",
        },
        (56, 16006) => LocalisedText {
            en: "Closed Position",
            ja: "クローズドポジション",
            de: "Tanzpartner",
            fr: "Position rapprochée",
        },
        (56, 16007) => LocalisedText {
            en: "Fan Dance",
            ja: "扇の舞い【序】",
            de: "Fächertanz I",
            fr: "Danse de l'éventail I",
        },
        (56, 16008) => LocalisedText {
            en: "Fan Dance II",
            ja: "扇の舞い【破】",
            de: "Fächertanz II",
            fr: "Danse de l'éventail II",
        },
        (56, 16009) => LocalisedText {
            en: "Fan Dance III",
            ja: "扇の舞い【急】",
            de: "Fächertanz III",
            fr: "Danse de l'éventail III",
        },
        (56, 16010) => LocalisedText {
            en: "En Avant",
            ja: "アン・アヴァン",
            de: "En Avant",
            fr: "En avant",
        },
        (56, 16011) => LocalisedText {
            en: "Devilment",
            ja: "攻めのタンゴ",
            de: "Todestango",
            fr: "Tango endiablé",
        },
        (56, 16012) => LocalisedText {
            en: "Shield Samba",
            ja: "守りのサンバ",
            de: "Schildsamba",
            fr: "Samba protectrice",
        },
        (56, 16013) => LocalisedText {
            en: "Flourish",
            ja: "フラリッシュ",
            de: "Trance",
            fr: "Apothéose",
        },
        (56, 16014) => LocalisedText {
            en: "Improvisation",
            ja: "インプロビゼーション",
            de: "Tanzimprovisation",
            fr: "Improvisation",
        },
        (56, 16015) => LocalisedText {
            en: "Curing Waltz",
            ja: "癒やしのワルツ",
            de: "Heilender Walzer",
            fr: "Valse revigorante",
        },
        (56, 16137) => LocalisedText {
            en: "Keen Edge",
            ja: "キーンエッジ",
            de: "Zerfetzer",
            fr: "Tranchant acéré",
        },
        (56, 16138) => LocalisedText {
            en: "No Mercy",
            ja: "ノー・マーシー",
            de: "Ohne Gnade",
            fr: "Sans pitié",
        },
        (56, 16139) => LocalisedText {
            en: "Brutal Shell",
            ja: "ブルータルシェル",
            de: "Brutaler Einschlag",
            fr: "Impact brutal",
        },
        (56, 16140) => LocalisedText {
            en: "Camouflage",
            ja: "カモフラージュ",
            de: "Camouflage",
            fr: "Camouflage",
        },
        (56, 16141) => LocalisedText {
            en: "Demon Slice",
            ja: "デーモンスライス",
            de: "Dämonenschnitter",
            fr: "Tranchage démoniaque",
        },
        (56, 16142) => LocalisedText {
            en: "Royal Guard",
            ja: "ロイヤルガード",
            de: "Eid der Leibgardisten",
            fr: "Garde royale",
        },
        (56, 16143) => LocalisedText {
            en: "Lightning Shot",
            ja: "サンダーバレット",
            de: "Donnerkugel",
            fr: "Barillet tonnerre",
        },
        (56, 16144) => LocalisedText {
            en: "Danger Zone",
            ja: "デンジャーゾーン",
            de: "Sprengzone",
            fr: "Zone de danger",
        },
        (56, 16145) => LocalisedText {
            en: "Solid Barrel",
            ja: "ソリッドバレル",
            de: "Stahlwalze",
            fr: "Salve tranchante",
        },
        (56, 16146) => LocalisedText {
            en: "Gnashing Fang",
            ja: "ビートファング",
            de: "Schlagfänge",
            fr: "Croc pugnace",
        },
        (56, 16147) => LocalisedText {
            en: "Savage Claw",
            ja: "サベッジクロウ",
            de: "Wilde Klauen",
            fr: "Griffe sauvage",
        },
        (56, 16148) => LocalisedText {
            en: "Nebula",
            ja: "ネビュラ",
            de: "Nebula",
            fr: "Nébuleuse",
        },
        (56, 16149) => LocalisedText {
            en: "Demon Slaughter",
            ja: "デーモンスローター",
            de: "Dämonenschlachter",
            fr: "Massacre démoniaque",
        },
        (56, 16150) => LocalisedText {
            en: "Wicked Talon",
            ja: "ウィケッドタロン",
            de: "Tückische Kralle",
            fr: "Serre vicieuse",
        },
        (56, 16151) => LocalisedText {
            en: "Aurora",
            ja: "オーロラ",
            de: "Aurora",
            fr: "Aurore",
        },
        (56, 16152) => LocalisedText {
            en: "Superbolide",
            ja: "ボーライド",
            de: "Meteoritenfall",
            fr: "Bolide",
        },
        (56, 16153) => LocalisedText {
            en: "Sonic Break",
            ja: "ソニックブレイク",
            de: "Schallbrecher",
            fr: "Rupture sonique",
        },
        (56, 16154) => LocalisedText {
            en: "Rough Divide",
            ja: "ラフディバイド",
            de: "Grobspalter",
            fr: "Lamineur",
        },
        (56, 16155) => LocalisedText {
            en: "Continuation",
            ja: "コンティニュエーション",
            de: "Triggerfolge",
            fr: "Consécution",
        },
        (56, 16156) => LocalisedText {
            en: "Jugular Rip",
            ja: "ジャギュラーリップ",
            de: "Kehlreißer",
            fr: "Arrachage de la jugulaire",
        },
        (56, 16157) => LocalisedText {
            en: "Abdomen Tear",
            ja: "アブドメンテアー",
            de: "Bauchschlitzer",
            fr: "Lacération de l'abdomen",
        },
        (56, 16158) => LocalisedText {
            en: "Eye Gouge",
            ja: "アイガウジ",
            de: "Augenstecher",
            fr: "Perforation des yeux",
        },
        (56, 16159) => LocalisedText {
            en: "Bow Shock",
            ja: "バウショック",
            de: "Schockpatrone",
            fr: "Arc de choc",
        },
        (56, 16160) => LocalisedText {
            en: "Heart of Light",
            ja: "ハート・オブ・ライト",
            de: "Wackeres Herz",
            fr: "Cœur de Lumière",
        },
        (56, 16161) => LocalisedText {
            en: "Heart of Stone",
            ja: "ハート・オブ・ストーン",
            de: "Steinernes Herz",
            fr: "Cœur de pierre",
        },
        (56, 16162) => LocalisedText {
            en: "Burst Strike",
            ja: "バーストストライク",
            de: "Explosiver Stoß",
            fr: "Frappe explosive",
        },
        (56, 16163) => LocalisedText {
            en: "Fated Circle",
            ja: "フェイテッドサークル",
            de: "Schicksalszirkel",
            fr: "Atomiseur",
        },
        (56, 16164) => LocalisedText {
            en: "Bloodfest",
            ja: "ブラッドソイル",
            de: "Blutpatronen",
            fr: "Cartouche de sang",
        },
        (56, 16165) => LocalisedText {
            en: "Blasting Zone",
            ja: "ブラスティングゾーン",
            de: "Erda-Detonation",
            fr: "Zone de destruction",
        },
        (56, 16191) => LocalisedText {
            en: "Single Standard Finish",
            ja: "シングル・スタンダードフィニッシュ",
            de: "Einfaches Finale I",
            fr: "Final classique simple",
        },
        (56, 16192) => LocalisedText {
            en: "Double Standard Finish",
            ja: "ダブル・スタンダードフィニッシュ",
            de: "Einfaches Finale II",
            fr: "Final classique doublé",
        },
        (56, 16193) => LocalisedText {
            en: "Single Technical Finish",
            ja: "シングル・テクニカルフィニッシュ",
            de: "Komplexes Finale I",
            fr: "Final technique simple",
        },
        (56, 16194) => LocalisedText {
            en: "Double Technical Finish",
            ja: "ダブル・テクニカルフィニッシュ",
            de: "Komplexes Finale II",
            fr: "Final technique doublé",
        },
        (56, 16195) => LocalisedText {
            en: "Triple Technical Finish",
            ja: "トリプル・テクニカルフィニッシュ",
            de: "Komplexes Finale III",
            fr: "Final technique triplé",
        },
        (56, 16196) => LocalisedText {
            en: "Quadruple Technical Finish",
            ja: "クワッド・テクニカルフィニッシュ",
            de: "Komplexes Finale IV",
            fr: "Final technique quadruplé",
        },
        (56, 16457) => LocalisedText {
            en: "Prominence",
            ja: "プロミネンス",
            de: "Protuberanz",
            fr: "Évidence",
        },
        (56, 16458) => LocalisedText {
            en: "Holy Circle",
            ja: "ホーリーサークル",
            de: "Heiliger Kreis",
            fr: "Cercle sacré",
        },
        (56, 16459) => LocalisedText {
            en: "Confiteor",
            ja: "コンフィテオル",
            de: "Confiteor",
            fr: "Confiteor",
        },
        (56, 16460) => LocalisedText {
            en: "Atonement",
            ja: "ロイエ",
            de: "Absolution",
            fr: "Absolution",
        },
        (56, 16461) => LocalisedText {
            en: "Intervene",
            ja: "インターヴィーン",
            de: "Einschreiten",
            fr: "Irruption brutale",
        },
        (56, 16462) => LocalisedText {
            en: "Mythril Tempest",
            ja: "ミスリルテンペスト",
            de: "Mithril-Gewitter",
            fr: "Tempête de mithril",
        },
        (56, 16463) => LocalisedText {
            en: "Chaotic Cyclone",
            ja: "カオティックサイクロン",
            de: "Chaosklinge",
            fr: "Cyclone chaotique",
        },
        (56, 16464) => LocalisedText {
            en: "Nascent Flash",
            ja: "原初の猛り",
            de: "Urflackern",
            fr: "Exaltation naissante",
        },
        (56, 16465) => LocalisedText {
            en: "Inner Chaos",
            ja: "インナーカオス",
            de: "Inneres Chaos",
            fr: "Chaos intérieur",
        },
        (56, 16466) => LocalisedText {
            en: "Flood of Darkness",
            ja: "暗黒の波動",
            de: "Finstere Flut",
            fr: "Déluge de ténèbres",
        },
        (56, 16467) => LocalisedText {
            en: "Edge of Darkness",
            ja: "暗黒の剣",
            de: "Finstere Klinge",
            fr: "Tranchant de ténèbres",
        },
        (56, 16468) => LocalisedText {
            en: "Stalwart Soul",
            ja: "ストルワートソウル",
            de: "Seelenstärke",
            fr: "Noirceur insoumise",
        },
        (56, 16469) => LocalisedText {
            en: "Flood of Shadow",
            ja: "漆黒の波動",
            de: "Schattenflut",
            fr: "Déluge d'ombre",
        },
        (56, 16470) => LocalisedText {
            en: "Edge of Shadow",
            ja: "漆黒の剣",
            de: "Schattenklinge",
            fr: "Tranchant d'ombre",
        },
        (56, 16471) => LocalisedText {
            en: "Dark Missionary",
            ja: "ダークミッショナリー",
            de: "Dunkler Bote",
            fr: "Missionnaire des Ténèbres",
        },
        (56, 16472) => LocalisedText {
            en: "Living Shadow",
            ja: "影身具現",
            de: "Schattenschemen",
            fr: "Ombre vivante",
        },
        (56, 16473) => LocalisedText {
            en: "Four-point Fury",
            ja: "四面脚",
            de: "Vierfaches Verhängnis",
            fr: "Furie cardinale",
        },
        (56, 16474) => LocalisedText {
            en: "Enlightenment",
            ja: "万象闘気圏",
            de: "Weg zur Erleuchtung",
            fr: "Illumination",
        },
        (56, 16475) => LocalisedText {
            en: "Anatman",
            ja: "無我",
            de: "Anatman",
            fr: "Anatman",
        },
        (56, 16476) => LocalisedText {
            en: "Six-sided Star",
            ja: "六合星導脚",
            de: "Sechsstern-Tritt",
            fr: "Étoile à six branches",
        },
        (56, 16477) => LocalisedText {
            en: "Coerthan Torment",
            ja: "クルザントーメント",
            de: "Coerthische Folter",
            fr: "Tourment du Coerthas",
        },
        (56, 16478) => LocalisedText {
            en: "High Jump",
            ja: "ハイジャンプ",
            de: "Hochsprung",
            fr: "Grand saut",
        },
        (56, 16479) => LocalisedText {
            en: "Raiden Thrust",
            ja: "竜眼雷電",
            de: "Überfallsstoß",
            fr: "Percée Raiden",
        },
        (56, 16480) => LocalisedText {
            en: "Stardiver",
            ja: "スターダイバー",
            de: "Sternensturz",
            fr: "Plongeon céleste",
        },
        (56, 16481) => LocalisedText {
            en: "Hissatsu: Senei",
            ja: "必殺剣・閃影",
            de: "Hissatsu: Senei",
            fr: "Hissatsu : Sen'ei",
        },
        (56, 16482) => LocalisedText {
            en: "Ikishoten",
            ja: "意気衝天",
            de: "Ikishoten",
            fr: "Ikishôten",
        },
        (56, 16483) => LocalisedText {
            en: "Tsubame-gaeshi",
            ja: "燕返し",
            de: "Tsubamegaeshi",
            fr: "Tsubamegaeshi",
        },
        (56, 16484) => LocalisedText {
            en: "Kaeshi: Higanbana",
            ja: "返し彼岸花",
            de: "Kaeshi Higanbana",
            fr: "Kaeshi Higanbana",
        },
        (56, 16485) => LocalisedText {
            en: "Kaeshi: Goken",
            ja: "返し五剣",
            de: "Kaeshi Goken",
            fr: "Kaeshi Goken",
        },
        (56, 16486) => LocalisedText {
            en: "Kaeshi: Setsugekka",
            ja: "返し雪月花",
            de: "Kaeshi Setsugekka",
            fr: "Kaeshi Setsugekka",
        },
        (56, 16487) => LocalisedText {
            en: "Shoha",
            ja: "照破",
            de: "Shoha",
            fr: "Shôha",
        },
        (56, 16488) => LocalisedText {
            en: "Hakke Mujinsatsu",
            ja: "八卦無刃殺",
            de: "Hakke Mujinsatsu",
            fr: "Hakke Mujinsatsu",
        },
        (56, 16489) => LocalisedText {
            en: "Meisui",
            ja: "命水",
            de: "Meisui",
            fr: "Meisui",
        },
        (56, 16491) => LocalisedText {
            en: "Goka Mekkyaku",
            ja: "劫火滅却の術",
            de: "Goka Mekkyaku",
            fr: "Gôka Mekkyaku",
        },
        (56, 16492) => LocalisedText {
            en: "Hyosho Ranryu",
            ja: "氷晶乱流の術",
            de: "Hyosho Ranryu",
            fr: "Hyôshô Ranryû",
        },
        (56, 16493) => LocalisedText {
            en: "Bunshin",
            ja: "分身の術",
            de: "Doppeltes Ich",
            fr: "Bunshin",
        },
        (56, 16494) => LocalisedText {
            en: "Shadowbite",
            ja: "シャドウバイト",
            de: "Schattenbiss",
            fr: "Morsure de l'ombre",
        },
        (56, 16495) => LocalisedText {
            en: "Burst Shot",
            ja: "バーストショット",
            de: "Berstender Schuss",
            fr: "Tir éclatant",
        },
        (56, 16496) => LocalisedText {
            en: "Apex Arrow",
            ja: "エイペックスアロー",
            de: "Apex-Pfeil",
            fr: "Trait alpha",
        },
        (56, 16497) => LocalisedText {
            en: "Auto Crossbow",
            ja: "オートボウガン",
            de: "Armbrust",
            fr: "Arbalète automatique",
        },
        (56, 16498) => LocalisedText {
            en: "Drill",
            ja: "ドリル",
            de: "Bohrer",
            fr: "Foreuse",
        },
        (56, 16499) => LocalisedText {
            en: "Bioblaster",
            ja: "バイオブラスト",
            de: "Bioblaster",
            fr: "Lance-toxine",
        },
        (56, 16500) => LocalisedText {
            en: "Air Anchor",
            ja: "エアアンカー",
            de: "Luftanker",
            fr: "Ancre aérienne",
        },
        (56, 16501) => LocalisedText {
            en: "Automaton Queen",
            ja: "オートマトン・クイーン",
            de: "Automaton DAME",
            fr: "Automate Reine",
        },
        (56, 16502) => LocalisedText {
            en: "Queen Overdrive",
            ja: "オーバードライブ・クイーン",
            de: "DAME-Überhitzung",
            fr: "Surmultiplication de Reine",
        },
        (56, 16503) => LocalisedText {
            en: "Pile Bunker",
            ja: "クイーン・パイルバンカー",
            de: "Pressluftramme",
            fr: "Marteau piqueur",
        },
        (56, 16504) => LocalisedText {
            en: "Arm Punch",
            ja: "クイーン・アームパンチ",
            de: "Eiserne Faust",
            fr: "Coup de poing",
        },
        (56, 16505) => LocalisedText {
            en: "Despair",
            ja: "デスペア",
            de: "Verzweiflung",
            fr: "Feu du désespoir",
        },
        (56, 16506) => LocalisedText {
            en: "Umbral Soul",
            ja: "アンブラルソウル",
            de: "Schattenseele",
            fr: "Âme ombrale",
        },
        (56, 16507) => LocalisedText {
            en: "Xenoglossy",
            ja: "ゼノグロシー",
            de: "Xenoglossie",
            fr: "Xénoglossie",
        },
        (56, 16510) => LocalisedText {
            en: "Energy Siphon",
            ja: "エナジーサイフォン",
            de: "Energiesiphon",
            fr: "Siphon d'énergie",
        },
        (56, 16511) => LocalisedText {
            en: "Outburst",
            ja: "アウトバースト",
            de: "Elementarkorona",
            fr: "Fracas élémentaire",
        },
        (56, 16514) => LocalisedText {
            en: "Fountain of Fire",
            ja: "霊泉の炎",
            de: "Quell des Feuers",
            fr: "Flamme de la vie",
        },
        (56, 16515) => LocalisedText {
            en: "Brand of Purgatory",
            ja: "煉獄の炎",
            de: "Zeichen der Läuterung",
            fr: "Tison du purgatoire",
        },
        (56, 16516) => LocalisedText {
            en: "Enkindle Phoenix",
            ja: "エンキンドル・フェニックス",
            de: "Phönix-Entflammung",
            fr: "Galvanisation Phénix",
        },
        (56, 16517) => LocalisedText {
            en: "Everlasting Flight",
            ja: "不死鳥の翼",
            de: "Schwingen der Unsterblichkeit",
            fr: "Vol éternel",
        },
        (56, 16518) => LocalisedText {
            en: "Revelation",
            ja: "リヴァレーション",
            de: "Offenbarung",
            fr: "Révélation",
        },
        (56, 16519) => LocalisedText {
            en: "Scarlet Flame",
            ja: "火焔",
            de: "Scharlachflamme",
            fr: "Flamme écarlate",
        },
        (56, 16524) => LocalisedText {
            en: "Verthunder II",
            ja: "ヴァルサンダラ",
            de: "Erzblitzra",
            fr: "Extra VerFoudre",
        },
        (56, 16525) => LocalisedText {
            en: "Veraero II",
            ja: "ヴァルエアロラ",
            de: "Erzwindra",
            fr: "Extra VerVent",
        },
        (56, 16526) => LocalisedText {
            en: "Impact",
            ja: "インパクト",
            de: "Impakt",
            fr: "Impact",
        },
        (56, 16527) => LocalisedText {
            en: "Engagement",
            ja: "アンガジェマン",
            de: "Arret-Stoß",
            fr: "Engagement",
        },
        (56, 16528) => LocalisedText {
            en: "Enchanted Reprise",
            ja: "エンルプリーズ",
            de: "Magische Reprise",
            fr: "Reprise enchantée",
        },
        (56, 16529) => LocalisedText {
            en: "Reprise",
            ja: "ルプリーズ",
            de: "Reprise",
            fr: "Reprise",
        },
        (56, 16530) => LocalisedText {
            en: "Scorch",
            ja: "スコーチ",
            de: "Versengung",
            fr: "Calcination",
        },
        (56, 16531) => LocalisedText {
            en: "Afflatus Solace",
            ja: "ハート・オブ・ソラス",
            de: "Afflatus solatii",
            fr: "Offrande de réconfort",
        },
        (56, 16532) => LocalisedText {
            en: "Dia",
            ja: "ディア",
            de: "Dia",
            fr: "Lumen",
        },
        (56, 16533) => LocalisedText {
            en: "Glare",
            ja: "グレア",
            de: "Blendung",
            fr: "Chatoiement",
        },
        (56, 16534) => LocalisedText {
            en: "Afflatus Rapture",
            ja: "ハート・オブ・ラプチャー",
            de: "Afflatus laetitiae",
            fr: "Offrande de ravissement",
        },
        (56, 16535) => LocalisedText {
            en: "Afflatus Misery",
            ja: "ハート・オブ・ミゼリ",
            de: "Afflatus miseriae",
            fr: "Offrande de misère",
        },
        (56, 16536) => LocalisedText {
            en: "Temperance",
            ja: "テンパランス",
            de: "Linderung",
            fr: "Tempérance",
        },
        (56, 16539) => LocalisedText {
            en: "Art of War",
            ja: "破陣法",
            de: "Kunst des Krieges",
            fr: "Loi de la guerre",
        },
        (56, 16540) => LocalisedText {
            en: "Biolysis",
            ja: "蠱毒法",
            de: "Causae et Curae",
            fr: "Loi de l'infection",
        },
        (56, 16541) => LocalisedText {
            en: "Broil III",
            ja: "死炎法",
            de: "Vernichtende Bravade",
            fr: "Loi de l'ébullition III",
        },
        (56, 16542) => LocalisedText {
            en: "Recitation",
            ja: "秘策",
            de: "Raffinesse",
            fr: "Récitation",
        },
        (56, 16543) => LocalisedText {
            en: "Fey Blessing",
            ja: "フェイブレッシング",
            de: "Gunst der Feen",
            fr: "Bénédiction féérique",
        },
        (56, 16545) => LocalisedText {
            en: "Summon Seraph",
            ja: "サモン・セラフィム",
            de: "Seraph-Beschwörung",
            fr: "Invocation Séraphin",
        },
        (56, 16546) => LocalisedText {
            en: "Consolation",
            ja: "コンソレイション",
            de: "Seelentrost",
            fr: "Dictame",
        },
        (56, 16548) => LocalisedText {
            en: "Seraphic Veil",
            ja: "セラフィックヴェール",
            de: "Schleier der Seraphim",
            fr: "Voile séraphique",
        },
        (56, 16550) => LocalisedText {
            en: "Angel's Whisper",
            ja: "光輝の囁き",
            de: "Hauch der Seraphim",
            fr: "Murmure des anges",
        },
        (56, 16551) => LocalisedText {
            en: "Seraphic Illumination",
            ja: "セラフィックイルミネーション",
            de: "Seraphische Illumination",
            fr: "Illumination séraphique",
        },
        (56, 16552) => LocalisedText {
            en: "Divination",
            ja: "ディヴィネーション",
            de: "Weissagung",
            fr: "Divination",
        },
        (56, 16553) => LocalisedText {
            en: "Celestial Opposition",
            ja: "星天対抗",
            de: "Opposition",
            fr: "Opposition céleste",
        },
        (56, 16554) => LocalisedText {
            en: "Combust III",
            ja: "コンバガ",
            de: "Konjunktion III",
            fr: "Conjonction supérieure III",
        },
        (56, 16555) => LocalisedText {
            en: "Malefic IV",
            ja: "マレフィジャ",
            de: "Spannungsaspekt IV",
            fr: "Malfaisance IV",
        },
        (56, 16556) => LocalisedText {
            en: "Celestial Intersection",
            ja: "星天交差",
            de: "Kongruenz",
            fr: "Rencontre céleste",
        },
        (56, 16557) => LocalisedText {
            en: "Horoscope",
            ja: "ホロスコープ",
            de: "Horoskop",
            fr: "Horoscope",
        },
        (56, 16559) => LocalisedText {
            en: "Neutral Sect",
            ja: "ニュートラルセクト",
            de: "Neutral",
            fr: "Adepte de la neutralité",
        },
        (56, 16560) => LocalisedText {
            en: "Repose",
            ja: "リポーズ",
            de: "Schlummer",
            fr: "Sérénité",
        },
        (56, 16578) => LocalisedText {
            en: "Falling Star",
            ja: "フォーリングスター",
            de: "Sternschnuppe",
            fr: "Étoile tombante",
        },
        (56, 16766) => LocalisedText {
            en: "Detonator",
            ja: "デトネーター",
            de: "Detonator",
            fr: "Détonateur",
        },
        (56, 16889) => LocalisedText {
            en: "Tactician",
            ja: "タクティシャン",
            de: "Taktiker",
            fr: "Tacticien",
        },
        (56, 17055) => LocalisedText {
            en: "Play",
            ja: "プレイ",
            de: "Ausspielen",
            fr: "Dévoilement",
        },
        (56, 17105) => LocalisedText {
            en: "Gunmetal Soul",
            ja: "ソウルガンメタル",
            de: "Cermet-Matrix",
            fr: "Âme de métal",
        },
        (56, 17106) => LocalisedText {
            en: "Crimson Lotus",
            ja: "クリムゾンロータス",
            de: "Blutroter Lotus",
            fr: "Lotus pourpre",
        },
        (56, 17206) => LocalisedText {
            en: "Roller Dash",
            ja: "クイーン・ローラーダッシュ",
            de: "Rollender Ansturm",
            fr: "Roues véloces",
        },
        (56, 17209) => LocalisedText {
            en: "Hypercharge",
            ja: "ハイパーチャージ",
            de: "Hyperladung",
            fr: "Hypercharge",
        },
        (56, 17215) => LocalisedText {
            en: "Summon Eos",
            ja: "サモン・エオス",
            de: "Eos-Beschwörung",
            fr: "Invocation Eos",
        },
        (56, 17216) => LocalisedText {
            en: "Summon Selene",
            ja: "サモン・セレネ",
            de: "Selene-Beschwörung",
            fr: "Invocation Selene",
        },
        (56, 17864) => LocalisedText {
            en: "Bio",
            ja: "バイオ",
            de: "Bio",
            fr: "Bactérie",
        },
        (56, 17865) => LocalisedText {
            en: "Bio II",
            ja: "バイオラ",
            de: "Biora",
            fr: "Extra Bactérie",
        },
        (56, 18073) => LocalisedText {
            en: "Ending",
            ja: "クローズドポジション解除",
            de: "Vom Tanzpartner lösen",
            fr: "Abandon de position rapprochée",
        },
        (56, 19004) => LocalisedText {
            en: "Innovation",
            ja: "イノベーション",
            de: "Innovation",
            fr: "Innovation",
        },
        (56, 19012) => LocalisedText {
            en: "Final Appraisal",
            ja: "最終確認",
            de: "Letzte Kontrolle",
            fr: "Dernières vérifications",
        },
        (56, 19264) => LocalisedText {
            en: "Release List",
            ja: "リリース一覧表示",
            de: "Freilassliste",
            fr: "Liste de remise à l'eau",
        },
        (56, 19297) => LocalisedText {
            en: "Veneration",
            ja: "ヴェネレーション",
            de: "Ehrfurcht",
            fr: "Vénération",
        },
        (56, 21177) => LocalisedText {
            en: "Mountaineer's Gift I",
            ja: "富鉱なる賜物I",
            de: "Ergiebige Ader I",
            fr: "Gisements fertiles I",
        },
        (56, 21178) => LocalisedText {
            en: "Pioneer's Gift I",
            ja: "肥沃なる賜物I",
            de: "Fruchtbarer Boden I",
            fr: "Sols fertiles I",
        },
        (56, 21203) => LocalisedText {
            en: "Nald'thal's Tidings",
            ja: "ナルザルの福音",
            de: "Gnade des Nald'thal",
            fr: "Évangile de Nald'thal",
        },
        (56, 21204) => LocalisedText {
            en: "Nophica's Tidings",
            ja: "ノフィカの福音",
            de: "Gnade der Nophica",
            fr: "Évangile de Nophica",
        },
        (56, 21205) => LocalisedText {
            en: "Collector's Focus",
            ja: "バリューフォーカス",
            de: "Intuitiver Fokus",
            fr: "Œil du collectionneur",
        },
        (56, 22182) => LocalisedText {
            en: "Scour",
            ja: "純化",
            de: "Lese",
            fr: "Sélection",
        },
        (56, 22183) => LocalisedText {
            en: "Brazen Prospector",
            ja: "大胆純化",
            de: "Intuitive Lese",
            fr: "Sélection instinctive",
        },
        (56, 22184) => LocalisedText {
            en: "Meticulous Prospector",
            ja: "慎重純化",
            de: "Fachkundige Lese",
            fr: "Sélection méthodique",
        },
        (56, 22185) => LocalisedText {
            en: "Scrutiny",
            ja: "集中検分",
            de: "Prüfender Blick",
            fr: "Inspection rigoureuse",
        },
        (56, 24283) => LocalisedText {
            en: "Dosis",
            ja: "ドシス",
            de: "Dosis",
            fr: "Dosis",
        },
        (56, 24284) => LocalisedText {
            en: "Diagnosis",
            ja: "ディアグノシス",
            de: "Diagnose",
            fr: "Diagnosis",
        },
        (56, 24285) => LocalisedText {
            en: "Kardia",
            ja: "カルディア",
            de: "Kardia",
            fr: "Kardia",
        },
        (56, 24286) => LocalisedText {
            en: "Prognosis",
            ja: "プログノシス",
            de: "Prognose",
            fr: "Prognosis",
        },
        (56, 24287) => LocalisedText {
            en: "Egeiro",
            ja: "エゲイロー",
            de: "Egeiro",
            fr: "Egeiro",
        },
        (56, 24288) => LocalisedText {
            en: "Physis",
            ja: "ピュシス",
            de: "Physis",
            fr: "Physis",
        },
        (56, 24289) => LocalisedText {
            en: "Phlegma",
            ja: "フレグマ",
            de: "Phlegma",
            fr: "Phlegma",
        },
        (56, 24290) => LocalisedText {
            en: "Eukrasia",
            ja: "エウクラシア",
            de: "Eukrasie",
            fr: "Eukrasia",
        },
        (56, 24291) => LocalisedText {
            en: "Eukrasian Diagnosis",
            ja: "エウクラシア・ディアグノシス",
            de: "Eukratische Diagnose",
            fr: "Diagnosis eucrasique",
        },
        (56, 24292) => LocalisedText {
            en: "Eukrasian Prognosis",
            ja: "エウクラシア・プログノシス",
            de: "Eukratische Prognose",
            fr: "Prognosis eucrasique",
        },
        (56, 24293) => LocalisedText {
            en: "Eukrasian Dosis",
            ja: "エウクラシア・ドシス",
            de: "Eukratische Dosis",
            fr: "Dosis eucrasique",
        },
        (56, 24294) => LocalisedText {
            en: "Soteria",
            ja: "ソーテリア",
            de: "Soteria",
            fr: "Soteria",
        },
        (56, 24295) => LocalisedText {
            en: "Icarus",
            ja: "イカロス",
            de: "Ikarus",
            fr: "Ikaros",
        },
        (56, 24296) => LocalisedText {
            en: "Druochole",
            ja: "ドルオコレ",
            de: "Druochole",
            fr: "Druochole",
        },
        (56, 24297) => LocalisedText {
            en: "Dyskrasia",
            ja: "ディスクラシア",
            de: "Dyskrasie",
            fr: "Dyskrasia",
        },
        (56, 24298) => LocalisedText {
            en: "Kerachole",
            ja: "ケーラコレ",
            de: "Kerachole",
            fr: "Kerachole",
        },
        (56, 24299) => LocalisedText {
            en: "Ixochole",
            ja: "イックソコレ",
            de: "Ixochole",
            fr: "Ixochole",
        },
        (56, 24300) => LocalisedText {
            en: "Zoe",
            ja: "ゾーエ",
            de: "Zoe",
            fr: "Zoe",
        },
        (56, 24301) => LocalisedText {
            en: "Pepsis",
            ja: "ペプシス",
            de: "Pepsis",
            fr: "Pepsis",
        },
        (56, 24302) => LocalisedText {
            en: "Physis II",
            ja: "ピュシスII",
            de: "Physis II",
            fr: "Physis II",
        },
        (56, 24303) => LocalisedText {
            en: "Taurochole",
            ja: "タウロコレ",
            de: "Taurochole",
            fr: "Taurochole",
        },
        (56, 24304) => LocalisedText {
            en: "Toxikon",
            ja: "トキシコン",
            de: "Toxikon",
            fr: "Toxikon",
        },
        (56, 24305) => LocalisedText {
            en: "Haima",
            ja: "ハイマ",
            de: "Haima",
            fr: "Haima",
        },
        (56, 24306) => LocalisedText {
            en: "Dosis II",
            ja: "ドシスII",
            de: "Dosis II",
            fr: "Dosis II",
        },
        (56, 24307) => LocalisedText {
            en: "Phlegma II",
            ja: "フレグマII",
            de: "Phlegma II",
            fr: "Phlegma II",
        },
        (56, 24308) => LocalisedText {
            en: "Eukrasian Dosis II",
            ja: "エウクラシア・ドシスII",
            de: "Eukratische Dosis II",
            fr: "Dosis eucrasique II",
        },
        (56, 24309) => LocalisedText {
            en: "Rhizomata",
            ja: "リゾーマタ",
            de: "Rizomata",
            fr: "Rizomata",
        },
        (56, 24310) => LocalisedText {
            en: "Holos",
            ja: "ホーリズム",
            de: "Holos",
            fr: "Holos",
        },
        (56, 24311) => LocalisedText {
            en: "Panhaima",
            ja: "パンハイマ",
            de: "Panhaima",
            fr: "Panhaima",
        },
        (56, 24312) => LocalisedText {
            en: "Dosis III",
            ja: "ドシスIII",
            de: "Dosis III",
            fr: "Dosis III",
        },
        (56, 24313) => LocalisedText {
            en: "Phlegma III",
            ja: "フレグマIII",
            de: "Phlegma III",
            fr: "Phlegma III",
        },
        (56, 24314) => LocalisedText {
            en: "Eukrasian Dosis III",
            ja: "エウクラシア・ドシスIII",
            de: "Eukratische Dosis III",
            fr: "Dosis eucrasique III",
        },
        (56, 24315) => LocalisedText {
            en: "Dyskrasia II",
            ja: "ディスクラシアII",
            de: "Dyskrasie II",
            fr: "Dyskrasia II",
        },
        (56, 24316) => LocalisedText {
            en: "Toxikon II",
            ja: "トキシコンII",
            de: "Toxikon II",
            fr: "Toxikon II",
        },
        (56, 24317) => LocalisedText {
            en: "Krasis",
            ja: "クラーシス",
            de: "Krasis",
            fr: "Krasis",
        },
        (56, 24318) => LocalisedText {
            en: "Pneuma",
            ja: "プネウマ",
            de: "Pneuma",
            fr: "Pneuma",
        },
        (56, 24373) => LocalisedText {
            en: "Slice",
            ja: "スライス",
            de: "Schlitzer",
            fr: "Tranchage",
        },
        (56, 24374) => LocalisedText {
            en: "Waxing Slice",
            ja: "ワクシングスライス",
            de: "Neulicht-Schlitzer",
            fr: "Tranchage blafard",
        },
        (56, 24375) => LocalisedText {
            en: "Infernal Slice",
            ja: "インファナルスライス",
            de: "Infernalischer Schlitzer",
            fr: "Tranchage infernal",
        },
        (56, 24376) => LocalisedText {
            en: "Spinning Scythe",
            ja: "スピニングサイズ",
            de: "Sichelwirbel",
            fr: "Faux tournoyante",
        },
        (56, 24377) => LocalisedText {
            en: "Nightmare Scythe",
            ja: "ナイトメアサイズ",
            de: "Schneidender Albtraum",
            fr: "Faux cauchemardesque",
        },
        (56, 24378) => LocalisedText {
            en: "Shadow of Death",
            ja: "シャドウ・オブ・デス",
            de: "Schatten des Todes",
            fr: "Ombre de la mort",
        },
        (56, 24379) => LocalisedText {
            en: "Whorl of Death",
            ja: "ワーラル・オブ・デス",
            de: "Sog des Todes",
            fr: "Tourbillon de la mort",
        },
        (56, 24380) => LocalisedText {
            en: "Soul Slice",
            ja: "ソウルスライス",
            de: "Seelenschlitzer",
            fr: "Tranchage animique",
        },
        (56, 24381) => LocalisedText {
            en: "Soul Scythe",
            ja: "ソウルサイズ",
            de: "Seelensense",
            fr: "Faux animique",
        },
        (56, 24382) => LocalisedText {
            en: "Gibbet",
            ja: "ジビトゥ",
            de: "Richtpfahl",
            fr: "Gibet",
        },
        (56, 24383) => LocalisedText {
            en: "Gallows",
            ja: "ギャロウズ",
            de: "Galgen",
            fr: "Potence",
        },
        (56, 24384) => LocalisedText {
            en: "Guillotine",
            ja: "ギロティン",
            de: "Guillotine",
            fr: "Guillotine",
        },
        (56, 24385) => LocalisedText {
            en: "Plentiful Harvest",
            ja: "プレンティフルハーベスト",
            de: "Große Mahd",
            fr: "Moisson abondante",
        },
        (56, 24386) => LocalisedText {
            en: "Harpe",
            ja: "ハルパー",
            de: "Harpe",
            fr: "Harpé",
        },
        (56, 24387) => LocalisedText {
            en: "Soulsow",
            ja: "ソウルソウ",
            de: "Seelensaat",
            fr: "Semence d'âmes",
        },
        (56, 24388) => LocalisedText {
            en: "Harvest Moon",
            ja: "ハーベストムーン",
            de: "Erntemond",
            fr: "Lune des moissons",
        },
        (56, 24389) => LocalisedText {
            en: "Blood Stalk",
            ja: "ストークスウェーズ",
            de: "Knochengarbe",
            fr: "Fétu ensanglanté",
        },
        (56, 24390) => LocalisedText {
            en: "Unveiled Gibbet",
            ja: "ジビトゥクロウ",
            de: "Richtergriff",
            fr: "Gibet suppliciant",
        },
        (56, 24391) => LocalisedText {
            en: "Unveiled Gallows",
            ja: "ギャロウズクロウ",
            de: "Galgengriff",
            fr: "Potence suppliciante",
        },
        (56, 24392) => LocalisedText {
            en: "Grim Swathe",
            ja: "シーフスウェーズ",
            de: "Nachtschwad",
            fr: "Andain sinistre",
        },
        (56, 24393) => LocalisedText {
            en: "Gluttony",
            ja: "グラトニー",
            de: "Völlerei",
            fr: "Gloutonnerie",
        },
        (56, 24394) => LocalisedText {
            en: "Enshroud",
            ja: "レムールシュラウド",
            de: "Lemurenschleier",
            fr: "Linceul du lémure",
        },
        (56, 24395) => LocalisedText {
            en: "Void Reaping",
            ja: "ヴォイドリーパー",
            de: "Nichtsmäher",
            fr: "Fauchage du néant",
        },
        (56, 24396) => LocalisedText {
            en: "Cross Reaping",
            ja: "クロスリーパー",
            de: "Kreuzmäher",
            fr: "Fauchage croisé",
        },
        (56, 24397) => LocalisedText {
            en: "Grim Reaping",
            ja: "グリムリーパー",
            de: "Gevatters Schnitt",
            fr: "Fauchage sinistre",
        },
        (56, 24398) => LocalisedText {
            en: "Communio",
            ja: "コムニオ",
            de: "Communio",
            fr: "Communio",
        },
        (56, 24399) => LocalisedText {
            en: "Lemure's Slice",
            ja: "レムールスライス",
            de: "Lemurenschlitzer",
            fr: "Tranchage du lémure",
        },
        (56, 24400) => LocalisedText {
            en: "Lemure's Scythe",
            ja: "レムールサイズ",
            de: "Lemurensense",
            fr: "Faux du lémure",
        },
        (56, 24401) => LocalisedText {
            en: "Hell's Ingress",
            ja: "ヘルズイングレス",
            de: "Höllischer Auftritt",
            fr: "Intrusion de l'enfer",
        },
        (56, 24402) => LocalisedText {
            en: "Hell's Egress",
            ja: "ヘルズイーグレス",
            de: "Höllischer Abgang",
            fr: "Retraite de l'enfer",
        },
        (56, 24403) => LocalisedText {
            en: "Regress",
            ja: "リターン",
            de: "Rückkehr",
            fr: "Repli",
        },
        (56, 24404) => LocalisedText {
            en: "Arcane Crest",
            ja: "アルケインクレスト",
            de: "Arkanes Wappen",
            fr: "Blason arcanique",
        },
        (56, 24405) => LocalisedText {
            en: "Arcane Circle",
            ja: "アルケインサークル",
            de: "Arkaner Kreis",
            fr: "Cercle arcanique",
        },
        (56, 24858) => LocalisedText {
            en: "the End",
            ja: "ジ・エンド",
            de: "Exitus",
            fr: "Dernier voyage",
        },
        (56, 24859) => LocalisedText {
            en: "Techne Makre",
            ja: "テクネー・マクレー",
            de: "Techne Makre",
            fr: "Techne Makre",
        },
        (56, 25589) => LocalisedText {
            en: "Mountaineer's Gift II",
            ja: "富鉱なる賜物II",
            de: "Ergiebige Ader II",
            fr: "Gisements fertiles II",
        },
        (56, 25590) => LocalisedText {
            en: "Pioneer's Gift II",
            ja: "肥沃なる賜物II",
            de: "Fruchtbarer Boden II",
            fr: "Sols fertiles II",
        },
        (56, 25746) => LocalisedText {
            en: "Holy Sheltron",
            ja: "ホーリーシェルトロン",
            de: "Heiliges Schiltron",
            fr: "Schiltron sacré",
        },
        (56, 25747) => LocalisedText {
            en: "Expiacion",
            ja: "エクスピアシオン",
            de: "Expiacion",
            fr: "Expiation",
        },
        (56, 25748) => LocalisedText {
            en: "Blade of Faith",
            ja: "ブレード・オブ・フェイス",
            de: "Klinge der Frömmigkeit",
            fr: "Lame de foi",
        },
        (56, 25749) => LocalisedText {
            en: "Blade of Truth",
            ja: "ブレード・オブ・トゥルース",
            de: "Klinge der Redlichkeit",
            fr: "Lame de vérité",
        },
        (56, 25750) => LocalisedText {
            en: "Blade of Valor",
            ja: "ブレード・オブ・ヴァラー",
            de: "Klinge der Tapferkeit",
            fr: "Lame de vaillance",
        },
        (56, 25751) => LocalisedText {
            en: "Bloodwhetting",
            ja: "原初の血気",
            de: "Urimpuls",
            fr: "Intuition fougueuse",
        },
        (56, 25752) => LocalisedText {
            en: "Orogeny",
            ja: "オロジェネシス",
            de: "Orogenese",
            fr: "Orogenèse",
        },
        (56, 25753) => LocalisedText {
            en: "Primal Rend",
            ja: "プライマルレンド",
            de: "Brachiale Gewalt",
            fr: "Griffade primitive",
        },
        (56, 25754) => LocalisedText {
            en: "Oblation",
            ja: "オブレーション",
            de: "Opfergabe",
            fr: "Oblation",
        },
        (56, 25755) => LocalisedText {
            en: "Salt and Darkness",
            ja: "ソルト・アンド・ダーク",
            de: "Salz und Schwärze",
            fr: "Sel et Ténèbres",
        },
        (56, 25757) => LocalisedText {
            en: "Shadowbringer",
            ja: "シャドウブリンガー",
            de: "Schattenbringer",
            fr: "Porteur d'ombre",
        },
        (56, 25758) => LocalisedText {
            en: "Heart of Corundum",
            ja: "ハート・オブ・コランダム",
            de: "Herz des Korunds",
            fr: "Cœur de corindon",
        },
        (56, 25759) => LocalisedText {
            en: "Hypervelocity",
            ja: "ハイパーヴェロシティ",
            de: "Überschall",
            fr: "Hypervitesse",
        },
        (56, 25760) => LocalisedText {
            en: "Double Down",
            ja: "ダブルダウン",
            de: "Doppelpack",
            fr: "Acharnement",
        },
        (56, 25761) => LocalisedText {
            en: "Steel Peak",
            ja: "鉄山靠",
            de: "Stahlzinnen",
            fr: "Charge d'acier",
        },
        (56, 25762) => LocalisedText {
            en: "Thunderclap",
            ja: "抜重歩法",
            de: "Donnersprung",
            fr: "Élan fulgurant",
        },
        (56, 25763) => LocalisedText {
            en: "Howling Fist",
            ja: "空鳴拳",
            de: "Heulende Faust",
            fr: "Poing hurlant",
        },
        (56, 25764) => LocalisedText {
            en: "Masterful Blitz",
            ja: "必殺技",
            de: "Blitzkünste",
            fr: "Blitz magistral",
        },
        (56, 25765) => LocalisedText {
            en: "Celestial Revolution",
            ja: "天宙脚",
            de: "Neuordnung der Himmel",
            fr: "Révolution céleste",
        },
        (56, 25766) => LocalisedText {
            en: "Riddle of Wind",
            ja: "疾風の極意",
            de: "Stürmendes Enigma",
            fr: "Énigme du vent",
        },
        (56, 25767) => LocalisedText {
            en: "Shadow of the Destroyer",
            ja: "壊神脚",
            de: "Schatten des Zerstörers",
            fr: "Ombre du Destructeur",
        },
        (56, 25768) => LocalisedText {
            en: "Rising Phoenix",
            ja: "鳳凰の舞",
            de: "Phönixtanz",
            fr: "Danse du Phénix",
        },
        (56, 25769) => LocalisedText {
            en: "Phantom Rush",
            ja: "夢幻闘舞",
            de: "Phantomstoß",
            fr: "Rixe évanescente",
        },
        (56, 25770) => LocalisedText {
            en: "Draconian Fury",
            ja: "竜眼蒼穹",
            de: "Groll des Drachen",
            fr: "Fureur du dragon",
        },
        (56, 25771) => LocalisedText {
            en: "Heavens' Thrust",
            ja: "ヘヴンスラスト",
            de: "Himmelsstoß",
            fr: "Percée céleste",
        },
        (56, 25772) => LocalisedText {
            en: "Chaotic Spring",
            ja: "桜華繚乱",
            de: "Chaotischer Sturm",
            fr: "Floraison chaotique",
        },
        (56, 25773) => LocalisedText {
            en: "Wyrmwind Thrust",
            ja: "天竜点睛",
            de: "Zwillingswyrm",
            fr: "Percée des dragons anciens",
        },
        (56, 25774) => LocalisedText {
            en: "Phantom Kamaitachi",
            ja: "残影鎌鼬",
            de: "Phantom-Kamaitachi",
            fr: "Kamaitachi fantôme",
        },
        (56, 25776) => LocalisedText {
            en: "Hollow Nozuchi",
            ja: "幻影野槌",
            de: "Nozuchi-Schemen",
            fr: "Nozuchi spectral",
        },
        (56, 25777) => LocalisedText {
            en: "Forked Raiju",
            ja: "月影雷獣爪",
            de: "Tanzendes Raiju",
            fr: "Griffe de Raijû",
        },
        (56, 25778) => LocalisedText {
            en: "Fleeting Raiju",
            ja: "月影雷獣牙",
            de: "Brüllendes Raiju",
            fr: "Croc de Raijû",
        },
        (56, 25779) => LocalisedText {
            en: "Shoha II",
            ja: "無明照破",
            de: "Mumyo Shoha",
            fr: "Mumyô Shôha",
        },
        (56, 25780) => LocalisedText {
            en: "Fuko",
            ja: "風光",
            de: "Fuko",
            fr: "Fûkô",
        },
        (56, 25781) => LocalisedText {
            en: "Ogi Namikiri",
            ja: "奥義波切",
            de: "Ogi Namikiri",
            fr: "Ôgi : Namikiri",
        },
        (56, 25782) => LocalisedText {
            en: "Kaeshi: Namikiri",
            ja: "返し波切",
            de: "Kaeshi Namikiri",
            fr: "Kaeshi Namikiri",
        },
        (56, 25783) => LocalisedText {
            en: "Ladonsbite",
            ja: "ラドンバイト",
            de: "Ladonsbiss",
            fr: "Morsure de Ladon",
        },
        (56, 25784) => LocalisedText {
            en: "Blast Arrow",
            ja: "ブラストアロー",
            de: "Stoßpfeil",
            fr: "Flèche explosive",
        },
        (56, 25785) => LocalisedText {
            en: "Radiant Finale",
            ja: "光神のフィナーレ",
            de: "Luminöses Finale",
            fr: "Final radieux",
        },
        (56, 25786) => LocalisedText {
            en: "Scattergun",
            ja: "スキャッターガン",
            de: "Schrotflinte",
            fr: "Fusil à pompe",
        },
        (56, 25787) => LocalisedText {
            en: "Crowned Collider",
            ja: "クイーン・コレダー",
            de: "DAME-Dynamitron",
            fr: "Collisionneur royal",
        },
        (56, 25788) => LocalisedText {
            en: "Chain Saw",
            ja: "回転のこぎり",
            de: "Kreissäge",
            fr: "Tronçonneuses rotatives",
        },
        (56, 25789) => LocalisedText {
            en: "Improvised Finish",
            ja: "インプロビゼーション・フィニッシュ",
            de: "Improvisiertes Finale",
            fr: "Final improvisé",
        },
        (56, 25790) => LocalisedText {
            en: "Tillana",
            ja: "ティラナ",
            de: "Tillana",
            fr: "Tillana",
        },
        (56, 25791) => LocalisedText {
            en: "Fan Dance IV",
            ja: "扇の舞い【終】",
            de: "Fächertanz IV",
            fr: "Danse de l'éventail IV",
        },
        (56, 25792) => LocalisedText {
            en: "Starfall Dance",
            ja: "流星の舞い",
            de: "Sternenfalltanz",
            fr: "Danse de la pluie étoilée",
        },
        (56, 25793) => LocalisedText {
            en: "Blizzard II",
            ja: "ブリザラ",
            de: "Eisra",
            fr: "Extra Glace",
        },
        (56, 25794) => LocalisedText {
            en: "High Fire II",
            ja: "ハイファイラ",
            de: "Hohes Feura",
            fr: "Extra Feu majeur",
        },
        (56, 25795) => LocalisedText {
            en: "High Blizzard II",
            ja: "ハイブリザラ",
            de: "Hohes Eisra",
            fr: "Extra Glace majeure",
        },
        (56, 25796) => LocalisedText {
            en: "Amplifier",
            ja: "アンプリファイア",
            de: "Verstärker",
            fr: "Amplificateur",
        },
        (56, 25797) => LocalisedText {
            en: "Paradox",
            ja: "パラドックス",
            de: "Paradox",
            fr: "Paradoxe",
        },
        (56, 25798) => LocalisedText {
            en: "Summon Carbuncle",
            ja: "サモン・カーバンクル",
            de: "Karfunkel-Beschwörung",
            fr: "Invocation Carbuncle",
        },
        (56, 25799) => LocalisedText {
            en: "Radiant Aegis",
            ja: "守りの光",
            de: "Schimmerschild",
            fr: "Égide rayonnante",
        },
        (56, 25800) => LocalisedText {
            en: "Aethercharge",
            ja: "エーテルチャージ",
            de: "Ätherinfusion",
            fr: "Infusion d'éther",
        },
        (56, 25801) => LocalisedText {
            en: "Searing Light",
            ja: "シアリングライト",
            de: "Gleißender Schein",
            fr: "Éclat ardent",
        },
        (56, 25802) => LocalisedText {
            en: "Summon Ruby",
            ja: "サモン・ルビー",
            de: "Rubin-Beschwörung",
            fr: "Invocation Rubis",
        },
        (56, 25803) => LocalisedText {
            en: "Summon Topaz",
            ja: "サモン・トパーズ",
            de: "Topas-Beschwörung",
            fr: "Invocation Topaze",
        },
        (56, 25804) => LocalisedText {
            en: "Summon Emerald",
            ja: "サモン・エメラルド",
            de: "Smaragd-Beschwörung",
            fr: "Invocation Émeraude",
        },
        (56, 25805) => LocalisedText {
            en: "Summon Ifrit",
            ja: "サモン・イフリート",
            de: "Ifrit-Beschwörung",
            fr: "Invocation Ifrit",
        },
        (56, 25806) => LocalisedText {
            en: "Summon Titan",
            ja: "サモン・タイタン",
            de: "Titan-Beschwörung",
            fr: "Invocation Titan",
        },
        (56, 25807) => LocalisedText {
            en: "Summon Garuda",
            ja: "サモン・ガルーダ",
            de: "Garuda-Beschwörung",
            fr: "Invocation Garuda",
        },
        (56, 25808) => LocalisedText {
            en: "Ruby Ruin",
            ja: "ルビー・ルイン",
            de: "Rubin-Ruin",
            fr: "Ruine rubis",
        },
        (56, 25809) => LocalisedText {
            en: "Topaz Ruin",
            ja: "トパーズ・ルイン",
            de: "Topas-Ruin",
            fr: "Ruine topaze",
        },
        (56, 25810) => LocalisedText {
            en: "Emerald Ruin",
            ja: "エメラルド・ルイン",
            de: "Smaragd-Ruin",
            fr: "Ruine émeraude",
        },
        (56, 25811) => LocalisedText {
            en: "Ruby Ruin II",
            ja: "ルビー・ルインラ",
            de: "Rubin-Ruinra",
            fr: "Extra Ruine rubis",
        },
        (56, 25812) => LocalisedText {
            en: "Topaz Ruin II",
            ja: "トパーズ・ルインラ",
            de: "Topas-Ruinra",
            fr: "Extra Ruine topaze",
        },
        (56, 25813) => LocalisedText {
            en: "Emerald Ruin II",
            ja: "エメラルド・ルインラ",
            de: "Smaragd-Ruinra",
            fr: "Extra Ruine émeraude",
        },
        (56, 25814) => LocalisedText {
            en: "Ruby Outburst",
            ja: "ルビー・アウトバースト",
            de: "Rubin-Korona",
            fr: "Fracas rubis",
        },
        (56, 25815) => LocalisedText {
            en: "Topaz Outburst",
            ja: "トパーズ・アウトバースト",
            de: "Topas-Korona",
            fr: "Fracas topaze",
        },
        (56, 25816) => LocalisedText {
            en: "Emerald Outburst",
            ja: "エメラルド・アウトバースト",
            de: "Smaragd-Korona",
            fr: "Fracas émeraude",
        },
        (56, 25817) => LocalisedText {
            en: "Ruby Ruin III",
            ja: "ルビー・ルインガ",
            de: "Rubin-Ruinga",
            fr: "Méga Ruine rubis",
        },
        (56, 25818) => LocalisedText {
            en: "Topaz Ruin III",
            ja: "トパーズ・ルインガ",
            de: "Topas-Ruinga",
            fr: "Méga Ruine topaze",
        },
        (56, 25819) => LocalisedText {
            en: "Emerald Ruin III",
            ja: "エメラルド・ルインガ",
            de: "Smaragd-Ruinga",
            fr: "Méga Ruine émeraude",
        },
        (56, 25820) => LocalisedText {
            en: "Astral Impulse",
            ja: "アストラルインパルス",
            de: "Lichtimpuls",
            fr: "Pulsar astral",
        },
        (56, 25821) => LocalisedText {
            en: "Astral Flare",
            ja: "アストラルフレア",
            de: "Lichtflare",
            fr: "Brasier astral",
        },
        (56, 25822) => LocalisedText {
            en: "Astral Flow",
            ja: "アストラルフロウ",
            de: "Lichtstrom",
            fr: "Flux astral",
        },
        (56, 25823) => LocalisedText {
            en: "Ruby Rite",
            ja: "ルビー・リチュアル",
            de: "Rubin-Ritus",
            fr: "Rite de rubis",
        },
        (56, 25824) => LocalisedText {
            en: "Topaz Rite",
            ja: "トパーズ・リチュアル",
            de: "Topas-Ritus",
            fr: "Rite de topaze",
        },
        (56, 25825) => LocalisedText {
            en: "Emerald Rite",
            ja: "エメラルド・リチュアル",
            de: "Smaragd-Ritus",
            fr: "Rite d'émeraude",
        },
        (56, 25826) => LocalisedText {
            en: "Tri-disaster",
            ja: "トライディザスター",
            de: "Trisaster",
            fr: "Tri-désastre",
        },
        (56, 25827) => LocalisedText {
            en: "Ruby Disaster",
            ja: "ルビー・ディザスター",
            de: "Rubin-Desaster",
            fr: "Désastre rubis",
        },
        (56, 25828) => LocalisedText {
            en: "Topaz Disaster",
            ja: "トパーズ・ディザスター",
            de: "Topas-Desaster",
            fr: "Désastre topaze",
        },
        (56, 25829) => LocalisedText {
            en: "Emerald Disaster",
            ja: "エメラルド・ディザスター",
            de: "Smaragd-Desaster",
            fr: "Désastre émeraude",
        },
        (56, 25830) => LocalisedText {
            en: "Rekindle",
            ja: "再生の炎",
            de: "Wiederaufleben",
            fr: "Ravivement",
        },
        (56, 25831) => LocalisedText {
            en: "Summon Phoenix",
            ja: "サモン・フェニックス",
            de: "Phönix-Beschwörung",
            fr: "Invocation Phénix",
        },
        (56, 25832) => LocalisedText {
            en: "Ruby Catastrophe",
            ja: "ルビー・カタストロフィ",
            de: "Rubin-Verderben",
            fr: "Cataclysme de rubis",
        },
        (56, 25833) => LocalisedText {
            en: "Topaz Catastrophe",
            ja: "トパーズ・カタストロフィ",
            de: "Topas-Verderben",
            fr: "Cataclysme de topaze",
        },
        (56, 25834) => LocalisedText {
            en: "Emerald Catastrophe",
            ja: "エメラルド・カタストロフィ",
            de: "Smaragd-Verderben",
            fr: "Cataclysme d'émeraude",
        },
        (56, 25835) => LocalisedText {
            en: "Crimson Cyclone",
            ja: "クリムゾンサイクロン",
            de: "Zinnober-Zyklon",
            fr: "Cyclone écarlate",
        },
        (56, 25836) => LocalisedText {
            en: "Mountain Buster",
            ja: "マウンテンバスター",
            de: "Bergsprenger",
            fr: "Casse-montagnes",
        },
        (56, 25837) => LocalisedText {
            en: "Slipstream",
            ja: "スリップストリーム",
            de: "Wirbelströmung",
            fr: "Sillage",
        },
        (56, 25838) => LocalisedText {
            en: "Summon Ifrit II",
            ja: "サモン・イフリートII",
            de: "Rubin-Ifrit-Beschwörung",
            fr: "Invocation Ifrit II",
        },
        (56, 25839) => LocalisedText {
            en: "Summon Titan II",
            ja: "サモン・タイタンII",
            de: "Topas-Titan-Beschwörung",
            fr: "Invocation Titan II",
        },
        (56, 25840) => LocalisedText {
            en: "Summon Garuda II",
            ja: "サモン・ガルーダII",
            de: "Smaragd-Garuda-Beschwörung",
            fr: "Invocation Garuda II",
        },
        (56, 25843) => LocalisedText {
            en: "Glittering Ruby",
            ja: "ルビーの煌き",
            de: "Rubinfunkeln",
            fr: "Scintillement de rubis",
        },
        (56, 25844) => LocalisedText {
            en: "Glittering Topaz",
            ja: "トパーズの煌き",
            de: "Topasfunkeln",
            fr: "Scintillement de topaze",
        },
        (56, 25845) => LocalisedText {
            en: "Glittering Emerald",
            ja: "エメラルドの煌き",
            de: "Smaragdfunkeln",
            fr: "Scintillement d'émeraude",
        },
        (56, 25846) => LocalisedText {
            en: "Burning Strike",
            ja: "バーニングストライク",
            de: "Brennender Schlag",
            fr: "Frappe ardente",
        },
        (56, 25847) => LocalisedText {
            en: "Rock Buster",
            ja: "ロックバスター",
            de: "Steinsprenger",
            fr: "Casse-roc",
        },
        (56, 25848) => LocalisedText {
            en: "Aerial Slash",
            ja: "エリアルスラッシュ",
            de: "Lufthieb",
            fr: "Taillade aérienne",
        },
        (56, 25849) => LocalisedText {
            en: "Inferno",
            ja: "地獄の火炎",
            de: "Inferno",
            fr: "Flammes de l'enfer",
        },
        (56, 25850) => LocalisedText {
            en: "Earthen Fury",
            ja: "大地の怒り",
            de: "Gaias Zorn",
            fr: "Fureur tellurique",
        },
        (56, 25851) => LocalisedText {
            en: "Aerial Blast",
            ja: "エリアルブラスト",
            de: "Windschlag",
            fr: "Rafale aérienne",
        },
        (56, 25855) => LocalisedText {
            en: "Verthunder III",
            ja: "ヴァルサンダガ",
            de: "Erzblitzga",
            fr: "Méga VerFoudre",
        },
        (56, 25856) => LocalisedText {
            en: "Veraero III",
            ja: "ヴァルエアロガ",
            de: "Erzwindga",
            fr: "Méga VerVent",
        },
        (56, 25857) => LocalisedText {
            en: "Magick Barrier",
            ja: "バマジク",
            de: "Magiebarriere",
            fr: "Barrière anti-magie",
        },
        (56, 25858) => LocalisedText {
            en: "Resolution",
            ja: "レゾリューション",
            de: "Entschlossenheit",
            fr: "Résolution",
        },
        (56, 25859) => LocalisedText {
            en: "Glare III",
            ja: "グレアガ",
            de: "Blendga",
            fr: "Méga Chatoiement",
        },
        (56, 25860) => LocalisedText {
            en: "Holy III",
            ja: "ホーリガ",
            de: "Sanctga",
            fr: "Méga Miracle",
        },
        (56, 25861) => LocalisedText {
            en: "Aquaveil",
            ja: "アクアヴェール",
            de: "Wasserschleier",
            fr: "Aquavoile",
        },
        (56, 25862) => LocalisedText {
            en: "Liturgy of the Bell",
            ja: "リタージー・オブ・ベル",
            de: "Glockenspiel",
            fr: "Tintinnabule",
        },
        (56, 25865) => LocalisedText {
            en: "Broil IV",
            ja: "極炎法",
            de: "Rabiate Bravade",
            fr: "Loi de l'ébullition IV",
        },
        (56, 25866) => LocalisedText {
            en: "Art of War II",
            ja: "裂陣法",
            de: "Kunst des Krieges II",
            fr: "Loi de la guerre II",
        },
        (56, 25867) => LocalisedText {
            en: "Protraction",
            ja: "生命回生法",
            de: "Protraktion",
            fr: "Loi de la protraction",
        },
        (56, 25868) => LocalisedText {
            en: "Expedient",
            ja: "疾風怒濤の計",
            de: "Sturm und Drang",
            fr: "Thèse fluidique",
        },
        (56, 25869) => LocalisedText {
            en: "Crown Play",
            ja: "クラウンプレイ",
            de: "Spiel der Kronen",
            fr: "Jeu des couronnes",
        },
        (56, 25870) => LocalisedText {
            en: "Astrodyne",
            ja: "アストロダイン",
            de: "Astrodyne",
            fr: "Astrodynamie",
        },
        (56, 25871) => LocalisedText {
            en: "Fall Malefic",
            ja: "フォールマレフィク",
            de: "Aspekt im Fall",
            fr: "Malfaisance en chute",
        },
        (56, 25872) => LocalisedText {
            en: "Gravity II",
            ja: "グラビラ",
            de: "Gravitra",
            fr: "Extra Gravité",
        },
        (56, 25873) => LocalisedText {
            en: "Exaltation",
            ja: "エクザルテーション",
            de: "Exaltation",
            fr: "Exaltation",
        },
        (56, 25874) => LocalisedText {
            en: "Macrocosmos",
            ja: "マクロコスモス",
            de: "Makrokosmos",
            fr: "Macrocosme",
        },
        (56, 25875) => LocalisedText {
            en: "Microcosmos",
            ja: "ミクロコスモス",
            de: "Mikrokosmos",
            fr: "Microcosme",
        },
        (56, 25876) => LocalisedText {
            en: "Huraijin",
            ja: "風来刃",
            de: "Huraijin",
            fr: "Fûraijin",
        },
        (56, 25880) => LocalisedText {
            en: "Sleep",
            ja: "スリプル",
            de: "Schlaf",
            fr: "Sommeil",
        },
        (56, 25882) => LocalisedText {
            en: "Flint Strike",
            ja: "爆裂脚",
            de: "Zündschlag",
            fr: "Départ de flammes",
        },
        (56, 25883) => LocalisedText {
            en: "Gemshine",
            ja: "ジェムシャイン",
            de: "Juwelenschein",
            fr: "Éclat des gemmes",
        },
        (56, 25884) => LocalisedText {
            en: "Precious Brilliance",
            ja: "ジェムブリリアンス",
            de: "Edelglanz",
            fr: "Resplendissement des gemmes",
        },
        (56, 25885) => LocalisedText {
            en: "Crimson Strike",
            ja: "クリムゾンストライク",
            de: "Zinnober-Schlag",
            fr: "Assaut écarlate",
        },
        (56, 26521) => LocalisedText {
            en: "Wise to the World",
            ja: "理知興起",
            de: "Triumph des Wissens",
            fr: "Étalage des connaissances",
        },
        (56, 26804) => LocalisedText {
            en: "Thaliak's Favor",
            ja: "サリャクの恩寵",
            de: "Thaliaks Huld",
            fr: "Faveur de Thaliak",
        },
        (56, 26805) => LocalisedText {
            en: "Makeshift Bait",
            ja: "熟漁の技",
            de: "Fischfinesse",
            fr: "Pêche au vif facile",
        },
        (56, 26806) => LocalisedText {
            en: "Prize Catch",
            ja: "大物狙い",
            de: "Fetter Fang",
            fr: "Pêche au gros",
        },
        (56, 26870) => LocalisedText {
            en: "Vital Sight",
            ja: "選魚眼",
            de: "Breitseite",
            fr: "Pupilles dilatées",
        },
        (56, 26871) => LocalisedText {
            en: "Baited Breath",
            ja: "ホールドブレス",
            de: "Atemtechnik",
            fr: "Apnée",
        },
        (56, 26872) => LocalisedText {
            en: "Electric Current",
            ja: "エレクトリックカレント",
            de: "Elektrischer Strom",
            fr: "Pêche électrique",
        },
        (56, 27523) => LocalisedText {
            en: "Triple Hook",
            ja: "トリプルフッキング",
            de: "Tripelhaken",
            fr: "Ferrage triple",
        },
        (57, 100001) => LocalisedText {
            en: "Basic Synthesis",
            ja: "作業",
            de: "Bearbeiten",
            fr: "Travail de base",
        },
        (57, 100002) => LocalisedText {
            en: "Basic Touch",
            ja: "加工",
            de: "Veredelung",
            fr: "Ouvrage de base",
        },
        (57, 100003) => LocalisedText {
            en: "Master's Mend",
            ja: "マスターズメンド",
            de: "Wiederherstellung",
            fr: "Réparation de maître",
        },
        (57, 100004) => LocalisedText {
            en: "Standard Touch",
            ja: "中級加工",
            de: "Solide Veredelung",
            fr: "Ouvrage standard",
        },
        (57, 100010) => LocalisedText {
            en: "Observe",
            ja: "経過観察",
            de: "Beobachten",
            fr: "Observation",
        },
        (57, 100128) => LocalisedText {
            en: "Precise Touch",
            ja: "集中加工",
            de: "Präzise Veredelung",
            fr: "Ouvrage précis",
        },
        (57, 100177) => LocalisedText {
            en: "Finishing Touches",
            ja: "フィニッシュワーク",
            de: "Letzter Schliff",
            fr: "Touche finale",
        },
        (57, 100203) => LocalisedText {
            en: "Careful Synthesis",
            ja: "模範作業",
            de: "Sorgfältige Bearbeitung",
            fr: "Travail prudent",
        },
        (57, 100227) => LocalisedText {
            en: "Prudent Touch",
            ja: "倹約加工",
            de: "Nachhaltige Veredelung",
            fr: "Ouvrage parcimonieux",
        },
        (57, 100235) => LocalisedText {
            en: "Focused Synthesis",
            ja: "注視作業",
            de: "Aufmerksame Bearbeitung",
            fr: "Travail attentif",
        },
        (57, 100243) => LocalisedText {
            en: "Focused Touch",
            ja: "注視加工",
            de: "Aufmerksame Veredelung",
            fr: "Ouvrage attentif",
        },
        (57, 100283) => LocalisedText {
            en: "Trained Eye",
            ja: "匠の早業",
            de: "Flinke Hand",
            fr: "Main preste",
        },
        (57, 100299) => LocalisedText {
            en: "Preparatory Touch",
            ja: "下地加工",
            de: "Basisveredelung",
            fr: "Ouvrage préparatoire",
        },
        (57, 100315) => LocalisedText {
            en: "Intensive Synthesis",
            ja: "集中作業",
            de: "Fokussierte Bearbeitung",
            fr: "Travail vigilant",
        },
        (57, 100323) => LocalisedText {
            en: "Delicate Synthesis",
            ja: "精密作業",
            de: "Akribische Bearbeitung",
            fr: "Travail minutieux",
        },
        (57, 100339) => LocalisedText {
            en: "Byregot's Blessing",
            ja: "ビエルゴの祝福",
            de: "Byregots Benediktion",
            fr: "Bénédiction de Byregot",
        },
        (57, 100355) => LocalisedText {
            en: "Hasty Touch",
            ja: "ヘイスティタッチ",
            de: "Hastige Veredelung",
            fr: "Ouvrage hâtif",
        },
        (57, 100363) => LocalisedText {
            en: "Rapid Synthesis",
            ja: "突貫作業",
            de: "Schnelle Bearbeitung",
            fr: "Travail rapide",
        },
        (57, 100371) => LocalisedText {
            en: "Tricks of the Trade",
            ja: "秘訣",
            de: "Kunstgriff",
            fr: "Ficelles du métier",
        },
        (57, 100379) => LocalisedText {
            en: "Muscle Memory",
            ja: "確信",
            de: "Motorisches Gedächtnis",
            fr: "Mémoire musculaire",
        },
        (57, 100387) => LocalisedText {
            en: "Reflect",
            ja: "真価",
            de: "Einkehr",
            fr: "Véritable valeur",
        },
        (57, 100395) => LocalisedText {
            en: "Careful Observation",
            ja: "設計変更",
            de: "Planänderung",
            fr: "Changement de patron",
        },
        (57, 100403) => LocalisedText {
            en: "Groundwork",
            ja: "下地作業",
            de: "Vorarbeit",
            fr: "Travail préparatoire",
        },
        (57, 100411) => LocalisedText {
            en: "Advanced Touch",
            ja: "上級加工",
            de: "Höhere Veredelung",
            fr: "Ouvrage avancé",
        },
        (57, 100419) => LocalisedText {
            en: "Heart and Soul",
            ja: "一心不乱",
            de: "Mit Leib und Seele",
            fr: "Attention totale",
        },
        (57, 100427) => LocalisedText {
            en: "Prudent Synthesis",
            ja: "倹約作業",
            de: "Rationelle Bearbeitung",
            fr: "Travail économe",
        },
        (57, 100435) => LocalisedText {
            en: "Trained Finesse",
            ja: "匠の神業",
            de: "Götter Werk",
            fr: "Main divine",
        },
        (59, 1) => LocalisedText {
            en: "Away",
            ja: "帰還",
            de: "Lauf",
            fr: "Renvoyer",
        },
        (59, 2) => LocalisedText {
            en: "Heel",
            ja: "追従",
            de: "Bei Fuß",
            fr: "Suivre",
        },
        (59, 3) => LocalisedText {
            en: "Place",
            ja: "移動",
            de: "Dorthin",
            fr: "Se placer",
        },
        (59, 4) => LocalisedText {
            en: "Stay",
            ja: "待機",
            de: "Bleib",
            fr: "Attendre",
        },
        (59, 5) => LocalisedText {
            en: "Guard",
            ja: "インターセプト",
            de: "Schütz",
            fr: "Défendre",
        },
        (59, 6) => LocalisedText {
            en: "Steady",
            ja: "ピース",
            de: "Aus",
            fr: "Passif",
        },
        (60, 1) => LocalisedText {
            en: "Clear Skies",
            ja: "快晴",
            de: "Klar",
            fr: "Dégagé",
        },
        (60, 2) => LocalisedText {
            en: "Fair Skies",
            ja: "晴れ",
            de: "Heiter",
            fr: "Clair",
        },
        (60, 3) => LocalisedText {
            en: "Clouds",
            ja: "曇り",
            de: "Wolkig",
            fr: "Couvert",
        },
        (60, 4) => LocalisedText {
            en: "Fog",
            ja: "霧",
            de: "Neblig",
            fr: "Brouillard",
        },
        (60, 5) => LocalisedText {
            en: "Wind",
            ja: "風",
            de: "Windig",
            fr: "Vent",
        },
        (60, 6) => LocalisedText {
            en: "Gales",
            ja: "暴風",
            de: "Stürmisch",
            fr: "Vents violents",
        },
        (60, 7) => LocalisedText {
            en: "Rain",
            ja: "雨",
            de: "Regnerisch",
            fr: "Pluie",
        },
        (60, 8) => LocalisedText {
            en: "Showers",
            ja: "暴雨",
            de: "Wolkenbruch",
            fr: "Pluie torrentielle",
        },
        (60, 9) => LocalisedText {
            en: "Thunder",
            ja: "雷",
            de: "Gewittrig",
            fr: "Orages",
        },
        (60, 10) => LocalisedText {
            en: "Thunderstorms",
            ja: "雷雨",
            de: "Gewitter",
            fr: "Orages violents",
        },
        (60, 11) => LocalisedText {
            en: "Dust Storms",
            ja: "砂塵",
            de: "Staubig",
            fr: "Tempêtes de poussière",
        },
        (60, 12) => LocalisedText {
            en: "Sandstorms",
            ja: "砂嵐",
            de: "Sandsturm",
            fr: "Tempêtes de sable",
        },
        (60, 13) => LocalisedText {
            en: "Hot Spells",
            ja: "熱波",
            de: "Heiß",
            fr: "Chaud",
        },
        (60, 14) => LocalisedText {
            en: "Heat Waves",
            ja: "灼熱波",
            de: "Gluthitze",
            fr: "Torride",
        },
        (60, 15) => LocalisedText {
            en: "Snow",
            ja: "雪",
            de: "Schnee",
            fr: "Neige",
        },
        (60, 16) => LocalisedText {
            en: "Blizzards",
            ja: "吹雪",
            de: "Schneesturm",
            fr: "Blizzard",
        },
        (60, 17) => LocalisedText {
            en: "Gloom",
            ja: "妖霧",
            de: "Unheimlich",
            fr: "Nébuleux",
        },
        (60, 18) => LocalisedText {
            en: "Auroras",
            ja: "オーロラ",
            de: "Aurora",
            fr: "Aurore",
        },
        (60, 19) => LocalisedText {
            en: "Darkness",
            ja: "闇",
            de: "Dunkel",
            fr: "Ténèbres",
        },
        (60, 20) => LocalisedText {
            en: "Tension",
            ja: "闘気",
            de: "Düsternis",
            fr: "Désespérance",
        },
        (60, 22) => LocalisedText {
            en: "Storm Clouds",
            ja: "雷雲",
            de: "Wetterleuchten",
            fr: "Orageux",
        },
        (60, 23) => LocalisedText {
            en: "Rough Seas",
            ja: "時化",
            de: "Tosend",
            fr: "Houle",
        },
        (60, 25) => LocalisedText {
            en: "Louring",
            ja: "曇天",
            de: "Finster",
            fr: "Nuageux",
        },
        (60, 29) => LocalisedText {
            en: "Eruptions",
            ja: "噴煙",
            de: "Rauchschwaden",
            fr: "Éruptions",
        },
        (60, 35) => LocalisedText {
            en: "Irradiance",
            ja: "極光",
            de: "Blendend",
            fr: "Rayonnement",
        },
        (60, 36) => LocalisedText {
            en: "Core Radiation",
            ja: "龍核",
            de: "Kernstrahlung",
            fr: "Radiations du Cœur",
        },
        (60, 40) => LocalisedText {
            en: "Shelf Clouds",
            ja: "群雲",
            de: "Wolkenschleier",
            fr: "Amas nuageux",
        },
        (60, 44) => LocalisedText {
            en: "Oppression",
            ja: "神意",
            de: "Bedrückend",
            fr: "Oppression",
        },
        (60, 49) => LocalisedText {
            en: "Umbral Wind",
            ja: "霊風",
            de: "Schattenwind",
            fr: "Vent ombral",
        },
        (60, 50) => LocalisedText {
            en: "Umbral Static",
            ja: "放電",
            de: "Schattengewitter",
            fr: "Charges ombrales",
        },
        (60, 51) => LocalisedText {
            en: "Smoke",
            ja: "煙霧",
            de: "Qualm",
            fr: "Fumées",
        },
        (60, 53) => LocalisedText {
            en: "Royal Levin",
            ja: "獣雷",
            de: "Königsgewitter",
            fr: "Éclairs royaux",
        },
        (60, 54) => LocalisedText {
            en: "Hyperelectricity",
            ja: "雷波",
            de: "Hochspannung",
            fr: "Hyperélectricité",
        },
        (60, 60) => LocalisedText {
            en: "Multiplicity",
            ja: "神秘",
            de: "Mystisch",
            fr: "Multiplicité",
        },
        (60, 66) => LocalisedText {
            en: "Dragonstorms",
            ja: "邪天",
            de: "Hass",
            fr: "Tempête de haine",
        },
        (60, 69) => LocalisedText {
            en: "Concordance",
            ja: "調和",
            de: "Gerechtigkeit",
            fr: "Harmonie",
        },
        (60, 71) => LocalisedText {
            en: "Beyond Time",
            ja: "時流",
            de: "Zeitfluss",
            fr: "Hors du temps",
        },
        (60, 74) => LocalisedText {
            en: "Demonic Infinity",
            ja: "鬼気",
            de: "Dämonisch unendlich",
            fr: "Infini démoniaque",
        },
        (60, 77) => LocalisedText {
            en: "Dimensional Disruption",
            ja: "次元",
            de: "Dimensionsspaltend",
            fr: "Perturbation dimensionnelle",
        },
        (60, 80) => LocalisedText {
            en: "Revelstorms",
            ja: "嵐天",
            de: "Feiersturm",
            fr: "Tempête festive",
        },
        (60, 82) => LocalisedText {
            en: "Eternal Bliss",
            ja: "美天",
            de: "Ewigschön",
            fr: "Béatitude éternelle",
        },
        (60, 84) => LocalisedText {
            en: "Wyrmstorms",
            ja: "龍天",
            de: "Drachensturm",
            fr: "Tempête draconique",
        },
        (60, 87) => LocalisedText {
            en: "Quicklevin",
            ja: "迅雷",
            de: "Blitzsturm",
            fr: "Vif-orage",
        },
        (60, 92) => LocalisedText {
            en: "White Cyclones",
            ja: "白嵐",
            de: "Weißer Zyklon",
            fr: "Tempête blanche",
        },
        (60, 95) => LocalisedText {
            en: "Ultimania",
            ja: "幻想",
            de: "Ultimativ",
            fr: "Fantasmagorique",
        },
        (60, 97) => LocalisedText {
            en: "Moonlight",
            ja: "月夜",
            de: "Dunkle Mondnacht",
            fr: "Clair de lune",
        },
        (60, 101) => LocalisedText {
            en: "Red Moon",
            ja: "紅月下",
            de: "Roter Mond",
            fr: "Lune rouge",
        },
        (60, 102) => LocalisedText {
            en: "Scarlet",
            ja: "朱炎",
            de: "Rotlodernd",
            fr: "Vermillon",
        },
        (60, 109) => LocalisedText {
            en: "Flames",
            ja: "火炎",
            de: "Feuer",
            fr: "Flammes",
        },
        (60, 110) => LocalisedText {
            en: "Tsunamis",
            ja: "津波",
            de: "Tsunami",
            fr: "Raz-de-marée",
        },
        (60, 111) => LocalisedText {
            en: "Cyclones",
            ja: "竜巻",
            de: "Tornado",
            fr: "Tornade",
        },
        (60, 112) => LocalisedText {
            en: "Geostorms",
            ja: "地震",
            de: "Erdbeben",
            fr: "Séisme",
        },
        (60, 113) => LocalisedText {
            en: "True Blue",
            ja: "青春",
            de: "Blauschimmernd",
            fr: "Azur",
        },
        (60, 116) => LocalisedText {
            en: "Umbral Turbulence",
            ja: "乱霊流",
            de: "Astrale Turbulenzen",
            fr: "Turbulences ombrales",
        },
        (60, 118) => LocalisedText {
            en: "Everlasting Light",
            ja: "無尽光",
            de: "Ewiges Licht",
            fr: "Clarté éternelle",
        },
        (60, 120) => LocalisedText {
            en: "Termination",
            ja: "終末",
            de: "Apokalyptisch",
            fr: "Apocalypse",
        },
        (60, 122) => LocalisedText {
            en: "Dreams",
            ja: "妖夢",
            de: "Feenträume",
            fr: "Rêverie",
        },
        (60, 125) => LocalisedText {
            en: "Brilliance",
            ja: "光天",
            de: "Strahlende Glanzhaftigkeit",
            fr: "Radiance divine",
        },
        (60, 133) => LocalisedText {
            en: "Umbral Flare",
            ja: "焔霊風",
            de: "Feuerseelenwinde",
            fr: "Vent ombral de feu",
        },
        (60, 134) => LocalisedText {
            en: "Umbral Duststorm",
            ja: "土霊風",
            de: "Erdseelenwinde",
            fr: "Vent ombral de terre",
        },
        (60, 135) => LocalisedText {
            en: "Umbral Levin",
            ja: "雷霊風",
            de: "Donnerseelenwinde",
            fr: "Vent ombral de foudre",
        },
        (60, 136) => LocalisedText {
            en: "Umbral Tempest",
            ja: "嵐霊風",
            de: "Sturmseelenwinde",
            fr: "Vent ombral des tempêtes",
        },
        (60, 137) => LocalisedText {
            en: "Starshower",
            ja: "流星雨",
            de: "Sternenregen",
            fr: "Pluie d'étoiles",
        },
        (60, 138) => LocalisedText {
            en: "Delirium",
            ja: "記憶乱流",
            de: "Erinnerungsturbulenzen",
            fr: "Tempête de souvenirs",
        },
        (60, 144) => LocalisedText {
            en: "Firestorm",
            ja: "焔嵐 ",
            de: "Flammensturm",
            fr: "Tempête de feu",
        },
        (60, 145) => LocalisedText {
            en: "Spectral Current",
            ja: "幻海流 ",
            de: "Phantomströmung",
            fr: "Courant spectral",
        },
        (60, 147) => LocalisedText {
            en: "Climactic",
            ja: "決戦",
            de: "Endzeitig",
            fr: "Fatidique",
        },
        (60, 148) => LocalisedText {
            en: "Moon Dust",
            ja: "月砂塵",
            de: "Mondstaubig",
            fr: "Tempêtes de régolithe",
        },
        (60, 149) => LocalisedText {
            en: "Astromagnetic Storm",
            ja: "磁気嵐",
            de: "Magnetsturm",
            fr: "Astromagnétique",
        },
        (60, 151) => LocalisedText {
            en: "Polarization",
            ja: "星霊",
            de: "Polarisierend",
            fr: "Polarisation",
        },
        (60, 156) => LocalisedText {
            en: "Projection",
            ja: "仮想",
            de: "Imaginär",
            fr: "Virtuel",
        },
        (60, 157) => LocalisedText {
            en: "Pandæmonium",
            ja: "万魔殿",
            de: "Pandæmonium",
            fr: "Pandæmonium",
        },
        (60, 160) => LocalisedText {
            en: "Ultimatum",
            ja: "終極",
            de: "Eschatologie",
            fr: "Eschatologique",
        },
        (60, 161) => LocalisedText {
            en: "Inevitability",
            ja: "絶望",
            de: "Verzweifelnd",
            fr: "Désespoir",
        },
        (60, 162) => LocalisedText {
            en: "Transcendence",
            ja: "神域",
            de: "Götterwetter",
            fr: "Mythologique",
        },
        (61, 1) => LocalisedText {
            en: "Stance",
            ja: "納刀する/抜刀する",
            de: "Initiative",
            fr: "Dégainer/Rengainer",
        },
        (61, 2) => LocalisedText {
            en: "Character",
            ja: "マイキャラクター",
            de: "Charakter",
            fr: "Personnage",
        },
        (61, 3) => LocalisedText {
            en: "Actions & Traits",
            ja: "アクションリスト",
            de: "Kommandoliste",
            fr: "Actions et traits",
        },
        (61, 4) => LocalisedText {
            en: "Journal",
            ja: "ジャーナル",
            de: "Archiv",
            fr: "Journal",
        },
        (61, 5) => LocalisedText {
            en: "Timers",
            ja: "コンテンツ情報一覧",
            de: "Timer",
            fr: "Calendrier",
        },
        (61, 6) => LocalisedText {
            en: "Achievements",
            ja: "アチーブメントリスト",
            de: "Errungenschaften",
            fr: "Hauts faits",
        },
        (61, 7) => LocalisedText {
            en: "Gathering Log",
            ja: "採集手帳",
            de: "Sammler-Notizbuch",
            fr: "Carnet de récolte",
        },
        (61, 8) => LocalisedText {
            en: "Hunting Log",
            ja: "討伐手帳",
            de: "Bestiarium",
            fr: "Bestiaire",
        },
        (61, 9) => LocalisedText {
            en: "Crafting Log",
            ja: "製作手帳",
            de: "Handwerker-Notizbuch",
            fr: "Carnet d'artisanat",
        },
        (61, 10) => LocalisedText {
            en: "Inventory",
            ja: "所持品",
            de: "Inventar",
            fr: "Inventaire",
        },
        (61, 11) => LocalisedText {
            en: "Key Items",
            ja: "イベントアイテムリスト",
            de: "Schlüsselgegenstände",
            fr: "Objets-clefs",
        },
        (61, 12) => LocalisedText {
            en: "Party Members",
            ja: "パーティメンバー",
            de: "Kontakte",
            fr: "Équipiers",
        },
        (61, 13) => LocalisedText {
            en: "Friend List",
            ja: "フレンドリスト管理",
            de: "Freunde",
            fr: "Liste d'amis",
        },
        (61, 14) => LocalisedText {
            en: "Blacklist",
            ja: "ブラックリスト管理",
            de: "Schwarze Liste",
            fr: "Liste noire",
        },
        (61, 15) => LocalisedText {
            en: "Player Search",
            ja: "プレイヤーサーチ",
            de: "Spielersuche",
            fr: "Recherche",
        },
        (61, 16) => LocalisedText {
            en: "Map",
            ja: "マップ",
            de: "Karte",
            fr: "Carte",
        },
        (61, 17) => LocalisedText {
            en: "Emotes",
            ja: "エモートアクション",
            de: "Emotes",
            fr: "Emotes",
        },
        (61, 18) => LocalisedText {
            en: "Signs",
            ja: "ターゲットマーカー",
            de: "Markierungen",
            fr: "Signes",
        },
        (61, 19) => LocalisedText {
            en: "System Configuration",
            ja: "システムコンフィグ",
            de: "Systemkonfiguration",
            fr: "Configuration système",
        },
        (61, 20) => LocalisedText {
            en: "Keybind",
            ja: "キーバインド変更",
            de: "Tastenbelegung",
            fr: "Attribution des touches",
        },
        (61, 21) => LocalisedText {
            en: "User Macros",
            ja: "マクロ管理",
            de: "Makros",
            fr: "Macros",
        },
        (61, 22) => LocalisedText {
            en: "HUD Layout",
            ja: "HUDレイアウト変更",
            de: "Layout",
            fr: "ATH",
        },
        (61, 23) => LocalisedText {
            en: "Log Out",
            ja: "ログアウトする",
            de: "Ausloggen",
            fr: "Déconnexion",
        },
        (61, 24) => LocalisedText {
            en: "Exit Game",
            ja: "ゲームを終了する",
            de: "Spiel beenden",
            fr: "Quitter le jeu",
        },
        (61, 25) => LocalisedText {
            en: "Armoury Chest",
            ja: "アーマリーチェスト",
            de: "Arsenal",
            fr: "Arsenal",
        },
        (61, 27) => LocalisedText {
            en: "Free Company",
            ja: "フリーカンパニー",
            de: "Freie Gesellschaft",
            fr: "Compagnie libre",
        },
        (61, 28) => LocalisedText {
            en: "Linkshells",
            ja: "リンクシェル",
            de: "Kontaktkreis",
            fr: "Linkshells",
        },
        (61, 29) => LocalisedText {
            en: "Fishing Log",
            ja: "釣り手帳",
            de: "Fischer-Notizbuch",
            fr: "Carnet de pêche",
        },
        (61, 30) => LocalisedText {
            en: "Active Help",
            ja: "HowToリストを開く",
            de: "Tutorial-Hilfstexte",
            fr: "Tutoriel",
        },
        (61, 31) => LocalisedText {
            en: "Recommendations",
            ja: "おすすめコンテンツ",
            de: "Empfehlungen",
            fr: "Recommandations",
        },
        (61, 32) => LocalisedText {
            en: "Support Desk",
            ja: "サポート&ニュース",
            de: "Kundendienst / Infos",
            fr: "Services et aide",
        },
        (61, 33) => LocalisedText {
            en: "Duty Finder",
            ja: "コンテンツファインダー",
            de: "Inhaltssuche",
            fr: "Outil de mission",
        },
        (61, 34) => LocalisedText {
            en: "Character Configuration",
            ja: "キャラクターコンフィグ",
            de: "Charakterkonfiguration",
            fr: "Configuration personnage",
        },
        (61, 35) => LocalisedText {
            en: "Teleport",
            ja: "テレポ",
            de: "Teleport",
            fr: "Téléportation",
        },
        (61, 36) => LocalisedText {
            en: "Return",
            ja: "デジョン",
            de: "Rückführung",
            fr: "Rapatriement",
        },
        (61, 41) => LocalisedText {
            en: "Fish Guide",
            ja: "魚類図鑑",
            de: "Fischverzeichnis",
            fr: "Nomenclature",
        },
        (61, 42) => LocalisedText {
            en: "Companion",
            ja: "バディ",
            de: "Mitstreiter",
            fr: "Compagnon",
        },
        (61, 44) => LocalisedText {
            en: "Housing",
            ja: "ハウジング",
            de: "Unterkünfte",
            fr: "Logement",
        },
        (61, 56) => LocalisedText {
            en: "PvP Profile",
            ja: "PvPプロフィール",
            de: "PvP-Profil",
            fr: "Profil JcJ",
        },
        (61, 57) => LocalisedText {
            en: "Party Finder",
            ja: "パーティ募集",
            de: "Gruppensuche",
            fr: "Recherche d'équipe",
        },
        (61, 58) => LocalisedText {
            en: "Waymarks",
            ja: "フィールドマーカー",
            de: "Geländemarkierung",
            fr: "Marques au sol",
        },
        (61, 59) => LocalisedText {
            en: "Ready Check",
            ja: "レディチェック",
            de: "Bereitschaftsanfrage",
            fr: "Appel de préparation",
        },
        (61, 60) => LocalisedText {
            en: "Challenge Log",
            ja: "攻略手帳",
            de: "Aktuelle Agenda",
            fr: "Carnet d'objectifs",
        },
        (61, 61) => LocalisedText {
            en: "Mount Guide",
            ja: "マウントリスト",
            de: "Reittier-Verzeichnis",
            fr: "Montures",
        },
        (61, 62) => LocalisedText {
            en: "Minion Guide",
            ja: "ミニオンリスト",
            de: "Begleiter-Verzeichnis",
            fr: "Mascottes",
        },
        (61, 64) => LocalisedText {
            en: "Sightseeing Log",
            ja: "探検手帳",
            de: "Eorzea Incognita",
            fr: "Carnet d'exploration",
        },
        (61, 65) => LocalisedText {
            en: "Gold Saucer",
            ja: "ゴールドソーサー",
            de: "Gold Saucer",
            fr: "Gold Saucer",
        },
        (61, 66) => LocalisedText {
            en: "Currency",
            ja: "通貨リスト",
            de: "Vermögen",
            fr: "Devises",
        },
        (61, 67) => LocalisedText {
            en: "Aether Currents",
            ja: "風脈の泉交感情報",
            de: "Windätherquellen",
            fr: "Vents éthérés",
        },
        (61, 68) => LocalisedText {
            en: "Playguide",
            ja: "プレイガイド",
            de: "Spielanleitung",
            fr: "Guides du jeu",
        },
        (61, 69) => LocalisedText {
            en: "Orchestrion List",
            ja: "オーケストリオン譜面帳",
            de: "Orchestrion-Liste",
            fr: "Morceaux de l'orchestrion",
        },
        (61, 70) => LocalisedText {
            en: "Hall of the Novice",
            ja: "初心者の館",
            de: "Anfänger-Arena",
            fr: "Salle des novices",
        },
        (61, 72) => LocalisedText {
            en: "Raid Finder",
            ja: "レイドファインダー",
            de: "Raid-Suche",
            fr: "Outil de raid",
        },
        (61, 73) => LocalisedText {
            en: "Countdown",
            ja: "戦闘開始カウント",
            de: "Countdown",
            fr: "Compte à rebours",
        },
        (61, 74) => LocalisedText {
            en: "Contacts",
            ja: "コンタクトリスト",
            de: "Gruppenverlauf",
            fr: "Liste de contacts",
        },
        (61, 75) => LocalisedText {
            en: "Mount Speed",
            ja: "マウントスピード",
            de: "Reittiergeschwindigkeit",
            fr: "Vitesse des montures",
        },
        (61, 76) => LocalisedText {
            en: "Duty Recorder",
            ja: "コンテンツリプレイ",
            de: "Inhaltsrückblende",
            fr: "Visionnage de mission",
        },
        (61, 77) => LocalisedText {
            en: "Chocobo Saddlebag",
            ja: "チョコボかばん",
            de: "Chocobo-Satteltasche",
            fr: "Sacoche chocobo",
        },
        (61, 78) => LocalisedText {
            en: "PvP Team",
            ja: "PvPチーム",
            de: "PvP-Team",
            fr: "Équipe JcJ",
        },
        (61, 79) => LocalisedText {
            en: "Record Ready Check",
            ja: "リプレイレディチェック",
            de: "Bereitschaftsanfrage (Rückblende)",
            fr: "Appel d'enregistrement",
        },
        (61, 80) => LocalisedText {
            en: "Cross-world Linkshells",
            ja: "クロスワールドリンクシェル",
            de: "Welten-Kontaktkreis",
            fr: "Linkshells inter-Monde",
        },
        (61, 81) => LocalisedText {
            en: "Blue Magic Spellbook",
            ja: "青魔道書",
            de: "Zauberbuch der Blaumagie",
            fr: "Grimoire de magie bleue",
        },
        (61, 82) => LocalisedText {
            en: "Trust",
            ja: "フェイス",
            de: "Seite an Seite",
            fr: "Adjuration",
        },
        (61, 84) => LocalisedText {
            en: "Shared FATE",
            ja: "F.A.T.E.達成度",
            de: "FATE-Fortschritt",
            fr: "ALÉA",
        },
        (61, 85) => LocalisedText {
            en: "Fellowships",
            ja: "フェローシップ",
            de: "Meine Zirkel",
            fr: "Communautés",
        },
        (61, 86) => LocalisedText {
            en: "Fellowship Finder",
            ja: "フェローシップ募集",
            de: "Zirkelsuche",
            fr: "Recherche de communautés",
        },
        (61, 87) => LocalisedText {
            en: "Collection",
            ja: "だいじなもの",
            de: "Kuriositäten",
            fr: "Objets précieux",
        },
        (61, 88) => LocalisedText {
            en: "New Game+",
            ja: "つよくてニューゲーム",
            de: "Neues Spiel+",
            fr: "Nouvelle partie +",
        },
        (61, 89) => LocalisedText {
            en: "Fashion Accessories",
            ja: "ファッションアクセサリー",
            de: "Modeaccessoires",
            fr: "Accessoires de mode",
        },
        (61, 90) => LocalisedText {
            en: "Official Sites",
            ja: "公式Webサイト",
            de: "Offizielle Webseiten",
            fr: "Liens vers le site officiel",
        },
        (61, 91) => LocalisedText {
            en: "Duty Support",
            ja: "コンテンツサポーター",
            de: "Inhaltshelfer",
            fr: "Renforts",
        },
        (61, 92) => LocalisedText {
            en: "Portraits",
            ja: "ポートレートの編集",
            de: "Portraits",
            fr: "Portraits",
        },
        (61, 93) => LocalisedText {
            en: "Adventurer Plate",
            ja: "アドベンチャラープレートの編集",
            de: "Abenteurerpass",
            fr: "Carte de personnage",
        },
        (62, 1) => LocalisedText {
            en: "/record",
            ja: "/replay",
            de: "/rückblende",
            fr: "/visionnage",
        },
        (62, 2) => LocalisedText {
            en: "/recordready",
            ja: "/replayready",
            de: "/rückbereit",
            fr: "/appelenregistrement",
        },
        (62, 3) => LocalisedText {
            en: "/pvpteamcmd",
            ja: "/pvpteamcmd",
            de: "/pvpteammenü",
            fr: "/fenêtreéquipejcj",
        },
        (62, 4) => LocalisedText {
            en: "/saddlebag",
            ja: "/chocobobag",
            de: "/satteltasche",
            fr: "/sacochechocobo",
        },
        (62, 5) => LocalisedText {
            en: "/magiaright",
            ja: "/magiaright",
            de: "/magiarechts",
            fr: "/magiadroite",
        },
        (62, 6) => LocalisedText {
            en: "/magialeft",
            ja: "/magialeft",
            de: "/magialinks",
            fr: "/magiagauche",
        },
        (62, 7) => LocalisedText {
            en: "/cwlinkshellcmd",
            ja: "/cwlinkshellcmd",
            de: "/wkkmenü",
            fr: "/lsimcom",
        },
        (62, 8) => LocalisedText {
            en: "/quickchat",
            ja: "/quickchat",
            de: "/schnellchat",
            fr: "/expressionfigée",
        },
        (62, 9) => LocalisedText {
            en: "/bluespellbook",
            ja: "/bluespellbook",
            de: "/blaumagie",
            fr: "/grimoiremagiebleue",
        },
        (62, 10) => LocalisedText {
            en: "/magiaattack",
            ja: "/magiaattack",
            de: "/magiaoffensiv",
            fr: "/magiaoffensif",
        },
        (62, 11) => LocalisedText {
            en: "/magiadefense",
            ja: "/magiadefense",
            de: "/magiadefensiv",
            fr: "/magiadéfensif",
        },
        (62, 12) => LocalisedText {
            en: "/magiaauto",
            ja: "/magiaauto",
            de: "/magiaauto",
            fr: "/magiaauto",
        },
        (62, 13) => LocalisedText {
            en: "/cwlinkshell1",
            ja: "/cwlinkshell1",
            de: "/weltenkontaktkreis1",
            fr: "/linkshellim1",
        },
        (62, 14) => LocalisedText {
            en: "/cwlinkshell2",
            ja: "/cwlinkshell2",
            de: "/weltenkontaktkreis2",
            fr: "/linkshellim2",
        },
        (62, 15) => LocalisedText {
            en: "/cwlinkshell3",
            ja: "/cwlinkshell3",
            de: "/weltenkontaktkreis3",
            fr: "/linkshellim3",
        },
        (62, 16) => LocalisedText {
            en: "/cwlinkshell4",
            ja: "/cwlinkshell4",
            de: "/weltenkontaktkreis4",
            fr: "/linkshellim4",
        },
        (62, 17) => LocalisedText {
            en: "/cwlinkshell5",
            ja: "/cwlinkshell5",
            de: "/weltenkontaktkreis5",
            fr: "/linkshellim5",
        },
        (62, 18) => LocalisedText {
            en: "/cwlinkshell6",
            ja: "/cwlinkshell6",
            de: "/weltenkontaktkreis6",
            fr: "/linkshellim6",
        },
        (62, 19) => LocalisedText {
            en: "/cwlinkshell7",
            ja: "/cwlinkshell7",
            de: "/weltenkontaktkreis7",
            fr: "/linkshellim7",
        },
        (62, 20) => LocalisedText {
            en: "/cwlinkshell8",
            ja: "/cwlinkshell8",
            de: "/weltenkontaktkreis8",
            fr: "/linkshellim8",
        },
        (62, 21) => LocalisedText {
            en: "/graphicpresets",
            ja: "/graphicpresets",
            de: "/grafikeinstellungen",
            fr: "/réggraphiques",
        },
        (62, 22) => LocalisedText {
            en: "/fellowship",
            ja: "/fellowship",
            de: "/zirkel",
            fr: "/communauté",
        },
        (62, 23) => LocalisedText {
            en: "/fellowshipfinder",
            ja: "/fellowshipfinder",
            de: "/zirkelsuche",
            fr: "/rechcommunauté",
        },
        (62, 24) => LocalisedText {
            en: "/collection",
            ja: "/collection",
            de: "/kuriositäten",
            fr: "/objetsprécieux",
        },
        (62, 25) => LocalisedText {
            en: "/newgame+",
            ja: "/newgame+",
            de: "/neuesspiel+",
            fr: "/nouvpartie+",
        },
        (62, 26) => LocalisedText {
            en: "/dice",
            ja: "/dice",
            de: "/zufall",
            fr: "/dé",
        },
        (62, 27) => LocalisedText {
            en: "/nameplatetype",
            ja: "/nameplatetype",
            de: "/namensanzeigeart",
            fr: "/typedeplaquedenom",
        },
        (62, 28) => LocalisedText {
            en: "/nameplatedisp",
            ja: "/nameplatedisp",
            de: "/namensanzeige",
            fr: "/affichageplaquedenom",
        },
        (62, 29) => LocalisedText {
            en: "/fashion",
            ja: "/fashion",
            de: "/mode",
            fr: "/accessoire",
        },
        (62, 30) => LocalisedText {
            en: "/fashionguide",
            ja: "/fashionlist",
            de: "/modeguide",
            fr: "/listeaccessoires",
        },
        (62, 31) => LocalisedText {
            en: "/officialsite",
            ja: "/officialsite",
            de: "/offiziellewebseiten",
            fr: "/siteofficiel",
        },
        (62, 33) => LocalisedText {
            en: "/petglamour",
            ja: "/petmirage",
            de: "/familiarprojektion",
            fr: "/miragefamilier",
        },
        (62, 34) => LocalisedText {
            en: "/petsize",
            ja: "/petsize",
            de: "/familiargröße",
            fr: "/taillefamilier",
        },
        (62, 35) => LocalisedText {
            en: "/ridepillion",
            ja: "/ridepillion",
            de: "/mitfahren",
            fr: "/monteràplusieurs",
        },
        (62, 36) => LocalisedText {
            en: "/patchnote",
            ja: "/patchnote",
            de: "/patchdetails",
            fr: "/notesdemiseàjour",
        },
        (62, 37) => LocalisedText {
            en: "/portrait",
            ja: "/portrait",
            de: "/portrait",
            fr: "/portrait",
        },
        (62, 38) => LocalisedText {
            en: "/adventurerplate",
            ja: "/adventurerplate",
            de: "/abenteurerpass",
            fr: "/cartepersonnage",
        },
        (62, 91) => LocalisedText {
            en: "/pvpteam",
            ja: "/pvpteam",
            de: "/pvpteam",
            fr: "/équipejcj",
        },
        (62, 92) => LocalisedText {
            en: "/cwlinkshell",
            ja: "/cwlinkshell",
            de: "/wkk",
            fr: "/linkshellim",
        },
        (62, 101) => LocalisedText {
            en: "/novice",
            ja: "/beginner",
            de: "/neuling",
            fr: "/novice",
        },
        (62, 102) => LocalisedText {
            en: "/say",
            ja: "/say",
            de: "/sagen",
            fr: "/dire",
        },
        (62, 103) => LocalisedText {
            en: "/shout",
            ja: "/shout",
            de: "/schreien",
            fr: "/hurler",
        },
        (62, 104) => LocalisedText {
            en: "/tell",
            ja: "/tell",
            de: "/flüstern",
            fr: "/murmurer",
        },
        (62, 105) => LocalisedText {
            en: "/party",
            ja: "/party",
            de: "/gruppe",
            fr: "/chatéq",
        },
        (62, 106) => LocalisedText {
            en: "/linkshell",
            ja: "/linkshell",
            de: "/kreis",
            fr: "/linkshell",
        },
        (62, 107) => LocalisedText {
            en: "/linkshell1",
            ja: "/linkshell1",
            de: "/kreis1",
            fr: "/linkshell1",
        },
        (62, 108) => LocalisedText {
            en: "/linkshell2",
            ja: "/linkshell2",
            de: "/kreis2",
            fr: "/linkshell2",
        },
        (62, 109) => LocalisedText {
            en: "/linkshell3",
            ja: "/linkshell3",
            de: "/kreis3",
            fr: "/linkshell3",
        },
        (62, 110) => LocalisedText {
            en: "/linkshell4",
            ja: "/linkshell4",
            de: "/kreis4",
            fr: "/linkshell4",
        },
        (62, 111) => LocalisedText {
            en: "/linkshell5",
            ja: "/linkshell5",
            de: "/kreis5",
            fr: "/linkshell5",
        },
        (62, 112) => LocalisedText {
            en: "/linkshell6",
            ja: "/linkshell6",
            de: "/kreis6",
            fr: "/linkshell6",
        },
        (62, 113) => LocalisedText {
            en: "/linkshell7",
            ja: "/linkshell7",
            de: "/kreis7",
            fr: "/linkshell7",
        },
        (62, 114) => LocalisedText {
            en: "/linkshell8",
            ja: "/linkshell8",
            de: "/kreis8",
            fr: "/linkshell8",
        },
        (62, 115) => LocalisedText {
            en: "/freecompany",
            ja: "/freecompany",
            de: "/freiegesellschaft",
            fr: "/compagnielibre",
        },
        (62, 116) => LocalisedText {
            en: "/echo",
            ja: "/echo",
            de: "/echo",
            fr: "/écho",
        },
        (62, 117) => LocalisedText {
            en: "/yell",
            ja: "/yell",
            de: "/rufen",
            fr: "/crier",
        },
        (62, 118) => LocalisedText {
            en: "/reply",
            ja: "/reply",
            de: "/antworten",
            fr: "/répondre",
        },
        (62, 119) => LocalisedText {
            en: "/alliance",
            ja: "/alliance",
            de: "/allianz",
            fr: "/all",
        },
        (62, 121) => LocalisedText {
            en: "/join",
            ja: "/join",
            de: "/ann",
            fr: "/rejoindre",
        },
        (62, 122) => LocalisedText {
            en: "/decline",
            ja: "/decline",
            de: "/abl",
            fr: "/décliner",
        },
        (62, 123) => LocalisedText {
            en: "/invite",
            ja: "/invite",
            de: "/einladen",
            fr: "/inviter",
        },
        (62, 124) => LocalisedText {
            en: "/kick",
            ja: "/kick",
            de: "/kick",
            fr: "/exclure",
        },
        (62, 125) => LocalisedText {
            en: "/leader",
            ja: "/leader",
            de: "/anführer",
            fr: "/chef",
        },
        (62, 126) => LocalisedText {
            en: "/leave",
            ja: "/leave",
            de: "/verlassen",
            fr: "/partir",
        },
        (62, 131) => LocalisedText {
            en: "/check",
            ja: "/check",
            de: "/check",
            fr: "/examiner",
        },
        (62, 132) => LocalisedText {
            en: "/trade",
            ja: "/trade",
            de: "/handel",
            fr: "/échanger",
        },
        (62, 133) => LocalisedText {
            en: "/automove",
            ja: "/automove",
            de: "/lauf",
            fr: "/courseauto",
        },
        (62, 134) => LocalisedText {
            en: "/follow",
            ja: "/follow",
            de: "/folg",
            fr: "/suivre",
        },
        (62, 136) => LocalisedText {
            en: "/meldrequest",
            ja: "/meldrequest",
            de: "/materiaanfrage",
            fr: "/sertirsvp",
        },
        (62, 141) => LocalisedText {
            en: "/character",
            ja: "/character",
            de: "/charakter",
            fr: "/personnage",
        },
        (62, 142) => LocalisedText {
            en: "/journal",
            ja: "/journal",
            de: "/archiv",
            fr: "/journal",
        },
        (62, 143) => LocalisedText {
            en: "/dutyfinder",
            ja: "/contentsfinder",
            de: "/inhaltssuche",
            fr: "/outilmission",
        },
        (62, 144) => LocalisedText {
            en: "/armourychest",
            ja: "/armourychest",
            de: "/arsenal",
            fr: "/arsenal",
        },
        (62, 145) => LocalisedText {
            en: "/huntinglog",
            ja: "/huntinglog",
            de: "/bestiarium",
            fr: "/bestiaire",
        },
        (62, 146) => LocalisedText {
            en: "/craftinglog",
            ja: "/craftinglog",
            de: "/handwerker",
            fr: "/cartisanat",
        },
        (62, 147) => LocalisedText {
            en: "/gatheringlog",
            ja: "/gatheringlog",
            de: "/sammler",
            fr: "/crécolte",
        },
        (62, 148) => LocalisedText {
            en: "/fishinglog",
            ja: "/fishinglog",
            de: "/fischer",
            fr: "/cpêche",
        },
        (62, 149) => LocalisedText {
            en: "/achievements",
            ja: "/achievements",
            de: "/errungenschaften",
            fr: "/hautsfaits",
        },
        (62, 150) => LocalisedText {
            en: "/partycmd",
            ja: "/partycmd",
            de: "/gruppenaktion",
            fr: "/cmdéq",
        },
        (62, 151) => LocalisedText {
            en: "/friendlist",
            ja: "/friendlist",
            de: "/freunde",
            fr: "/amis",
        },
        (62, 152) => LocalisedText {
            en: "/blacklist",
            ja: "/blacklist",
            de: "/schwarzeliste",
            fr: "/listenoire",
        },
        (62, 153) => LocalisedText {
            en: "/search",
            ja: "/search",
            de: "/suche",
            fr: "/recherche",
        },
        (62, 154) => LocalisedText {
            en: "/map",
            ja: "/map",
            de: "/karte",
            fr: "/carte",
        },
        (62, 155) => LocalisedText {
            en: "/teleport",
            ja: "/teleport",
            de: "/teleport",
            fr: "/téléportation",
        },
        (62, 156) => LocalisedText {
            en: "/enemysign",
            ja: "/marking",
            de: "/mark",
            fr: "/marquer",
        },
        (62, 157) => LocalisedText {
            en: "/actionlist",
            ja: "/actionlist",
            de: "/kommandoliste",
            fr: "/actionstraits",
        },
        (62, 158) => LocalisedText {
            en: "/timers",
            ja: "/timers",
            de: "/timer",
            fr: "/calendrier",
        },
        (62, 159) => LocalisedText {
            en: "/recommended",
            ja: "/recommended",
            de: "/empfehlung",
            fr: "/recommandations",
        },
        (62, 160) => LocalisedText {
            en: "/inventory",
            ja: "/inventory",
            de: "/inventar",
            fr: "/inventaire",
        },
        (62, 161) => LocalisedText {
            en: "/freecompanycmd",
            ja: "/freecompanycmd",
            de: "/freiegesmenü",
            fr: "/cmdcl",
        },
        (62, 162) => LocalisedText {
            en: "/linkshellcmd",
            ja: "/linkshellcmd",
            de: "/kkmenü",
            fr: "/lscom",
        },
        (62, 163) => LocalisedText {
            en: "/return",
            ja: "/return",
            de: "/heim",
            fr: "/rapatriement",
        },
        (62, 164) => LocalisedText {
            en: "/emotelist",
            ja: "/emotelist",
            de: "/emotes",
            fr: "/emotes",
        },
        (62, 165) => LocalisedText {
            en: "/activehelp",
            ja: "/howtolist",
            de: "/tutorial",
            fr: "/tutoriel",
        },
        (62, 166) => LocalisedText {
            en: "/supportdesk",
            ja: "/supportdesk",
            de: "/kundendienst",
            fr: "/assistance",
        },
        (62, 167) => LocalisedText {
            en: "/characterconfig",
            ja: "/characterconfig",
            de: "/charakterkonfig",
            fr: "/configperso",
        },
        (62, 168) => LocalisedText {
            en: "/systemconfig",
            ja: "/systemconfig",
            de: "/systemkonfig",
            fr: "/configsys",
        },
        (62, 169) => LocalisedText {
            en: "/keybind",
            ja: "/keybind",
            de: "/tastenbelegung",
            fr: "/attribtouches",
        },
        (62, 170) => LocalisedText {
            en: "/macros",
            ja: "/macros",
            de: "/makros",
            fr: "/macro",
        },
        (62, 171) => LocalisedText {
            en: "/hudlayout",
            ja: "/hudlayout",
            de: "/bildschirmlayout",
            fr: "/configath",
        },
        (62, 172) => LocalisedText {
            en: "/logout",
            ja: "/logout",
            de: "/logout",
            fr: "/déco",
        },
        (62, 173) => LocalisedText {
            en: "/shutdown",
            ja: "/shutdown",
            de: "/ende",
            fr: "/quitter",
        },
        (62, 174) => LocalisedText {
            en: "/partysort",
            ja: "/partysort",
            de: "/gruppensort",
            fr: "/triéquipe",
        },
        (62, 175) => LocalisedText {
            en: "/readycheck",
            ja: "/readycheck",
            de: "/bereitschaft",
            fr: "/appelpréparation",
        },
        (62, 176) => LocalisedText {
            en: "/ready",
            ja: "/readyok",
            de: "/bereit",
            fr: "/prêt",
        },
        (62, 177) => LocalisedText {
            en: "/hold",
            ja: "/readyng",
            de: "/halt",
            fr: "/nonprêt",
        },
        (62, 178) => LocalisedText {
            en: "/waymark",
            ja: "/fieldmarking",
            de: "/wegmark",
            fr: "/marqausol",
        },
        (62, 179) => LocalisedText {
            en: "/fishguide",
            ja: "/fishguide",
            de: "/fischverzeichnis",
            fr: "/nomenclature",
        },
        (62, 180) => LocalisedText {
            en: "/housing",
            ja: "/housing",
            de: "/unterkunft",
            fr: "/logement",
        },
        (62, 181) => LocalisedText {
            en: "/minionguide",
            ja: "/minionlist",
            de: "/begleiterverzeichnis",
            fr: "/mascottes",
        },
        (62, 182) => LocalisedText {
            en: "/mountguide",
            ja: "/mountlist",
            de: "/reittierverzeichnis",
            fr: "/montures",
        },
        (62, 183) => LocalisedText {
            en: "/partyfinder",
            ja: "/partyfinder",
            de: "/gruppensuche",
            fr: "/rechéq",
        },
        (62, 184) => LocalisedText {
            en: "/pvpprofile",
            ja: "/pvpprofile",
            de: "/pvpprofil",
            fr: "/profiljcj",
        },
        (62, 185) => LocalisedText {
            en: "/companion",
            ja: "/buddy",
            de: "/mitstreiter",
            fr: "/compagnon",
        },
        (62, 186) => LocalisedText {
            en: "/keyitem",
            ja: "/eventitem",
            de: "/schlüsselgegenstand",
            fr: "/objetsclefs",
        },
        (62, 187) => LocalisedText {
            en: "/challengelog",
            ja: "/challengelog",
            de: "/aktuelleagenda",
            fr: "/cobjectifs",
        },
        (62, 188) => LocalisedText {
            en: "/sightseeinglog",
            ja: "/sightseeinglog",
            de: "/incognita",
            fr: "/cexploration",
        },
        (62, 189) => LocalisedText {
            en: "/goldsaucer",
            ja: "/goldsaucer",
            de: "/goldsaucer",
            fr: "/goldsaucer",
        },
        (62, 190) => LocalisedText {
            en: "/currency",
            ja: "/currency",
            de: "/vermögen",
            fr: "/devises",
        },
        (62, 191) => LocalisedText {
            en: "/aethercurrent",
            ja: "/aethercurrent",
            de: "/äther",
            fr: "/ventséthérés",
        },
        (62, 192) => LocalisedText {
            en: "/logcolor",
            ja: "/logcolor",
            de: "/logfarbe",
            fr: "/couleurslog",
        },
        (62, 193) => LocalisedText {
            en: "/playguide",
            ja: "/playguide",
            de: "/spielanleitung",
            fr: "/guidesjeu",
        },
        (62, 194) => LocalisedText {
            en: "/generaldutykey",
            ja: "/contentskey",
            de: "/allgemeinefunktion",
            fr: "/toucheusagegénéral",
        },
        (62, 195) => LocalisedText {
            en: "/raidfinder",
            ja: "/raidfinder",
            de: "/raidsuche",
            fr: "/outilraid",
        },
        (62, 196) => LocalisedText {
            en: "/orchestrion",
            ja: "/orchestrion",
            de: "/orchestrion",
            fr: "/orchestrion",
        },
        (62, 197) => LocalisedText {
            en: "/countdown",
            ja: "/countdown",
            de: "/countdown",
            fr: "/compteàrebours",
        },
        (62, 198) => LocalisedText {
            en: "/contactlist",
            ja: "/contactlist",
            de: "/gruppenverlauf",
            fr: "/listecontacts",
        },
        (62, 199) => LocalisedText {
            en: "/mountspeed",
            ja: "/mountspeed",
            de: "/reitgeschwindigkeit",
            fr: "/vitessemontures",
        },
        (62, 201) => LocalisedText {
            en: "/?",
            ja: "/?",
            de: "/?",
            fr: "/infocmd",
        },
        (62, 202) => LocalisedText {
            en: "/wait",
            ja: "/wait",
            de: "/warte",
            fr: "/attendre",
        },
        (62, 204) => LocalisedText {
            en: "/legacymark",
            ja: "/legacymark",
            de: "/legacy",
            fr: "/marquelegacy",
        },
        (62, 206) => LocalisedText {
            en: "/gcsalute",
            ja: "/gcsalute",
            de: "/gsalut",
            fr: "/salutgc",
        },
        (62, 207) => LocalisedText {
            en: "/macroicon",
            ja: "/macroicon",
            de: "/makroicon",
            fr: "/iconemacro",
        },
        (62, 208) => LocalisedText {
            en: "/recast",
            ja: "/recast",
            de: "/wann",
            fr: "/voirdélai",
        },
        (62, 209) => LocalisedText {
            en: "/gearset",
            ja: "/gearset",
            de: "/set",
            fr: "/tenue",
        },
        (62, 210) => LocalisedText {
            en: "/itemsort",
            ja: "/itemsort",
            de: "/sortieren",
            fr: "/triobjet",
        },
        (62, 211) => LocalisedText {
            en: "/random",
            ja: "/random",
            de: "/würfeln",
            fr: "/dés",
        },
        (62, 213) => LocalisedText {
            en: "/cleartellhistory",
            ja: "/cleartellhistory",
            de: "/flüsterverlauf",
            fr: "/effacerinterlocuteurs",
        },
        (62, 214) => LocalisedText {
            en: "/macrolock",
            ja: "/macrolock",
            de: "/makroblock",
            fr: "/verroumacro",
        },
        (62, 215) => LocalisedText {
            en: "/macroerror",
            ja: "/macroerror",
            de: "/macrofehler",
            fr: "/erreurmacro",
        },
        (62, 216) => LocalisedText {
            en: "/clearlog",
            ja: "/clearlog",
            de: "/loglöschen",
            fr: "/effacerlog",
        },
        (62, 217) => LocalisedText {
            en: "/macrocancel",
            ja: "/macrocancel",
            de: "/makrostopp",
            fr: "/annulermacro",
        },
        (62, 218) => LocalisedText {
            en: "/playtime",
            ja: "/playtime",
            de: "/spielzeit",
            fr: "/tempsjeu",
        },
        (62, 219) => LocalisedText {
            en: "/grouppose",
            ja: "/grouppose",
            de: "/gruppenpose",
            fr: "/posegroupe",
        },
        (62, 220) => LocalisedText {
            en: "/itemsearch",
            ja: "/itemsearch",
            de: "/ggsuche",
            fr: "/rechercheobjet",
        },
        (62, 221) => LocalisedText {
            en: "/idlingcamera",
            ja: "/idlingcamera",
            de: "/knips",
            fr: "/cameradiaporama",
        },
        (62, 223) => LocalisedText {
            en: "/duelswitch",
            ja: "/duelswitch",
            de: "/duell",
            fr: "/refuscombatsingulier",
        },
        (62, 224) => LocalisedText {
            en: "/novicenetworkinvitation",
            ja: "/beginnerchannelinvitation",
            de: "/neulingschateinladung",
            fr: "/invitationréseaunovices",
        },
        (62, 225) => LocalisedText {
            en: "/novicenetworkleave",
            ja: "/beginnerchannelleave",
            de: "/neulingschatverlassen",
            fr: "/quitterréseaunovices",
        },
        (62, 226) => LocalisedText {
            en: "/egiglamour",
            ja: "/egimirage",
            de: "/egiprojektion",
            fr: "/mirageegi",
        },
        (62, 227) => LocalisedText {
            en: "/title",
            ja: "/title",
            de: "/titel",
            fr: "/titre",
        },
        (62, 228) => LocalisedText {
            en: "/bahamutsize",
            ja: "/bahamutsize",
            de: "/bahamut",
            fr: "/taillebahamut",
        },
        (62, 229) => LocalisedText {
            en: "/alarm",
            ja: "/alarm",
            de: "/alarm",
            fr: "/alerte",
        },
        (62, 231) => LocalisedText {
            en: "/busy",
            ja: "/busy",
            de: "/beschäftigt",
            fr: "/occupé",
        },
        (62, 232) => LocalisedText {
            en: "/away",
            ja: "/away",
            de: "/abwesend",
            fr: "/absent",
        },
        (62, 233) => LocalisedText {
            en: "/lookingforparty",
            ja: "/lookingforparty",
            de: "/suchtgruppe",
            fr: "/cherchéquipe",
        },
        (62, 234) => LocalisedText {
            en: "/lookingtomeld",
            ja: "/lookingformeld",
            de: "/setztein",
            fr: "/sertismatéria",
        },
        (62, 235) => LocalisedText {
            en: "/searchcomment",
            ja: "/searchcomment",
            de: "/suchkomm",
            fr: "/commentairerecherche",
        },
        (62, 236) => LocalisedText {
            en: "/nastatus",
            ja: "/beginnerstatus",
            de: "/neulingsstatus",
            fr: "/étatnovice",
        },
        (62, 237) => LocalisedText {
            en: "/novicenetwork",
            ja: "/beginnerchannel",
            de: "/neulingschat",
            fr: "/réseaunovices",
        },
        (62, 238) => LocalisedText {
            en: "/returnerstatusoff",
            ja: "/returnerstatusoff",
            de: "/rückkehrerstatusaus",
            fr: "/désactiveétatrappelé",
        },
        (62, 239) => LocalisedText {
            en: "/roleplaying",
            ja: "/roleplaying",
            de: "/rollenspiel",
            fr: "/jeuderôle",
        },
        (62, 251) => LocalisedText {
            en: "/action",
            ja: "/action",
            de: "/kommando",
            fr: "/act",
        },
        (62, 252) => LocalisedText {
            en: "/battlemode",
            ja: "/battlemode",
            de: "/kampf",
            fr: "/modecombat",
        },
        (62, 253) => LocalisedText {
            en: "/target",
            ja: "/target",
            de: "/ziel",
            fr: "/cibler",
        },
        (62, 254) => LocalisedText {
            en: "/targetpc",
            ja: "/targetpc",
            de: "/zielpc",
            fr: "/ciblerpj",
        },
        (62, 255) => LocalisedText {
            en: "/targetnpc",
            ja: "/targetnpc",
            de: "/zielnpc",
            fr: "/ciblerpnj",
        },
        (62, 256) => LocalisedText {
            en: "/targetenemy",
            ja: "/targetenemy",
            de: "/zielgeg",
            fr: "/ciblerenn",
        },
        (62, 257) => LocalisedText {
            en: "/battletarget",
            ja: "/battletarget",
            de: "/zielfeind",
            fr: "/ciblecombat",
        },
        (62, 258) => LocalisedText {
            en: "/assist",
            ja: "/assist",
            de: "/assist",
            fr: "/aider",
        },
        (62, 259) => LocalisedText {
            en: "/facetarget",
            ja: "/facetarget",
            de: "/hin",
            fr: "/faireface",
        },
        (62, 260) => LocalisedText {
            en: "/nexttarget",
            ja: "/nexttarget",
            de: "/nächstesziel",
            fr: "/ciblesuivante",
        },
        (62, 261) => LocalisedText {
            en: "/previoustarget",
            ja: "/previoustarget",
            de: "/vorigesziel",
            fr: "/cibleprécédente",
        },
        (62, 262) => LocalisedText {
            en: "/targetlasttarget",
            ja: "/targetlasttarget",
            de: "/letztesziel",
            fr: "/dernièrecible",
        },
        (62, 263) => LocalisedText {
            en: "/targetlastenemy",
            ja: "/targetlastenemy",
            de: "/letzterfeind",
            fr: "/dernierennemi",
        },
        (62, 264) => LocalisedText {
            en: "/lockon",
            ja: "/lockon",
            de: "/feststellen",
            fr: "/verrouillercible",
        },
        (62, 265) => LocalisedText {
            en: "/focustarget",
            ja: "/focustarget",
            de: "/fokusziel",
            fr: "/focalisercible",
        },
        (62, 267) => LocalisedText {
            en: "/petaction",
            ja: "/petaction",
            de: "/familiarkomm",
            fr: "/actionfamilier",
        },
        (62, 268) => LocalisedText {
            en: "/companionaction",
            ja: "/buddyaction",
            de: "/mitstreiterkomm",
            fr: "/actioncompagnon",
        },
        (62, 269) => LocalisedText {
            en: "/facecamera",
            ja: "/facecamera",
            de: "/zukamera",
            fr: "/visagecaméra",
        },
        (62, 270) => LocalisedText {
            en: "/levelsync",
            ja: "/levelsync",
            de: "/stufenanpassung",
            fr: "/synchroniveau",
        },
        (62, 271) => LocalisedText {
            en: "/mount",
            ja: "/mount",
            de: "/reit",
            fr: "/monture",
        },
        (62, 272) => LocalisedText {
            en: "/additionalaction",
            ja: "/additionalaction",
            de: "/ergkmd",
            fr: "/actionrôle",
        },
        (62, 273) => LocalisedText {
            en: "/generalaction",
            ja: "/generalaction",
            de: "/allgkom",
            fr: "/actiongénérale",
        },
        (62, 274) => LocalisedText {
            en: "/minion",
            ja: "/minion",
            de: "/begleiter",
            fr: "/mascotte",
        },
        (62, 275) => LocalisedText {
            en: "/statusoff",
            ja: "/statusoff",
            de: "/statusaus",
            fr: "/annuleffet",
        },
        (62, 276) => LocalisedText {
            en: "/addpvpaction",
            ja: "/addpvpaction",
            de: "/pluspvpkmd",
            fr: "/ajoutactionjcj",
        },
        (62, 277) => LocalisedText {
            en: "/pvpaction",
            ja: "/pvpaction",
            de: "/pvpkommando",
            fr: "/actionjcj",
        },
        (62, 279) => LocalisedText {
            en: "/blueaction",
            ja: "/blueaction",
            de: "/blaukommando",
            fr: "/actionmagiebleue",
        },
        (62, 301) => LocalisedText {
            en: "/autolockon",
            ja: "/autolockon",
            de: "/autofest",
            fr: "/verrouillerauto",
        },
        (62, 302) => LocalisedText {
            en: "/autofacetarget",
            ja: "/autofacetarget",
            de: "/autohin",
            fr: "/pivoteauto",
        },
        (62, 303) => LocalisedText {
            en: "/targetring",
            ja: "/targetring",
            de: "/ring",
            fr: "/cercleciblage",
        },
        (62, 304) => LocalisedText {
            en: "/targetline",
            ja: "/targetline",
            de: "/aggro",
            fr: "/ligneciblage",
        },
        (62, 305) => LocalisedText {
            en: "/aggroline",
            ja: "/linkline",
            de: "/verbündete",
            fr: "/liensmonstres",
        },
        (62, 306) => LocalisedText {
            en: "/autotarget",
            ja: "/autotarget",
            de: "/autoziel",
            fr: "/ciblageauto",
        },
        (62, 307) => LocalisedText {
            en: "/displayhead",
            ja: "/displayhead",
            de: "/helm",
            fr: "/affichagecouvrechef",
        },
        (62, 308) => LocalisedText {
            en: "/displayarms",
            ja: "/displayarms",
            de: "/waffe",
            fr: "/affichagearmes",
        },
        (62, 309) => LocalisedText {
            en: "/autosheathe",
            ja: "/autosheathe",
            de: "/waffeweg",
            fr: "/rengainerauto",
        },
        (62, 310) => LocalisedText {
            en: "/targetself",
            ja: "/targetself",
            de: "/selbstklick",
            fr: "/cliquersursoi",
        },
        (62, 311) => LocalisedText {
            en: "/groundclick",
            ja: "/groundclick",
            de: "/bodenklick",
            fr: "/cliquervide",
        },
        (62, 312) => LocalisedText {
            en: "/chatlog",
            ja: "/chatlog",
            de: "/chatlog",
            fr: "/fenêtrelog",
        },
        (62, 313) => LocalisedText {
            en: "/battleeffect",
            ja: "/battleeffect",
            de: "/kampfeffekte",
            fr: "/effetscombats",
        },
        (62, 314) => LocalisedText {
            en: "/hud",
            ja: "/hud",
            de: "/hud",
            fr: "/ath",
        },
        (62, 315) => LocalisedText {
            en: "/hotbar",
            ja: "/hotbar",
            de: "/kommandomenü",
            fr: "/barracc",
        },
        (62, 316) => LocalisedText {
            en: "/crosshotbar",
            ja: "/crosshotbar",
            de: "/kreuzkommandomenü",
            fr: "/croixracc",
        },
        (62, 317) => LocalisedText {
            en: "/hudreset",
            ja: "/hudreset",
            de: "/bildschirmreset",
            fr: "/réinitath",
        },
        (62, 318) => LocalisedText {
            en: "/uireset",
            ja: "/uireset",
            de: "/fensterreset",
            fr: "/réiniiu",
        },
        (62, 319) => LocalisedText {
            en: "/uiscale",
            ja: "/uiscale",
            de: "/uiskalierung",
            fr: "/tailleiu",
        },
        (62, 320) => LocalisedText {
            en: "/actionerror",
            ja: "/actionerror",
            de: "/kmdwarnung",
            fr: "/erreuraction",
        },
        (62, 321) => LocalisedText {
            en: "/recasterror",
            ja: "/recasterror",
            de: "/reaktwarnung",
            fr: "/erreurrecharge",
        },
        (62, 323) => LocalisedText {
            en: "/crosshotbartype",
            ja: "/crosshotbartype",
            de: "/kreuzkommandomenütyp",
            fr: "/typecroixraccourcis",
        },
        (62, 324) => LocalisedText {
            en: "/crosshotbardisplay",
            ja: "/crosshotbardisplay",
            de: "/kreuzkommandomenüeinstellungen",
            fr: "/affichagecroixraccourcis",
        },
        (62, 325) => LocalisedText {
            en: "/pvphotbar",
            ja: "/pvphotbar",
            de: "/pvpkommandomenü",
            fr: "/barraccjcj",
        },
        (62, 326) => LocalisedText {
            en: "/pvpcrosshotbar",
            ja: "/pvpcrosshotbar",
            de: "/pvpkreuzkommandomenü",
            fr: "/croixraccjcj",
        },
        (62, 327) => LocalisedText {
            en: "/emotelog",
            ja: "/emotelog",
            de: "/emotelog",
            fr: "/emotelog",
        },
        (62, 328) => LocalisedText {
            en: "/jobhudmode",
            ja: "/jobhudmode",
            de: "/jobbalkenmodus",
            fr: "/affichageinterfacejob",
        },
        (62, 329) => LocalisedText {
            en: "/mastervolume",
            ja: "/mastervolume",
            de: "/lautstärke",
            fr: "/vgénéral",
        },
        (62, 330) => LocalisedText {
            en: "/bgm",
            ja: "/bgm",
            de: "/musik",
            fr: "/vmusique",
        },
        (62, 331) => LocalisedText {
            en: "/soundeffects",
            ja: "/soundeffects",
            de: "/soundeffekte",
            fr: "/veffetssonores",
        },
        (62, 332) => LocalisedText {
            en: "/voice",
            ja: "/voice",
            de: "/stimmen",
            fr: "/vvoix",
        },
        (62, 333) => LocalisedText {
            en: "/systemsounds",
            ja: "/systemsounds",
            de: "/systemtöne",
            fr: "/vsonssystèmes",
        },
        (62, 334) => LocalisedText {
            en: "/ambientsounds",
            ja: "/ambientsounds",
            de: "/umgebung",
            fr: "/vambiancesonore",
        },
        (62, 335) => LocalisedText {
            en: "/soundeffectsself",
            ja: "/soundeffectsself",
            de: "/soundselbst",
            fr: "/veffetssoi",
        },
        (62, 336) => LocalisedText {
            en: "/soundeffectsparty",
            ja: "/soundeffectsparty",
            de: "/soundgruppe",
            fr: "/veffetséquipe",
        },
        (62, 337) => LocalisedText {
            en: "/soundeffectsother",
            ja: "/soundeffectsother",
            de: "/soundandere",
            fr: "/veffetsautres",
        },
        (62, 338) => LocalisedText {
            en: "/systemsoundsspeaker",
            ja: "/systemsoundsspeaker",
            de: "/systemlautsprecher",
            fr: "/vhautparleur",
        },
        (62, 339) => LocalisedText {
            en: "/tiltcamera",
            ja: "/tiltcamera",
            de: "/kamerawinkel",
            fr: "/hcaméraobjective",
        },
        (62, 340) => LocalisedText {
            en: "/mountbgm",
            ja: "/mountbgm",
            de: "/reitbgm",
            fr: "/musiquemonture",
        },
        (62, 341) => LocalisedText {
            en: "/performsounds",
            ja: "/performsounds",
            de: "/komplaut",
            fr: "/actionsinterprétation",
        },
        (62, 401) => LocalisedText {
            en: "/emote",
            ja: "/emote",
            de: "/emote",
            fr: "/cmdémotion",
        },
        (62, 402) => LocalisedText {
            en: "/surprised",
            ja: "/surprised",
            de: "/überrascht",
            fr: "/surprise",
        },
        (62, 403) => LocalisedText {
            en: "/angry",
            ja: "/angry",
            de: "/ärger",
            fr: "/fâché",
        },
        (62, 404) => LocalisedText {
            en: "/furious",
            ja: "/furious",
            de: "/zorn",
            fr: "/furieux",
        },
        (62, 405) => LocalisedText {
            en: "/blush",
            ja: "/blush",
            de: "/rot",
            fr: "/rougir",
        },
        (62, 406) => LocalisedText {
            en: "/bow",
            ja: "/bow",
            de: "/verbeug",
            fr: "/révérence",
        },
        (62, 407) => LocalisedText {
            en: "/cheer",
            ja: "/cheer",
            de: "/feuer",
            fr: "/encourager",
        },
        (62, 408) => LocalisedText {
            en: "/clap",
            ja: "/clap",
            de: "/klatsch",
            fr: "/applaudir",
        },
        (62, 409) => LocalisedText {
            en: "/beckon",
            ja: "/beckon",
            de: "/herbei",
            fr: "/viens",
        },
        (62, 410) => LocalisedText {
            en: "/comfort",
            ja: "/comfort",
            de: "/trost",
            fr: "/consoler",
        },
        (62, 411) => LocalisedText {
            en: "/cry",
            ja: "/cry",
            de: "/weinen",
            fr: "/pleurer",
        },
        (62, 412) => LocalisedText {
            en: "/dance",
            ja: "/dance",
            de: "/tanz",
            fr: "/danser",
        },
        (62, 413) => LocalisedText {
            en: "/doubt",
            ja: "/doubt",
            de: "/schuld",
            fr: "/questionner",
        },
        (62, 414) => LocalisedText {
            en: "/doze",
            ja: "/doze",
            de: "/müde",
            fr: "/somnoler",
        },
        (62, 415) => LocalisedText {
            en: "/fume",
            ja: "/fume",
            de: "/frust",
            fr: "/dépit",
        },
        (62, 416) => LocalisedText {
            en: "/goodbye",
            ja: "/goodbye",
            de: "/tschüs",
            fr: "/aurevoir",
        },
        (62, 417) => LocalisedText {
            en: "/wave",
            ja: "/wave",
            de: "/wink",
            fr: "/coucou",
        },
        (62, 418) => LocalisedText {
            en: "/huh",
            ja: "/huh",
            de: "/tja",
            fr: "/consterner",
        },
        (62, 419) => LocalisedText {
            en: "/joy",
            ja: "/joy",
            de: "/froh",
            fr: "/joie",
        },
        (62, 420) => LocalisedText {
            en: "/kneel",
            ja: "/kneel",
            de: "/knie",
            fr: "/àgenoux",
        },
        (62, 421) => LocalisedText {
            en: "/chuckle",
            ja: "/chuckle",
            de: "/lach",
            fr: "/rire",
        },
        (62, 422) => LocalisedText {
            en: "/laugh",
            ja: "/laugh",
            de: "/totlach",
            fr: "/sesclaffer",
        },
        (62, 423) => LocalisedText {
            en: "/lookout",
            ja: "/lookout",
            de: "/späh",
            fr: "/scruter",
        },
        (62, 424) => LocalisedText {
            en: "/me",
            ja: "/me",
            de: "/ich",
            fr: "/moi",
        },
        (62, 425) => LocalisedText {
            en: "/no",
            ja: "/no",
            de: "/nein",
            fr: "/non",
        },
        (62, 426) => LocalisedText {
            en: "/deny",
            ja: "/deny",
            de: "/niemals",
            fr: "/refuser",
        },
        (62, 427) => LocalisedText {
            en: "/panic",
            ja: "/panic",
            de: "/panik",
            fr: "/paniquer",
        },
        (62, 428) => LocalisedText {
            en: "/point",
            ja: "/point",
            de: "/zeig",
            fr: "/montrer",
        },
        (62, 429) => LocalisedText {
            en: "/poke",
            ja: "/poke",
            de: "/stups",
            fr: "/toucher",
        },
        (62, 430) => LocalisedText {
            en: "/congratulate",
            ja: "/congratulate",
            de: "/super",
            fr: "/féliciter",
        },
        (62, 431) => LocalisedText {
            en: "/psych",
            ja: "/psych",
            de: "/aufgehts",
            fr: "/motivation",
        },
        (62, 432) => LocalisedText {
            en: "/salute",
            ja: "/salute",
            de: "/salut",
            fr: "/saluer",
        },
        (62, 433) => LocalisedText {
            en: "/shocked",
            ja: "/shocked",
            de: "/schock",
            fr: "/choc",
        },
        (62, 434) => LocalisedText {
            en: "/shrug",
            ja: "/shrug",
            de: "/schulter",
            fr: "/épaules",
        },
        (62, 435) => LocalisedText {
            en: "/rally",
            ja: "/rally",
            de: "/aufmunter",
            fr: "/stimuler",
        },
        (62, 436) => LocalisedText {
            en: "/soothe",
            ja: "/soothe",
            de: "/ruhig",
            fr: "/apaiser",
        },
        (62, 437) => LocalisedText {
            en: "/stagger",
            ja: "/stagger",
            de: "/taumel",
            fr: "/tituber",
        },
        (62, 438) => LocalisedText {
            en: "/stretch",
            ja: "/stretch",
            de: "/streck",
            fr: "/étirer",
        },
        (62, 439) => LocalisedText {
            en: "/sulk",
            ja: "/sulk",
            de: "/schmoll",
            fr: "/bouder",
        },
        (62, 440) => LocalisedText {
            en: "/think",
            ja: "/think",
            de: "/denk",
            fr: "/penser",
        },
        (62, 441) => LocalisedText {
            en: "/upset",
            ja: "/upset",
            de: "/natoll",
            fr: "/déçu",
        },
        (62, 442) => LocalisedText {
            en: "/welcome",
            ja: "/welcome",
            de: "/willkommen",
            fr: "/bienvenue",
        },
        (62, 443) => LocalisedText {
            en: "/yes",
            ja: "/yes",
            de: "/ja",
            fr: "/oui",
        },
        (62, 444) => LocalisedText {
            en: "/thumbsup",
            ja: "/thumbsup",
            de: "/daumen",
            fr: "/approuver",
        },
        (62, 445) => LocalisedText {
            en: "/examineself",
            ja: "/examineself",
            de: "/selbst",
            fr: "/seregarder",
        },
        (62, 446) => LocalisedText {
            en: "/pose",
            ja: "/pose",
            de: "/pose",
            fr: "/poser",
        },
        (62, 447) => LocalisedText {
            en: "/blowkiss",
            ja: "/blowkiss",
            de: "/kusshand",
            fr: "/lancerbaiser",
        },
        (62, 448) => LocalisedText {
            en: "/grovel",
            ja: "/grovel",
            de: "/demut",
            fr: "/prosterner",
        },
        (62, 449) => LocalisedText {
            en: "/happy",
            ja: "/happy",
            de: "/juhu",
            fr: "/heureux",
        },
        (62, 450) => LocalisedText {
            en: "/disappointed",
            ja: "/disappointed",
            de: "/allesaus",
            fr: "/désespoir",
        },
        (62, 451) => LocalisedText {
            en: "/sit",
            ja: "/sit",
            de: "/sitz",
            fr: "/asseoir",
        },
        (62, 453) => LocalisedText {
            en: "/groundsit",
            ja: "/groundsit",
            de: "/bodensitz",
            fr: "/asseoirparterre",
        },
        (62, 455) => LocalisedText {
            en: "/airquotes",
            ja: "/airquotes",
            de: "/sogenannt",
            fr: "/guillemets",
        },
        (62, 459) => LocalisedText {
            en: "/pray",
            ja: "/pray",
            de: "/beten",
            fr: "/prier",
        },
        (62, 460) => LocalisedText {
            en: "/imperialsalute",
            ja: "/imperialsalute",
            de: "/garsalut",
            fr: "/salutimp",
        },
        (62, 461) => LocalisedText {
            en: "/visor",
            ja: "/visor",
            de: "/visier",
            fr: "/visière",
        },
        (62, 463) => LocalisedText {
            en: "/megaflare",
            ja: "/megaflare",
            de: "/megaflare",
            fr: "/mégabrasier",
        },
        (62, 464) => LocalisedText {
            en: "/crimsonlotus",
            ja: "/crimsonlotus",
            de: "/purpurlotos",
            fr: "/chorémartiale",
        },
        (62, 465) => LocalisedText {
            en: "/charmed",
            ja: "/charmed",
            de: "/betört",
            fr: "/tombersouslecharme",
        },
        (62, 466) => LocalisedText {
            en: "/cheeron",
            ja: "/cheeron",
            de: "/anfeuernblau",
            fr: "/encouragerbleu",
        },
        (62, 467) => LocalisedText {
            en: "/cheerwave",
            ja: "/cheerwave",
            de: "/anfeuerngelb",
            fr: "/encouragerjaune",
        },
        (62, 468) => LocalisedText {
            en: "/cheerjump",
            ja: "/cheerjump",
            de: "/anfeuernrot",
            fr: "/encouragerrouge",
        },
        (62, 469) => LocalisedText {
            en: "/straightface",
            ja: "/straightface",
            de: "/neutral",
            fr: "/neutre",
        },
        (62, 470) => LocalisedText {
            en: "/smile",
            ja: "/smile",
            de: "/lächel",
            fr: "/sourire",
        },
        (62, 471) => LocalisedText {
            en: "/grin",
            ja: "/grin",
            de: "/grins",
            fr: "/radieux",
        },
        (62, 472) => LocalisedText {
            en: "/smirk",
            ja: "/smirk",
            de: "/zuversicht",
            fr: "/confiant",
        },
        (62, 473) => LocalisedText {
            en: "/taunt",
            ja: "/taunt",
            de: "/siegessicher",
            fr: "/arrogant",
        },
        (62, 474) => LocalisedText {
            en: "/shuteyes",
            ja: "/shuteyes",
            de: "/augenzu",
            fr: "/yeuxfermés",
        },
        (62, 475) => LocalisedText {
            en: "/sad",
            ja: "/sad",
            de: "/traurig",
            fr: "/triste",
        },
        (62, 476) => LocalisedText {
            en: "/scared",
            ja: "/scared",
            de: "/ängst",
            fr: "/peur",
        },
        (62, 477) => LocalisedText {
            en: "/amazed",
            ja: "/amazed",
            de: "/sprachlos",
            fr: "/stupéfait",
        },
        (62, 478) => LocalisedText {
            en: "/ouch",
            ja: "/ouch",
            de: "/schmerz",
            fr: "/douleur",
        },
        (62, 479) => LocalisedText {
            en: "/annoyed",
            ja: "/annoyed",
            de: "/genervt",
            fr: "/grognon",
        },
        (62, 480) => LocalisedText {
            en: "/alert",
            ja: "/alert",
            de: "/alarmiert",
            fr: "/interloqué",
        },
        (62, 481) => LocalisedText {
            en: "/worried",
            ja: "/worried",
            de: "/besorgt",
            fr: "/inquiet",
        },
        (62, 482) => LocalisedText {
            en: "/biggrin",
            ja: "/biggrin",
            de: "/feistgrinsen",
            fr: "/rieur",
        },
        (62, 483) => LocalisedText {
            en: "/reflect",
            ja: "/reflect",
            de: "/einkehr",
            fr: "/méditatif",
        },
        (62, 484) => LocalisedText {
            en: "/furrow",
            ja: "/furrow",
            de: "/augenbrauen",
            fr: "/froncersourcils",
        },
        (62, 485) => LocalisedText {
            en: "/scoff",
            ja: "/scoff",
            de: "/überheblich",
            fr: "/sourirenavré",
        },
        (62, 486) => LocalisedText {
            en: "/throw",
            ja: "/throw",
            de: "/werfen",
            fr: "/lancer",
        },
        (62, 491) => LocalisedText {
            en: "/changepose",
            ja: "/changepose",
            de: "/pose2",
            fr: "/changerpose",
        },
        (62, 502) => LocalisedText {
            en: "/stepdance",
            ja: "/stepdance",
            de: "/stepptanz",
            fr: "/claquettes",
        },
        (62, 503) => LocalisedText {
            en: "/harvestdance",
            ja: "/harvestdance",
            de: "/fruchttanz",
            fr: "/dansemoissons",
        },
        (62, 504) => LocalisedText {
            en: "/balldance",
            ja: "/balldance",
            de: "/höflingstanz",
            fr: "/dansecour",
        },
        (62, 505) => LocalisedText {
            en: "/mandervilledance",
            ja: "/mandervilledance",
            de: "/chevalierstanz",
            fr: "/dansegentilhomme",
        },
        (62, 506) => LocalisedText {
            en: "/pet",
            ja: "/stroke",
            de: "/streicheln",
            fr: "/caresser",
        },
        (62, 507) => LocalisedText {
            en: "/handover",
            ja: "/handover",
            de: "/übergeben",
            fr: "/donner",
        },
        (62, 510) => LocalisedText {
            en: "/bombdance",
            ja: "/bombdance",
            de: "/feuertanz",
            fr: "/dansebombo",
        },
        (62, 511) => LocalisedText {
            en: "/huzzah",
            ja: "/hurray",
            de: "/hurra",
            fr: "/hourra",
        },
        (62, 512) => LocalisedText {
            en: "/slap",
            ja: "/slap",
            de: "/schlag",
            fr: "/claque",
        },
        (62, 513) => LocalisedText {
            en: "/hug",
            ja: "/hug",
            de: "/umarmen",
            fr: "/accolade",
        },
        (62, 514) => LocalisedText {
            en: "/embrace",
            ja: "/embrace",
            de: "/herz",
            fr: "/étreinte",
        },
        (62, 515) => LocalisedText {
            en: "/hildibrand",
            ja: "/hildibrand",
            de: "/hildibrand",
            fr: "/hildibrand",
        },
        (62, 516) => LocalisedText {
            en: "/fistbump",
            ja: "/fistbump",
            de: "/faustgruß",
            fr: "/salutpoing",
        },
        (62, 519) => LocalisedText {
            en: "/thavdance",
            ja: "/thavdance",
            de: "/thavnairtanz",
            fr: "/dansethavnairoise",
        },
        (62, 520) => LocalisedText {
            en: "/golddance",
            ja: "/golddance",
            de: "/goldtanz",
            fr: "/dansedelor",
        },
        (62, 521) => LocalisedText {
            en: "/sundropdance",
            ja: "/sundance",
            de: "/sonnentanz",
            fr: "/dansedusoleil",
        },
        (62, 522) => LocalisedText {
            en: "/battlestance",
            ja: "/battlestance",
            de: "/aufwärmen",
            fr: "/seprépareraucombat",
        },
        (62, 523) => LocalisedText {
            en: "/victorypose",
            ja: "/victorypose",
            de: "/triumph",
            fr: "/seréjouirdelavictoire",
        },
        (62, 524) => LocalisedText {
            en: "/backflip",
            ja: "/backflip",
            de: "/salto",
            fr: "/saltoarrière",
        },
        (62, 525) => LocalisedText {
            en: "/easterngreeting",
            ja: "/easterngreeting",
            de: "/fernöstlichergruß",
            fr: "/salutoriental",
        },
        (62, 526) => LocalisedText {
            en: "/eureka",
            ja: "/eureka",
            de: "/erleuchtung",
            fr: "/éclairdegénie",
        },
        (62, 527) => LocalisedText {
            en: "/mogdance",
            ja: "/mogdance",
            de: "/mogrytanz",
            fr: "/dansemog",
        },
        (62, 528) => LocalisedText {
            en: "/haurchefant",
            ja: "/haurchefant",
            de: "/haurchefant",
            fr: "/haurchefant",
        },
        (62, 529) => LocalisedText {
            en: "/easternstretch",
            ja: "/warmup",
            de: "/fernöstlichesstrecken",
            fr: "/étirementsorientaux",
        },
        (62, 530) => LocalisedText {
            en: "/easterndance",
            ja: "/easterndance",
            de: "/fernöstlichertanz",
            fr: "/danseorientale",
        },
        (62, 531) => LocalisedText {
            en: "/redrangerposea",
            ja: "/rangerposered1",
            de: "/roterbombattant1",
            fr: "/bomboforcerouge",
        },
        (62, 532) => LocalisedText {
            en: "/blackrangerposea",
            ja: "/rangerposeblack1",
            de: "/schwarzebombattantin1",
            fr: "/bomboforcenoire",
        },
        (62, 533) => LocalisedText {
            en: "/yellowrangerposea",
            ja: "/rangerposeyellow1",
            de: "/gelberbombattant1",
            fr: "/bomboforcejaune",
        },
        (62, 534) => LocalisedText {
            en: "/rightwink",
            ja: "/rightwink",
            de: "/zwinkerrechts",
            fr: "/clindoeildroit",
        },
        (62, 535) => LocalisedText {
            en: "/redrangerposeb",
            ja: "/rangerposered2",
            de: "/roterbombattant2",
            fr: "/bomboforcerougeinversé",
        },
        (62, 536) => LocalisedText {
            en: "/blackrangerposeb",
            ja: "/rangerposeblack2",
            de: "/schwarzebombattantin2",
            fr: "/bomboforcenoireinversé",
        },
        (62, 537) => LocalisedText {
            en: "/yellowrangerposeb",
            ja: "/rangerposeyellow2",
            de: "/gelberbombattant2",
            fr: "/bomboforcejauneinversé",
        },
        (62, 538) => LocalisedText {
            en: "/facepalm",
            ja: "/facepalm",
            de: "/vergraben",
            fr: "/seprendrelatête",
        },
        (62, 539) => LocalisedText {
            en: "/zantetsuken",
            ja: "/zantetsuken",
            de: "/zantetsuken",
            fr: "/zantetsuken",
        },
        (62, 540) => LocalisedText {
            en: "/flex",
            ja: "/flex",
            de: "/muskeln",
            fr: "/contractersesmuscles",
        },
        (62, 541) => LocalisedText {
            en: "/respect",
            ja: "/respect",
            de: "/respekt",
            fr: "/serecueillir",
        },
        (62, 542) => LocalisedText {
            en: "/sneer",
            ja: "/sneer",
            de: "/verächtlich",
            fr: "/vicieux",
        },
        (62, 543) => LocalisedText {
            en: "/prettyplease",
            ja: "/prettyplease",
            de: "/liebbitten",
            fr: "/attendrir",
        },
        (62, 544) => LocalisedText {
            en: "/playdead",
            ja: "/playdead",
            de: "/totstellen",
            fr: "/feindrelamort",
        },
        (62, 545) => LocalisedText {
            en: "/iceheart",
            ja: "/iceheart",
            de: "/eisherz",
            fr: "/illusionsglacées",
        },
        (62, 546) => LocalisedText {
            en: "/moonlift",
            ja: "/moonlift",
            de: "/sonneundmondtanz",
            fr: "/dansesoleiletlune",
        },
        (62, 547) => LocalisedText {
            en: "/dote",
            ja: "/dote",
            de: "/kussregen",
            fr: "/exprimersonamour",
        },
        (62, 549) => LocalisedText {
            en: "/spectacles",
            ja: "/spectacles",
            de: "/brille",
            fr: "/ajusterseslunettes",
        },
        (62, 550) => LocalisedText {
            en: "/songbird",
            ja: "/songbird",
            de: "/singdrossel",
            fr: "/dansedeséglantines",
        },
        (62, 551) => LocalisedText {
            en: "/waterfloat",
            ja: "/waterfloat",
            de: "/totermann",
            fr: "/flottersurleau",
        },
        (62, 552) => LocalisedText {
            en: "/waterflip",
            ja: "/waterflip",
            de: "/unterwasserrolle",
            fr: "/saltodansleau",
        },
        (62, 553) => LocalisedText {
            en: "/puckerup",
            ja: "/puckerup",
            de: "/mundkräuseln",
            fr: "/tendreleslèvres",
        },
        (62, 554) => LocalisedText {
            en: "/powerup",
            ja: "/powerup",
            de: "/kraft",
            fr: "/concentrersesforces",
        },
        (62, 555) => LocalisedText {
            en: "/easternbow",
            ja: "/easternbow",
            de: "/fernöstlicheverbeugung",
            fr: "/révérenceorientale",
        },
        (62, 556) => LocalisedText {
            en: "/squats",
            ja: "/squats",
            de: "/kniebeugen",
            fr: "/flexionsjambes",
        },
        (62, 557) => LocalisedText {
            en: "/pushups",
            ja: "/pushups",
            de: "/liegestütze",
            fr: "/pompes",
        },
        (62, 558) => LocalisedText {
            en: "/situps",
            ja: "/situps",
            de: "/situps",
            fr: "/abdos",
        },
        (62, 559) => LocalisedText {
            en: "/breathcontrol",
            ja: "/breathcontrol",
            de: "/atemübung",
            fr: "/respirationprofonde",
        },
        (62, 560) => LocalisedText {
            en: "/converse",
            ja: "/converse",
            de: "/erzählen",
            fr: "/parler",
        },
        (62, 561) => LocalisedText {
            en: "/concentrate",
            ja: "/concentrate",
            de: "/ernst",
            fr: "/sérieux",
        },
        (62, 562) => LocalisedText {
            en: "/disturbed",
            ja: "/disturbed",
            de: "/betrübt",
            fr: "/perplexe",
        },
        (62, 563) => LocalisedText {
            en: "/simper",
            ja: "/simper",
            de: "/sanftmütig",
            fr: "/souriretendre",
        },
        (62, 564) => LocalisedText {
            en: "/beam",
            ja: "/beam",
            de: "/zufrieden",
            fr: "/satisfait",
        },
        (62, 565) => LocalisedText {
            en: "/attention",
            ja: "/attention",
            de: "/habacht",
            fr: "/gardeàvous",
        },
        (62, 566) => LocalisedText {
            en: "/atease",
            ja: "/atease",
            de: "/rühren",
            fr: "/repos",
        },
        (62, 567) => LocalisedText {
            en: "/box",
            ja: "/box",
            de: "/boxen",
            fr: "/boxe",
        },
        (62, 568) => LocalisedText {
            en: "/ritualprayer",
            ja: "/ritualprayer",
            de: "/beschwören",
            fr: "/prièrerituelle",
        },
        (62, 570) => LocalisedText {
            en: "/tremble",
            ja: "/tremble",
            de: "/zittern",
            fr: "/tremblerdepeur",
        },
        (62, 571) => LocalisedText {
            en: "/winded",
            ja: "/winded",
            de: "/außeratem",
            fr: "/genouàterre",
        },
        (62, 572) => LocalisedText {
            en: "/aback",
            ja: "/aback",
            de: "/huch",
            fr: "/stupéfaction",
        },
        (62, 573) => LocalisedText {
            en: "/greet",
            ja: "/greet",
            de: "/salopp",
            fr: "/salutdécontracté",
        },
        (62, 574) => LocalisedText {
            en: "/boxstep",
            ja: "/boxstep",
            de: "/tanzgrundschritt",
            fr: "/pasdebase",
        },
        (62, 575) => LocalisedText {
            en: "/sidestep",
            ja: "/sidestep",
            de: "/seitwärtsschritt",
            fr: "/pasdecôté",
        },
        (62, 576) => LocalisedText {
            en: "/ultima",
            ja: "/ultima",
            de: "/ultima",
            fr: "/ultima",
        },
        (62, 577) => LocalisedText {
            en: "/yoldance",
            ja: "/yoldance",
            de: "/yoltanz",
            fr: "/dansedesyols",
        },
        (62, 579) => LocalisedText {
            en: "/splash",
            ja: "/splash",
            de: "/wasserschlacht",
            fr: "/éclabousser",
        },
        (62, 581) => LocalisedText {
            en: "/sweat",
            ja: "/sweat",
            de: "/heiß",
            fr: "/transpirer",
        },
        (62, 582) => LocalisedText {
            en: "/shiver",
            ja: "/shiver",
            de: "/kalt",
            fr: "/grelotter",
        },
        (62, 583) => LocalisedText {
            en: "/elucidate",
            ja: "/elucidate",
            de: "/erklären",
            fr: "/élucider",
        },
        (62, 584) => LocalisedText {
            en: "/ponder",
            ja: "/ponder",
            de: "/ahnungslos",
            fr: "/songeur",
        },
        (62, 585) => LocalisedText {
            en: "/leftwink",
            ja: "/leftwink",
            de: "/zwinkerlinks",
            fr: "/clindoeilgauche",
        },
        (62, 586) => LocalisedText {
            en: "/getfantasy",
            ja: "/getfantasy",
            de: "/funkyfantasy",
            fr: "/fantaisieurbaine",
        },
        (62, 587) => LocalisedText {
            en: "/popotostep",
            ja: "/popotostep",
            de: "/toffeltanz",
            fr: "/poppingpopoto",
        },
        (62, 588) => LocalisedText {
            en: "/hum",
            ja: "/hum",
            de: "/summen",
            fr: "/fredonner",
        },
        (62, 589) => LocalisedText {
            en: "/confirm",
            ja: "/confirm",
            de: "/vergewissern",
            fr: "/confirmer",
        },
        (62, 590) => LocalisedText {
            en: "/scheme",
            ja: "/scheme",
            de: "/plan",
            fr: "/expliquerplan",
        },
        (62, 591) => LocalisedText {
            en: "/endure",
            ja: "/endure",
            de: "/tapfer",
            fr: "/crispé",
        },
        (62, 592) => LocalisedText {
            en: "/tomestone",
            ja: "/tomestone",
            de: "/allagischerstein",
            fr: "/mémoquartz",
        },
        (62, 593) => LocalisedText {
            en: "/heeltoe",
            ja: "/heeltoe",
            de: "/salontanz",
            fr: "/dansesautillante",
        },
        (62, 594) => LocalisedText {
            en: "/goobbuedo",
            ja: "/goobbuedo",
            de: "/goobbuepoplock",
            fr: "/dansegoobbue",
        },
        (62, 595) => LocalisedText {
            en: "/gratuity",
            ja: "/gratuity",
            de: "/spendierhose",
            fr: "/charitéchafouine",
        },
        (62, 596) => LocalisedText {
            en: "/fistpump",
            ja: "/fistpump",
            de: "/gewinnerpose",
            fr: "/poingair",
        },
        (62, 597) => LocalisedText {
            en: "/reprimand",
            ja: "/reprimand",
            de: "/ermahnen",
            fr: "/réprimander",
        },
        (62, 598) => LocalisedText {
            en: "/sabotender",
            ja: "/sabotender",
            de: "/kaktor",
            fr: "/pasdepampa",
        },
        (62, 599) => LocalisedText {
            en: "/mandervillemambo",
            ja: "/mandervillemambo",
            de: "/mandervillemambo",
            fr: "/mambogentilhomme",
        },
        (62, 600) => LocalisedText {
            en: "/laliho",
            ja: "/laliho",
            de: "/holladrio",
            fr: "/laliho",
        },
        (62, 601) => LocalisedText {
            en: "/simulationm",
            ja: "/simulationm",
            de: "/omegam",
            fr: "/simulationm",
        },
        (62, 602) => LocalisedText {
            en: "/simulationf",
            ja: "/simulationf",
            de: "/omegaf",
            fr: "/simulationf",
        },
        (62, 603) => LocalisedText {
            en: "/toast",
            ja: "/toast",
            de: "/prosit",
            fr: "/trinquer",
        },
        (62, 604) => LocalisedText {
            en: "/lean",
            ja: "/lean",
            de: "/anlehnen",
            fr: "/sepencher",
        },
        (62, 605) => LocalisedText {
            en: "/headache",
            ja: "/headache",
            de: "/kopfschmerz",
            fr: "/migraine",
        },
        (62, 606) => LocalisedText {
            en: "/snap",
            ja: "/snap",
            de: "/fingerschnippen",
            fr: "/claquerdesdoigts",
        },
        (62, 607) => LocalisedText {
            en: "/breakfast",
            ja: "/breakfast",
            de: "/brotessen",
            fr: "/mangerdupain",
        },
        (62, 608) => LocalisedText {
            en: "/read",
            ja: "/read",
            de: "/buchlektüre",
            fr: "/lireunlivre",
        },
        (62, 609) => LocalisedText {
            en: "/insist",
            ja: "/insist",
            de: "/insistieren",
            fr: "/insister",
        },
        (62, 610) => LocalisedText {
            en: "/consider",
            ja: "/consider",
            de: "/hm",
            fr: "/sinterroger",
        },
        (62, 611) => LocalisedText {
            en: "/wasshoi",
            ja: "/wasshoi",
            de: "/wasshoi",
            fr: "/ohhisse",
        },
        (62, 612) => LocalisedText {
            en: "/flowershower",
            ja: "/flowershower",
            de: "/blütenregen",
            fr: "/pluiedepétales",
        },
        (62, 613) => LocalisedText {
            en: "/flamedance",
            ja: "/flamedance",
            de: "/flammentanz",
            fr: "/dansedesflammes",
        },
        (62, 614) => LocalisedText {
            en: "/highfive",
            ja: "/highfive",
            de: "/schlagein",
            fr: "/topelà",
        },
        (62, 615) => LocalisedText {
            en: "/guard",
            ja: "/guard",
            de: "/wachsam",
            fr: "/monterlagarde",
        },
        (62, 616) => LocalisedText {
            en: "/malevolence",
            ja: "/malevolence",
            de: "/fluch",
            fr: "/occultiste",
        },
        (62, 617) => LocalisedText {
            en: "/beesknees",
            ja: "/beesknees",
            de: "/bienenswing",
            fr: "/dansedesabeilles",
        },
        (62, 618) => LocalisedText {
            en: "/lalihop",
            ja: "/lalihop",
            de: "/holladrihopsasa",
            fr: "/lali-hop",
        },
        (62, 621) => LocalisedText {
            en: "/eatriceball",
            ja: "/eatriceball",
            de: "/onigiriessen",
            fr: "/mangerunonigiri",
        },
        (62, 622) => LocalisedText {
            en: "/eatapple",
            ja: "/eatapple",
            de: "/apfelessen",
            fr: "/mangerunepomme",
        },
        (62, 623) => LocalisedText {
            en: "/wringhands",
            ja: "/wringhands",
            de: "/händereiben",
            fr: "/sefrotterlesmains",
        },
        (62, 624) => LocalisedText {
            en: "/sweep",
            ja: "/sweep",
            de: "/kehren",
            fr: "/balayer",
        },
        (62, 625) => LocalisedText {
            en: "/paintblack",
            ja: "/paintblack",
            de: "/pinselschwarz",
            fr: "/pinchoconoir",
        },
        (62, 626) => LocalisedText {
            en: "/paintred",
            ja: "/paintred",
            de: "/pinselrot",
            fr: "/pinchocorouge",
        },
        (62, 627) => LocalisedText {
            en: "/paintyellow",
            ja: "/paintyellow",
            de: "/pinselgelb",
            fr: "/pinchocojaune",
        },
        (62, 628) => LocalisedText {
            en: "/paintblue",
            ja: "/paintblue",
            de: "/pinselblau",
            fr: "/pinchocobleu",
        },
        (62, 629) => LocalisedText {
            en: "/fakesmile",
            ja: "/fakesmile",
            de: "/aufgesetzteslächeln",
            fr: "/sourireforcé",
        },
        (62, 630) => LocalisedText {
            en: "/pantomime",
            ja: "/pantomime",
            de: "/pmime",
            fr: "/pantomime",
        },
        (62, 631) => LocalisedText {
            en: "/vexed",
            ja: "/vexed",
            de: "/grübeln",
            fr: "/mécontentement",
        },
        (62, 632) => LocalisedText {
            en: "/shush",
            ja: "/shush",
            de: "/pst",
            fr: "/chut",
        },
        (62, 633) => LocalisedText {
            en: "/eatpizza",
            ja: "/eatpizza",
            de: "/pizzaessen",
            fr: "/mangerunepizza",
        },
        (62, 634) => LocalisedText {
            en: "/clutchhead",
            ja: "/clutchhead",
            de: "/kopffassen",
            fr: "/setenirlatête",
        },
        (62, 637) => LocalisedText {
            en: "/content",
            ja: "/content",
            de: "/glückselig",
            fr: "/content",
        },
        (62, 638) => LocalisedText {
            en: "/sheathe",
            ja: "/sheathe",
            de: "/wegstecken",
            fr: "/rengainer",
        },
        (62, 639) => LocalisedText {
            en: "/draw",
            ja: "/draw",
            de: "/ziehen",
            fr: "/dégainer",
        },
        (62, 641) => LocalisedText {
            en: "/determined",
            ja: "/determined",
            de: "/entschlossen",
            fr: "/détermination",
        },
        (63, 29054) => LocalisedText {
            en: "Guard",
            ja: "防御",
            de: "Wehr",
            fr: "Garde",
        },
        (63, 29055) => LocalisedText {
            en: "Standard-issue Elixir",
            ja: "軍用エリクサー",
            de: "Frontenelixier ",
            fr: "Élixir militaire",
        },
        (63, 29056) => LocalisedText {
            en: "Purify",
            ja: "浄化",
            de: "Purifikation",
            fr: "Dépuration",
        },
        (63, 29057) => LocalisedText {
            en: "Sprint",
            ja: "スプリント",
            de: "Sprint",
            fr: "Sprint",
        },
        (63, 29058) => LocalisedText {
            en: "Fast Blade",
            ja: "ファストブレード",
            de: "Vortexschnitt",
            fr: "Lame rapide",
        },
        (63, 29059) => LocalisedText {
            en: "Riot Blade",
            ja: "ライオットソード",
            de: "Donnerklinge",
            fr: "Lame déchaînée",
        },
        (63, 29060) => LocalisedText {
            en: "Royal Authority",
            ja: "ロイヤルアソリティ",
            de: "Königsklinge",
            fr: "Autorité royale",
        },
        (63, 29061) => LocalisedText {
            en: "Atonement",
            ja: "ロイエ",
            de: "Absolution",
            fr: "Absolution",
        },
        (63, 29064) => LocalisedText {
            en: "Shield Bash",
            ja: "シールドバッシュ",
            de: "Schildschlag",
            fr: "Coup de bouclier",
        },
        (63, 29065) => LocalisedText {
            en: "Intervene",
            ja: "インターヴィーン",
            de: "Einschreiten",
            fr: "Irruption brutale",
        },
        (63, 29066) => LocalisedText {
            en: "Guardian",
            ja: "ガーディアン",
            de: "Wächter",
            fr: "Gardien",
        },
        (63, 29067) => LocalisedText {
            en: "Holy Sheltron",
            ja: "ホーリーシェルトロン",
            de: "Heiliges Schiltron",
            fr: "Schiltron sacré",
        },
        (63, 29069) => LocalisedText {
            en: "Phalanx",
            ja: "ファランクス",
            de: "Phalanx",
            fr: "Phalanx",
        },
        (63, 29070) => LocalisedText {
            en: "Confiteor",
            ja: "コンフィテオル",
            de: "Confiteor",
            fr: "Confiteor",
        },
        (63, 29071) => LocalisedText {
            en: "Blade of Faith",
            ja: "ブレード・オブ・フェイス",
            de: "Klinge der Frömmigkeit",
            fr: "Lame de foi",
        },
        (63, 29072) => LocalisedText {
            en: "Blade of Truth",
            ja: "ブレード・オブ・トゥルース",
            de: "Klinge der Redlichkeit",
            fr: "Lame de vérité",
        },
        (63, 29073) => LocalisedText {
            en: "Blade of Valor",
            ja: "ブレード・オブ・ヴァラー",
            de: "Klinge der Tapferkeit",
            fr: "Lame de vaillance",
        },
        (63, 29074) => LocalisedText {
            en: "Heavy Swing",
            ja: "ヘヴィスウィング",
            de: "Gewaltiger Hieb",
            fr: "Coup puissant",
        },
        (63, 29075) => LocalisedText {
            en: "Maim",
            ja: "メイム",
            de: "Verstümmelung",
            fr: "Mutilation",
        },
        (63, 29076) => LocalisedText {
            en: "Storm's Path",
            ja: "シュトルムヴィント",
            de: "Sturmkeil",
            fr: "Couperet de justice",
        },
        (63, 29078) => LocalisedText {
            en: "Fell Cleave",
            ja: "フェルクリーヴ",
            de: "Niedermähen",
            fr: "Sape-fendeur",
        },
        (63, 29079) => LocalisedText {
            en: "Onslaught",
            ja: "オンスロート",
            de: "Sturmlauf",
            fr: "Assaut violent",
        },
        (63, 29080) => LocalisedText {
            en: "Orogeny",
            ja: "オロジェネシス",
            de: "Orogenese",
            fr: "Orogenèse",
        },
        (63, 29081) => LocalisedText {
            en: "Blota",
            ja: "ブロート",
            de: "Blot",
            fr: "Libation",
        },
        (63, 29082) => LocalisedText {
            en: "Bloodwhetting",
            ja: "原初の血気",
            de: "Urimpuls",
            fr: "Intuition fougueuse",
        },
        (63, 29083) => LocalisedText {
            en: "Primal Scream",
            ja: "原初の怒号",
            de: "Urschrei",
            fr: "Rugissement bestial",
        },
        (63, 29084) => LocalisedText {
            en: "Primal Rend",
            ja: "プライマルレンド",
            de: "Brachiale Gewalt",
            fr: "Griffade primitive",
        },
        (63, 29085) => LocalisedText {
            en: "Hard Slash",
            ja: "ヘヴィスラッシュ",
            de: "Gewaltiger Schlitzer",
            fr: "Taillade puissante",
        },
        (63, 29086) => LocalisedText {
            en: "Syphon Strike",
            ja: "サイフォンストライク",
            de: "Osmose",
            fr: "Frappe siphon",
        },
        (63, 29087) => LocalisedText {
            en: "Souleater",
            ja: "ソウルイーター",
            de: "Seelenfresser",
            fr: "Dévoreur d'âme",
        },
        (63, 29088) => LocalisedText {
            en: "Bloodspiller",
            ja: "ブラッドスピラー",
            de: "Blutvergießen",
            fr: "Marée de sang",
        },
        (63, 29091) => LocalisedText {
            en: "Shadowbringer",
            ja: "シャドウブリンガー",
            de: "Schattenbringer",
            fr: "Porteur d'ombre",
        },
        (63, 29092) => LocalisedText {
            en: "Plunge",
            ja: "プランジカット",
            de: "Hiebsprung",
            fr: "Coupe plongeante",
        },
        (63, 29093) => LocalisedText {
            en: "the Blackest Night",
            ja: "ブラックナイト",
            de: "Schwärzeste Nacht",
            fr: "Nuit noirissime",
        },
        (63, 29094) => LocalisedText {
            en: "Salted Earth",
            ja: "ソルトアース",
            de: "Salzige Erde",
            fr: "Terre salée",
        },
        (63, 29095) => LocalisedText {
            en: "Salt and Darkness",
            ja: "ソルト・アンド・ダーク",
            de: "Salz und Schwärze",
            fr: "Sel et Ténèbres",
        },
        (63, 29097) => LocalisedText {
            en: "Eventide",
            ja: "イーブンタイド",
            de: "Dämmerung",
            fr: "Crépuscule",
        },
        (63, 29098) => LocalisedText {
            en: "Keen Edge",
            ja: "キーンエッジ",
            de: "Zerfetzer",
            fr: "Tranchant acéré",
        },
        (63, 29099) => LocalisedText {
            en: "Brutal Shell",
            ja: "ブルータルシェル",
            de: "Brutaler Einschlag",
            fr: "Impact brutal",
        },
        (63, 29100) => LocalisedText {
            en: "Solid Barrel",
            ja: "ソリッドバレル",
            de: "Stahlwalze",
            fr: "Salve tranchante",
        },
        (63, 29101) => LocalisedText {
            en: "Burst Strike",
            ja: "バーストストライク",
            de: "Explosiver Stoß",
            fr: "Frappe explosive",
        },
        (63, 29102) => LocalisedText {
            en: "Gnashing Fang",
            ja: "ビートファング",
            de: "Schlagfänge",
            fr: "Croc pugnace",
        },
        (63, 29103) => LocalisedText {
            en: "Savage Claw",
            ja: "サベッジクロウ",
            de: "Wilde Klauen",
            fr: "Griffe sauvage",
        },
        (63, 29104) => LocalisedText {
            en: "Wicked Talon",
            ja: "ウィケッドタロン",
            de: "Tückische Kralle",
            fr: "Serre vicieuse",
        },
        (63, 29105) => LocalisedText {
            en: "Double Down",
            ja: "ダブルダウン",
            de: "Doppelpack",
            fr: "Acharnement",
        },
        (63, 29106) => LocalisedText {
            en: "Continuation",
            ja: "コンティニュエーション",
            de: "Triggerfolge",
            fr: "Consécution",
        },
        (63, 29107) => LocalisedText {
            en: "Hypervelocity",
            ja: "ハイパーヴェロシティ",
            de: "Überschall",
            fr: "Hypervitesse",
        },
        (63, 29108) => LocalisedText {
            en: "Jugular Rip",
            ja: "ジャギュラーリップ",
            de: "Kehlreißer",
            fr: "Arrachage de la jugulaire",
        },
        (63, 29109) => LocalisedText {
            en: "Abdomen Tear",
            ja: "アブドメンテアー",
            de: "Bauchschlitzer",
            fr: "Lacération de l'abdomen",
        },
        (63, 29110) => LocalisedText {
            en: "Eye Gouge",
            ja: "アイガウジ",
            de: "Augenstecher",
            fr: "Perforation des yeux",
        },
        (63, 29123) => LocalisedText {
            en: "Rough Divide",
            ja: "ラフディバイド",
            de: "Grobspalter",
            fr: "Lamineur",
        },
        (63, 29124) => LocalisedText {
            en: "Draw and Junction",
            ja: "ブラッドドロー",
            de: "Blut-Erda",
            fr: "Giclée de sang",
        },
        (63, 29125) => LocalisedText {
            en: "Junctioned Cast",
            ja: "ブラッドキャスト",
            de: "Gekoppelter Zauber",
            fr: "Prise de sang",
        },
        (63, 29126) => LocalisedText {
            en: "Nebula",
            ja: "ネビュラ",
            de: "Nebula",
            fr: "Nébuleuse",
        },
        (63, 29128) => LocalisedText {
            en: "Blasting Zone",
            ja: "ブラスティングゾーン",
            de: "Erda-Detonation",
            fr: "Zone de destruction",
        },
        (63, 29129) => LocalisedText {
            en: "Aurora",
            ja: "オーロラ",
            de: "Aurora",
            fr: "Aurore",
        },
        (63, 29130) => LocalisedText {
            en: "Relentless Rush",
            ja: "連続剣",
            de: "Multi",
            fr: "Multi-taillade",
        },
        (63, 29223) => LocalisedText {
            en: "Glare III",
            ja: "グレアガ",
            de: "Blendga",
            fr: "Méga Chatoiement",
        },
        (63, 29224) => LocalisedText {
            en: "Cure II",
            ja: "ケアルラ",
            de: "Vitra",
            fr: "Extra Soin",
        },
        (63, 29225) => LocalisedText {
            en: "Cure III",
            ja: "ケアルガ",
            de: "Vitaga",
            fr: "Méga Soin",
        },
        (63, 29226) => LocalisedText {
            en: "Afflatus Misery",
            ja: "ハート・オブ・ミゼリ",
            de: "Afflatus miseriae",
            fr: "Offrande de misère",
        },
        (63, 29227) => LocalisedText {
            en: "Aquaveil",
            ja: "アクアヴェール",
            de: "Wasserschleier",
            fr: "Aquavoile",
        },
        (63, 29228) => LocalisedText {
            en: "Miracle of Nature",
            ja: "ミラクル・オブ・ネイチャー",
            de: "Wunder der Natur",
            fr: "Miracle de la nature",
        },
        (63, 29229) => LocalisedText {
            en: "Seraph Strike",
            ja: "セラフストライク",
            de: "Seraphenschlag",
            fr: "Frappe séraphine",
        },
        (63, 29230) => LocalisedText {
            en: "Afflatus Purgation",
            ja: "ハート・オブ・パーゲーション",
            de: "Afflatus purgatio",
            fr: "Purge fondamentale",
        },
        (63, 29231) => LocalisedText {
            en: "Broil IV",
            ja: "極炎法",
            de: "Rabiate Bravade",
            fr: "Loi de l'ébullition IV",
        },
        (63, 29232) => LocalisedText {
            en: "Adloquium",
            ja: "鼓舞激励の策",
            de: "Adloquium",
            fr: "Traité du réconfort",
        },
        (63, 29233) => LocalisedText {
            en: "Biolysis",
            ja: "蠱毒法",
            de: "Causae et Curae",
            fr: "Loi de l'infection",
        },
        (63, 29234) => LocalisedText {
            en: "Deployment Tactics",
            ja: "展開戦術",
            de: "Dislokation",
            fr: "Stratagème du déploiement",
        },
        (63, 29235) => LocalisedText {
            en: "Mummification",
            ja: "枯骨法",
            de: "Mumifizierung",
            fr: "Momification",
        },
        (63, 29236) => LocalisedText {
            en: "Expedient",
            ja: "疾風怒濤の計",
            de: "Sturm und Drang",
            fr: "Thèse fluidique",
        },
        (63, 29237) => LocalisedText {
            en: "Summon Seraph",
            ja: "サモン・セラフィム",
            de: "Seraph-Beschwörung",
            fr: "Invocation Séraphin",
        },
        (63, 29238) => LocalisedText {
            en: "Consolation",
            ja: "コンソレイション",
            de: "Seelentrost",
            fr: "Dictame",
        },
        (63, 29239) => LocalisedText {
            en: "Seraph Flight",
            ja: "セラフィムウィング",
            de: "Flug der Seraphim",
            fr: "Aile de Séraphin",
        },
        (63, 29240) => LocalisedText {
            en: "Seraphic Veil",
            ja: "セラフィックヴェール",
            de: "Schleier der Seraphim",
            fr: "Voile séraphique",
        },
        (63, 29242) => LocalisedText {
            en: "Fall Malefic",
            ja: "フォールマレフィク",
            de: "Aspekt im Fall",
            fr: "Malfaisance en chute",
        },
        (63, 29243) => LocalisedText {
            en: "Aspected Benefic",
            ja: "アスペクト・ベネフィク",
            de: "Harmonischer Orbis",
            fr: "Bienfaisance aspectée",
        },
        (63, 29244) => LocalisedText {
            en: "Gravity II",
            ja: "グラビラ",
            de: "Gravitra",
            fr: "Extra Gravité",
        },
        (63, 29245) => LocalisedText {
            en: "Double Cast",
            ja: "ダブルキャスト",
            de: "Zweifacher Zauber",
            fr: "Double incantation",
        },
        (63, 29249) => LocalisedText {
            en: "Draw",
            ja: "ドロー",
            de: "Ziehen",
            fr: "Tirage",
        },
        (63, 29250) => LocalisedText {
            en: "the Balance",
            ja: "アーゼマの均衡",
            de: "Waage",
            fr: "La Balance",
        },
        (63, 29251) => LocalisedText {
            en: "the Ewer",
            ja: "サリャクの水瓶",
            de: "Krug",
            fr: "L'Aiguière",
        },
        (63, 29252) => LocalisedText {
            en: "the Spire",
            ja: "ビエルゴの塔",
            de: "Turm",
            fr: "La Tour",
        },
        (63, 29253) => LocalisedText {
            en: "Macrocosmos",
            ja: "マクロコスモス",
            de: "Makrokosmos",
            fr: "Macrocosme",
        },
        (63, 29254) => LocalisedText {
            en: "Microcosmos",
            ja: "ミクロコスモス",
            de: "Mikrokosmos",
            fr: "Microcosme",
        },
        (63, 29255) => LocalisedText {
            en: "Celestial River",
            ja: "星河一天",
            de: "Säule der Herrlichkeit",
            fr: "Rivière céleste",
        },
        (63, 29256) => LocalisedText {
            en: "Dosis III",
            ja: "ドシスIII",
            de: "Dosis III",
            fr: "Dosis III",
        },
        (63, 29257) => LocalisedText {
            en: "Eukrasian Dosis III",
            ja: "エウクラシア・ドシスIII",
            de: "Eukratische Dosis III",
            fr: "Dosis eucrasique III",
        },
        (63, 29258) => LocalisedText {
            en: "Eukrasia",
            ja: "エウクラシア",
            de: "Eukrasie",
            fr: "Eukrasia",
        },
        (63, 29259) => LocalisedText {
            en: "Phlegma III",
            ja: "フレグマIII",
            de: "Phlegma III",
            fr: "Phlegma III",
        },
        (63, 29260) => LocalisedText {
            en: "Pneuma",
            ja: "プネウマ",
            de: "Pneuma",
            fr: "Pneuma",
        },
        (63, 29261) => LocalisedText {
            en: "Icarus",
            ja: "イカロス",
            de: "Ikarus",
            fr: "Ikaros",
        },
        (63, 29262) => LocalisedText {
            en: "Toxikon",
            ja: "トキシコン",
            de: "Toxikon",
            fr: "Toxikon",
        },
        (63, 29263) => LocalisedText {
            en: "Toxikon II",
            ja: "トキシコンII",
            de: "Toxikon II",
            fr: "Toxikon II",
        },
        (63, 29264) => LocalisedText {
            en: "Kardia",
            ja: "カルディア",
            de: "Kardia",
            fr: "Kardia",
        },
        (63, 29266) => LocalisedText {
            en: "Mesotes",
            ja: "メソテース",
            de: "Mesotes",
            fr: "Mesotes",
        },
        (63, 29371) => LocalisedText {
            en: "Foul",
            ja: "ファウル",
            de: "Moder",
            fr: "Infect",
        },
        (63, 29391) => LocalisedText {
            en: "Powerful Shot",
            ja: "パワーショット",
            de: "Mächtiger Schuss",
            fr: "Tir puissant",
        },
        (63, 29392) => LocalisedText {
            en: "Pitch Perfect",
            ja: "ピッチパーフェクト",
            de: "Absolutes Gehör",
            fr: "Ton parfait",
        },
        (63, 29393) => LocalisedText {
            en: "Apex Arrow",
            ja: "エイペックスアロー",
            de: "Apex-Pfeil",
            fr: "Trait alpha",
        },
        (63, 29394) => LocalisedText {
            en: "Blast Arrow",
            ja: "ブラストアロー",
            de: "Stoßpfeil",
            fr: "Flèche explosive",
        },
        (63, 29395) => LocalisedText {
            en: "Silent Nocturne",
            ja: "黙者のノクターン",
            de: "Stumme Nocturne",
            fr: "Nocturne muet",
        },
        (63, 29396) => LocalisedText {
            en: "Empyreal Arrow",
            ja: "エンピリアルアロー",
            de: "Empyreischer Pfeil",
            fr: "Flèche empyréenne",
        },
        (63, 29399) => LocalisedText {
            en: "Repelling Shot",
            ja: "リペリングショット",
            de: "Mannstopper",
            fr: "Tir repoussant",
        },
        (63, 29400) => LocalisedText {
            en: "the Warden's Paean",
            ja: "時神のピーアン",
            de: "Päan des Hüters",
            fr: "Péon du Contemplateur",
        },
        (63, 29401) => LocalisedText {
            en: "Final Fantasia",
            ja: "英雄のファンタジア",
            de: "Fantasie des Helden",
            fr: "Fantaisie héroïque",
        },
        (63, 29402) => LocalisedText {
            en: "Blast Charge",
            ja: "チャージブラスト",
            de: "Explosiver Schuss",
            fr: "Souffle explosif",
        },
        (63, 29403) => LocalisedText {
            en: "Heat Blast",
            ja: "ヒートブラスト",
            de: "Hitzeschuss",
            fr: "Pyro-explosion",
        },
        (63, 29404) => LocalisedText {
            en: "Scattergun",
            ja: "スキャッターガン",
            de: "Schrotflinte",
            fr: "Fusil à pompe",
        },
        (63, 29405) => LocalisedText {
            en: "Drill",
            ja: "ドリル",
            de: "Bohrer",
            fr: "Foreuse",
        },
        (63, 29406) => LocalisedText {
            en: "Bioblaster",
            ja: "バイオブラスト",
            de: "Bioblaster",
            fr: "Lance-toxine",
        },
        (63, 29407) => LocalisedText {
            en: "Air Anchor",
            ja: "エアアンカー",
            de: "Luftanker",
            fr: "Ancre aérienne",
        },
        (63, 29408) => LocalisedText {
            en: "Chain Saw",
            ja: "回転のこぎり",
            de: "Kreissäge",
            fr: "Tronçonneuses rotatives",
        },
        (63, 29409) => LocalisedText {
            en: "Wildfire",
            ja: "ワイルドファイア",
            de: "Wildfeuer",
            fr: "Flambée",
        },
        (63, 29412) => LocalisedText {
            en: "Bishop Autoturret",
            ja: "オートタレット・ビショップ",
            de: "Selbstschuss-Gyrocopter LÄUFER",
            fr: "Auto-tourelle Fou",
        },
        (63, 29413) => LocalisedText {
            en: "Aether Mortar",
            ja: "エーテルモーター",
            de: "Äthermörser",
            fr: "Mortier d'éther",
        },
        (63, 29414) => LocalisedText {
            en: "Analysis",
            ja: "アナライズ",
            de: "Analyse",
            fr: "Analyse",
        },
        (63, 29415) => LocalisedText {
            en: "Marksman's Spite",
            ja: "魔弾の射手",
            de: "Freischütz",
            fr: "Fusil mystique",
        },
        (63, 29416) => LocalisedText {
            en: "Cascade",
            ja: "カスケード",
            de: "Cascade",
            fr: "Cascade",
        },
        (63, 29417) => LocalisedText {
            en: "Fountain",
            ja: "ファウンテン",
            de: "Fontäne",
            fr: "Fontaine",
        },
        (63, 29418) => LocalisedText {
            en: "Reverse Cascade",
            ja: "リバースカスケード",
            de: "Invertierte Cascade",
            fr: "Cascade inversée",
        },
        (63, 29419) => LocalisedText {
            en: "Fountainfall",
            ja: "ファウンテンフォール",
            de: "Fallende Fontäne",
            fr: "Fontaine en cascade",
        },
        (63, 29420) => LocalisedText {
            en: "Saber Dance",
            ja: "剣の舞い",
            de: "Schwerttanz",
            fr: "Danse du sabre",
        },
        (63, 29421) => LocalisedText {
            en: "Starfall Dance",
            ja: "流星の舞い",
            de: "Sternenfalltanz",
            fr: "Danse de la pluie étoilée",
        },
        (63, 29422) => LocalisedText {
            en: "Honing Dance",
            ja: "刃の舞い",
            de: "Klingentanz",
            fr: "Danse tranchante",
        },
        (63, 29423) => LocalisedText {
            en: "Honing Ovation",
            ja: "刃の舞い【終】",
            de: "Scharfes Finale",
            fr: "Final tranchant",
        },
        (63, 29428) => LocalisedText {
            en: "Fan Dance",
            ja: "扇の舞い",
            de: "Fächertanz",
            fr: "Danse de l'éventail",
        },
        (63, 29429) => LocalisedText {
            en: "Curing Waltz",
            ja: "癒やしのワルツ",
            de: "Heilender Walzer",
            fr: "Valse revigorante",
        },
        (63, 29430) => LocalisedText {
            en: "En Avant",
            ja: "アン・アヴァン",
            de: "En Avant",
            fr: "En avant",
        },
        (63, 29431) => LocalisedText {
            en: "Closed Position",
            ja: "クローズドポジション",
            de: "Tanzpartner",
            fr: "Position rapprochée",
        },
        (63, 29432) => LocalisedText {
            en: "Contradance",
            ja: "コントラダンス",
            de: "Kontratanz",
            fr: "Contredanse",
        },
        (63, 29469) => LocalisedText {
            en: "Terminal Trigger",
            ja: "フィニッシュトリガー",
            de: "Finaler Trigger",
            fr: "Gâchette terminale",
        },
        (63, 29472) => LocalisedText {
            en: "Bootshine",
            ja: "連撃",
            de: "Schlag auf Schlag",
            fr: "Volée de coups",
        },
        (63, 29473) => LocalisedText {
            en: "True Strike",
            ja: "正拳突き",
            de: "Wahrer Schlag",
            fr: "Coup de poing direct",
        },
        (63, 29474) => LocalisedText {
            en: "Snap Punch",
            ja: "崩拳",
            de: "Peitschenhieb",
            fr: "Coup de poing brusque",
        },
        (63, 29475) => LocalisedText {
            en: "Dragon Kick",
            ja: "双竜脚",
            de: "Drachentritt",
            fr: "Tacle du dragon",
        },
        (63, 29476) => LocalisedText {
            en: "Twin Snakes",
            ja: "双掌打",
            de: "Doppelviper",
            fr: "Serpents jumeaux",
        },
        (63, 29477) => LocalisedText {
            en: "Demolish",
            ja: "破砕拳",
            de: "Demolieren",
            fr: "Démolition",
        },
        (63, 29478) => LocalisedText {
            en: "Phantom Rush",
            ja: "夢幻闘舞",
            de: "Phantomstoß",
            fr: "Rixe évanescente",
        },
        (63, 29479) => LocalisedText {
            en: "Six-sided Star",
            ja: "六合星導脚",
            de: "Sechsstern-Tritt",
            fr: "Étoile à six branches",
        },
        (63, 29480) => LocalisedText {
            en: "Enlightenment",
            ja: "万象闘気圏",
            de: "Weg zur Erleuchtung",
            fr: "Illumination",
        },
        (63, 29481) => LocalisedText {
            en: "Rising Phoenix",
            ja: "鳳凰の舞",
            de: "Phönixtanz",
            fr: "Danse du Phénix",
        },
        (63, 29482) => LocalisedText {
            en: "Riddle of Earth",
            ja: "金剛の極意",
            de: "Steinernes Enigma",
            fr: "Énigme de la terre",
        },
        (63, 29483) => LocalisedText {
            en: "Earth's Reply",
            ja: "金剛転輪",
            de: "Echo der Erde",
            fr: "Cercles de terre",
        },
        (63, 29484) => LocalisedText {
            en: "Thunderclap",
            ja: "抜重歩法",
            de: "Donnersprung",
            fr: "Élan fulgurant",
        },
        (63, 29485) => LocalisedText {
            en: "Meteodrive",
            ja: "メテオドライヴ",
            de: "Meteorschub",
            fr: "Plongeon météore",
        },
        (63, 29486) => LocalisedText {
            en: "Raiden Thrust",
            ja: "竜眼雷電",
            de: "Überfallsstoß",
            fr: "Percée Raiden",
        },
        (63, 29487) => LocalisedText {
            en: "Fang and Claw",
            ja: "竜牙竜爪",
            de: "Fang und Klaue",
            fr: "Croc-et-griffe",
        },
        (63, 29488) => LocalisedText {
            en: "Wheeling Thrust",
            ja: "竜尾大車輪",
            de: "Fächerstoß",
            fr: "Percée tournante",
        },
        (63, 29489) => LocalisedText {
            en: "Heavens' Thrust",
            ja: "ヘヴンスラスト",
            de: "Himmelsstoß",
            fr: "Percée céleste",
        },
        (63, 29490) => LocalisedText {
            en: "Chaotic Spring",
            ja: "桜華繚乱",
            de: "Chaotischer Sturm",
            fr: "Floraison chaotique",
        },
        (63, 29491) => LocalisedText {
            en: "Geirskogul",
            ja: "ゲイルスコグル",
            de: "Geirskogul",
            fr: "Geirskögul",
        },
        (63, 29492) => LocalisedText {
            en: "Nastrond",
            ja: "ナーストレンド",
            de: "Nastrond",
            fr: "Naströnd",
        },
        (63, 29493) => LocalisedText {
            en: "High Jump",
            ja: "ハイジャンプ",
            de: "Hochsprung",
            fr: "Grand saut",
        },
        (63, 29494) => LocalisedText {
            en: "Elusive Jump",
            ja: "イルーシブジャンプ",
            de: "Ausweichsprung",
            fr: "Saut insaisissable",
        },
        (63, 29495) => LocalisedText {
            en: "Wyrmwind Thrust",
            ja: "天竜点睛",
            de: "Zwillingswyrm",
            fr: "Percée des dragons anciens",
        },
        (63, 29496) => LocalisedText {
            en: "Horrid Roar",
            ja: "ホリッドロア",
            de: "Entsetzliches Brüllen",
            fr: "Rugissement horrible",
        },
        (63, 29497) => LocalisedText {
            en: "Sky High",
            ja: "スカイハイ",
            de: "Himmelhoch",
            fr: "Tourbillon vertigineux",
        },
        (63, 29499) => LocalisedText {
            en: "Sky Shatter",
            ja: "スカイシャッター",
            de: "Himmelsbrecher",
            fr: "Briseur des cieux",
        },
        (63, 29500) => LocalisedText {
            en: "Spinning Edge",
            ja: "双刃旋",
            de: "Wirbelklinge",
            fr: "Lame tournoyante",
        },
        (63, 29501) => LocalisedText {
            en: "Gust Slash",
            ja: "風断ち",
            de: "Windschlitzer",
            fr: "Tourbillon tailladant",
        },
        (63, 29502) => LocalisedText {
            en: "Aeolian Edge",
            ja: "旋風刃",
            de: "Äolusklinge",
            fr: "Lame éolienne",
        },
        (63, 29503) => LocalisedText {
            en: "Assassinate",
            ja: "終撃",
            de: "Meuchelsprung",
            fr: "Assassinement",
        },
        (63, 29504) => LocalisedText {
            en: "Goka Mekkyaku",
            ja: "劫火滅却の術",
            de: "Goka Mekkyaku",
            fr: "Gôka Mekkyaku",
        },
        (63, 29505) => LocalisedText {
            en: "Fuma Shuriken",
            ja: "風魔手裏剣",
            de: "Fuma Shuriken",
            fr: "Fûma Shuriken",
        },
        (63, 29506) => LocalisedText {
            en: "Hyosho Ranryu",
            ja: "氷晶乱流の術",
            de: "Hyosho Ranryu",
            fr: "Hyôshô Ranryû",
        },
        (63, 29507) => LocalisedText {
            en: "Three Mudra",
            ja: "三印自在",
            de: "Tri-Mudra",
            fr: "Trimudrâ",
        },
        (63, 29508) => LocalisedText {
            en: "Meisui",
            ja: "命水",
            de: "Meisui",
            fr: "Meisui",
        },
        (63, 29509) => LocalisedText {
            en: "Mug",
            ja: "ぶんどる",
            de: "Ausrauben",
            fr: "Rapine",
        },
        (63, 29510) => LocalisedText {
            en: "Forked Raiju",
            ja: "月影雷獣爪",
            de: "Tanzendes Raiju",
            fr: "Griffe de Raijû",
        },
        (63, 29511) => LocalisedText {
            en: "Bunshin",
            ja: "分身の術",
            de: "Doppeltes Ich",
            fr: "Bunshin",
        },
        (63, 29512) => LocalisedText {
            en: "Huton",
            ja: "風遁の術",
            de: "Huton",
            fr: "Fûton",
        },
        (63, 29513) => LocalisedText {
            en: "Shukuchi",
            ja: "縮地",
            de: "Shukuchi",
            fr: "Shukuchi",
        },
        (63, 29514) => LocalisedText {
            en: "Doton",
            ja: "土遁の術",
            de: "Doton",
            fr: "Doton",
        },
        (63, 29515) => LocalisedText {
            en: "Seiton Tenchu",
            ja: "星遁天誅",
            de: "Seiton Tenchu",
            fr: "Seiton Tenchû",
        },
        (63, 29523) => LocalisedText {
            en: "Yukikaze",
            ja: "雪風",
            de: "Yukikaze",
            fr: "Yukikaze",
        },
        (63, 29524) => LocalisedText {
            en: "Gekko",
            ja: "月光",
            de: "Gekko",
            fr: "Gekkô",
        },
        (63, 29525) => LocalisedText {
            en: "Kasha",
            ja: "花車",
            de: "Kasha",
            fr: "Kasha",
        },
        (63, 29526) => LocalisedText {
            en: "Hyosetsu",
            ja: "氷雪",
            de: "Hyosetsu",
            fr: "Hyosetsu",
        },
        (63, 29527) => LocalisedText {
            en: "Mangetsu",
            ja: "満月",
            de: "Mangetsu",
            fr: "Mangetsu",
        },
        (63, 29528) => LocalisedText {
            en: "Oka",
            ja: "桜花",
            de: "Oka",
            fr: "Ôka",
        },
        (63, 29529) => LocalisedText {
            en: "Midare Setsugekka",
            ja: "乱れ雪月花",
            de: "Midare Setsugekka",
            fr: "Midare Setsugekka",
        },
        (63, 29530) => LocalisedText {
            en: "Ogi Namikiri",
            ja: "奥義波切",
            de: "Ogi Namikiri",
            fr: "Ôgi : Namikiri",
        },
        (63, 29531) => LocalisedText {
            en: "Kaeshi: Namikiri",
            ja: "返し波切",
            de: "Kaeshi Namikiri",
            fr: "Kaeshi Namikiri",
        },
        (63, 29532) => LocalisedText {
            en: "Hissatsu: Soten",
            ja: "必殺剣・早天",
            de: "Hissatsu: Soten",
            fr: "Hissatsu : Sôten",
        },
        (63, 29533) => LocalisedText {
            en: "Hissatsu: Chiten",
            ja: "必殺剣・地天",
            de: "Hissatsu: Chiten",
            fr: "Hissatsu : Chiten",
        },
        (63, 29535) => LocalisedText {
            en: "Mineuchi",
            ja: "峰打ち",
            de: "Mineuchi",
            fr: "Mineuchi",
        },
        (63, 29536) => LocalisedText {
            en: "Meikyo Shisui",
            ja: "明鏡止水",
            de: "Meikyo Shisui",
            fr: "Meikyô Shisui",
        },
        (63, 29537) => LocalisedText {
            en: "Zantetsuken",
            ja: "斬鉄剣",
            de: "Zantetsuken",
            fr: "Zantetsuken",
        },
        (63, 29538) => LocalisedText {
            en: "Slice",
            ja: "スライス",
            de: "Schlitzer",
            fr: "Tranchage",
        },
        (63, 29539) => LocalisedText {
            en: "Waxing Slice",
            ja: "ワクシングスライス",
            de: "Neulicht-Schlitzer",
            fr: "Tranchage blafard",
        },
        (63, 29540) => LocalisedText {
            en: "Infernal Slice",
            ja: "インファナルスライス",
            de: "Infernalischer Schlitzer",
            fr: "Tranchage infernal",
        },
        (63, 29541) => LocalisedText {
            en: "Gibbet",
            ja: "ジビトゥ",
            de: "Richtpfahl",
            fr: "Gibet",
        },
        (63, 29542) => LocalisedText {
            en: "Gallows",
            ja: "ギャロウズ",
            de: "Galgen",
            fr: "Potence",
        },
        (63, 29543) => LocalisedText {
            en: "Void Reaping",
            ja: "ヴォイドリーパー",
            de: "Nichtsmäher",
            fr: "Fauchage du néant",
        },
        (63, 29544) => LocalisedText {
            en: "Cross Reaping",
            ja: "クロスリーパー",
            de: "Kreuzmäher",
            fr: "Fauchage croisé",
        },
        (63, 29545) => LocalisedText {
            en: "Harvest Moon",
            ja: "ハーベストムーン",
            de: "Erntemond",
            fr: "Lune des moissons",
        },
        (63, 29546) => LocalisedText {
            en: "Plentiful Harvest",
            ja: "プレンティフルハーベスト",
            de: "Große Mahd",
            fr: "Moisson abondante",
        },
        (63, 29547) => LocalisedText {
            en: "Grim Swathe",
            ja: "シーフスウェーズ",
            de: "Nachtschwad",
            fr: "Andain sinistre",
        },
        (63, 29548) => LocalisedText {
            en: "Lemure's Slice",
            ja: "レムールサイズ",
            de: "Lemurensense",
            fr: "Faux du lémure",
        },
        (63, 29549) => LocalisedText {
            en: "Death Warrant",
            ja: "デスワラント",
            de: "Richtbefehl",
            fr: "Arrêt de mort",
        },
        (63, 29550) => LocalisedText {
            en: "Hell's Ingress",
            ja: "ヘルズイングレス",
            de: "Höllischer Auftritt",
            fr: "Intrusion de l'enfer",
        },
        (63, 29551) => LocalisedText {
            en: "Regress",
            ja: "リターン",
            de: "Rückkehr",
            fr: "Repli",
        },
        (63, 29552) => LocalisedText {
            en: "Arcane Crest",
            ja: "アルケインクレスト",
            de: "Arkanes Wappen",
            fr: "Blason arcanique",
        },
        (63, 29553) => LocalisedText {
            en: "Tenebrae Lemurum",
            ja: "レムール・テネブレー",
            de: "Tenebrae Lemurum",
            fr: "Ténèbres du lémure",
        },
        (63, 29554) => LocalisedText {
            en: "Communio",
            ja: "コムニオ",
            de: "Communio",
            fr: "Communio",
        },
        (63, 29566) => LocalisedText {
            en: "Soul Slice",
            ja: "ソウルスライス",
            de: "Seelenschlitzer",
            fr: "Tranchage animique",
        },
        (63, 29649) => LocalisedText {
            en: "Fire",
            ja: "ファイア",
            de: "Feuer",
            fr: "Feu",
        },
        (63, 29650) => LocalisedText {
            en: "Fire IV",
            ja: "ファイジャ",
            de: "Feuka",
            fr: "Giga Feu",
        },
        (63, 29651) => LocalisedText {
            en: "Flare",
            ja: "フレア",
            de: "Flare",
            fr: "Brasier",
        },
        (63, 29653) => LocalisedText {
            en: "Blizzard",
            ja: "ブリザド",
            de: "Eis",
            fr: "Glace",
        },
        (63, 29654) => LocalisedText {
            en: "Blizzard IV",
            ja: "ブリザジャ",
            de: "Eiska",
            fr: "Giga glace",
        },
        (63, 29655) => LocalisedText {
            en: "Freeze",
            ja: "フリーズ",
            de: "Einfrieren",
            fr: "Gel",
        },
        (63, 29657) => LocalisedText {
            en: "Burst",
            ja: "バースト",
            de: "Schub",
            fr: "Bouillonnement",
        },
        (63, 29659) => LocalisedText {
            en: "Night Wing",
            ja: "ナイトウィング",
            de: "Nachtschwinge",
            fr: "Aile nocturne",
        },
        (63, 29660) => LocalisedText {
            en: "Aetherial Manipulation",
            ja: "エーテリアルステップ",
            de: "Äthermanipulation",
            fr: "Manipulation de l'éther",
        },
        (63, 29661) => LocalisedText {
            en: "Superflare",
            ja: "ヒートショック",
            de: "Supereruption",
            fr: "Choc thermique",
        },
        (63, 29662) => LocalisedText {
            en: "Soul Resonance",
            ja: "ソウルレゾナンス",
            de: "Seelenresonanz",
            fr: "Résonance de l'âme",
        },
        (63, 29663) => LocalisedText {
            en: "Paradox",
            ja: "パラドックス",
            de: "Paradox",
            fr: "Paradoxe",
        },
        (63, 29664) => LocalisedText {
            en: "Ruin III",
            ja: "ルインガ",
            de: "Ruinga",
            fr: "Méga Ruine",
        },
        (63, 29665) => LocalisedText {
            en: "Astral Impulse",
            ja: "アストラルインパルス",
            de: "Lichtimpuls",
            fr: "Pulsar astral",
        },
        (63, 29666) => LocalisedText {
            en: "Fountain of Fire",
            ja: "霊泉の炎",
            de: "Quell des Feuers",
            fr: "Flamme de la vie",
        },
        (63, 29667) => LocalisedText {
            en: "Crimson Cyclone",
            ja: "クリムゾンサイクロン",
            de: "Zinnober-Zyklon",
            fr: "Cyclone écarlate",
        },
        (63, 29668) => LocalisedText {
            en: "Crimson Strike",
            ja: "クリムゾンストライク",
            de: "Zinnober-Schlag",
            fr: "Assaut écarlate",
        },
        (63, 29669) => LocalisedText {
            en: "Slipstream",
            ja: "スリップストリーム",
            de: "Wirbelströmung",
            fr: "Sillage",
        },
        (63, 29670) => LocalisedText {
            en: "Radiant Aegis",
            ja: "守りの光",
            de: "Schimmerschild",
            fr: "Égide rayonnante",
        },
        (63, 29671) => LocalisedText {
            en: "Mountain Buster",
            ja: "マウンテンバスター",
            de: "Bergsprenger",
            fr: "Casse-montagnes",
        },
        (63, 29672) => LocalisedText {
            en: "Fester",
            ja: "ミアズマバースト",
            de: "Schwäre",
            fr: "Suppuration",
        },
        (63, 29673) => LocalisedText {
            en: "Summon Bahamut",
            ja: "サモン・バハムート",
            de: "Bahamut-Beschwörung",
            fr: "Invocation Bahamut",
        },
        (63, 29674) => LocalisedText {
            en: "Enkindle Bahamut",
            ja: "エンキンドル・バハムート",
            de: "Bahamut-Entflammung",
            fr: "Galvanisation Bahamut",
        },
        (63, 29675) => LocalisedText {
            en: "Megaflare",
            ja: "メガフレア",
            de: "Megaflare",
            fr: "MégaBrasier",
        },
        (63, 29676) => LocalisedText {
            en: "Wyrmwave",
            ja: "ウィルムウェーブ",
            de: "Drachenwelle",
            fr: "Onde draconique",
        },
        (63, 29677) => LocalisedText {
            en: "Akh Morn",
            ja: "アク・モーン",
            de: "Akh Morn",
            fr: "Akh Morn",
        },
        (63, 29678) => LocalisedText {
            en: "Summon Phoenix",
            ja: "サモン・フェニックス",
            de: "Phönix-Beschwörung",
            fr: "Invocation Phénix",
        },
        (63, 29679) => LocalisedText {
            en: "Enkindle Phoenix",
            ja: "エンキンドル・フェニックス",
            de: "Phönix-Entflammung",
            fr: "Galvanisation Phénix",
        },
        (63, 29680) => LocalisedText {
            en: "Everlasting Flight",
            ja: "不死鳥の翼",
            de: "Schwingen der Unsterblichkeit",
            fr: "Vol éternel",
        },
        (63, 29681) => LocalisedText {
            en: "Scarlet Flame",
            ja: "火焔",
            de: "Scharlachflamme",
            fr: "Flamme écarlate",
        },
        (63, 29682) => LocalisedText {
            en: "Revelation",
            ja: "リヴァレーション",
            de: "Offenbarung",
            fr: "Révélation",
        },
        (63, 29683) => LocalisedText {
            en: "Verstone",
            ja: "ヴァルストーン",
            de: "Erzstein",
            fr: "VerTerre",
        },
        (63, 29684) => LocalisedText {
            en: "Veraero III",
            ja: "ヴァルエアロガ",
            de: "Erzwindga",
            fr: "Méga VerVent",
        },
        (63, 29685) => LocalisedText {
            en: "Verholy",
            ja: "ヴァルホーリー",
            de: "Erz-Sanctus",
            fr: "VerMiracle",
        },
        (63, 29686) => LocalisedText {
            en: "Verfire",
            ja: "ヴァルファイア",
            de: "Erzfeuer",
            fr: "VerFeu",
        },
        (63, 29687) => LocalisedText {
            en: "Verthunder III",
            ja: "ヴァルサンダガ",
            de: "Erzblitzga",
            fr: "Méga VerFoudre",
        },
        (63, 29688) => LocalisedText {
            en: "Verflare",
            ja: "ヴァルフレア",
            de: "Erzflare",
            fr: "VerBrasier",
        },
        (63, 29689) => LocalisedText {
            en: "Enchanted Riposte",
            ja: "エンリポスト",
            de: "Magische Riposte",
            fr: "Riposte enchantée",
        },
        (63, 29690) => LocalisedText {
            en: "Enchanted Zwerchhau",
            ja: "エンツヴェルクハウ",
            de: "Magischer Zwerchhau",
            fr: "Zwerchhau enchanté",
        },
        (63, 29691) => LocalisedText {
            en: "Enchanted Redoublement",
            ja: "エンルドゥブルマン",
            de: "Magisches Redoublement",
            fr: "Redoublement enchanté",
        },
        (63, 29695) => LocalisedText {
            en: "Resolution",
            ja: "レゾリューション",
            de: "Entschlossenheit",
            fr: "Résolution",
        },
        (63, 29697) => LocalisedText {
            en: "Magick Barrier",
            ja: "バマジク",
            de: "Magiebarriere",
            fr: "Barrière anti-magie",
        },
        (63, 29698) => LocalisedText {
            en: "Frazzle",
            ja: "フラズル",
            de: "Fix und fertig",
            fr: "Éreintage",
        },
        (63, 29699) => LocalisedText {
            en: "Corps-a-corps",
            ja: "コル・ア・コル",
            de: "Corps-a-corps",
            fr: "Corps-à-corps",
        },
        (63, 29700) => LocalisedText {
            en: "Displacement",
            ja: "デプラスマン",
            de: "Körperparade",
            fr: "Déplacement",
        },
        (63, 29702) => LocalisedText {
            en: "Black Shift",
            ja: "シフトブラック",
            de: "Schwarze Gabe",
            fr: "Transfert noir",
        },
        (63, 29703) => LocalisedText {
            en: "White Shift",
            ja: "シフトホワイト",
            de: "Weiße Gabe",
            fr: "Transfert blanc",
        },
        (63, 29704) => LocalisedText {
            en: "Southern Cross",
            ja: "サザンクロス",
            de: "Kreuz des Südens",
            fr: "Croix du sud",
        },
        (63, 29707) => LocalisedText {
            en: "Fleeting Raiju",
            ja: "月影雷獣牙",
            de: "Brüllendes Raiju
",
            fr: "Croc de Raijû",
        },
        (63, 29711) => LocalisedText {
            en: "Recuperate",
            ja: "快気",
            de: "Genesung",
            fr: "Rétablissement",
        },
        (63, 29736) => LocalisedText {
            en: "Chaotic Cyclone",
            ja: "カオティックサイクロン",
            de: "Chaosklinge",
            fr: "Cyclone chaotique",
        },
        (63, 29737) => LocalisedText {
            en: "Quietus",
            ja: "クワイタス",
            de: "Quietus",
            fr: "Quietus",
        },
        (64, 1) => LocalisedText {
            en: "Emerald Carbuncle",
            ja: "カーバンクル・エメラルド",
            de: "Smaragd-Karfunkel",
            fr: "Carbuncle émeraude",
        },
        (64, 2) => LocalisedText {
            en: "Topaz Carbuncle",
            ja: "カーバンクル・トパーズ",
            de: "Topas-Karfunkel",
            fr: "Carbuncle topaze",
        },
        (64, 6) => LocalisedText {
            en: "Eos",
            ja: "フェアリー・エオス",
            de: "Eos",
            fr: "Eos",
        },
        (64, 7) => LocalisedText {
            en: "Selene",
            ja: "フェアリー・セレネ",
            de: "Selene",
            fr: "Selene",
        },
        (64, 8) => LocalisedText {
            en: "Rook Autoturret",
            ja: "オートタレット・ルーク",
            de: "Selbstschuss-Gyrocopter TURM",
            fr: "Auto-tourelle Tour",
        },
        (64, 10) => LocalisedText {
            en: "Demi-Bahamut",
            ja: "デミ・バハムート",
            de: "Demi-Bahamut",
            fr: "Demi-Bahamut",
        },
        (64, 14) => LocalisedText {
            en: "Demi-Phoenix",
            ja: "デミ・フェニックス",
            de: "Demi-Phönix",
            fr: "Demi-Phénix",
        },
        (64, 15) => LocalisedText {
            en: "Seraph",
            ja: "セラフィム",
            de: "Seraph",
            fr: "Séraphin",
        },
        (64, 17) => LocalisedText {
            en: "Esteem",
            ja: "英雄の影身",
            de: "Schattenschemen",
            fr: "Estime",
        },
        (64, 18) => LocalisedText {
            en: "Automaton Queen",
            ja: "オートマトン・クイーン",
            de: "Automaton DAME",
            fr: "Automate Reine",
        },
        (64, 19) => LocalisedText {
            en: "Bunshin",
            ja: "分身",
            de: "Gedoppeltes Ich",
            fr: "Ombre",
        },
        (64, 23) => LocalisedText {
            en: "Carbuncle",
            ja: "カーバンクル",
            de: "Karfunkel",
            fr: "Carbuncle",
        },
        (64, 24) => LocalisedText {
            en: "Ruby Carbuncle",
            ja: "カーバンクル・ルビー",
            de: "Rubin-Karfunkel",
            fr: "Carbuncle rubis",
        },
        (64, 27) => LocalisedText {
            en: "Ifrit-Egi",
            ja: "イフリート・エギ",
            de: "Ifrit-Egi",
            fr: "Ifrit-Egi",
        },
        (64, 28) => LocalisedText {
            en: "Titan-Egi",
            ja: "タイタン・エギ",
            de: "Titan-Egi",
            fr: "Titan-Egi",
        },
        (64, 29) => LocalisedText {
            en: "Garuda-Egi",
            ja: "ガルーダ・エギ",
            de: "Garuda-Egi",
            fr: "Garuda-Egi",
        },
        (64, 30) => LocalisedText {
            en: "Ruby Ifrit",
            ja: "イフリート・ルビー",
            de: "Rubin-Ifrit",
            fr: "Ifrit rubis",
        },
        (64, 31) => LocalisedText {
            en: "Topaz Titan",
            ja: "タイタン・トパーズ",
            de: "Topas-Titan",
            fr: "Titan topaze",
        },
        (64, 32) => LocalisedText {
            en: "Emerald Garuda",
            ja: "ガルーダ・エメラルド",
            de: "Smaragd-Garuda",
            fr: "Garuda émeraude",
        },
        (65, 1) => LocalisedText {
            en: "cherry bomb",
            ja: "タイニーボム",
            de: "Kirschbomber",
            fr: "bombino",
        },
        (65, 2) => LocalisedText {
            en: "mammet #001",
            ja: "マメット001",
            de: "Marionette Nr. 001",
            fr: "pantin P001",
        },
        (65, 3) => LocalisedText {
            en: "wayward hatchling",
            ja: "チョコチョコボ",
            de: "Chocobo-Küken",
            fr: "bébé chocobo",
        },
        (65, 4) => LocalisedText {
            en: "Cait Sith doll",
            ja: "マメット・ケットシー",
            de: "Cait Sith-Puppe",
            fr: "poupée Cait Sith",
        },
        (65, 5) => LocalisedText {
            en: "baby behemoth",
            ja: "ベビーモス",
            de: "Baby-Behemoth",
            fr: "bébéhémoth",
        },
        (65, 6) => LocalisedText {
            en: "mammet #003L",
            ja: "マメット003L",
            de: "Marionette Nr. 003L",
            fr: "pantin P003L",
        },
        (65, 7) => LocalisedText {
            en: "mammet #003G",
            ja: "マメット003G",
            de: "Marionette Nr. 003G",
            fr: "pantin P003G",
        },
        (65, 8) => LocalisedText {
            en: "mammet #003U",
            ja: "マメット003U",
            de: "Marionette Nr. 003U",
            fr: "pantin P003U",
        },
        (65, 9) => LocalisedText {
            en: "storm hatchling",
            ja: "黒渦のチョコチョコボ",
            de: "Mahlstrom-Küken",
            fr: "bébé chocobo du Maelstrom",
        },
        (65, 10) => LocalisedText {
            en: "serpent hatchling",
            ja: "双蛇のチョコチョコボ",
            de: "Bruderschafts-Küken",
            fr: "bébé chocobo des Deux Vipères",
        },
        (65, 11) => LocalisedText {
            en: "flame hatchling",
            ja: "不滅のチョコチョコボ",
            de: "Legions-Küken",
            fr: "bébé chocobo des Immortels",
        },
        (65, 12) => LocalisedText {
            en: "morbol seedling",
            ja: "モルボルバルブ",
            de: "Morbol-Knolle",
            fr: "bulbe de morbol",
        },
        (65, 13) => LocalisedText {
            en: "tiny rat",
            ja: "タイニースクウィレル",
            de: "klein[a] Ratte",
            fr: "écureuil nain",
        },
        (65, 14) => LocalisedText {
            en: "baby bun",
            ja: "ベイビーファンガー",
            de: "Mini-Pilz",
            fr: "fungus miniature",
        },
        (65, 15) => LocalisedText {
            en: "chigoe larva",
            ja: "チゴーラーヴァ",
            de: "Stechmücken-Larve",
            fr: "chigoe apprivoisé",
        },
        (65, 16) => LocalisedText {
            en: "bluebird",
            ja: "ブルーバード",
            de: "blau[a] Vögelchen",
            fr: "oiseau bleu",
        },
        (65, 17) => LocalisedText {
            en: "wide-eyed fawn",
            ja: "アンテロープ・フォーン",
            de: "Rehkitz",
            fr: "faon aux yeux de biche",
        },
        (65, 18) => LocalisedText {
            en: "infant imp",
            ja: "ベイビーインプ",
            de: "Baby-Imp",
            fr: "bébé imp",
        },
        (65, 19) => LocalisedText {
            en: "coeurl kitten",
            ja: "タイニークァール",
            de: "Mini-Coeurl",
            fr: "joli coeurl",
        },
        (65, 20) => LocalisedText {
            en: "black coeurl",
            ja: "ブラッククァール",
            de: "schwarz[a] Coeurl",
            fr: "coeurl noir",
        },
        (65, 21) => LocalisedText {
            en: "wind-up gentleman",
            ja: "マメット・ヒルディ",
            de: "Hildibrand-Puppe",
            fr: "Hildibrand miniature",
        },
        (65, 22) => LocalisedText {
            en: "gravel golem",
            ja: "グラベルゴーレム",
            de: "Kiesgolem",
            fr: "golem graveleux",
        },
        (65, 23) => LocalisedText {
            en: "wind-up tonberry",
            ja: "マメット・トンベリ",
            de: "Tomberry-Stofftier",
            fr: "tomberry mécanique",
        },
        (65, 24) => LocalisedText {
            en: "tiny tortoise",
            ja: "タイニーアダマンタス",
            de: "Zwergschildkröte",
            fr: "tortue naine",
        },
        (65, 25) => LocalisedText {
            en: "baby raptor",
            ja: "ベイビーラプトル",
            de: "Baby-Raptor",
            fr: "bébé raptor",
        },
        (65, 26) => LocalisedText {
            en: "baby bat",
            ja: "ベイビーバット",
            de: "Baby-Fledermaus",
            fr: "bébé chauve-souris",
        },
        (65, 27) => LocalisedText {
            en: "tiny bulb",
            ja: "タイニーバルブ",
            de: "Mini-Knolle",
            fr: "bulbe miniature",
        },
        (65, 28) => LocalisedText {
            en: "dust bunny",
            ja: "ダストバニー",
            de: "Staubhäschen",
            fr: "lapin mutin",
        },
        (65, 29) => LocalisedText {
            en: "wind-up dullahan",
            ja: "マメット・デュラハン",
            de: "mechanisch[a] Dullahan",
            fr: "dullahan nain",
        },
        (65, 30) => LocalisedText {
            en: "gigantpole",
            ja: "ギガンジャクシ",
            de: "Kaulquappe",
            fr: "têtard géant",
        },
        (65, 31) => LocalisedText {
            en: "pudgy puk",
            ja: "パジープーク",
            de: "Schwabbelpuk",
            fr: "puk potelé",
        },
        (65, 32) => LocalisedText {
            en: "buffalo calf",
            ja: "バッファロー・カフ",
            de: "Büffelkalb",
            fr: "bufflon",
        },
        (65, 33) => LocalisedText {
            en: "cactuar cutting",
            ja: "ミニテンダー",
            de: "Kaktorfigur",
            fr: "mini-pampa",
        },
        (65, 34) => LocalisedText {
            en: "smallshell",
            ja: "スモールシェル",
            de: "Mini-Krabbe",
            fr: "coquépique",
        },
        (65, 35) => LocalisedText {
            en: "wolf pup",
            ja: "ウルフ・パップ",
            de: "Wolfshundwelpe",
            fr: "bébé chacal",
        },
        (65, 36) => LocalisedText {
            en: "beady eye",
            ja: "タイニーアイ",
            de: "Wächterauge",
            fr: "œil apprivoisé",
        },
        (65, 37) => LocalisedText {
            en: "fledgling dodo",
            ja: "タイニードードー",
            de: "Dodo-Küken",
            fr: "dodu dodo",
        },
        (65, 38) => LocalisedText {
            en: "coblyn larva",
            ja: "コブランラーヴァ",
            de: "Kobalos-Larve",
            fr: "larve de coblyn",
        },
        (65, 39) => LocalisedText {
            en: "wind-up aldgoat",
            ja: "マメット・ゴート",
            de: "Steinbock-Stofftier",
            fr: "mini-bouquetin",
        },
        (65, 40) => LocalisedText {
            en: "fledgling apkallu",
            ja: "タイニーアプカル",
            de: "Apkallu-Küken",
            fr: "apkallu nain",
        },
        (65, 41) => LocalisedText {
            en: "goobbue sproutling",
            ja: "グゥーブー・スプラウト",
            de: "Baby-Goobbue",
            fr: "goobbue goulu",
        },
        (65, 42) => LocalisedText {
            en: "bite-sized pudding",
            ja: "ミニプリン",
            de: "Mini-Pudding",
            fr: "mini-pudding",
        },
        (65, 43) => LocalisedText {
            en: "model vanguard",
            ja: "魔導ヴァンガード追従型",
            de: "Aufzieh-Frontbrecher",
            fr: "maquette magitek",
        },
        (65, 44) => LocalisedText {
            en: "demon brick",
            ja: "デモンズブロック",
            de: "Dämonenwürfel",
            fr: "bloc démonique",
        },
        (65, 45) => LocalisedText {
            en: "mini mole",
            ja: "ミニモール",
            de: "Mini-Mull",
            fr: "taupe trapue",
        },
        (65, 46) => LocalisedText {
            en: "tender lamb",
            ja: "タイニーラム",
            de: "Schäfchen",
            fr: "belin bêlant",
        },
        (65, 47) => LocalisedText {
            en: "slime puddle",
            ja: "ミニスライム",
            de: "Schleimpfütze",
            fr: "mini-gluant",
        },
        (65, 48) => LocalisedText {
            en: "kidragora",
            ja: "ミニマンドラゴラ",
            de: "Kindragora",
            fr: "mini-mini-mandragore",
        },
        (65, 49) => LocalisedText {
            en: "wind-up goblin",
            ja: "マメット・ゴブリン",
            de: "Goblin-Stofftier",
            fr: "Gobelin articulé",
        },
        (65, 50) => LocalisedText {
            en: "wind-up sylph",
            ja: "マメット・シルフ",
            de: "Sylphen-Puppe",
            fr: "Sylphe nain",
        },
        (65, 51) => LocalisedText {
            en: "wind-up cursor",
            ja: "マメット・カーソル",
            de: "Zeigerhändchen",
            fr: "curseur flottant",
        },
        (65, 52) => LocalisedText {
            en: "wind-up airship",
            ja: "マメット・インビンシブル",
            de: "Aufzieh-Luftschiff",
            fr: "aéronef mécanique",
        },
        (65, 53) => LocalisedText {
            en: "wind-up Qiqirn",
            ja: "マメット・キキルン",
            de: "Qiqirn-Stofftier",
            fr: "Qiqirn mécanique",
        },
        (65, 54) => LocalisedText {
            en: "black chocobo chick",
            ja: "黒チョコチョコボ",
            de: "schwarz[a] Chocobo-Küken",
            fr: "bébé chocobo noir",
        },
        (65, 55) => LocalisedText {
            en: "wind-up Dalamud",
            ja: "マメット・ダラガブ",
            de: "mechanisch[a] Dalamud",
            fr: "Dalamud mécanique",
        },
        (65, 56) => LocalisedText {
            en: "minute mindflayer",
            ja: "カインドフレア",
            de: "Mini-Gedankenschinder",
            fr: "gentil flagelleur",
        },
        (65, 57) => LocalisedText {
            en: "tight-beaked parrot",
            ja: "アントーキング・コリブリ",
            de: "schnatterfaul[a] Kolibri",
            fr: "perroquet taciturne",
        },
        (65, 58) => LocalisedText {
            en: "wind-up Amalj'aa",
            ja: "マメット・アマルジャ",
            de: "Aufzieh-Amalj'aa",
            fr: "Amalj'aa mécanique",
        },
        (65, 59) => LocalisedText {
            en: "wind-up Ixal",
            ja: "マメット・イクサル",
            de: "Ixal-Puppe",
            fr: "Ixal mécanique",
        },
        (65, 60) => LocalisedText {
            en: "wind-up kobold",
            ja: "マメット・コボルド",
            de: "Kobold-Puppe",
            fr: "poupée kobolde",
        },
        (65, 61) => LocalisedText {
            en: "wind-up Sahagin",
            ja: "マメット・サハギン",
            de: "Jung-Sahagin",
            fr: "pantin sahuagin",
        },
        (65, 62) => LocalisedText {
            en: "wind-up brickman",
            ja: "ゴレムス",
            de: "Golem",
            fr: "Siman mécanique",
        },
        (65, 63) => LocalisedText {
            en: "wind-up Shantotto",
            ja: "マメット・シャントット",
            de: "Aufzieh-Shantotto",
            fr: "poupée Shantotto",
        },
        (65, 64) => LocalisedText {
            en: "wind-up Edvya",
            ja: "マメット・エドヴィア",
            de: "Aufzieh-Edvya",
            fr: "mini-princesse Edvya",
        },
        (65, 65) => LocalisedText {
            en: "wind-up sun",
            ja: "クロックワークサン",
            de: "portabl[a] Studiosonne",
            fr: "soleil mécanique",
        },
        (65, 66) => LocalisedText {
            en: "plush cushion",
            ja: "ふかふかクッション",
            de: "Sitzkissen",
            fr: "coussin moelleux",
        },
        (65, 67) => LocalisedText {
            en: "Minion of Light",
            ja: "マメット・オブ・ライト",
            de: "Begleiter[p] des Lichts",
            fr: "héros de la Lumière",
        },
        (65, 71) => LocalisedText {
            en: "wind-up leader",
            ja: "マメット・リーダー",
            de: "Aufzieh-Anführer",
            fr: "dirigeant miniature",
        },
        (65, 72) => LocalisedText {
            en: "wind-up Merlwyb",
            ja: "マメット・メルウィブ",
            de: "Aufzieh-Merlwyb",
            fr: "Merlwyb miniature",
        },
        (65, 73) => LocalisedText {
            en: "wind-up Kan-E",
            ja: "マメット・カヌ・エ",
            de: "Aufzieh-Kan-E",
            fr: "Kan-E miniature",
        },
        (65, 74) => LocalisedText {
            en: "wind-up Raubahn",
            ja: "マメット・ラウバーン",
            de: "Aufzieh-Raubahn",
            fr: "Raubahn miniature",
        },
        (65, 75) => LocalisedText {
            en: "princely hatchling",
            ja: "チョコチョコボ・プリンス",
            de: "Chocobo-Prinz",
            fr: "prince bébé chocobo",
        },
        (65, 76) => LocalisedText {
            en: "wind-up Odin",
            ja: "マメット・オーディン",
            de: "Mini-Odin",
            fr: "Odin mécanique",
        },
        (65, 77) => LocalisedText {
            en: "wind-up Warrior of Light",
            ja: "ウォーリア・オブ・ライト",
            de: "Mini-Lichtkrieger",
            fr: "guerrier de la Lumière",
        },
        (65, 78) => LocalisedText {
            en: "wind-up Bahamut",
            ja: "マメット・バハムート",
            de: "Aufzieh-Bahamut",
            fr: "bébé Bahamut",
        },
        (65, 79) => LocalisedText {
            en: "wind-up moogle",
            ja: "マメット・モーグリ",
            de: "Mogry-Stofftier",
            fr: "poupée mog",
        },
        (65, 80) => LocalisedText {
            en: "baby opo-opo",
            ja: "ベイビーオポオポ",
            de: "Stoff-Opo-Opo",
            fr: "bébé opo-opo",
        },
        (65, 81) => LocalisedText {
            en: "magic broom",
            ja: "マメット・ブルーム",
            de: "emsig[a] Besen",
            fr: "balai enchanté",
        },
        (65, 82) => LocalisedText {
            en: "wind-up succubus",
            ja: "マメット・サキュバス",
            de: "Sukkubus-Puppe",
            fr: "poupée succube",
        },
        (65, 83) => LocalisedText {
            en: "zu hatchling",
            ja: "ズー・ハッチリング",
            de: "Zu-Küken",
            fr: "poussin zu",
        },
        (65, 84) => LocalisedText {
            en: "wind-up Nanamo",
            ja: "マメット・ナナモ",
            de: "Nanamo-Puppe",
            fr: "Nanamo miniature",
        },
        (65, 85) => LocalisedText {
            en: "wind-up Gilgamesh",
            ja: "マメット・ギルガメッシュ",
            de: "Aufzieh-Gilgamesch",
            fr: "pantin Gilgamesh",
        },
        (65, 86) => LocalisedText {
            en: "Onion Prince",
            ja: "オニオンプリンス",
            de: "Prinz Zwiebel",
            fr: "Prince oignon",
        },
        (65, 87) => LocalisedText {
            en: "Eggplant Knight",
            ja: "エッグナイト",
            de: "Ritter Aubergine",
            fr: "Chevalier aubergine",
        },
        (65, 88) => LocalisedText {
            en: "Garlic Jester",
            ja: "ガーリックスター",
            de: "Sir Knoblauch",
            fr: "Baron ail",
        },
        (65, 89) => LocalisedText {
            en: "Tomato King",
            ja: "キングトマト",
            de: "König Tomate",
            fr: "Roi tomate",
        },
        (65, 90) => LocalisedText {
            en: "Mandragora Queen",
            ja: "マンドラクイーン",
            de: "Königin Mandragora",
            fr: "Reine mandragore",
        },
        (65, 91) => LocalisedText {
            en: "wind-up Y'shtola",
            ja: "マメット・ヤ・シュトラ",
            de: "Y'shtola-Puppe",
            fr: "Y'shtola miniature",
        },
        (65, 92) => LocalisedText {
            en: "wind-up onion knight",
            ja: "マメット・オニオン",
            de: "Aufzieh-Zwiebelritter",
            fr: "poupée chevalier oignon",
        },
        (65, 93) => LocalisedText {
            en: "treasure box",
            ja: "トレジャーボックス",
            de: "Schatzkiste",
            fr: "coffre au trésor mécanique",
        },
        (65, 94) => LocalisedText {
            en: "tiny tapir",
            ja: "タイニーバク",
            de: "Tapirjung[a]",
            fr: "petit tapir",
        },
        (65, 95) => LocalisedText {
            en: "Nana bear",
            ja: "ナナッコ・ベアー",
            de: "Nana-Stoffbär",
            fr: "ourson de Nanakko",
        },
        (65, 96) => LocalisedText {
            en: "miniature minecart",
            ja: "クロックワークトロッコ",
            de: "Miniaturlore",
            fr: "wagonnet mécanique",
        },
        (65, 97) => LocalisedText {
            en: "nutkin",
            ja: "ナッツイーター",
            de: "Nussfresser",
            fr: "écureuil grignoteur",
        },
        (65, 98) => LocalisedText {
            en: "wind-up Minfilia",
            ja: "マメット・ミンフィリア",
            de: "Minfilia-Puppe",
            fr: "Minfilia miniature",
        },
        (65, 99) => LocalisedText {
            en: "wind-up Thancred",
            ja: "マメット・サンクレッド",
            de: "Thancred-Puppe",
            fr: "Thancred miniature",
        },
        (65, 100) => LocalisedText {
            en: "model magitek bit",
            ja: "魔導ビット自律型",
            de: "Aufzieh-Magitek-Drohne",
            fr: "micro-drone magitek",
        },
        (65, 101) => LocalisedText {
            en: "puff of darkness",
            ja: "薄闇の雲",
            de: "Wölkchen[p] der Dunkelheit",
            fr: "nuage noir miniature",
        },
        (65, 102) => LocalisedText {
            en: "naughty nanka",
            ja: "タイニーナンカ",
            de: "Ninki-Nanka-Puppe",
            fr: "nanka narquois",
        },
        (65, 103) => LocalisedText {
            en: "panda cub",
            ja: "マメット・パンダ",
            de: "Panda-Stofftier",
            fr: "petit panda pataud",
        },
        (65, 104) => LocalisedText {
            en: "wind-up Ultros",
            ja: "マメット・オルトロス",
            de: "Aufzieh-Ultros",
            fr: "Orthros articulé",
        },
        (65, 105) => LocalisedText {
            en: "Hoary the Snowman",
            ja: "マメット・スノーマン",
            de: "Zuckerschneemann",
            fr: "bébé bonhomme de neige",
        },
        (65, 106) => LocalisedText {
            en: "heavy hatchling",
            ja: "ポチャチョコボ",
            de: "Chocopummelchen",
            fr: "bébé chocobo grassouillet",
        },
        (65, 107) => LocalisedText {
            en: "wind-up Yda",
            ja: "マメット・イダ",
            de: "Yda-Puppe",
            fr: "Yda miniature",
        },
        (65, 108) => LocalisedText {
            en: "wind-up Papalymo",
            ja: "マメット・パパリモ",
            de: "Papalymo-Puppe",
            fr: "Papalymo miniature",
        },
        (65, 109) => LocalisedText {
            en: "wind-up Urianger",
            ja: "マメット・ウリエンジェ",
            de: "Urianger-Puppe",
            fr: "Urianger miniature",
        },
        (65, 110) => LocalisedText {
            en: "fat cat",
            ja: "ファットキャット",
            de: "dick[a] Katze",
            fr: "gros minou",
        },
        (65, 111) => LocalisedText {
            en: "Littlefoot",
            ja: "リトルフット",
            de: "Kleinfuß",
            fr: "petit yéti",
        },
        (65, 112) => LocalisedText {
            en: "mummy's little mummy",
            ja: "リトルマミー",
            de: "Mini-Mumie",
            fr: "mini-momie",
        },
        (65, 113) => LocalisedText {
            en: "model Enterprise",
            ja: "マメット・エンタープライズ",
            de: "Aufzieh-Enterprise",
            fr: "maquette de l'Entreprise",
        },
        (65, 114) => LocalisedText {
            en: "wind-up delivery moogle",
            ja: "マメット・レターモーグリ",
            de: "Mogry-Stoffpostbote",
            fr: "poupée Mog postier",
        },
        (65, 115) => LocalisedText {
            en: "assassin fry",
            ja: "アサシンフライ",
            de: "fliegend[a] Kampffisch",
            fr: "fretin assassin",
        },
        (65, 116) => LocalisedText {
            en: "demon box",
            ja: "プレゼンター",
            de: "beseelt[a] Geschenkpäckchen",
            fr: "cadeau ambulant",
        },
        (65, 117) => LocalisedText {
            en: "water imp",
            ja: "カッパッパ",
            de: "Flusskobold",
            fr: "petit kappa",
        },
        (65, 118) => LocalisedText {
            en: "wind-up Louisoix",
            ja: "マメット・ルイゾワ",
            de: "Aufzieh-Louisoix",
            fr: "Louisoix miniature",
        },
        (65, 119) => LocalisedText {
            en: "Midgardsormr",
            ja: "ミドガルズオルム",
            de: "Midgardsormr",
            fr: "Midgardsormr",
        },
        (65, 121) => LocalisedText {
            en: "Doman magpie",
            ja: "ピカピカ",
            de: "klein[a] Elster",
            fr: "pie pimpante",
        },
        (65, 122) => LocalisedText {
            en: "Enkidu",
            ja: "エンキドゥ",
            de: "Aufzieh-Enkidu",
            fr: "Enkidu",
        },
        (65, 123) => LocalisedText {
            en: "wind-up Violet",
            ja: "マメット・バイオレット",
            de: "Finstrigen-Puppe",
            fr: "poupée de méchant Sylphe",
        },
        (65, 124) => LocalisedText {
            en: "wind-up founder",
            ja: "マメット・ファウンダー",
            de: "Aufzieh-Gründer",
            fr: "poupée du maître fondateur",
        },
        (65, 125) => LocalisedText {
            en: "wind-up Dezul Qualan",
            ja: "マメット・デズル",
            de: "Dezul-Qualan-Puppe",
            fr: "Dezul miniature",
        },
        (65, 126) => LocalisedText {
            en: "wind-up kobolder",
            ja: "マメット・コボルド2号",
            de: "Aufzieh-Kobold[p] Nr. 2",
            fr: "poupée kobolde n°2",
        },
        (65, 127) => LocalisedText {
            en: "wind-up Sea Devil",
            ja: "マメット・シーデビル",
            de: "Aufzieh-Seeteufel",
            fr: "démon des mers mécanique",
        },
        (65, 128) => LocalisedText {
            en: "chocobo chick courier",
            ja: "黒チョコチョコボ・フライヤー",
            de: "Brief-Chocobo-Küken",
            fr: "bébé chocobo noir volant",
        },
        (65, 129) => LocalisedText {
            en: "wind-up Kain",
            ja: "マメット・カイン",
            de: "Kain-Puppe",
            fr: "Kaïn miniature",
        },
        (65, 130) => LocalisedText {
            en: "wind-up Alphinaud",
            ja: "マメット・アルフィノ",
            de: "Aufzieh-Alphinaud",
            fr: "Alphinaud miniature",
        },
        (65, 131) => LocalisedText {
            en: "wind-up Alisaie",
            ja: "マメット・アリゼー",
            de: "Aufzieh-Alisaie",
            fr: "Alisaie miniature",
        },
        (65, 132) => LocalisedText {
            en: "wind-up Tataru",
            ja: "マメット・タタル",
            de: "Tataru-Puppe",
            fr: "Tataru miniature",
        },
        (65, 133) => LocalisedText {
            en: "wind-up Cid",
            ja: "マメット・シド",
            de: "Cid-Puppe",
            fr: "Cid miniature",
        },
        (65, 134) => LocalisedText {
            en: "unicolt",
            ja: "ユニコルト",
            de: "Kleinhorn",
            fr: "bébé licorne",
        },
        (65, 135) => LocalisedText {
            en: "wind-up Gundu warrior",
            ja: "マメット・グンド",
            de: "Aufzieh-Gundo",
            fr: "poupée gundu",
        },
        (65, 136) => LocalisedText {
            en: "atrophied Atomos",
            ja: "ミニマムアトモス",
            de: "Mini-Atomos",
            fr: "atomos atrophié",
        },
        (65, 137) => LocalisedText {
            en: "owlet",
            ja: "オウレット",
            de: "Eulette",
            fr: "bébé hibou",
        },
        (65, 138) => LocalisedText {
            en: "ugly duckling",
            ja: "アグリー・ダックリング",
            de: "hässlich[a] Entlein",
            fr: "vilain petit canard",
        },
        (65, 139) => LocalisedText {
            en: "gaelikitten",
            ja: "ゲイラキトン",
            de: "Geira-Kätzchen",
            fr: "gaélichaton",
        },
        (65, 140) => LocalisedText {
            en: "clockwork barrow",
            ja: "クロックワークカート",
            de: "Aufzieh-Schubkarre",
            fr: "brouette mécanique",
        },
        (65, 141) => LocalisedText {
            en: "lesser panda",
            ja: "レッサーパンダ",
            de: "Rotpanda-Stofftier",
            fr: "petit panda roux",
        },
        (65, 142) => LocalisedText {
            en: "page 63",
            ja: "63ページ",
            de: "Seite[p] 63",
            fr: "page 63",
        },
        (65, 143) => LocalisedText {
            en: "iron dwarf",
            ja: "鉄小人",
            de: "Eisenzwerg",
            fr: "nain d'acier",
        },
        (65, 144) => LocalisedText {
            en: "griffin hatchling",
            ja: "グリフィン・ハッチリング",
            de: "Greifen-Küken",
            fr: "bébé griffon",
        },
        (65, 145) => LocalisedText {
            en: "wind-up Iceheart",
            ja: "マメット・イゼル",
            de: "Eisherz-Puppe",
            fr: "Ysayle miniature",
        },
        (65, 146) => LocalisedText {
            en: "Gestahl",
            ja: "ガストラ",
            de: "Gestahl-Schoßhund",
            fr: "Gestahl",
        },
        (65, 147) => LocalisedText {
            en: "steam-powered gobwalker G-VII",
            ja: "VII号スチームウォーカー",
            de: "Dampf-Stampfer",
            fr: "gobblindé G-VII à vapeur",
        },
        (65, 148) => LocalisedText {
            en: "behemoth heir",
            ja: "ロイヤルベビーモス",
            de: "blaublütig[a] Behemoth",
            fr: "prince bébéhémoth",
        },
        (65, 149) => LocalisedText {
            en: "accompaniment node",
            ja: "追従システム",
            de: "Begleitmodul",
            fr: "sphère de guidage",
        },
        (65, 150) => LocalisedText {
            en: "wind-up Yugiri",
            ja: "マメット・ユウギリ",
            de: "Yugiri-Puppe",
            fr: "Yugiri miniature",
        },
        (65, 151) => LocalisedText {
            en: "Peridot Carbuncle",
            ja: "カーバンクル・ペリドット",
            de: "Peridot-Karfunkel",
            fr: "Carbuncle péridot",
        },
        (65, 152) => LocalisedText {
            en: "Heliodor Carbuncle",
            ja: "カーバンクル・ヘリオドール",
            de: "Heliodor-Karfunkel",
            fr: "Carbuncle héliodore",
        },
        (65, 154) => LocalisedText {
            en: "gold rush minecart",
            ja: "ゴールドラッシュ",
            de: "Goldrausch-Karre",
            fr: "chariot de mine mécanique",
        },
        (65, 155) => LocalisedText {
            en: "set of Primogs",
            ja: "ザ・プライモグズ",
            de: "Primog-Kapelle",
            fr: "fanfare des Primogs",
        },
        (65, 156) => LocalisedText {
            en: "wind-up Gnath",
            ja: "マメット・グナース",
            de: "Gnath-Puppe",
            fr: "poupée gnathe",
        },
        (65, 157) => LocalisedText {
            en: "paissa brat",
            ja: "パイッサ・ブラット",
            de: "Paissa-Fratz",
            fr: "petiot païssa",
        },
        (65, 158) => LocalisedText {
            en: "wind-up Illuminatus",
            ja: "マメット・ブルーハンド",
            de: "Indigohand-Puppe",
            fr: "Gobelin de la Mimine bleue",
        },
        (65, 159) => LocalisedText {
            en: "Pumpkin Butler",
            ja: "パンプキンバトラー",
            de: "Kürbis-Butler",
            fr: "majordome citrouille",
        },
        (65, 160) => LocalisedText {
            en: "wind-up Echidna",
            ja: "マメット・エキドナ",
            de: "Aufzieh-Echidna",
            fr: "Echidna miniature",
        },
        (65, 161) => LocalisedText {
            en: "wind-up Relm",
            ja: "マメット・リルム",
            de: "Relm-Puppe",
            fr: "Relm miniature",
        },
        (65, 162) => LocalisedText {
            en: "hunting hawk",
            ja: "ハンティング・ホーク",
            de: "Jagdfalke",
            fr: "buse chasseuse",
        },
        (65, 163) => LocalisedText {
            en: "shalloweye",
            ja: "シャローアイ",
            de: "Stumpfauge",
            fr: "poussin oculus",
        },
        (65, 164) => LocalisedText {
            en: "penguin prince",
            ja: "プリンスペンギン",
            de: "Pinguinprinz",
            fr: "manchot prince",
        },
        (65, 165) => LocalisedText {
            en: "clockwork Twintania",
            ja: "マメット・ツインタニア",
            de: "Uhrwerk-Twintania",
            fr: "mini-Gémellia",
        },
        (65, 166) => LocalisedText {
            en: "korpokkur kid",
            ja: "コロコロポックル",
            de: "Korpokkur-Pflänzchen",
            fr: "pon-korpokkur",
        },
        (65, 167) => LocalisedText {
            en: "wind-up Firion",
            ja: "マメット・フリオニール",
            de: "Firion-Puppe",
            fr: "Firion miniature",
        },
        (65, 168) => LocalisedText {
            en: "wind-up Ifrit",
            ja: "マメット・イフリート",
            de: "Mini-Ifrit",
            fr: "petit Ifrit",
        },
        (65, 169) => LocalisedText {
            en: "wind-up Garuda",
            ja: "マメット・ガルーダ",
            de: "Mini-Garuda",
            fr: "petite Garuda",
        },
        (65, 170) => LocalisedText {
            en: "wind-up Titan",
            ja: "マメット・タイタン",
            de: "Mini-Titan",
            fr: "petit Titan",
        },
        (65, 171) => LocalisedText {
            en: "wind-up Leviathan",
            ja: "マメット・リヴァイアサン",
            de: "Mini-Leviathan",
            fr: "petit Léviathan",
        },
        (65, 172) => LocalisedText {
            en: "wind-up Zundu warrior",
            ja: "マメット・ズンド",
            de: "Aufzieh-Zundo-Krieger",
            fr: "poupée zundu",
        },
        (65, 173) => LocalisedText {
            en: "wind-up Haurchefant",
            ja: "マメット・オルシュファン",
            de: "Haurchefant-Puppe",
            fr: "Haurchefant miniature",
        },
        (65, 174) => LocalisedText {
            en: "wind-up Nero tol Scaeva",
            ja: "マメット・ネロ",
            de: "Nero-Puppe",
            fr: "Nero tol Scaeva miniature",
        },
        (65, 175) => LocalisedText {
            en: "wind-up Vath",
            ja: "マメット・ヴァス",
            de: "Vath-Puppe",
            fr: "poupée vathe",
        },
        (65, 176) => LocalisedText {
            en: "faustlet",
            ja: "ミニファウスト",
            de: "Fäustchen",
            fr: "faust articulé",
        },
        (65, 177) => LocalisedText {
            en: "dress-up Y'shtola",
            ja: "ニュー・ヤ・シュトラ",
            de: "neu[a] Y'shtola-Puppe",
            fr: "nouvelle Y'shtola miniature",
        },
        (65, 178) => LocalisedText {
            en: "Calca",
            ja: "カルコ",
            de: "Calco-Puppe",
            fr: "Calca",
        },
        (65, 179) => LocalisedText {
            en: "Brina",
            ja: "ブリーナ",
            de: "Brina-Puppe",
            fr: "Brina",
        },
        (65, 180) => LocalisedText {
            en: "morpho",
            ja: "モルフォ",
            de: "Morpho",
            fr: "morpho",
        },
        (65, 181) => LocalisedText {
            en: "Poro Roggo",
            ja: "ポロ・ロッゴ",
            de: "Poro Roggo",
            fr: "Poro Roggo",
        },
        (65, 182) => LocalisedText {
            en: "spoony bard",
            ja: "マメット・ギルバート",
            de: "Aufzieh-Edward",
            fr: "Edward miniature",
        },
        (65, 183) => LocalisedText {
            en: "Fenrir pup",
            ja: "フェンリル・パップ",
            de: "Fenrir-Welpe",
            fr: "bébé Fenrir",
        },
        (65, 184) => LocalisedText {
            en: "wind-up dragonet",
            ja: "マメット・ドラゴネット",
            de: "Drachenjung[a]",
            fr: "mini-dragonnet",
        },
        (65, 185) => LocalisedText {
            en: "wind-up Ramuh",
            ja: "マメット・ラムウ",
            de: "Mini-Ramuh",
            fr: "petit Ramuh",
        },
        (65, 186) => LocalisedText {
            en: "wind-up Shiva",
            ja: "マメット・シヴァ",
            de: "Mini-Shiva",
            fr: "petite Shiva",
        },
        (65, 187) => LocalisedText {
            en: "Piggy",
            ja: "ポーキー",
            de: "Ferkelchen",
            fr: "porcelet promeneur",
        },
        (65, 188) => LocalisedText {
            en: "magic bucket",
            ja: "マジックバケツ",
            de: "magisch[a] Eimer",
            fr: "seau d'eau magique",
        },
        (65, 189) => LocalisedText {
            en: "calamari",
            ja: "ゲソ",
            de: "Tentakel",
            fr: "tentacule minuscule",
        },
        (65, 190) => LocalisedText {
            en: "baby brachiosaur",
            ja: "マメット・ブラキオ",
            de: "Brachiosaurus-Baby",
            fr: "bébé brachiosaure",
        },
        (65, 191) => LocalisedText {
            en: "wind-up cheerleader",
            ja: "マメット・チアガール",
            de: "Aufzieh-Cheerleaderin",
            fr: "marionnette majorette",
        },
        (65, 192) => LocalisedText {
            en: "wind-up Krile",
            ja: "マメット・クルル",
            de: "Krile-Puppe",
            fr: "Krile miniature",
        },
        (65, 193) => LocalisedText {
            en: "wind-up Aymeric",
            ja: "マメット・アイメリク",
            de: "Aymeric-Puppe",
            fr: "Aymeric miniature",
        },
        (65, 194) => LocalisedText {
            en: "pegasus colt",
            ja: "ペガサスコルト",
            de: "Pegasus-Fohlen",
            fr: "poulain pégase",
        },
        (65, 195) => LocalisedText {
            en: "wind-up Calofisteri",
            ja: "マメット・カロフィステリ",
            de: "Mini-Calofisteri",
            fr: "poupée Calofisteri",
        },
        (65, 196) => LocalisedText {
            en: "wind-up sasquatch",
            ja: "マメット・サスカッチ",
            de: "Sasquatch-Stofftier",
            fr: "mini-sasquatch",
        },
        (65, 197) => LocalisedText {
            en: "dwarf rabbit",
            ja: "ドワーフラビット",
            de: "Zwergkaninchen",
            fr: "lapin nain",
        },
        (65, 198) => LocalisedText {
            en: "hecteye",
            ja: "ヘクトアイ",
            de: "Hektokulinchen",
            fr: "hectoculus monoculaire",
        },
        (65, 199) => LocalisedText {
            en: "Taoist moogle",
            ja: "道士モーグリ",
            de: "Daoisten-Mogry",
            fr: "Mog dàoshi",
        },
        (65, 200) => LocalisedText {
            en: "Jibanyan",
            ja: "ジバニャン",
            de: "Jibanyan",
            fr: "Jibanyan",
        },
        (65, 201) => LocalisedText {
            en: "Komasan",
            ja: "コマさん",
            de: "Komasan",
            fr: "Komasan",
        },
        (65, 202) => LocalisedText {
            en: "Whisper",
            ja: "ウィスパー",
            de: "Whisper",
            fr: "Whisper",
        },
        (65, 203) => LocalisedText {
            en: "Blizzaria",
            ja: "ふぶき姫",
            de: "Glazia",
            fr: "Blizzaria",
        },
        (65, 204) => LocalisedText {
            en: "Kyubi",
            ja: "キュウビ",
            de: "Kyubi",
            fr: "Kyubi",
        },
        (65, 205) => LocalisedText {
            en: "Komajiro",
            ja: "コマじろう",
            de: "Komajiro",
            fr: "Komajiro",
        },
        (65, 206) => LocalisedText {
            en: "Manjimutt",
            ja: "じんめん犬",
            de: "Möter",
            fr: "Corniot",
        },
        (65, 207) => LocalisedText {
            en: "Noko",
            ja: "ツチノコ",
            de: "Noko",
            fr: "Noko",
        },
        (65, 208) => LocalisedText {
            en: "Venoct",
            ja: "オロチ",
            de: "Viptor",
            fr: "Octorgone",
        },
        (65, 209) => LocalisedText {
            en: "Shogunyan",
            ja: "ブシニャン",
            de: "Shogunyan",
            fr: "Shogunyan",
        },
        (65, 210) => LocalisedText {
            en: "Hovernyan",
            ja: "フユニャン",
            de: "Hovernyan",
            fr: "Hovernyan",
        },
        (65, 211) => LocalisedText {
            en: "Robonyan F-type",
            ja: "ロボニャンＦ型",
            de: "Robonyan[p] F-Type",
            fr: "Robonyan F-Type",
        },
        (65, 212) => LocalisedText {
            en: "USApyon",
            ja: "ＵＳＡピョン",
            de: "Usapyon",
            fr: "Usapyon",
        },
        (65, 214) => LocalisedText {
            en: "Namingway",
            ja: "ネミングウェイ",
            de: "Namingway-Puppe",
            fr: "poupée Namingway",
        },
        (65, 215) => LocalisedText {
            en: "toy Alexander",
            ja: "ミニアレキサンダー",
            de: "Mini-Alexander",
            fr: "Alexander articulé",
        },
        (65, 216) => LocalisedText {
            en: "shaggy shoat",
            ja: "うり坊",
            de: "Frischling",
            fr: "marcassin",
        },
        (65, 217) => LocalisedText {
            en: "dress-up Thancred",
            ja: "ニュー・サンクレッド",
            de: "neu[a] Thancred-Puppe",
            fr: "nouveau Thancred miniature",
        },
        (65, 218) => LocalisedText {
            en: "dress-up Alisaie",
            ja: "ニュー・アリゼー",
            de: "neu[a] Alisaie-Puppe",
            fr: "nouvelle Alisaie miniature",
        },
        (65, 219) => LocalisedText {
            en: "wind-up Edda",
            ja: "マメット・エッダ",
            de: "Edda-Puppe",
            fr: "Edda miniature",
        },
        (65, 220) => LocalisedText {
            en: "wind-up Yuna",
            ja: "マメット・ユウナ",
            de: "Yuna-Puppe",
            fr: "Yuna miniature",
        },
        (65, 221) => LocalisedText {
            en: "wind-up Rikku",
            ja: "マメット・リュック",
            de: "Rikku-Puppe",
            fr: "Rikku miniature",
        },
        (65, 222) => LocalisedText {
            en: "wind-up Lulu",
            ja: "マメット・ルールー",
            de: "Lulu-Puppe",
            fr: "Lulu miniature",
        },
        (65, 224) => LocalisedText {
            en: "wind-up Moenbryda",
            ja: "マメット・ムーンブリダ",
            de: "Moenbryda-Puppe",
            fr: "Moenbryda miniature",
        },
        (65, 225) => LocalisedText {
            en: "Continental eye",
            ja: "コンチネンタルアイ",
            de: "Circus-Auge",
            fr: "Œil du Cirque",
        },
        (65, 226) => LocalisedText {
            en: "bullpup",
            ja: "ブルパピー",
            de: "Bulldoggenwelpe",
            fr: "chiot bulldog",
        },
        (65, 227) => LocalisedText {
            en: "angel of mercy",
            ja: "レイズエンジェル",
            de: "Liebesengel",
            fr: "ange de la résurrection",
        },
        (65, 228) => LocalisedText {
            en: "wind-up Estinien",
            ja: "マメット・エスティニアン",
            de: "Estinien-Puppe",
            fr: "Estinien miniature",
        },
        (65, 229) => LocalisedText {
            en: "wind-up Hraesvelgr",
            ja: "マメット・フレースヴェルグ",
            de: "Hraesvelgr-Stofftier",
            fr: "Hraesvelgr miniature",
        },
        (65, 230) => LocalisedText {
            en: "Gigi",
            ja: "ギギ",
            de: "Gigi",
            fr: "Gigi",
        },
        (65, 231) => LocalisedText {
            en: "anima",
            ja: "人造精霊",
            de: "Anima",
            fr: "esprit synthétique",
        },
        (65, 232) => LocalisedText {
            en: "wind-up Scathach",
            ja: "マメット・スカアハ",
            de: "Aufzieh-Scathach",
            fr: "Scáthach miniature",
        },
        (65, 233) => LocalisedText {
            en: "wind-up Nidhogg",
            ja: "マメット・ニーズヘッグ",
            de: "Nidhogg-Stofftier",
            fr: "Nidhogg miniature",
        },
        (65, 234) => LocalisedText {
            en: "hellpup",
            ja: "ヘルパピー",
            de: "Höllenhundwelpe",
            fr: "chiot des enfers",
        },
        (65, 235) => LocalisedText {
            en: "wind-up Ohl Deeh",
            ja: "マメット・オル・ディー",
            de: "Aufzieh-Ohl Deeh",
            fr: "Ohl Deeh miniature",
        },
        (65, 236) => LocalisedText {
            en: "wind-up moon",
            ja: "クロックワークムーン",
            de: "portabl[a] Studiomond",
            fr: "lune mécanique",
        },
        (65, 237) => LocalisedText {
            en: "castaway chocobo chick",
            ja: "海チョコチョコボ",
            de: "schiffbrüchig[a] Chocobo-Küken",
            fr: "bébé chocobo des mers",
        },
        (65, 238) => LocalisedText {
            en: "wind-up Bartz",
            ja: "マメット・バッツ",
            de: "Bartz-Puppe",
            fr: "Bartz miniature",
        },
        (65, 239) => LocalisedText {
            en: "wind-up red mage",
            ja: "マメット・レッドメイジ",
            de: "Rotmagier-Puppe",
            fr: "mage rouge mécanique",
        },
        (65, 240) => LocalisedText {
            en: "faepup",
            ja: "フェイパピー",
            de: "Feenhundwelpe",
            fr: "chiot fantasmagorique",
        },
        (65, 241) => LocalisedText {
            en: "odder otter",
            ja: "ウソウソ",
            de: "obskur[a] Otter",
            fr: "loutre choute",
        },
        (65, 242) => LocalisedText {
            en: "fox kit",
            ja: "豆妖狐",
            de: "Fuchswelpe",
            fr: "bébé renard",
        },
        (65, 243) => LocalisedText {
            en: "Tora-jiro",
            ja: "トラジロウ",
            de: "Tora-jiro",
            fr: "Tora-jiro",
        },
        (65, 244) => LocalisedText {
            en: "tiny tatsunoko",
            ja: "タツマゴ",
            de: "winzig[a] Tatsunoko",
            fr: "hippocampoupon",
        },
        (65, 245) => LocalisedText {
            en: "bombfish",
            ja: "ハリセンボム",
            de: "Igelbomber",
            fr: "poisson-bombo",
        },
        (65, 246) => LocalisedText {
            en: "Bom Boko",
            ja: "ブンブク",
            de: "Bom Boko",
            fr: "Bunbuku",
        },
        (65, 247) => LocalisedText {
            en: "road sparrow",
            ja: "タビスズメ",
            de: "Straßenspatz",
            fr: "moineau voyageur",
        },
        (65, 248) => LocalisedText {
            en: "wind-up Lyse",
            ja: "マメット・リセ",
            de: "Lyse-Puppe",
            fr: "Lyse miniature",
        },
        (65, 249) => LocalisedText {
            en: "dress-up Yugiri",
            ja: "ニュー・ユウギリ",
            de: "neu[a] Yugiri-Puppe",
            fr: "Yugiri miniature revisitée",
        },
        (65, 250) => LocalisedText {
            en: "wind-up Gosetsu",
            ja: "マメット・ゴウセツ",
            de: "Gosetsu-Puppe",
            fr: "Gosetsu miniature",
        },
        (65, 251) => LocalisedText {
            en: "wind-up Yotsuyu",
            ja: "マメット・ヨツユ",
            de: "Yotsuyu-Puppe",
            fr: "Yotsuyu miniature",
        },
        (65, 252) => LocalisedText {
            en: "mock-up Grynewaht",
            ja: "マメット・グリーンワート",
            de: "Aufzieh-Grynewaht",
            fr: "Grynewaht miniature",
        },
        (65, 253) => LocalisedText {
            en: "wind-up Namazu",
            ja: "マメット・ナマズオ",
            de: "Aufzieh-Namazuo",
            fr: "Namazu mécanique",
        },
        (65, 254) => LocalisedText {
            en: "Ivon Coeurlfist doll",
            ja: "イヴォン人形",
            de: "Ivon Coeurlfaust-Puppe",
            fr: "poupée Ivon le Coeurl",
        },
        (65, 255) => LocalisedText {
            en: "wind-up chimera",
            ja: "マメットキメラ",
            de: "Aufzieh-Chimära",
            fr: "chimère mécanique",
        },
        (65, 256) => LocalisedText {
            en: "wind-up meateater",
            ja: "マメット・ミートイーター",
            de: "Aufzieh-Fleischfresser",
            fr: "croque-viande mécanique",
        },
        (65, 257) => LocalisedText {
            en: "magitek avenger F1",
            ja: "魔導アヴェンジャー追従型",
            de: "A-Rächer[p] der XII. Legion",
            fr: "vengeur magitek S1",
        },
        (65, 258) => LocalisedText {
            en: "Ghido",
            ja: "ギード",
            de: "Ghido",
            fr: "Ghido",
        },
        (65, 259) => LocalisedText {
            en: "wind-up Exdeath",
            ja: "マメット・エクスデス",
            de: "Aufzieh-Exdeath",
            fr: "Exdeath miniature",
        },
        (65, 260) => LocalisedText {
            en: "wind-up Khloe",
            ja: "マメット・クロ",
            de: "Khloe-Puppe",
            fr: "Khloe miniature",
        },
        (65, 261) => LocalisedText {
            en: "wind-up Susano",
            ja: "マメット・スサノオ",
            de: "Susano-Puppe",
            fr: "petit Susano",
        },
        (65, 262) => LocalisedText {
            en: "wind-up Lakshmi",
            ja: "マメット・ラクシュミ",
            de: "Lakshmi-Puppe",
            fr: "petite Lakshmi",
        },
        (65, 263) => LocalisedText {
            en: "wind-up Bismarck",
            ja: "マメット・ビスマルク",
            de: "Aufzieh-Bismarck",
            fr: "petit Bismarck",
        },
        (65, 264) => LocalisedText {
            en: "wind-up Hien",
            ja: "マメット・ヒエン",
            de: "Hien-Puppe",
            fr: "Hien miniature",
        },
        (65, 265) => LocalisedText {
            en: "wind-up Ravana",
            ja: "マメット・ラーヴァナ",
            de: "Mini-Ravana",
            fr: "petit Ravana",
        },
        (65, 266) => LocalisedText {
            en: "wind-up Kojin",
            ja: "マメット・コウジン",
            de: "Kojin-Puppe",
            fr: "Kojin mécanique",
        },
        (65, 267) => LocalisedText {
            en: "mameshiba",
            ja: "豆柴",
            de: "Mameshiba",
            fr: "mame shiba",
        },
        (65, 268) => LocalisedText {
            en: "tengu doll",
            ja: "天狗人形",
            de: "Tengu-Puppe",
            fr: "poupée Tengu",
        },
        (65, 269) => LocalisedText {
            en: "wind-up Matanga",
            ja: "マメット・マタンガ",
            de: "Matanga-Puppe",
            fr: "Matanga mécanique",
        },
        (65, 270) => LocalisedText {
            en: "wind-up Ramza",
            ja: "マメット・ラムザ",
            de: "Ramza-Puppe",
            fr: "Ramza miniature",
        },
        (65, 271) => LocalisedText {
            en: "koala joey",
            ja: "子コアラ",
            de: "Koala-Bärchen",
            fr: "bébé koala",
        },
        (65, 272) => LocalisedText {
            en: "salt & pepper seal",
            ja: "ゴマ塩アザラシ",
            de: "Gomashio-Seehund",
            fr: "bébé phoque",
        },
        (65, 273) => LocalisedText {
            en: "axolotl eft",
            ja: "ウーパールーパー",
            de: "Axolotl-Molch",
            fr: "axolotl",
        },
        (65, 274) => LocalisedText {
            en: "wind-up Ixion",
            ja: "マメット・イクシオン",
            de: "Ixion-Puppe",
            fr: "petit Ixion",
        },
        (65, 275) => LocalisedText {
            en: "wind-up Shinryu",
            ja: "マメット・神龍",
            de: "Mini-Shinryu",
            fr: "mini-Shinryu",
        },
        (65, 276) => LocalisedText {
            en: "dress-up Raubahn",
            ja: "ニュー・ラウバーン",
            de: "neu[a] Raubahn-Puppe",
            fr: "nouveau Raubahn miniature",
        },
        (65, 277) => LocalisedText {
            en: "wind-up Ananta",
            ja: "マメット・アナンタ",
            de: "Aufzieh-Ananta",
            fr: "poupée ananta",
        },
        (65, 278) => LocalisedText {
            en: "private moai",
            ja: "ミニモアイ",
            de: "Moai-Statuette",
            fr: "mini moai",
        },
        (65, 279) => LocalisedText {
            en: "white whittret",
            ja: "シロイタチ",
            de: "Schneewiesel",
            fr: "fouine blanche",
        },
        (65, 280) => LocalisedText {
            en: "motley egg",
            ja: "ジョーカーエッグ",
            de: "Joker-Ei",
            fr: "œuf bouffon",
        },
        (65, 281) => LocalisedText {
            en: "wind-up Kefka",
            ja: "マメット・ケフカ",
            de: "Kefka-Puppe",
            fr: "Kefka miniature",
        },
        (65, 282) => LocalisedText {
            en: "wind-up Magnai",
            ja: "マメット・マグナイ",
            de: "Magnai-Puppe",
            fr: "Magnai miniature",
        },
        (65, 283) => LocalisedText {
            en: "aurelia polyp",
            ja: "舞海月",
            de: "Aurelia-Polyp",
            fr: "aurélie polype",
        },
        (65, 284) => LocalisedText {
            en: "Byakko cub",
            ja: "白小虎",
            de: "Byakko-Jungtier",
            fr: "bébé Byakko",
        },
        (65, 285) => LocalisedText {
            en: "wind-up Fafnir",
            ja: "マメット・ファヴニル",
            de: "Mini-Fafnir",
            fr: "Fafnir mécanique",
        },
        (65, 286) => LocalisedText {
            en: "wind-up Mithra",
            ja: "マメット・ミスラ",
            de: "Mithra-Puppe",
            fr: "poupée mithra",
        },
        (65, 287) => LocalisedText {
            en: "the Prince of Anemos",
            ja: "プリンス・オブ・アネモス",
            de: "Prinz[p] von Anemos",
            fr: "Prince d'Anemos",
        },
        (65, 288) => LocalisedText {
            en: "komainu",
            ja: "狛犬",
            de: "Komainu",
            fr: "petit Komainu",
        },
        (65, 289) => LocalisedText {
            en: "the Gold Whisker",
            ja: "ナマキン",
            de: "Goldbartel",
            fr: "Namazu nain doré",
        },
        (65, 290) => LocalisedText {
            en: "monkey king",
            ja: "セイテンショウセイ",
            de: "Affenkönig",
            fr: "petit Qitian Dasheng",
        },
        (65, 291) => LocalisedText {
            en: "clockwork lantern",
            ja: "クロックワークランタン",
            de: "Uhrwerk-Laterne",
            fr: "lanterne mécanique",
        },
        (65, 292) => LocalisedText {
            en: "frilled dragon",
            ja: "クラミドサウルス",
            de: "Kragenechse",
            fr: "Lézard à collerette",
        },
        (65, 293) => LocalisedText {
            en: "wind-up Cirina",
            ja: "マメット・シリナ",
            de: "Cirina-Puppe",
            fr: "Cirina miniature",
        },
        (65, 294) => LocalisedText {
            en: "wind-up Sadu",
            ja: "マメット・サドゥ",
            de: "Sadu-Puppe",
            fr: "Sadu miniature",
        },
        (65, 295) => LocalisedText {
            en: "copycat bulb",
            ja: "キャシーバルブ",
            de: "Cassie-Knolle",
            fr: "pousse de Cassie",
        },
        (65, 296) => LocalisedText {
            en: "wind-up Tarutaru",
            ja: "マメット・タルタル",
            de: "Tarutaru-Puppe",
            fr: "poupée tarutaru",
        },
        (65, 297) => LocalisedText {
            en: "wind-up Tsukuyomi",
            ja: "マメット・ツクヨミ",
            de: "Tsukuyomi-Puppe",
            fr: "poupée Tsukuyomi",
        },
        (65, 298) => LocalisedText {
            en: "wind-up Zhloe",
            ja: "マメット・シロ",
            de: "Zhloe-Puppe",
            fr: "Zhloe miniature",
        },
        (65, 299) => LocalisedText {
            en: "Construct 8",
            ja: "労働八号",
            de: "Automat[p] Nr. 8",
            fr: "Bâtisseur n°8",
        },
        (65, 300) => LocalisedText {
            en: "Palico",
            ja: "アイルー",
            de: "Felyne",
            fr: "Felyne",
        },
        (65, 301) => LocalisedText {
            en: "Poogie",
            ja: "プーギー",
            de: "Poogie",
            fr: "Poogie",
        },
        (65, 302) => LocalisedText {
            en: "attendee #777",
            ja: "祭ナマズオ",
            de: "Festival-Namazuo",
            fr: "Namazu nain festif",
        },
        (65, 303) => LocalisedText {
            en: "scarlet peacock",
            ja: "朱孔雀",
            de: "Feuerpfau",
            fr: "paonne vermillon",
        },
        (65, 304) => LocalisedText {
            en: "wind-up Alpha",
            ja: "マメット・アルファ",
            de: "Alpha-Puppe",
            fr: "Alpha mécanique",
        },
        (65, 305) => LocalisedText {
            en: "OMG",
            ja: "プチオメガ",
            de: "Mini-Omega",
            fr: "mini-Oméga",
        },
        (65, 306) => LocalisedText {
            en: "wind-up G'raha Tia",
            ja: "マメット・グ・ラハ",
            de: "G'raha-Tia-Puppe",
            fr: "G'raha Tia miniature",
        },
        (65, 307) => LocalisedText {
            en: "Little Yin",
            ja: "陰気なゴースト人形",
            de: "mürrisch[a] Geistlein",
            fr: "fantôme triste",
        },
        (65, 308) => LocalisedText {
            en: "Little Yang",
            ja: "陽気なゴースト人形",
            de: "munter[a] Geistlein",
            fr: "fantôme joyeux",
        },
        (65, 309) => LocalisedText {
            en: "wind-up Cloud",
            ja: "マメット・クラウド",
            de: "Cloud-Puppe",
            fr: "Cloud miniature",
        },
        (65, 310) => LocalisedText {
            en: "wind-up Aerith",
            ja: "マメット・エアリス",
            de: "Aerith-Puppe",
            fr: "Aerith miniature",
        },
        (65, 311) => LocalisedText {
            en: "wind-up Tifa",
            ja: "マメット・ティファ",
            de: "Tifa-Puppe",
            fr: "Tifa miniature",
        },
        (65, 312) => LocalisedText {
            en: "mudpie",
            ja: "ドロップチ",
            de: "Schlammprinz",
            fr: "pâté de boue",
        },
        (65, 313) => LocalisedText {
            en: "bridesmoogle",
            ja: "ブーケモーグリ",
            de: "Brautjungfer-Mogry",
            fr: "Mog d'honneur",
        },
        (65, 314) => LocalisedText {
            en: "wind-up Elvaan",
            ja: "マメット・エルヴァーン",
            de: "Elvaan-Marionette",
            fr: "poupée elvaan",
        },
        (65, 315) => LocalisedText {
            en: "dhalmel calf",
            ja: "ダルメル・カフ",
            de: "Dhalmel-Kalb",
            fr: "bébé dhalmel",
        },
        (65, 316) => LocalisedText {
            en: "capybara pup",
            ja: "カピバラ・パップ",
            de: "Capybara-Jungtier",
            fr: "bébé cabiaï",
        },
        (65, 317) => LocalisedText {
            en: "baby gremlin",
            ja: "ベイビーグレムリン",
            de: "Baby-Gremlin",
            fr: "bébé gremlin",
        },
        (65, 318) => LocalisedText {
            en: "Conditional Virtue",
            ja: "プチヴァーチュー",
            de: "Tugendhaft[a] Begleiter",
            fr: "mini-vertu",
        },
        (65, 319) => LocalisedText {
            en: "yukinko snowflake",
            ja: "コユキンコ",
            de: "Schnee-Korpokkur",
            fr: "ko-yukinko",
        },
        (65, 320) => LocalisedText {
            en: "dress-up Tataru",
            ja: "ニュー・タタル",
            de: "neu[a] Tataru-Puppe",
            fr: "nouvelle Tataru miniature",
        },
        (65, 321) => LocalisedText {
            en: "wind-up weapon",
            ja: "アームドミニオン",
            de: "Panzerwaffen-Miniatur",
            fr: "mascotte blindée",
        },
        (65, 322) => LocalisedText {
            en: "wind-up Qalyana",
            ja: "マメット・カリヤナ",
            de: "Aufzieh-Qalyana",
            fr: "poupée qalyana",
        },
        (65, 323) => LocalisedText {
            en: "wind-up Redback",
            ja: "マメット・ベニコウラ",
            de: "Rotpanzer-Puppe",
            fr: "Kojin rouge mécanique",
        },
        (65, 324) => LocalisedText {
            en: "minitek conveyor",
            ja: "マメット・キャリアー",
            de: "Mini-Magitek-Transporter",
            fr: "transporteur miniature",
        },
        (65, 325) => LocalisedText {
            en: "wind-up Fran",
            ja: "マメット・フラン",
            de: "Fran-Puppe",
            fr: "Fran miniature",
        },
        (65, 326) => LocalisedText {
            en: "wind-up Nu Mou",
            ja: "マメット・ン・モゥ",
            de: "Nu-Mou-Stofftier",
            fr: "Nu Mou miniature",
        },
        (65, 327) => LocalisedText {
            en: "Seitei",
            ja: "青帝",
            de: "Seitei",
            fr: "dragonnet azur",
        },
        (65, 328) => LocalisedText {
            en: "zephyrous zabuton",
            ja: "ふかふか座布団",
            de: "Zabuton-Sitzkissen",
            fr: "coussin zabuton moelleux",
        },
        (65, 329) => LocalisedText {
            en: "abroader otter",
            ja: "アロハウソウソ",
            de: "Aloha-Otter",
            fr: "loutre aloha",
        },
        (65, 330) => LocalisedText {
            en: "hedgehoglet",
            ja: "ヘッジホッグ",
            de: "Igel",
            fr: "hérisson mignon",
        },
        (65, 331) => LocalisedText {
            en: "brave new Y'shtola",
            ja: "ファースト・ヤ・シュトラ",
            de: "Splitterwelt-Y'shtola",
            fr: "Y'shtola miniature (premier reflet)",
        },
        (65, 332) => LocalisedText {
            en: "wind-up Ryne",
            ja: "マメット・リーン",
            de: "Ryne-Puppe",
            fr: "Ryne miniature",
        },
        (65, 333) => LocalisedText {
            en: "black hayate",
            ja: "黒柴",
            de: "schwarz[a] Shiba-Welpe",
            fr: "kuro shiba",
        },
        (65, 334) => LocalisedText {
            en: "chameleon",
            ja: "カメレオン",
            de: "Zwergchamäleon",
            fr: "caméléon",
        },
        (65, 335) => LocalisedText {
            en: "wind-up Fuath",
            ja: "マメット・フーア",
            de: "Fuath-Puppe",
            fr: "Fuath miniature",
        },
        (65, 336) => LocalisedText {
            en: "armadillo bowler",
            ja: "アルマジロ・ボーラー",
            de: "artistisch[a] Gürteltier",
            fr: "tatou-boule",
        },
        (65, 337) => LocalisedText {
            en: "the Great Serpent of Ronka",
            ja: "ロンカの水蛇？",
            de: "Klein[a] Schlange[p] von Ronka",
            fr: "serpent marin de Ronka ?",
        },
        (65, 338) => LocalisedText {
            en: "micro gigantender",
            ja: "ミニギガテンダー",
            de: "Mini-Riesenkaktor",
            fr: "mini-gigapampa",
        },
        (65, 339) => LocalisedText {
            en: "clionid larva",
            ja: "クリオニッド・ラーヴァ",
            de: "Clionidenlarve",
            fr: "jeune clionide",
        },
        (65, 340) => LocalisedText {
            en: "bitty duckbill",
            ja: "タイニーダックビル",
            de: "Baby-Schnabeltier",
            fr: "ornithorynque nain",
        },
        (65, 341) => LocalisedText {
            en: "Eden Minor",
            ja: "ミニエデン",
            de: "Mini-Eden",
            fr: "mini-Éden",
        },
        (65, 342) => LocalisedText {
            en: "giant beaver",
            ja: "ジャイアントビーバー",
            de: "Riesenbiber",
            fr: "castor géant",
        },
        (65, 343) => LocalisedText {
            en: "wind-up Suzaku",
            ja: "からくり朱雀",
            de: "Aufzieh-Suzaku",
            fr: "Suzaku mécanique",
        },
        (65, 344) => LocalisedText {
            en: "wind-up Omega-M",
            ja: "マメット・オメガM",
            de: "männlich[a] Omega-Puppe",
            fr: "Oméga-M miniature",
        },
        (65, 345) => LocalisedText {
            en: "wind-up Omega-F",
            ja: "マメット・オメガF",
            de: "weiblich[a] Omega-Puppe",
            fr: "Oméga-F miniature",
        },
        (65, 346) => LocalisedText {
            en: "tinker's bell",
            ja: "ティンカーベル",
            de: "Glöckchen",
            fr: "Clochette",
        },
        (65, 347) => LocalisedText {
            en: "tiny echevore",
            ja: "タイニーエケボア",
            de: "klein[a] Echevore",
            fr: "échevore nain",
        },
        (65, 348) => LocalisedText {
            en: "sharksucker-class insubmersible",
            ja: "コバンザメ級潜水艦",
            de: "Tauchboot[p] der Haiklasse",
            fr: "sous-marin de classe Rémora",
        },
        (65, 349) => LocalisedText {
            en: "shoebill",
            ja: "ハシビロコウ",
            de: "Schuhschnäbelchen",
            fr: "bec-en-sabot",
        },
        (65, 350) => LocalisedText {
            en: "butterfly effect",
            ja: "バタフライエフェクト",
            de: "Schmetterlingspaar",
            fr: "couple de papillons",
        },
        (65, 351) => LocalisedText {
            en: "ironfrog ambler",
            ja: "フロッガーA",
            de: "Eisenfrosch",
            fr: "grenouilleur A",
        },
        (65, 352) => LocalisedText {
            en: "forgiven hate",
            ja: "フォーギヴン・ヘイト",
            de: "Mini-Sündenbereuer",
            fr: "haine pardonnée",
        },
        (65, 353) => LocalisedText {
            en: "bacon bits",
            ja: "ポークシー・ビッツ",
            de: "Mini-Quiexie",
            fr: "bout-de-cochon",
        },
        (65, 354) => LocalisedText {
            en: "wind-up pixie",
            ja: "マメット・ピクシー",
            de: "Pixie-Puppe",
            fr: "Pixie miniature",
        },
        (65, 355) => LocalisedText {
            en: "mystic weapon",
            ja: "ミスティックウェポン",
            de: "mystisch[a] Waffe",
            fr: "arme mystique",
        },
        (65, 356) => LocalisedText {
            en: "meerkat",
            ja: "ミーアキャット",
            de: "Erdmännchen",
            fr: "suricate",
        },
        (65, 357) => LocalisedText {
            en: "miniature white knight",
            ja: "ホワイト・ミニナイト",
            de: "weiß[a] Miniaturritter",
            fr: "mini-cavalier blanc",
        },
        (65, 358) => LocalisedText {
            en: "Cute Justice",
            ja: "極合金ジャスティス",
            de: "Mini-Brutalus",
            fr: "Ultragokin Justice",
        },
        (65, 359) => LocalisedText {
            en: "domakin",
            ja: "ドマキン",
            de: "Domakin",
            fr: "domakin",
        },
        (65, 360) => LocalisedText {
            en: "dress-up Estinien",
            ja: "ニュー・エスティニアン",
            de: "neu[a] Estinien-Puppe",
            fr: "nouvel Estinien miniature",
        },
        (65, 361) => LocalisedText {
            en: "little leannan",
            ja: "プチリャナンシー",
            de: "klein[a] Leanan",
            fr: "mini-Leannan Sith",
        },
        (65, 362) => LocalisedText {
            en: "wind-up hobgoblin",
            ja: "マメット・ホブゴブリン",
            de: "Hobgoblin-Puppe",
            fr: "hobgobelin miniature",
        },
        (65, 363) => LocalisedText {
            en: "Cerberpup",
            ja: "ケルベロス・パピー",
            de: "Cerberus-Welpe",
            fr: "cerbère chiot",
        },
        (65, 364) => LocalisedText {
            en: "pod 054",
            ja: "ポッド054",
            de: "Pod[p] 054",
            fr: "pod 054",
        },
        (65, 365) => LocalisedText {
            en: "pod 316",
            ja: "ポッド316",
            de: "Pod[p] 316",
            fr: "pod 316",
        },
        (65, 366) => LocalisedText {
            en: "sungold Talos",
            ja: "ゴールドタロース",
            de: "Gold-Talos",
            fr: "talos d'or",
        },
        (65, 367) => LocalisedText {
            en: "Malone",
            ja: "マローン",
            de: "Malone",
            fr: "Malone",
        },
        (65, 368) => LocalisedText {
            en: "little leafman",
            ja: "草こびと",
            de: "Grasgrummelchen",
            fr: "enfeuillé miniature",
        },
        (65, 369) => LocalisedText {
            en: "the Behelmeted Serpent of Ronka",
            ja: "ホパルの水蛇？",
            de: "Wasserschlange[p] des Hopl",
            fr: "Grand Serpent de Hopl ?",
        },
        (65, 370) => LocalisedText {
            en: "the Behatted Serpent of Ronka",
            ja: "キタリの水蛇？",
            de: "Wasserschlange[p] der Qitari",
            fr: "Grand Serpent des Qitari ?",
        },
        (65, 371) => LocalisedText {
            en: "silver dasher",
            ja: "シマエナガ",
            de: "Langschwanzmeise",
            fr: "mésange à longue queue",
        },
        (65, 372) => LocalisedText {
            en: "paissa patissier",
            ja: "パイッサ・クリナリアン",
            de: "Paissa-Gourmet",
            fr: "païssa cuisinier",
        },
        (65, 373) => LocalisedText {
            en: "paissa threadpuller",
            ja: "パイッサ・ウィーバー",
            de: "Paissa-Weber",
            fr: "païssa couturier",
        },
        (65, 374) => LocalisedText {
            en: "ancient one",
            ja: "古代人形",
            de: "Ascian-Puppe",
            fr: "poupée ascienne",
        },
        (65, 375) => LocalisedText {
            en: "the Major-General",
            ja: "コマンダー・シャーク",
            de: "Haikrieger",
            fr: "requin commandeur",
        },
        (65, 376) => LocalisedText {
            en: "wind-up Dulia-Chai",
            ja: "マメット・ドゥリア",
            de: "Dulia-Chai-Puppe",
            fr: "Dulia-Chai miniature",
        },
        (65, 377) => LocalisedText {
            en: "amaro hatchling",
            ja: "アマロ・ハッチリング",
            de: "Amaro-Küken",
            fr: "bébé amaro",
        },
        (65, 378) => LocalisedText {
            en: "Laladile",
            ja: "ララダイル",
            de: "Laladil",
            fr: "laladile",
        },
        (65, 379) => LocalisedText {
            en: "unlucky rabbit",
            ja: "アンラッキーラビット",
            de: "Unglückshase",
            fr: "lapin malchanceux",
        },
        (65, 380) => LocalisedText {
            en: "Lalinator 5.H0",
            ja: "ラリホードワーフ",
            de: "Holladrio-Zwerg",
            fr: "Nain lali-ho",
        },
        (65, 381) => LocalisedText {
            en: "wind-up Mystel",
            ja: "マメット・ミステル",
            de: "Mystel-Puppe",
            fr: "Mystel miniature",
        },
        (65, 382) => LocalisedText {
            en: "wind-up Ardbert",
            ja: "マメット・アルバート",
            de: "Aufzieh-Ardbert",
            fr: "Ardbert miniature",
        },
        (65, 383) => LocalisedText {
            en: "magitek helldiver F1",
            ja: "魔導ヘルダイバー追従型",
            de: "A-Magitek-Höllentaucher",
            fr: "plongeur infernal S1",
        },
        (65, 384) => LocalisedText {
            en: "weatherproof gaelicat",
            ja: "もちもちゲイラキャット",
            de: "Geira-Flauschekatze",
            fr: "gaélichat emmitouflé",
        },
        (65, 385) => LocalisedText {
            en: "ephemeral necromancer",
            ja: "幻影のネクロマンサー",
            de: "Phantom-Nekromantin",
            fr: "nécromancienne évanescente",
        },
        (65, 386) => LocalisedText {
            en: "Allagan melon",
            ja: "アラガンメロン",
            de: "allagisch[a] Melone",
            fr: "melon allagois",
        },
        (65, 387) => LocalisedText {
            en: "sand fox",
            ja: "サンドフォックス",
            de: "Sandfuchs",
            fr: "renard des sables",
        },
        (65, 388) => LocalisedText {
            en: "petit pteranodon",
            ja: "プチラノドン",
            de: "Pteranoknirps",
            fr: "ptiranodon",
        },
        (65, 389) => LocalisedText {
            en: "Dáinsleif F1",
            ja: "ダーインスレイヴ追従型",
            de: "A-Dáinsleif",
            fr: "Dáinsleif S1",
        },
        (65, 390) => LocalisedText {
            en: "Lord Enma",
            ja: "エンマ",
            de: "Meister Enma[p]",
            fr: "Baron Enma",
        },
        (65, 391) => LocalisedText {
            en: "Lord Ananta",
            ja: "蛇王カイラ",
            de: "Meister Ananta[p]",
            fr: "Baron Ananta",
        },
        (65, 392) => LocalisedText {
            en: "Zazel",
            ja: "ぬらりひょん",
            de: "Sasel",
            fr: "Zazel",
        },
        (65, 393) => LocalisedText {
            en: "Damona",
            ja: "百鬼姫",
            de: "Dämona",
            fr: "Damona",
        },
        (65, 394) => LocalisedText {
            en: "2B automaton",
            ja: "オートマトン２Ｂ",
            de: "Aufzieh-2B",
            fr: "automate 2B",
        },
        (65, 395) => LocalisedText {
            en: "2P automaton",
            ja: "オートマトン２Ｐ",
            de: "Aufzieh-2P",
            fr: "automate 2P",
        },
        (65, 396) => LocalisedText {
            en: "much-coveted Mora",
            ja: "モモラ・モラ！！！",
            de: "anhänglich[a] Momora Mora",
            fr: "Momo Ramora",
        },
        (65, 397) => LocalisedText {
            en: "Syldrion-class insubmersible",
            ja: "シルドラ級潜水艦P型",
            de: "P-Tauchboot[p] der Klasse „Syldra“",
            fr: "sous-marin aérien de classe Syldra",
        },
        (65, 398) => LocalisedText {
            en: "wind-up Gaia",
            ja: "マメット・ガイア",
            de: "Gaia-Puppe",
            fr: "Gaïa miniature",
        },
        (65, 399) => LocalisedText {
            en: "wind-up Palom",
            ja: "マメット・パロム",
            de: "Palom-Puppe",
            fr: "Palom miniature",
        },
        (65, 400) => LocalisedText {
            en: "wind-up Porom",
            ja: "マメット・ポロム",
            de: "Porom-Puppe",
            fr: "Porom miniature",
        },
        (65, 401) => LocalisedText {
            en: "wind-up Edge",
            ja: "マメット・エッジ",
            de: "Edge-Puppe",
            fr: "Edge miniature",
        },
        (65, 402) => LocalisedText {
            en: "wind-up Rydia",
            ja: "マメット・リディア",
            de: "Rydia-Puppe",
            fr: "Rydia miniature",
        },
        (65, 403) => LocalisedText {
            en: "wind-up Rosa",
            ja: "マメット・ローザ",
            de: "Rosa-Puppe",
            fr: "Rosa miniature",
        },
        (65, 404) => LocalisedText {
            en: "Save the Princess",
            ja: "セイブ・ザ・プリンセス",
            de: "Prinzessinnenwache",
            fr: "Garde-la-Princesse",
        },
        (65, 405) => LocalisedText {
            en: "Drippy",
            ja: "ドリッピー",
            de: "Tröpfel",
            fr: "Drippy",
        },
        (65, 406) => LocalisedText {
            en: "Trike",
            ja: "トライク",
            de: "Tricero",
            fr: "Trike",
        },
        (65, 407) => LocalisedText {
            en: "golden beaver",
            ja: "ゴールデンビーバー",
            de: "Goldbiber",
            fr: "castor doré",
        },
        (65, 408) => LocalisedText {
            en: "gull",
            ja: "ガル",
            de: "Möwe",
            fr: "Goé",
        },
        (65, 409) => LocalisedText {
            en: "anteater",
            ja: "アントイーター",
            de: "Ameisenbär",
            fr: "Tamandua",
        },
        (65, 410) => LocalisedText {
            en: "dolphin calf",
            ja: "ドルフィン・カフ",
            de: "Delphinkalb",
            fr: "bébé dauphin",
        },
        (65, 411) => LocalisedText {
            en: "magitek predator F1",
            ja: "魔導プレデター追従型",
            de: "A-Magitek-Prädator",
            fr: "prédateur magitek S1",
        },
        (65, 412) => LocalisedText {
            en: "Senorita Sabotender",
            ja: "セニョリータ・サボテンダー",
            de: "Señorita[p] Kaktor",
            fr: "Señorita Pampa",
        },
        (65, 413) => LocalisedText {
            en: "Benben stone",
            ja: "ベンベン",
            de: "Benben-Pyramide",
            fr: "Benben",
        },
        (65, 414) => LocalisedText {
            en: "wanderer's campfire",
            ja: "放浪者の焚き火",
            de: "Reise-Lagerfeuer",
            fr: "feu de camp de voyageur",
        },
        (65, 415) => LocalisedText {
            en: "smaller stubby",
            ja: "機械生命体",
            de: "Maschinenwesen-Puppe",
            fr: "forme de vie mécanique",
        },
        (65, 416) => LocalisedText {
            en: "wind-up Lyna",
            ja: "マメット・ライナ",
            de: "Lyna-Puppe",
            fr: "Lyna miniature",
        },
        (65, 417) => LocalisedText {
            en: "wind-up Runar",
            ja: "マメット・ルナル",
            de: "Runar-Puppe",
            fr: "Runar miniature",
        },
        (65, 418) => LocalisedText {
            en: "wind-up Gunnhildr",
            ja: "マメット・グンヒルド",
            de: "Gunnhildr-Puppe",
            fr: "Gunnhildr miniature",
        },
        (65, 419) => LocalisedText {
            en: "9S automaton",
            ja: "オートマトン９Ｓ",
            de: "Aufzieh-9S",
            fr: "automate 9S",
        },
        (65, 420) => LocalisedText {
            en: "wind-up Rudy",
            ja: "マメット・ルディ",
            de: "Rudy-Stofftier",
            fr: "Rudy miniature",
        },
        (65, 421) => LocalisedText {
            en: "squirrel emperor",
            ja: "幻栗斎",
            de: "Phantom-Rissai",
            fr: "Rissai chimérique",
        },
        (65, 422) => LocalisedText {
            en: "Flag",
            ja: "フラッグ",
            de: "Fähnchen",
            fr: "Drapo",
        },
        (65, 423) => LocalisedText {
            en: "wee Ea",
            ja: "プチイーア",
            de: "Klein-Ea",
            fr: "mini-Éa",
        },
        (65, 424) => LocalisedText {
            en: "wind-up Magus Sisters",
            ja: "マメット・メーガス",
            de: "Magus-Schwestern-Puppentrio",
            fr: "sœurs Magus miniatures",
        },
        (65, 425) => LocalisedText {
            en: "optimus Omicron",
            ja: "オプティマス・オミクロン",
            de: "Optimus Omikron[p]",
            fr: "Omicron Optimus",
        },
        (65, 426) => LocalisedText {
            en: "wind-up Anima",
            ja: "マメット・アニマ",
            de: "Aufzieh-Anima",
            fr: "Anima miniature",
        },
        (65, 427) => LocalisedText {
            en: "starbird",
            ja: "スターバード",
            de: "Sternenvogel",
            fr: "oiseau stellaire",
        },
        (65, 428) => LocalisedText {
            en: "Nagxian cat",
            ja: "ナグサキャット",
            de: "nagxianisch[a] Katze",
            fr: "chat nagxien",
        },
        (65, 429) => LocalisedText {
            en: "brave new Urianger",
            ja: "ニュー・ウリエンジェ",
            de: "neu[a] Urianger-Puppe",
            fr: "nouvel Urianger miniature",
        },
        (65, 430) => LocalisedText {
            en: "wind-up Vrtra",
            ja: "マメット・ヴリトラ",
            de: "Aufzieh-Vrtra",
            fr: "Vrtra miniature",
        },
        (65, 431) => LocalisedText {
            en: "hippo calf",
            ja: "ヒッポカフ",
            de: "Flusspferdkalb",
            fr: "bébé hippo",
        },
        (65, 432) => LocalisedText {
            en: "crabe de la crabe",
            ja: "カニカニ",
            de: "Krabbenkrebs",
            fr: "crabe du crabe",
        },
        (65, 433) => LocalisedText {
            en: "Prince Lunatender",
            ja: "ルナテンダー・プリンス",
            de: "Lunatender-Prinz",
            fr: "Prince pampa sélénien",
        },
        (65, 434) => LocalisedText {
            en: "tiny troll",
            ja: "リトルトロル",
            de: "Tröllchen",
            fr: "petit troll",
        },
        (65, 435) => LocalisedText {
            en: "Caduceus",
            ja: "カドゥケウス",
            de: "Caduceus",
            fr: "Caducée",
        },
        (65, 436) => LocalisedText {
            en: "adventure basket",
            ja: "アドベンチャーバスケット",
            de: "Abenteurerkorb",
            fr: "malle d'aventure",
        },
        (65, 437) => LocalisedText {
            en: "Golden Dhyata",
            ja: "黄金のチャタ号",
            de: "Golden[a] Dhyata",
            fr: "Dhyata dorée",
        },
        (65, 438) => LocalisedText {
            en: "greener gleaner",
            ja: "グリーングリーナー",
            de: "Laub-Klauber",
            fr: "glaneur vert",
        },
        (65, 439) => LocalisedText {
            en: "wind-up Grebuloff",
            ja: "マメット・グレビュオフ",
            de: "Typhoid-Puppe",
            fr: "Grébuloff miniature",
        },
        (65, 440) => LocalisedText {
            en: "nosferatu",
            ja: "ノスフェラトゥ",
            de: "Nosferatu-Puppe",
            fr: "Nosferatu",
        },
        (65, 441) => LocalisedText {
            en: "wind-up herois",
            ja: "マメット・ヘロイス",
            de: "heroisch[a] Puppe",
            fr: "héroïne miniature",
        },
        (65, 442) => LocalisedText {
            en: "wind-up Daivadipa",
            ja: "マメット・ダイヴァディーパ",
            de: "Daivadipa-Puppe",
            fr: "Daïvadipa miniature",
        },
        (65, 443) => LocalisedText {
            en: "royal lunatender",
            ja: "ルナテンダー・ロイヤル",
            de: "königlich[a] Lunatender",
            fr: "Roi pampa sélénien",
        },
        (65, 445) => LocalisedText {
            en: "wind-up kangaroo",
            ja: "マメット・カンガルー",
            de: "Aufzieh-Känguru",
            fr: "kangourou miniature",
        },
        (65, 446) => LocalisedText {
            en: "protonaught",
            ja: "プロト・ドレッドノート",
            de: "Proto-Brummonaut",
            fr: "proto-Dreadnaught",
        },
        (65, 447) => LocalisedText {
            en: "teacup Kapikulu",
            ja: "カププクル",
            de: "Kapipikulu",
            fr: "kapipikulu",
        },
        (65, 448) => LocalisedText {
            en: "Chewy",
            ja: "チワワン",
            de: "Chiwauwau",
            fr: "chihuaouaf",
        },
        (65, 449) => LocalisedText {
            en: "clockwork Novus D",
            ja: "クロックワーク・ノウスD",
            de: "Aufzieh-Novus[p] D",
            fr: "Novus D. mécanique",
        },
        (65, 451) => LocalisedText {
            en: "wind-up Azeyma",
            ja: "マメット・アーゼマ",
            de: "Azeyma-Puppe",
            fr: "Azeyma miniature",
        },
        (65, 453) => LocalisedText {
            en: "blue-footed booby",
            ja: "青足のカツォ",
            de: "Blaufußtölpel",
            fr: "fou à pattes bleues",
        },
        (67, 50) => LocalisedText {
            en: "Royal Authority Combo",
            ja: "コンボ：ロイヤルアソリティ",
            de: "Kombo Königsklinge",
            fr: "Combo Autorité royale",
        },
        (67, 51) => LocalisedText {
            en: "Storm's Path Combo",
            ja: "コンボ：シュトルムヴィント",
            de: "Kombo Sturmkeil",
            fr: "Combo Couperet de justice",
        },
        (67, 52) => LocalisedText {
            en: "Souleater Combo",
            ja: "コンボ：ソウルイーター",
            de: "Kombo Seelenfresser",
            fr: "Combo Dévoreur d'âme",
        },
        (67, 53) => LocalisedText {
            en: "Solid Barrel Combo",
            ja: "コンボ：ソリッドバレル",
            de: "Kombo Stahlwalze",
            fr: "Combo Salve tranchante",
        },
        (67, 54) => LocalisedText {
            en: "Fountain Combo",
            ja: "コンボ：ファウンテン",
            de: "Kombo Fontäne",
            fr: "Combo Fontaine",
        },
        (67, 55) => LocalisedText {
            en: "Phantom Rush Combo",
            ja: "コンボ：夢幻闘舞",
            de: "Kombo Phantomstoß",
            fr: "Combo Rixe évanescente",
        },
        (67, 56) => LocalisedText {
            en: "Wheeling Thrust Combo",
            ja: "コンボ：竜尾大車輪",
            de: "Kombo Fächerstoß",
            fr: "Combo Percée tournante",
        },
        (67, 57) => LocalisedText {
            en: "Aeolian Edge Combo",
            ja: "コンボ：旋風刃",
            de: "Kombo Äolusklinge",
            fr: "Combo Lame éolienne",
        },
        (67, 58) => LocalisedText {
            en: "Kasha Combo",
            ja: "コンボ：花車",
            de: "Kombo Kasha",
            fr: "Combo Kasha",
        },
        (67, 59) => LocalisedText {
            en: "Infernal Slice Combo",
            ja: "コンボ：インファナルスライス",
            de: "Kombo Infernalischer Schlitzer",
            fr: "Combo Tranchage infernal",
        },
        (68, 12958) => LocalisedText {
            en: "Wisdom of the Aetherweaver",
            ja: "術士の記憶",
            de: "Vermächtnis des Zauberers",
            fr: "Mémoire d'un prêtre guerrier",
        },
        (68, 12959) => LocalisedText {
            en: "Wisdom of the Martialist",
            ja: "闘士の記憶",
            de: "Vermächtnis des Kämpfers",
            fr: "Mémoire d'un protecteur intrépide",
        },
        (68, 12960) => LocalisedText {
            en: "Wisdom of the Platebearer",
            ja: "重騎兵の記憶",
            de: "Vermächtnis des Ritters",
            fr: "Mémoire d'un soldat cuirassé",
        },
        (68, 12961) => LocalisedText {
            en: "Wisdom of the Guardian",
            ja: "守護者の記憶",
            de: "Vermächtnis des Wächters",
            fr: "Mémoire d'un gardien",
        },
        (68, 12962) => LocalisedText {
            en: "Wisdom of the Ordained",
            ja: "司祭の記憶",
            de: "Vermächtnis des Priesters",
            fr: "Mémoire d'un prêtre",
        },
        (68, 12963) => LocalisedText {
            en: "Wisdom of the Skirmisher",
            ja: "武人の記憶",
            de: "Vermächtnis des Kampfkünstlers",
            fr: "Mémoire d'un combattant",
        },
        (68, 12964) => LocalisedText {
            en: "Wisdom of the Watcher",
            ja: "斥候の記憶",
            de: "Vermächtnis des Spähers",
            fr: "Mémoire d'un éclaireur",
        },
        (68, 12965) => LocalisedText {
            en: "Wisdom of the Templar",
            ja: "聖騎士の記憶",
            de: "Vermächtnis des heiligen Ritters",
            fr: "Mémoire d'un chevalier sacré",
        },
        (68, 12966) => LocalisedText {
            en: "Wisdom of the Irregular",
            ja: "狂戦士の記憶",
            de: "Vermächtnis des Berserkers",
            fr: "Mémoire d'un guerrier enragé",
        },
        (68, 12967) => LocalisedText {
            en: "Wisdom of the Breathtaker",
            ja: "盗賊の記憶",
            de: "Vermächtnis des Diebes",
            fr: "Mémoire d'un voleur",
        },
        (68, 12968) => LocalisedText {
            en: "Spirit of the Remembered",
            ja: "英傑の加護",
            de: "Segen des Helden",
            fr: "Égide des héros vaillants",
        },
        (68, 12969) => LocalisedText {
            en: "Protect L",
            ja: "ロゴス・プロテス",
            de: "Logos-Protes",
            fr: "Évocation Logos : Bouclier",
        },
        (68, 12970) => LocalisedText {
            en: "Shell L",
            ja: "ロゴス・シェル",
            de: "Logos-Vallum",
            fr: "Évocation Logos : Barrière",
        },
        (68, 12971) => LocalisedText {
            en: "Death L",
            ja: "ロゴス・デス",
            de: "Logos-Tod",
            fr: "Évocation Logos : Mort",
        },
        (68, 12972) => LocalisedText {
            en: "Focus L",
            ja: "ロゴス・チャージ",
            de: "Logos-Ladung",
            fr: "Évocation Logos : Focalisation",
        },
        (68, 12973) => LocalisedText {
            en: "Paralyze L",
            ja: "ロゴス・パライズ",
            de: "Logos-Paralyse",
            fr: "Évocation Logos : Paralysie",
        },
        (68, 12974) => LocalisedText {
            en: "Paralyze L III",
            ja: "ロゴス・パライガ",
            de: "Logos-Paralyga",
            fr: "Évocation Logos : Méga Paralysie",
        },
        (68, 12975) => LocalisedText {
            en: "Swift L",
            ja: "ロゴス・スウィフト",
            de: "Logos-Eile",
            fr: "Évocation Logos : Vélocité",
        },
        (68, 12976) => LocalisedText {
            en: "Featherfoot L",
            ja: "ロゴス・フェザーステップ",
            de: "Logos-Leichtfuß",
            fr: "Évocation Logos : Pieds légers",
        },
        (68, 12977) => LocalisedText {
            en: "Spirit Dart L",
            ja: "ロゴス・スピリットダート",
            de: "Logos-Geistpfeil",
            fr: "Évocation Logos : Flèche spirituelle",
        },
        (68, 12978) => LocalisedText {
            en: "Catastrophe L",
            ja: "ロゴス・カタスト",
            de: "Logos-Magieexplosion",
            fr: "Évocation Logos : Catastrophe",
        },
        (68, 12979) => LocalisedText {
            en: "Dispel L",
            ja: "ロゴス・ディスペル",
            de: "Logos-Entzaubern",
            fr: "Évocation Logos : Dissipation",
        },
        (68, 12980) => LocalisedText {
            en: "Feint L",
            ja: "ロゴス・フェイント",
            de: "Logos-Finte",
            fr: "Évocation Logos : Feinte",
        },
        (68, 12981) => LocalisedText {
            en: "Stealth L",
            ja: "ロゴス・ステルス",
            de: "Logos-Tarnung",
            fr: "Évocation Logos : Furtivité",
        },
        (68, 12982) => LocalisedText {
            en: "Aetherial Manipulation L",
            ja: "ロゴス・ステップ",
            de: "Logos-Äthermanipulation",
            fr: "Évocation Logos : Bond",
        },
        (68, 12983) => LocalisedText {
            en: "Backstep L",
            ja: "ロゴス・スウェー",
            de: "Logos-Entrinnung",
            fr: "Évocation Logos : Bond arrière",
        },
        (68, 12984) => LocalisedText {
            en: "Tranquilizer L",
            ja: "ロゴス・トランキライザー",
            de: "Logos-Betäubung",
            fr: "Évocation Logos : Tranquillisant",
        },
        (68, 12985) => LocalisedText {
            en: "Bloodbath L",
            ja: "ロゴス・ブラッドバス",
            de: "Logos-Blutbad",
            fr: "Évocation Logos : Bain de sang",
        },
        (68, 12986) => LocalisedText {
            en: "Rejuvenate L",
            ja: "ロゴス・リチャージ",
            de: "Logos-Aufladung",
            fr: "Évocation Logos : Restauration",
        },
        (68, 12987) => LocalisedText {
            en: "Haymaker L",
            ja: "ロゴス・カウンター",
            de: "Logos-Konter",
            fr: "Évocation Logos : Haymaker",
        },
        (68, 12988) => LocalisedText {
            en: "Rapid Recast L",
            ja: "ロゴス・ファストリキャスト",
            de: "Logos-Velocitas",
            fr: "Évocation Logos : Recharge rapide",
        },
        (68, 12989) => LocalisedText {
            en: "Cure L",
            ja: "ロゴス・ケアル",
            de: "Logos-Vita",
            fr: "Évocation Logos : Soin",
        },
        (68, 12990) => LocalisedText {
            en: "Cure L II",
            ja: "ロゴス・ケアルラ",
            de: "Logos-Vitra",
            fr: "Évocation Logos : Extra Soin",
        },
        (68, 12991) => LocalisedText {
            en: "Stoneskin L",
            ja: "ロゴス・ストンスキン",
            de: "Logos-Steinhaut",
            fr: "Évocation Logos : Cuirasse",
        },
        (68, 12992) => LocalisedText {
            en: "Cure L III",
            ja: "ロゴス・ケアルガ",
            de: "Logos-Vitaga",
            fr: "Évocation Logos : Méga Soin",
        },
        (68, 12993) => LocalisedText {
            en: "Regen L",
            ja: "ロゴス・リジェネ",
            de: "Logos-Regena",
            fr: "Évocation Logos : Récup",
        },
        (68, 12994) => LocalisedText {
            en: "Esuna L",
            ja: "ロゴス・エスナ",
            de: "Logos-Medica",
            fr: "Évocation Logos : Guérison",
        },
        (68, 12995) => LocalisedText {
            en: "Incense L",
            ja: "ロゴス・エンミティ",
            de: "Logos-Hohn",
            fr: "Évocation Logos : Incitation",
        },
        (68, 12996) => LocalisedText {
            en: "Raise L",
            ja: "ロゴス・レイズ",
            de: "Logos-Wiederbeleben",
            fr: "Évocation Logos : Vie",
        },
        (68, 12997) => LocalisedText {
            en: "Bravery L",
            ja: "ロゴス・ブレイブ",
            de: "Logos-Courage",
            fr: "Évocation Logos : Bravoure",
        },
        (68, 12998) => LocalisedText {
            en: "Solid Shield L",
            ja: "ロゴス・フィジカルシールド",
            de: "Logos-Schild",
            fr: "Évocation Logos : Protection physique",
        },
        (68, 12999) => LocalisedText {
            en: "Spell Shield L",
            ja: "ロゴス・マジックシールド",
            de: "Logos-Magieschild",
            fr: "Évocation Logos : Protection magique",
        },
        (68, 13000) => LocalisedText {
            en: "Reflect L",
            ja: "ロゴス・リフレク",
            de: "Logos-Reflektion",
            fr: "Évocation Logos : Miroir",
        },
        (68, 13001) => LocalisedText {
            en: "Smite L",
            ja: "ロゴス・スマイト",
            de: "Logos-Schattenhieb",
            fr: "Évocation Logos : Châtiment",
        },
        (68, 13002) => LocalisedText {
            en: "Refresh L",
            ja: "ロゴス・リフレッシュ",
            de: "Logos-Erfrischung",
            fr: "Évocation Logos : Tonification",
        },
        (68, 13003) => LocalisedText {
            en: "Banish L",
            ja: "ロゴス・バニシュ",
            de: "Logos-Bann",
            fr: "Évocation Logos : Bannissement",
        },
        (68, 13004) => LocalisedText {
            en: "Banish L III",
            ja: "ロゴス・バニシュガ",
            de: "Logos-Bannga",
            fr: "Évocation Logos : Méga Bannissement",
        },
        (68, 13005) => LocalisedText {
            en: "Magic Burst L",
            ja: "ロゴス・マジックバースト",
            de: "Logos-Magieexplosion",
            fr: "Évocation Logos : Explosion magique",
        },
        (68, 13006) => LocalisedText {
            en: "Double Edge L",
            ja: "ロゴス・ダブルエッジ",
            de: "Logos-Zweischneid",
            fr: "Évocation Logos : Double tranchant",
        },
        (68, 13007) => LocalisedText {
            en: "Eagle Eye Shot L",
            ja: "ロゴス・イーグルアイ",
            de: "Logos-Adlerauge",
            fr: "Évocation Logos : Œil d'aigle",
        },
        (68, 14476) => LocalisedText {
            en: "Perception L",
            ja: "ロゴス・サイトラ",
            de: "Logos-Tigeraugen",
            fr: "Évocation Logos : Vision radar",
        },
        (68, 14477) => LocalisedText {
            en: "Wisdom of the Elder",
            ja: "賢者の記憶",
            de: "Vermächtnis des Weisen",
            fr: "Mémoire d'un sage",
        },
        (68, 14478) => LocalisedText {
            en: "Wisdom of the Duelist",
            ja: "剣豪の記憶",
            de: "Vermächtnis des Fechtmeisters",
            fr: "Mémoire d'une fine lame",
        },
        (68, 14479) => LocalisedText {
            en: "Wisdom of the Fiendhunter",
            ja: "弓聖の記憶",
            de: "Vermächtnis des Bogengeweihten",
            fr: "Mémoire d'un archer émérite",
        },
        (68, 14480) => LocalisedText {
            en: "Wisdom of the Indomitable",
            ja: "豪傑の記憶",
            de: "Vermächtnis des Helden",
            fr: "Mémoire d'un héros illustre",
        },
        (68, 14481) => LocalisedText {
            en: "Sacrifice L",
            ja: "ロゴス・サクリファイス",
            de: "Logos-Aufopferung",
            fr: "Évocation Logos : Sacrifice",
        },
        (69, 11383) => LocalisedText {
            en: "Snort",
            ja: "鼻息",
            de: "Nieser",
            fr: "Souffle nasal",
        },
        (69, 11384) => LocalisedText {
            en: "4-tonze Weight",
            ja: "4トンズ",
            de: "4-Tonzen-Gewicht",
            fr: "Poids de 4 tonz",
        },
        (69, 11385) => LocalisedText {
            en: "Water Cannon",
            ja: "水鉄砲",
            de: "Wasserkanone",
            fr: "Tir d'eau",
        },
        (69, 11386) => LocalisedText {
            en: "Song of Torment",
            ja: "苦悶の歌",
            de: "Gesang der Gemarterten",
            fr: "Chant de l'agonie",
        },
        (69, 11387) => LocalisedText {
            en: "High Voltage",
            ja: "高圧電流",
            de: "Hochstrom",
            fr: "Haute tension",
        },
        (69, 11388) => LocalisedText {
            en: "Bad Breath",
            ja: "臭い息",
            de: "Schlechter Atem",
            fr: "Mauvaise haleine",
        },
        (69, 11389) => LocalisedText {
            en: "Flying Frenzy",
            ja: "狂乱",
            de: "Rasender Sturz",
            fr: "Démence",
        },
        (69, 11390) => LocalisedText {
            en: "Aqua Breath",
            ja: "アクアブレス",
            de: "Aqua-Atem",
            fr: "Aquasouffle",
        },
        (69, 11391) => LocalisedText {
            en: "Plaincracker",
            ja: "プレーンクラッカー",
            de: "Plattensprenger",
            fr: "Brise-plaine",
        },
        (69, 11392) => LocalisedText {
            en: "Acorn Bomb",
            ja: "どんぐり爆弾",
            de: "Eichelbombe",
            fr: "Gland explosif",
        },
        (69, 11393) => LocalisedText {
            en: "Bristle",
            ja: "怒髪天",
            de: "Borstigkeit",
            fr: "Hérissement",
        },
        (69, 11394) => LocalisedText {
            en: "Mind Blast",
            ja: "マインドブラスト",
            de: "Geiststoß",
            fr: "Explosion mentale",
        },
        (69, 11395) => LocalisedText {
            en: "Blood Drain",
            ja: "吸血",
            de: "Blutsauger",
            fr: "Suceur de sang",
        },
        (69, 11396) => LocalisedText {
            en: "Bomb Toss",
            ja: "爆弾投げ",
            de: "Bombenwurf",
            fr: "Lancer de bombe",
        },
        (69, 11397) => LocalisedText {
            en: "1000 Needles",
            ja: "針千本",
            de: "1000 Nadeln",
            fr: "1000 aiguilles",
        },
        (69, 11398) => LocalisedText {
            en: "Drill Cannons",
            ja: "ドリルキャノン",
            de: "Bohrkanone",
            fr: "Canon-foreur",
        },
        (69, 11399) => LocalisedText {
            en: "the Look",
            ja: "怪視線",
            de: "Schiefer Blick",
            fr: "Regard intense",
        },
        (69, 11400) => LocalisedText {
            en: "Sharpened Knife",
            ja: "とぎたて",
            de: "Blankes Messer",
            fr: "Couteau aiguisé",
        },
        (69, 11401) => LocalisedText {
            en: "Loom",
            ja: "ルーム",
            de: "Böse Ahnung",
            fr: "Approche masquée",
        },
        (69, 11402) => LocalisedText {
            en: "Flame Thrower",
            ja: "火炎放射",
            de: "Flammensturm",
            fr: "Crache-flammes",
        },
        (69, 11403) => LocalisedText {
            en: "Faze",
            ja: "猫だまし",
            de: "Verwirrung",
            fr: "Désorientation",
        },
        (69, 11404) => LocalisedText {
            en: "Glower",
            ja: "グラワー",
            de: "Finsterer Blick",
            fr: "Regard noir",
        },
        (69, 11405) => LocalisedText {
            en: "Missile",
            ja: "ミサイル",
            de: "Rakete",
            fr: "Missile",
        },
        (69, 11406) => LocalisedText {
            en: "White Wind",
            ja: "ホワイトウィンド",
            de: "Weißer Wind",
            fr: "Vent blanc",
        },
        (69, 11407) => LocalisedText {
            en: "Final Sting",
            ja: "ファイナルスピア",
            de: "Schlussstich",
            fr: "Dard final",
        },
        (69, 11408) => LocalisedText {
            en: "Self-destruct",
            ja: "自爆",
            de: "Selbstzerstörung",
            fr: "Auto-destruction",
        },
        (69, 11409) => LocalisedText {
            en: "Transfusion",
            ja: "融合",
            de: "Assimilation",
            fr: "Fusion",
        },
        (69, 11410) => LocalisedText {
            en: "Toad Oil",
            ja: "ガマの脂",
            de: "Schleimiges Sekret",
            fr: "Huile de crapaud",
        },
        (69, 11411) => LocalisedText {
            en: "Off-guard",
            ja: "ガードオファ",
            de: "Einschüchtern",
            fr: "Sans défense",
        },
        (69, 11412) => LocalisedText {
            en: "Sticky Tongue",
            ja: "スティッキータン",
            de: "Spitze Zunge",
            fr: "Langue poisseuse",
        },
        (69, 11413) => LocalisedText {
            en: "Tail Screw",
            ja: "テールスクリュー",
            de: "Schweifschraube",
            fr: "Queue forante",
        },
        (69, 11414) => LocalisedText {
            en: "Level 5 Petrify",
            ja: "レベル5石化",
            de: "Petrifikation Stufe 5",
            fr: "Pétrification niveau 5",
        },
        (69, 11415) => LocalisedText {
            en: "Moon Flute",
            ja: "月の笛",
            de: "Mondflöte",
            fr: "Flûte lunaire",
        },
        (69, 11416) => LocalisedText {
            en: "Doom",
            ja: "死の宣告",
            de: "Verhängnis",
            fr: "Glas",
        },
        (69, 11417) => LocalisedText {
            en: "Mighty Guard",
            ja: "マイティガード",
            de: "Totalabwehr",
            fr: "Garde puissante",
        },
        (69, 11418) => LocalisedText {
            en: "Ice Spikes",
            ja: "アイススパイク",
            de: "Eisstachel",
            fr: "Pointes de glace",
        },
        (69, 11419) => LocalisedText {
            en: "the Ram's Voice",
            ja: "氷結の咆哮",
            de: "Stimme des Widders",
            fr: "Voix du bélier",
        },
        (69, 11420) => LocalisedText {
            en: "the Dragon's Voice",
            ja: "雷電の咆哮",
            de: "Stimme des Drachen",
            fr: "Voix du dragon",
        },
        (69, 11421) => LocalisedText {
            en: "Peculiar Light",
            ja: "不思議な光",
            de: "Unheimliche Aura",
            fr: "Lumière sinistre",
        },
        (69, 11422) => LocalisedText {
            en: "Ink Jet",
            ja: "インクジェット",
            de: "Tintenstrahl",
            fr: "Tir d'encre",
        },
        (69, 11423) => LocalisedText {
            en: "Flying Sardine",
            ja: "フライングサーディン",
            de: "Spucksardine",
            fr: "Sardine volante",
        },
        (69, 11424) => LocalisedText {
            en: "Diamondback",
            ja: "超硬化",
            de: "Diamantrücken",
            fr: "Blindage",
        },
        (69, 11425) => LocalisedText {
            en: "Fire Angon",
            ja: "ファイアアンゴン",
            de: "Feuer-Ango",
            fr: "Angon enflammé",
        },
        (69, 11426) => LocalisedText {
            en: "Feather Rain",
            ja: "フェザーレイン",
            de: "Federregen",
            fr: "Pluie de plumes",
        },
        (69, 11427) => LocalisedText {
            en: "Eruption",
            ja: "エラプション",
            de: "Eruption",
            fr: "Éruption",
        },
        (69, 11428) => LocalisedText {
            en: "Mountain Buster",
            ja: "マウンテンバスター",
            de: "Bergsprenger",
            fr: "Casse-montagnes",
        },
        (69, 11429) => LocalisedText {
            en: "Shock Strike",
            ja: "ショックストライク",
            de: "Schockschlag",
            fr: "Frappe de choc",
        },
        (69, 11430) => LocalisedText {
            en: "Glass Dance",
            ja: "氷雪乱舞",
            de: "Gläserner Tanz",
            fr: "Danse de glace",
        },
        (69, 11431) => LocalisedText {
            en: "Veil of the Whorl",
            ja: "水神のヴェール",
            de: "Wogenschleier",
            fr: "Voile du Déchaîneur",
        },
        (69, 18295) => LocalisedText {
            en: "Alpine Draft",
            ja: "アルペンドラフト",
            de: "Gletscherluft",
            fr: "Vent alpin",
        },
        (69, 18296) => LocalisedText {
            en: "Protean Wave",
            ja: "プロティアンウェイブ",
            de: "Proteische Welle",
            fr: "Vague inconstante",
        },
        (69, 18297) => LocalisedText {
            en: "Northerlies",
            ja: "猛吹雪",
            de: "Schneesturm",
            fr: "Aquilons",
        },
        (69, 18298) => LocalisedText {
            en: "Electrogenesis",
            ja: "エレクトロジェネシス",
            de: "Elektrogenese",
            fr: "Électrogenèse",
        },
        (69, 18299) => LocalisedText {
            en: "Kaltstrahl",
            ja: "カルトシュトラール",
            de: "Kaltstrahl",
            fr: "Kaltstrahl",
        },
        (69, 18300) => LocalisedText {
            en: "Abyssal Transfixion",
            ja: "アビサルトランスフィクション",
            de: "Abyssal-Transfixion",
            fr: "Transfixion abyssale",
        },
        (69, 18301) => LocalisedText {
            en: "Chirp",
            ja: "チャープ",
            de: "Fiepsen",
            fr: "Gazouillement",
        },
        (69, 18302) => LocalisedText {
            en: "Eerie Soundwave",
            ja: "怪音波",
            de: "Ekplexie",
            fr: "Onde lugubre",
        },
        (69, 18303) => LocalisedText {
            en: "Pom Cure",
            ja: "ポンポンケアル",
            de: "Vitakupo",
            fr: "Pompon soignant",
        },
        (69, 18304) => LocalisedText {
            en: "Gobskin",
            ja: "ゴブスキン",
            de: "Gobhaut",
            fr: "Gobépiderme",
        },
        (69, 18305) => LocalisedText {
            en: "Magic Hammer",
            ja: "マジックハンマー",
            de: "Zauberhammer",
            fr: "Marteau magique",
        },
        (69, 18306) => LocalisedText {
            en: "Avail",
            ja: "防御指示",
            de: "Absichern",
            fr: "Ordre défensif",
        },
        (69, 18307) => LocalisedText {
            en: "Frog Legs",
            ja: "フロッグレッグ",
            de: "Froschkonzert",
            fr: "Ritournelle batracienne",
        },
        (69, 18308) => LocalisedText {
            en: "Sonic Boom",
            ja: "ソニックブーム",
            de: "Schallwelle",
            fr: "Bang supersonique",
        },
        (69, 18309) => LocalisedText {
            en: "Whistle",
            ja: "ホイッスル",
            de: "Pfeife",
            fr: "Sifflement",
        },
        (69, 18310) => LocalisedText {
            en: "White Knight's Tour",
            ja: "ホワイトナイトツアー",
            de: "Weißer Rösselsprung",
            fr: "Tour de cavalier blanc",
        },
        (69, 18311) => LocalisedText {
            en: "Black Knight's Tour",
            ja: "ブラックナイトツアー",
            de: "Schwarzer Rösselsprung",
            fr: "Tour de cavalier noir",
        },
        (69, 18312) => LocalisedText {
            en: "Level 5 Death",
            ja: "レベル5デス",
            de: "Todesstufe 5",
            fr: "Mort niveau 5",
        },
        (69, 18313) => LocalisedText {
            en: "Launcher",
            ja: "ランチャー",
            de: "Raketenwerfer",
            fr: "Lance-roquettes",
        },
        (69, 18314) => LocalisedText {
            en: "Perpetual Ray",
            ja: "パーペチュアルレイ",
            de: "Perpetueller Strahl",
            fr: "Rayon perpétuel",
        },
        (69, 18315) => LocalisedText {
            en: "Cactguard",
            ja: "カクトガード",
            de: "Sukkulentenschild",
            fr: "Cactugarde",
        },
        (69, 18316) => LocalisedText {
            en: "Revenge Blast",
            ja: "リベンジブラスト",
            de: "Rachestoß",
            fr: "Éclat vengeur",
        },
        (69, 18317) => LocalisedText {
            en: "Angel Whisper",
            ja: "天使のささやき",
            de: "Engelsflüstern",
            fr: "Grâce angélique",
        },
        (69, 18318) => LocalisedText {
            en: "Exuviation",
            ja: "イグジュビエーション",
            de: "Exuviation",
            fr: "Exuviation",
        },
        (69, 18319) => LocalisedText {
            en: "Reflux",
            ja: "リフラックス",
            de: "Reflux",
            fr: "Reflux",
        },
        (69, 18320) => LocalisedText {
            en: "Devour",
            ja: "捕食",
            de: "Verschlingen",
            fr: "Dévoration",
        },
        (69, 18321) => LocalisedText {
            en: "Condensed Libra",
            ja: "プチライブラ",
            de: "Kurzanalyse",
            fr: "Acuité concentrée",
        },
        (69, 18322) => LocalisedText {
            en: "Aetheric Mimicry",
            ja: "エーテルコピー",
            de: "Äther-Mimikry",
            fr: "Imitation éthéréenne",
        },
        (69, 18323) => LocalisedText {
            en: "Surpanakha",
            ja: "徹甲散弾",
            de: "Surpanakha",
            fr: "Surpanakha",
        },
        (69, 18324) => LocalisedText {
            en: "Quasar",
            ja: "クエーサー",
            de: "Quasar",
            fr: "Quasar",
        },
        (69, 18325) => LocalisedText {
            en: "J Kick",
            ja: "ジャスティスキック",
            de: "Gewissenstritt",
            fr: "Pied justicier",
        },
        (69, 23264) => LocalisedText {
            en: "Triple Trident",
            ja: "銛三段",
            de: "Dreizack",
            fr: "Triple trident",
        },
        (69, 23265) => LocalisedText {
            en: "Tingle",
            ja: "ビリビリ",
            de: "Kribbeln",
            fr: "Fourmillement",
        },
        (69, 23266) => LocalisedText {
            en: "Tatami-gaeshi",
            ja: "畳返しの術",
            de: "Tatami-gaeshi",
            fr: "Tatami-gaeshi",
        },
        (69, 23267) => LocalisedText {
            en: "Cold Fog",
            ja: "冷たい霧",
            de: "Eisiger Nebel",
            fr: "Brume glaciale",
        },
        (69, 23269) => LocalisedText {
            en: "Stotram",
            ja: "ストトラム",
            de: "Stotram",
            fr: "Stotram",
        },
        (69, 23270) => LocalisedText {
            en: "Saintly Beam",
            ja: "セイントビーム",
            de: "Heiligenstrahl",
            fr: "Faisceaux sacrés",
        },
        (69, 23271) => LocalisedText {
            en: "Feculent Flood",
            ja: "ドロドロ掬い投げ",
            de: "Modrige Flut",
            fr: "Sukuinage-gadoue",
        },
        (69, 23272) => LocalisedText {
            en: "Angel's Snack",
            ja: "天使のおやつ",
            de: "Engelspeise",
            fr: "Angélique",
        },
        (69, 23273) => LocalisedText {
            en: "Chelonian Gate",
            ja: "玄結界",
            de: "Barriere",
            fr: "Barrière mystique",
        },
        (69, 23275) => LocalisedText {
            en: "The Rose of Destruction",
            ja: "闘霊弾",
            de: "Zermalmender Geist",
            fr: "Rose de la destruction",
        },
        (69, 23276) => LocalisedText {
            en: "Basic Instinct",
            ja: "闘争本能",
            de: "Kampfinstinkt",
            fr: "Instinct profond",
        },
        (69, 23277) => LocalisedText {
            en: "Ultravibration",
            ja: "超振動",
            de: "Ultravibration",
            fr: "Ultravibration",
        },
        (69, 23278) => LocalisedText {
            en: "Blaze",
            ja: "ブレイズ",
            de: "Frost",
            fr: "Givre",
        },
        (69, 23279) => LocalisedText {
            en: "Mustard Bomb",
            ja: "マスタードボム",
            de: "Senfbombe",
            fr: "Obus d'ypérite",
        },
        (69, 23280) => LocalisedText {
            en: "Dragon Force",
            ja: "ドラゴンフォース",
            de: "Drachenkraft",
            fr: "Dracoforce",
        },
        (69, 23281) => LocalisedText {
            en: "Aetherial Spark",
            ja: "エーテリックスパーク",
            de: "Ätherfunken",
            fr: "Étincelle éthérée",
        },
        (69, 23282) => LocalisedText {
            en: "Hydro Pull",
            ja: "ハイドロプル",
            de: "Hydrosog",
            fr: "Hydrotraction",
        },
        (69, 23283) => LocalisedText {
            en: "Malediction of Water",
            ja: "水脈の呪詛",
            de: "Verfluchung des Wassers",
            fr: "Malédiction de l'eau",
        },
        (69, 23284) => LocalisedText {
            en: "Choco Meteor",
            ja: "チョコメテオ",
            de: "Choco-Meteo",
            fr: "Choco-météore",
        },
        (69, 23285) => LocalisedText {
            en: "Matra Magic",
            ja: "マトラマジック",
            de: "Matra-Magie",
            fr: "Magie Matra",
        },
        (69, 23286) => LocalisedText {
            en: "Peripheral Synthesis",
            ja: "プリントアウト",
            de: "Replikator",
            fr: "Impression",
        },
        (69, 23287) => LocalisedText {
            en: "Both Ends",
            ja: "如意大旋風",
            de: "Beide Enden",
            fr: "Coup de bâton tournicotant",
        },
        (69, 23288) => LocalisedText {
            en: "Phantom Flurry",
            ja: "鬼宿脚",
            de: "Phantomhast",
            fr: "Frénésie spectrale",
        },
        (69, 23290) => LocalisedText {
            en: "Nightbloom",
            ja: "月下彼岸花",
            de: "Monddämmerung",
            fr: "Lis araignée",
        },
        (70, 20701) => LocalisedText {
            en: "Lost Paralyze III",
            ja: "ロスト・パライガ",
            de: "Verschollenes Paralyga",
            fr: "Méga Paralysie oubliée",
        },
        (70, 20702) => LocalisedText {
            en: "Lost Banish III",
            ja: "ロスト・バニシュガ",
            de: "Verschollenes Verbannga",
            fr: "Méga Bannissement oublié",
        },
        (70, 20703) => LocalisedText {
            en: "Lost Manawall",
            ja: "ロスト・ウォール",
            de: "Verschollene Mana-Wand",
            fr: "Mur de mana oublié",
        },
        (70, 20704) => LocalisedText {
            en: "Lost Dispel",
            ja: "ロスト・ディスペル",
            de: "Verschollenes Entzaubern",
            fr: "Dissipation oubliée",
        },
        (70, 20705) => LocalisedText {
            en: "Lost Stealth",
            ja: "ロスト・ステルス",
            de: "Verschollene Coeurl-Pfoten",
            fr: "Furtivité oubliée",
        },
        (70, 20706) => LocalisedText {
            en: "Lost Spellforge",
            ja: "ロスト・スペルフォージ",
            de: "Verschollene Zauberschmiede",
            fr: "Forgesort oublié",
        },
        (70, 20707) => LocalisedText {
            en: "Lost Steelsting",
            ja: "ロスト・スチールスティング",
            de: "Verschollener Stahlstachel",
            fr: "Morsure de l'acier oubliée",
        },
        (70, 20708) => LocalisedText {
            en: "Lost Swift",
            ja: "ロスト・スウィフト",
            de: "Verschollene Eile",
            fr: "Vélocité oubliée",
        },
        (70, 20709) => LocalisedText {
            en: "Lost Protect",
            ja: "ロスト・プロテス",
            de: "Verschollenes Protes",
            fr: "Bouclier oublié",
        },
        (70, 20710) => LocalisedText {
            en: "Lost Shell",
            ja: "ロスト・シェル",
            de: "Verschollenes Vallum",
            fr: "Barrière oubliée",
        },
        (70, 20711) => LocalisedText {
            en: "Lost Reflect",
            ja: "ロスト・リフレク",
            de: "Verschollene Reflektion",
            fr: "Miroir oublié",
        },
        (70, 20712) => LocalisedText {
            en: "Lost Stoneskin",
            ja: "ロスト・ストンスキン",
            de: "Verschollene Steinhaut",
            fr: "Cuirasse oubliée",
        },
        (70, 20713) => LocalisedText {
            en: "Lost Bravery",
            ja: "ロスト・ブレイブ",
            de: "Verschollene Courage",
            fr: "Bravoure oubliée",
        },
        (70, 20714) => LocalisedText {
            en: "Lost Focus",
            ja: "ロスト・チャージ",
            de: "Verschollene Ladung",
            fr: "Focalisation oubliée",
        },
        (70, 20715) => LocalisedText {
            en: "Lost Font of Magic",
            ja: "ロスト・マジックフォント",
            de: "Verschollene Magiefontäne",
            fr: "Fontaine de magie oubliée",
        },
        (70, 20716) => LocalisedText {
            en: "Lost Font of Skill",
            ja: "ロスト・スキルフォント",
            de: "Verschollene Technikfontäne",
            fr: "Fontaine de technique oubliée",
        },
        (70, 20717) => LocalisedText {
            en: "Lost Font of Power",
            ja: "ロスト・パワーフォント",
            de: "Verschollene Machtfontäne",
            fr: "Fontaine de puissance oubliée",
        },
        (70, 20718) => LocalisedText {
            en: "Lost Slash",
            ja: "ロスト・スラッシュ",
            de: "Verschollener Schlitzer",
            fr: "Taillade oubliée",
        },
        (70, 20719) => LocalisedText {
            en: "Lost Death",
            ja: "ロスト・デス",
            de: "Verschollener Tod",
            fr: "Mort oubliée",
        },
        (70, 20720) => LocalisedText {
            en: "Banner of Noble Ends",
            ja: "背水の境地",
            de: "Gemüt der Verdrossenheit",
            fr: "Seuil de la frénésie",
        },
        (70, 20721) => LocalisedText {
            en: "Banner of Honored Sacrifice",
            ja: "捨身の境地",
            de: "Gemüt der Ungehaltenheit",
            fr: "Seuil de l'abnégation",
        },
        (70, 20722) => LocalisedText {
            en: "Banner of Tireless Conviction",
            ja: "忍耐の境地",
            de: "Gemüt der Beharrlichkeit",
            fr: "Seuil de la patience",
        },
        (70, 20723) => LocalisedText {
            en: "Banner of Firm Resolve",
            ja: "堅守の境地",
            de: "Gemüt der Folgsamkeit",
            fr: "Seuil de la préservation",
        },
        (70, 20724) => LocalisedText {
            en: "Banner of Solemn Clarity",
            ja: "瞑想の境地",
            de: "Gemüt der Besinnlichkeit",
            fr: "Seuil de la contemplation",
        },
        (70, 20725) => LocalisedText {
            en: "Banner of Honed Acuity",
            ja: "鋭敏の境地",
            de: "Gemüt der Empfindsamkeit",
            fr: "Seuil de la perception",
        },
        (70, 20726) => LocalisedText {
            en: "Lost Cure",
            ja: "ロスト・ケアル",
            de: "Verschollene Vita",
            fr: "Soin oublié",
        },
        (70, 20727) => LocalisedText {
            en: "Lost Cure II",
            ja: "ロスト・ケアルラ",
            de: "Verschollene Vitra",
            fr: "Extra Soin oublié",
        },
        (70, 20728) => LocalisedText {
            en: "Lost Cure III",
            ja: "ロスト・ケアルガ",
            de: "Verschollene Vitaga",
            fr: "Méga Soin oublié",
        },
        (70, 20729) => LocalisedText {
            en: "Lost Cure IV",
            ja: "ロスト・ケアルジャ",
            de: "Verschollene Vitaka",
            fr: "Giga Soin oublié",
        },
        (70, 20730) => LocalisedText {
            en: "Lost Arise",
            ja: "ロスト・アレイズ",
            de: "Verschollener Erzengel",
            fr: "Maxi-Vie oubliée",
        },
        (70, 20731) => LocalisedText {
            en: "Lost Incense",
            ja: "ロスト・エンミティ",
            de: "Verschollener Hohn",
            fr: "Incitation oubliée",
        },
        (70, 20732) => LocalisedText {
            en: "Lost Fair Trade",
            ja: "ロスト・フェアトレード",
            de: "Verschollener Handel",
            fr: "Échange équitable oublié",
        },
        (70, 20733) => LocalisedText {
            en: "Mimic",
            ja: "ものまね",
            de: "Nachahmung",
            fr: "Mime",
        },
        (70, 20734) => LocalisedText {
            en: "Dynamis Dice",
            ja: "運命のダイス",
            de: "Würfel des Schicksals",
            fr: "Dé du destin",
        },
        (70, 20735) => LocalisedText {
            en: "Resistance Phoenix",
            ja: "レジスタンスフェニックスフェザー",
            de: "Phönixfeder des Widerstands",
            fr: "Plume de phénix de la résistance",
        },
        (70, 20736) => LocalisedText {
            en: "Resistance Reraiser",
            ja: "レジスタンスリレイザー",
            de: "Lebensfeder des Widerstands",
            fr: "Vitaplume de la résistance",
        },
        (70, 20737) => LocalisedText {
            en: "Resistance Potion Kit",
            ja: "レジスタンスポーションキット",
            de: "Tränkesammlung des Widerstands",
            fr: "Kit de potion de la résistance",
        },
        (70, 20738) => LocalisedText {
            en: "Resistance Ether Kit",
            ja: "レジスタンスエーテルキット",
            de: "Äthersammlung des Widerstands",
            fr: "Kit d'éther de la résistance",
        },
        (70, 20739) => LocalisedText {
            en: "Resistance Medikit",
            ja: "レジスタンスレメディキット",
            de: "Kräutersammlung des Widerstands",
            fr: "Kit médical de la résistance",
        },
        (70, 20740) => LocalisedText {
            en: "Resistance Potion",
            ja: "レジスタンスポーション",
            de: "Heiltrank des Widerstands",
            fr: "Potion de la résistance",
        },
        (70, 20741) => LocalisedText {
            en: "Essence of the Aetherweaver",
            ja: "術士の秘薬",
            de: "Zaubertrank des Veteranen",
            fr: "Breuvage magique de prêtre guerrier",
        },
        (70, 20742) => LocalisedText {
            en: "Essence of the Martialist",
            ja: "闘士の秘薬",
            de: "Zaubertrank des Kämpfers",
            fr: "Breuvage magique de protecteur intrépide",
        },
        (70, 20743) => LocalisedText {
            en: "Essence of the Savior",
            ja: "治癒者の秘薬",
            de: "Zaubertrank des Heilers",
            fr: "Breuvage magique de vivificateur",
        },
        (70, 20744) => LocalisedText {
            en: "Essence of the Veteran",
            ja: "魔戦士の秘薬",
            de: "Zaubertrank des Kampfmagiers",
            fr: "Breuvage magique de mage guerrier",
        },
        (70, 20745) => LocalisedText {
            en: "Essence of the Platebearer",
            ja: "重騎兵の秘薬",
            de: "Zaubertrank des Ritters",
            fr: "Breuvage magique de soldat cuirassé",
        },
        (70, 20746) => LocalisedText {
            en: "Essence of the Guardian",
            ja: "守護者の秘薬",
            de: "Zaubertrank des Beschützers",
            fr: "Breuvage magique de gardien",
        },
        (70, 20747) => LocalisedText {
            en: "Essence of the Ordained",
            ja: "司祭の秘薬",
            de: "Zaubertrank des Priesters",
            fr: "Breuvage magique de prêtre",
        },
        (70, 20748) => LocalisedText {
            en: "Essence of the Skirmisher",
            ja: "武人の秘薬",
            de: "Zaubertrank des Kriegers",
            fr: "Breuvage magique de combattant",
        },
        (70, 20749) => LocalisedText {
            en: "Essence of the Watcher",
            ja: "斥候の秘薬",
            de: "Zaubertrank des Spähers",
            fr: "Breuvage magique d'éclaireur",
        },
        (70, 20750) => LocalisedText {
            en: "Essence of the Profane",
            ja: "破戒僧の秘薬",
            de: "Zaubertrank des Mönchs",
            fr: "Breuvage magique de prêtre hérétique",
        },
        (70, 20751) => LocalisedText {
            en: "Essence of the Irregular",
            ja: "狂戦士の秘薬",
            de: "Zaubertrank des Berserkers",
            fr: "Breuvage magique de guerrier enragé",
        },
        (70, 20752) => LocalisedText {
            en: "Essence of the Breathtaker",
            ja: "盗賊の秘薬",
            de: "Zaubertrank des Diebes",
            fr: "Breuvage magique de voleur",
        },
        (70, 20753) => LocalisedText {
            en: "Essence of the Bloodsucker",
            ja: "吸血鬼の秘薬",
            de: "Zaubertrank des Blutsaugers",
            fr: "Breuvage magique de vampire",
        },
        (70, 20754) => LocalisedText {
            en: "Essence of the Beast",
            ja: "人狼の秘薬",
            de: "Zaubertrank der Wolfsleute",
            fr: "Breuvage magique d'homme-loup",
        },
        (70, 20755) => LocalisedText {
            en: "Essence of the Templar",
            ja: "聖騎士の秘薬",
            de: "Zaubertrank des Paladins",
            fr: "Breuvage magique de chevalier sacré",
        },
        (70, 20756) => LocalisedText {
            en: "Deep Essence of the Aetherweaver",
            ja: "術士の霊薬",
            de: "Wundertrank des Veteranen",
            fr: "Breuvage miraculeux de prêtre guerrier",
        },
        (70, 20757) => LocalisedText {
            en: "Deep Essence of the Martialist",
            ja: "闘士の霊薬",
            de: "Wundertrank des Kämpfers",
            fr: "Breuvage miraculeux de protecteur intrépide",
        },
        (70, 20758) => LocalisedText {
            en: "Deep Essence of the Savior",
            ja: "治癒者の霊薬",
            de: "Wundertrank des Heilers",
            fr: "Breuvage miraculeux de vivificateur",
        },
        (70, 20759) => LocalisedText {
            en: "Deep Essence of the Veteran",
            ja: "魔戦士の霊薬",
            de: "Wundertrank des Kampfmagiers",
            fr: "Breuvage miraculeux de mage guerrier",
        },
        (70, 20760) => LocalisedText {
            en: "Deep Essence of the Platebearer",
            ja: "重騎兵の霊薬",
            de: "Wundertrank des Ritters",
            fr: "Breuvage miraculeux de soldat cuirassé",
        },
        (70, 20761) => LocalisedText {
            en: "Deep Essence of the Guardian ",
            ja: "守護者の霊薬",
            de: "Wundertrank des Beschützers",
            fr: "Breuvage miraculeux de gardien",
        },
        (70, 20762) => LocalisedText {
            en: "Deep Essence of the Ordained",
            ja: "司祭の霊薬",
            de: "Wundertrank des Priesters",
            fr: "Breuvage miraculeux de prêtre",
        },
        (70, 20763) => LocalisedText {
            en: "Deep Essence of the Skirmisher",
            ja: "武人の霊薬",
            de: "Wundertrank des Kriegers",
            fr: "Breuvage miraculeux de combattant",
        },
        (70, 20764) => LocalisedText {
            en: "Deep Essence of the Watcher",
            ja: "斥候の霊薬",
            de: "Wundertrank des Spähers",
            fr: "Breuvage miraculeux d'éclaireur",
        },
        (70, 20765) => LocalisedText {
            en: "Deep Essence of the Profane",
            ja: "破戒僧の霊薬",
            de: "Wundertrank des Mönchs",
            fr: "Breuvage miraculeux de prêtre hérétique",
        },
        (70, 20766) => LocalisedText {
            en: "Deep Essence of the Irregular",
            ja: "狂戦士の霊薬",
            de: "Wundertrank des Berserkers",
            fr: "Breuvage miraculeux de guerrier enragé",
        },
        (70, 20767) => LocalisedText {
            en: "Deep Essence of the Breathtaker",
            ja: "盗賊の霊薬",
            de: "Wundertrank des Diebes",
            fr: "Breuvage miraculeux de voleur",
        },
        (70, 20768) => LocalisedText {
            en: "Deep Essence of the Bloodsucker",
            ja: "吸血鬼の霊薬",
            de: "Wundertrank des Blutsaugers",
            fr: "Breuvage miraculeux de vampire",
        },
        (70, 20769) => LocalisedText {
            en: "Deep Essence of the Beast",
            ja: "人狼の霊薬",
            de: "Wundertrank der Wolfsleute",
            fr: "Breuvage miraculeux d'homme-loup",
        },
        (70, 20770) => LocalisedText {
            en: "Deep Essence of the Templar",
            ja: "聖騎士の霊薬",
            de: "Wundertrank des Paladins",
            fr: "Breuvage miraculeux de chevalier sacré",
        },
        (70, 22344) => LocalisedText {
            en: "Lost Perception",
            ja: "ロスト・サイトラ",
            de: "Verschollene Tigeraugen",
            fr: "Vision radar oubliée",
        },
        (70, 22345) => LocalisedText {
            en: "Lost Sacrifice",
            ja: "ロスト・サクリファイス",
            de: "Verschollene Aufopferung",
            fr: "Sacrifice oublié",
        },
        (70, 22346) => LocalisedText {
            en: "Pure Essence of the Gambler",
            ja: "勝負師の仙薬",
            de: "Maxtrank des Draufgängers",
            fr: "Breuvage érémitique de risque-tout",
        },
        (70, 22347) => LocalisedText {
            en: "Pure Essence of the Elder",
            ja: "賢者の仙薬",
            de: "Maxtrank des Weisen",
            fr: "Breuvage érémitique de sage",
        },
        (70, 22348) => LocalisedText {
            en: "Pure Essence of the Duelist",
            ja: "剣豪の仙薬",
            de: "Maxtrank des Fechtmeisters",
            fr: "Breuvage érémitique de fine lame",
        },
        (70, 22349) => LocalisedText {
            en: "Pure Essence of the Fiendhunter",
            ja: "弓聖の仙薬",
            de: "Maxtrank des Bogengeweihten",
            fr: "Breuvage érémitique d'archer émérite",
        },
        (70, 22350) => LocalisedText {
            en: "Pure Essence of the Indomitable",
            ja: "豪傑の仙薬",
            de: "Maxtrank des Helden",
            fr: "Breuvage érémitique de héros illustre",
        },
        (70, 22351) => LocalisedText {
            en: "Pure Essence of the Divine",
            ja: "聖人の仙薬",
            de: "Maxtrank des Heiligen",
            fr: "Breuvage érémitique de saint",
        },
        (70, 22352) => LocalisedText {
            en: "Lost Flare Star",
            ja: "ロスト・フレアスター",
            de: "Verschollener Flare-Stern",
            fr: "Astre flamboyant oublié",
        },
        (70, 22353) => LocalisedText {
            en: "Lost Rend Armor",
            ja: "ロスト・アーマーブレイク",
            de: "Verschollener Panzerbruch",
            fr: "Brise-armure oublié",
        },
        (70, 22354) => LocalisedText {
            en: "Lost Seraph Strike",
            ja: "ロスト・セラフストライク",
            de: "Verschollener Seraphenschlag",
            fr: "Frappe séraphine oubliée",
        },
        (70, 22355) => LocalisedText {
            en: "Lost Aethershield",
            ja: "ロスト・エーテルシールド",
            de: "Verschollener Ätherschild",
            fr: "Égide d'éther oubliée",
        },
        (70, 22356) => LocalisedText {
            en: "Lost Dervish",
            ja: "ロスト・スピードスター",
            de: "Verschollener Rapidus",
            fr: "Célérité extrême oubliée",
        },
        (70, 23907) => LocalisedText {
            en: "Lodestone",
            ja: "デジョンストーン",
            de: "Stein der Rückführung",
            fr: "Pierre de rapatriement",
        },
        (70, 23908) => LocalisedText {
            en: "Lost Stoneskin II",
            ja: "ロスト・ストンラスキン",
            de: "Verschollene Steinrahaut",
            fr: "Extra Cuirasse oubliée",
        },
        (70, 23909) => LocalisedText {
            en: "Lost Burst",
            ja: "ロスト・バースト",
            de: "Verschollener Knall",
            fr: "Bouillonnement oublié",
        },
        (70, 23910) => LocalisedText {
            en: "Lost Rampage",
            ja: "ロスト・ランページ",
            de: "Verschollener Amok",
            fr: "Ravage oublié",
        },
        (70, 23911) => LocalisedText {
            en: "Light Curtain",
            ja: "光のカーテン",
            de: "Lichtvorhang",
            fr: "Rideau de lumière",
        },
        (70, 23912) => LocalisedText {
            en: "Lost Reraise",
            ja: "ロスト・リレイズ",
            de: "Verschollene Rückkehr aus dem Jenseits",
            fr: "Auréole oubliée",
        },
        (70, 23913) => LocalisedText {
            en: "Lost Chainspell",
            ja: "ロスト・チェインスペル",
            de: "Verschollene Zauberkette",
            fr: "Avalanche de sorts oubliée",
        },
        (70, 23914) => LocalisedText {
            en: "Lost Assassination",
            ja: "ロスト・アサシネイト",
            de: "Verschollener Meuchelmord",
            fr: "Assassinat oublié",
        },
        (70, 23915) => LocalisedText {
            en: "Lost Protect II",
            ja: "ロスト・プロテスII",
            de: "Verschollenes Protes II",
            fr: "Bouclier II oublié",
        },
        (70, 23916) => LocalisedText {
            en: "Lost Shell II",
            ja: "ロスト・シェルII",
            de: "Verschollenes Vallum II",
            fr: "Barrière II oubliée",
        },
        (70, 23917) => LocalisedText {
            en: "Lost Bubble",
            ja: "ロスト・バブル",
            de: "Verschollene Blase",
            fr: "Vitalité oubliée",
        },
        (70, 23918) => LocalisedText {
            en: "Lost Impetus",
            ja: "ロスト・インペタス",
            de: "Verschollener Impuls",
            fr: "Impulsion oubliée",
        },
        (70, 23919) => LocalisedText {
            en: "Lost Excellence",
            ja: "ロスト・エクセレンス",
            de: "Verschollene Exzellenz",
            fr: "Excellence oubliée",
        },
        (70, 23920) => LocalisedText {
            en: "Lost Full Cure",
            ja: "ロスト・フルケア",
            de: "Verschollene Erneuerung",
            fr: "Maxi Soin oublié",
        },
        (70, 23921) => LocalisedText {
            en: "Lost Blood Rage",
            ja: "ロスト・ブラッドレイジ",
            de: "Verschollener Blutzorn",
            fr: "Rage sanguinaire oubliée",
        },
        (70, 23922) => LocalisedText {
            en: "Resistance Elixir",
            ja: "レジスタンスエリクサー",
            de: "Elixier des Widerstands",
            fr: "Élixir de la résistance",
        },
        (28, 2900) => LocalisedText {
            en: "【Job Gauge】",
            ja: "【ジョブHUD】",
            de: "【Job-Balken】",
            fr: "【Interfaces de job】",
        },
        (28, 2901) => LocalisedText {
            en: "Oath Gauge",
            ja: "オウスゲージ",
            de: "Eidbalken",
            fr: "Jauge du serment",
        },
        (28, 2902) => LocalisedText {
            en: "Oath",
            ja: "オウス",
            de: "Eid",
            fr: "Jauge du serment (coût)",
        },
        (28, 2904) => LocalisedText {
            en: "Chakra Gauge",
            ja: "闘気シンボル",
            de: "Chakra-Balken",
            fr: "Jauge des chakras",
        },
        (28, 2905) => LocalisedText {
            en: "Beast Gauge",
            ja: "ビーストゲージ",
            de: "Urkraftbalken",
            fr: "Jauge de la bête",
        },
        (28, 2906) => LocalisedText {
            en: "Wrath",
            ja: "インナービースト",
            de: "Urkraft",
            fr: "Jauge de la bête (coût)",
        },
        (28, 2907) => LocalisedText {
            en: "Dragon Gauge",
            ja: "竜血ゲージ",
            de: "Drachenbalken",
            fr: "Jauge du dragon",
        },
        (28, 2908) => LocalisedText {
            en: "Gaze of the First Brood",
            ja: "ドラゴンアイ",
            de: "Drachenauge",
            fr: "Œil du dragon",
        },
        (28, 2909) => LocalisedText {
            en: "Song Gauge",
            ja: "戦歌ゲージ",
            de: "Liedbalken",
            fr: "Jauge de chant",
        },
        (28, 2910) => LocalisedText {
            en: "Repertoire",
            ja: "詩心",
            de: "Virtuose",
            fr: "Inspiration poétique",
        },
        (28, 2911) => LocalisedText {
            en: "Healing Gauge",
            ja: "ヒーリングケーン",
            de: "Heilbalken",
            fr: "Canne des soins",
        },
        (28, 2912) => LocalisedText {
            en: "Lily",
            ja: "ヒーリングリリー",
            de: "Lilie",
            fr: "Lys",
        },
        (28, 2913) => LocalisedText {
            en: "Elemental Gauge",
            ja: "エレメンタルゲージ",
            de: "Elementarbalken",
            fr: "Jauge des éléments",
        },
        (28, 2914) => LocalisedText {
            en: "Umbral Heart",
            ja: "アンブラルハート",
            de: "Schattenherz",
            fr: "Cœur ombral",
        },
        (28, 2915) => LocalisedText {
            en: "Aetherflow Gauge",
            ja: "エーテルフローシンボル",
            de: "Ätherstrombalken",
            fr: "Jauge de flux d'éther",
        },
        (28, 2916) => LocalisedText {
            en: "Aethertrail Attunement",
            ja: "バハムートエーテル",
            de: "Bahamut-Äther",
            fr: "Éther de Bahamut",
        },
        (28, 2917) => LocalisedText {
            en: "Trance Gauge",
            ja: "トランスゲージ",
            de: "Trancebalken",
            fr: "Jauge de transe",
        },
        (28, 2919) => LocalisedText {
            en: "Faerie Gauge",
            ja: "エーテルゲージ",
            de: "Feenbalken",
            fr: "Jauge d'éther féerique",
        },
        (28, 2920) => LocalisedText {
            en: "Fae Aether",
            ja: "フェイエーテル",
            de: "Feenäther",
            fr: "Jauge d'éther féerique (coût)",
        },
        (28, 2921) => LocalisedText {
            en: "Huton Gauge",
            ja: "風遁シンボル",
            de: "Huton-Balken",
            fr: "Jauge de Fûton",
        },
        (28, 2922) => LocalisedText {
            en: "Ninki Gauge",
            ja: "忍気ゲージ",
            de: "Ninki-Balken",
            fr: "Jauge de Ninki",
        },
        (28, 2923) => LocalisedText {
            en: "Ninki",
            ja: "忍気",
            de: "Ninki",
            fr: "Jauge de Ninki (coût)",
        },
        (28, 2926) => LocalisedText {
            en: "Heat Gauge",
            ja: "ヒートゲージ",
            de: "Hitzebalken",
            fr: "Jauge de chaleur",
        },
        (28, 2927) => LocalisedText {
            en: "Overheat",
            ja: "オーバーヒート",
            de: "Überhitzung",
            fr: "Surchauffe",
        },
        (28, 2928) => LocalisedText {
            en: "Blood Gauge",
            ja: "ブラッドゲージ",
            de: "Blutbalken",
            fr: "Jauge de sang",
        },
        (28, 2930) => LocalisedText {
            en: "Arcana Gauge",
            ja: "アルカナシンボル",
            de: "Arkana-Balken",
            fr: "Table des arcanes",
        },
        (28, 2931) => LocalisedText {
            en: "Sen Gauge",
            ja: "雪月花シンボル",
            de: "Setsugekka-Balken",
            fr: "Marque Setsugekka",
        },
        (28, 2932) => LocalisedText {
            en: "Sen",
            ja: "閃",
            de: "Setsugekka-Symbol",
            fr: "effets Sen",
        },
        (28, 2933) => LocalisedText {
            en: "Kenki Gauge",
            ja: "剣気ゲージ",
            de: "Kenki-Balken",
            fr: "Jauge de Kenki",
        },
        (28, 2934) => LocalisedText {
            en: "Kenki",
            ja: "剣気",
            de: "Kenki",
            fr: "Jauge de Kenki (coût)",
        },
        (28, 2935) => LocalisedText {
            en: "Balance Gauge",
            ja: "バランスゲージ",
            de: "Balancebalken",
            fr: "Jauge d'équilibre",
        },
        (28, 2936) => LocalisedText {
            en: "White Mana",
            ja: "ホワイトマナ",
            de: "Weißes Mana",
            fr: "Mana noire",
        },
        (28, 2937) => LocalisedText {
            en: "Black Mana",
            ja: "ブラックマナ",
            de: "Schwarzes Mana",
            fr: "Mana blanche",
        },
        (28, 2938) => LocalisedText {
            en: "Darkside Gauge",
            ja: "ダークサイドシンボル",
            de: "Mal der Dunklen Seite",
            fr: "Jauge de ténèbres intérieures",
        },
        (28, 2939) => LocalisedText {
            en: "Blood Lily",
            ja: "ブラッドリリー",
            de: "Blutlilie",
            fr: "Lys de sang",
        },
        (28, 2940) => LocalisedText {
            en: "Heat Gauge",
            ja: "エンジニアゲージ",
            de: "Hitzebalken",
            fr: "Thermovoltmètre",
        },
        (28, 2941) => LocalisedText {
            en: "Battery Gauge",
            ja: "バッテリーゲージ",
            de: "Batteriebalken",
            fr: "Jauge de batterie",
        },
        (28, 2942) => LocalisedText {
            en: "Powder Gauge",
            ja: "ソイルシンボル",
            de: "Erda-Trommel",
            fr: "Barillet éthéréen",
        },
        (28, 2943) => LocalisedText {
            en: "Step Gauge",
            ja: "ステップシンボル",
            de: "Schrittindikator",
            fr: "Indicateur de pas",
        },
        (28, 2944) => LocalisedText {
            en: "Fourfold Feathers",
            ja: "幻扇シンボル",
            de: "Tanzfächer",
            fr: "Éventail onirique",
        },
        (28, 2946) => LocalisedText {
            en: "Meditation Gauge",
            ja: "剣圧シンボル",
            de: "Meditationsbalken",
            fr: "Jauge de Tension de la lame",
        },
        (28, 2947) => LocalisedText {
            en: "Soul Gauge",
            ja: "ソウルゲージ",
            de: "Seelenbalken",
            fr: "Jauge des âmes",
        },
        (28, 2948) => LocalisedText {
            en: "Death Gauge",
            ja: "レムールシンボル",
            de: "Lemurenmal",
            fr: "Phylactère du lémure",
        },
        (28, 2949) => LocalisedText {
            en: "Eukrasia",
            ja: "エウクラシアシンボル",
            de: "Eukratisches Äthergramm",
            fr: "Diagramme eucrasique",
        },
        (28, 2950) => LocalisedText {
            en: "Addersgall Gauge",
            ja: "アダーゲージ",
            de: "Humor-Balken",
            fr: "Bile de vipère",
        },
        (28, 2951) => LocalisedText {
            en: "Master's Gauge",
            ja: "必殺技シンボル",
            de: "Blitzkunstbalken",
            fr: "Chakras bestiaux et Nadis",
        },
        (28, 2952) => LocalisedText {
            en: "Mana Stacks",
            ja: "マナスタックシンボル",
            de: "Magiesplitter",
            fr: "Cristal de mana",
        },
        (28, 2953) => LocalisedText {
            en: "Esprit Gauge",
            ja: "エスプリゲージ",
            de: "Tanzfieberbalken",
            fr: "Jauge de fascination",
        },
        (28, 2954) => LocalisedText {
            en: "Coda",
            ja: "コーダシンボル",
            de: "Coda",
            fr: "Coda",
        },
        (28, 2955) => LocalisedText {
            en: "Soul Voice Gauge",
            ja: "ソウルボイスゲージ",
            de: "Seelenstimme",
            fr: "Jauge de voix de l'âme",
        },
        (28, 2956) => LocalisedText {
            en: "Paradox",
            ja: "パラドックスシンボル",
            de: "Paradoxie",
            fr: "Symbole paradoxal",
        },
        (28, 2957) => LocalisedText {
            en: "Addersgall",
            ja: "アダーガル",
            de: "Humor",
            fr: "Jauge de la vipère",
        },
        (28, 2958) => LocalisedText {
            en: "Addersting",
            ja: "アダースティング",
            de: "Komplexion",
            fr: "Croc de vipère",
        },
        (28, 2959) => LocalisedText {
            en: "Dark Arts",
            ja: "ダークアーツ",
            de: "Dunkle Künste",
            fr: "Arts ténébreux",
        },
        (28, 2960) => LocalisedText {
            en: "Polyglot",
            ja: "ポリグロット",
            de: "Polyglott",
            fr: "Polyglotte",
        },
        (28, 2961) => LocalisedText {
            en: "Meditation",
            ja: "剣圧",
            de: "Meditationsebene",
            fr: "Tension de la lame",
        },
        (28, 2962) => LocalisedText {
            en: "Astrosigns",
            ja: "アストロサイン",
            de: "Asterismen",
            fr: "Signes astraux",
        },
        (28, 2963) => LocalisedText {
            en: "Beast Chakra",
            ja: "チャクラ",
            de: "Chakra",
            fr: "Chakra bestial",
        },
        (28, 2964) => LocalisedText {
            en: "Chakra",
            ja: "闘気",
            de: "Schatten-Chakra",
            fr: "Chakra",
        },
        (28, 2965) => LocalisedText {
            en: "Shroud",
            ja: "シュラウド",
            de: "Schleier",
            fr: "Linceul du lémure",
        },
        (28, 2966) => LocalisedText {
            en: "Shroud Gauge",
            ja: "シュラウドゲージ",
            de: "Schleierbalken",
            fr: "Jauge du linceul",
        },
        (28, 2967) => LocalisedText {
            en: "Esprit",
            ja: "エスプリ",
            de: "Tanzfieber",
            fr: "Fascination",
        },
        (28, 2968) => LocalisedText {
            en: "Void Shroud",
            ja: "ヴォイドソウル",
            de: "Nichtsseele",
            fr: "Âme du néant",
        },
        (28, 2969) => LocalisedText {
            en: "Lemure Shroud",
            ja: "レムールソウル",
            de: "Lemurenseele",
            fr: "Âme de lémure",
        },
        (28, 2970) => LocalisedText {
            en: "Soul Voice",
            ja: "ソウルボイス",
            de: "Seelenstimme",
            fr: "Jauge de voix de l'âme",
        },
        (25, 3000) => LocalisedText {
            en: "【Gold Saucer】",
            ja: "【ゴールドソーサー】",
            de: "【Gold Saucer】",
            fr: "【Gold Saucer】",
        },
        (25, 3001) => LocalisedText {
            en: "chocobo race",
            ja: "チョコボレース",
            de: "Chocobo-Rennen",
            fr: "course de chocobos",
        },
        (25, 3002) => LocalisedText {
            en: "race chocobo",
            ja: "競走羽",
            de: "Renn-Chocobo",
            fr: "chocobo de course",
        },
        (25, 3003) => LocalisedText {
            en: "fledgling chocobo",
            ja: "新羽チョコボ",
            de: "Jung-Chocobo",
            fr: "jeune chocobo",
        },
        (25, 3004) => LocalisedText {
            en: "retired chocobo",
            ja: "引退チョコボ",
            de: "Chocobo im Ruhestand",
            fr: "chocobo à la retraite",
        },
        (25, 3005) => LocalisedText {
            en: "covering",
            ja: "カップリング",
            de: "Paarung",
            fr: "accouplement",
        },
        (25, 3006) => LocalisedText {
            en: "rating",
            ja: "総合力",
            de: "Wertung",
            fr: "cote",
        },
        (25, 3007) => LocalisedText {
            en: "pedigree level",
            ja: "血統レベル",
            de: "Zuchtqualität",
            fr: "niveau du pedigree",
        },
        (25, 3008) => LocalisedText {
            en: "course roulette",
            ja: "コースルーレット",
            de: "Zufallsstrecke",
            fr: "course aléatoire",
        },
        (25, 3009) => LocalisedText {
            en: "Sagolii Road",
            ja: "サゴリーロード",
            de: "Sagolii-Straße",
            fr: "Route de Sagolii",
        },
        (25, 3010) => LocalisedText {
            en: "Tranquil Paths",
            ja: "トランキルパス",
            de: "Pfad der Seelenruhe",
            fr: "Sentes tranquilles",
        },
        (25, 3011) => LocalisedText {
            en: "Triple Triad",
            ja: "トリプルトライアド",
            de: "Triple Triad",
            fr: "Triple Triade",
        },
        (25, 3012) => LocalisedText {
            en: "card",
            ja: "カード",
            de: "Karte",
            fr: "carte de Triple Triade",
        },
        (25, 3013) => LocalisedText {
            en: "card type",
            ja: "カードタイプ",
            de: "Kartentyp",
            fr: "famille",
        },
        (25, 3014) => LocalisedText {
            en: "rarity",
            ja: "レアリティ",
            de: "Seltenheit",
            fr: "rareté",
        },
        (25, 3015) => LocalisedText {
            en: "match",
            ja: "対戦",
            de: "Partie",
            fr: "duel",
        },
        (25, 3016) => LocalisedText {
            en: "tournament",
            ja: "大会",
            de: "Wettkampf",
            fr: "compétition",
        },
        (25, 3017) => LocalisedText {
            en: "tourney",
            ja: "トーナメント",
            de: "Turnier",
            fr: "tournoi",
        },
        (25, 3018) => LocalisedText {
            en: "points",
            ja: "勝ち点",
            de: "Punkte",
            fr: "points du tournoi",
        },
        (25, 3019) => LocalisedText {
            en: "rule variations",
            ja: "特殊ルール",
            de: "Regelvarianten",
            fr: "règles spéciales",
        },
        (25, 3020) => LocalisedText {
            en: "regional rules",
            ja: "流行ルール",
            de: "Tagesregeln",
            fr: "règles locales",
        },
        (25, 3021) => LocalisedText {
            en: "match rules",
            ja: "対戦ルール",
            de: "Match-Regeln",
            fr: "règles initiales",
        },
        (25, 3022) => LocalisedText {
            en: "tournament rules",
            ja: "大会ルール",
            de: "Turnier-Regeln",
            fr: "règles du tournoi",
        },
        (25, 3023) => LocalisedText {
            en: "All Open",
            ja: "オールオープン",
            de: "Alle offen",
            fr: "Toutes sur table",
        },
        (25, 3024) => LocalisedText {
            en: "Three Open",
            ja: "スリーオープン",
            de: "Drei offen",
            fr: "Trois sur table",
        },
        (25, 3025) => LocalisedText {
            en: "Same",
            ja: "セイム",
            de: "Gleich",
            fr: "Identique",
        },
        (25, 3026) => LocalisedText {
            en: "Sudden Death",
            ja: "サドンデス",
            de: "Verlängerung",
            fr: "Mort subite",
        },
        (25, 3027) => LocalisedText {
            en: "Plus",
            ja: "プラス",
            de: "Plus",
            fr: "Plus",
        },
        (25, 3028) => LocalisedText {
            en: "Random",
            ja: "ランダムハンド",
            de: "Zufallshand",
            fr: "Aléatoire",
        },
        (25, 3029) => LocalisedText {
            en: "Order",
            ja: "オーダー",
            de: "Ordnung",
            fr: "Ordre",
        },
        (25, 3030) => LocalisedText {
            en: "Chaos",
            ja: "カオス",
            de: "Chaos",
            fr: "Chaos",
        },
        (25, 3031) => LocalisedText {
            en: "Reverse",
            ja: "リバース",
            de: "Umkehrung",
            fr: "Inversion",
        },
        (25, 3032) => LocalisedText {
            en: "Fallen Ace",
            ja: "エースキラー",
            de: "Assfresser",
            fr: "Tueur d'as",
        },
        (25, 3033) => LocalisedText {
            en: "Ascension",
            ja: "タイプアセンド",
            de: "Bonus",
            fr: "Bonus",
        },
        (25, 3034) => LocalisedText {
            en: "Descension",
            ja: "タイプディセンド",
            de: "Malus",
            fr: "Malus",
        },
        (25, 3035) => LocalisedText {
            en: "Swap",
            ja: "スワップ",
            de: "Tauschen",
            fr: "Échange",
        },
        (25, 3036) => LocalisedText {
            en: "Roulette",
            ja: "ルーレット",
            de: "Zufällig",
            fr: "Roulette",
        },
        (25, 3037) => LocalisedText {
            en: "deck",
            ja: "デッキ",
            de: "Deck",
            fr: "jeu",
        },
        (25, 3038) => LocalisedText {
            en: "desired rule",
            ja: "希望ルール：",
            de: "Gewünschte Regel",
            fr: "Règle souhaitée : ",
        },
        (25, 3039) => LocalisedText {
            en: "GATE",
            ja: "GATE",
            de: "GATE",
            fr: "JACTA",
        },
        (25, 3040) => LocalisedText {
            en: "Cactpot",
            ja: "くじテンダー",
            de: "Glückskaktor",
            fr: "Cactpot",
        },
        (25, 3041) => LocalisedText {
            en: "Jumbo Cactpot",
            ja: "ジャンボ・くじテンダー",
            de: "Jumbo-Glückskaktor",
            fr: "Méga Cactpot",
        },
        (25, 3042) => LocalisedText {
            en: "Mini Cactpot",
            ja: "ミニ・くじテンダー",
            de: "Mini-Glückskaktor",
            fr: "Mini Cactpot",
        },
        (25, 3043) => LocalisedText {
            en: "winning number",
            ja: "当選番号",
            de: "Gewinnzahl",
            fr: "numéros gagnants",
        },
        (25, 3044) => LocalisedText {
            en: "Lord of Verminion",
            ja: "ロード・オブ・ヴァーミニオン",
            de: "Kampf der Trabanten",
            fr: "Choc des mascottes",
        },
        (25, 3045) => LocalisedText {
            en: "Strengths",
            ja: "施設特効",
            de: "Stärke",
            fr: "destruction",
        },
        (25, 3046) => LocalisedText {
            en: "Arcana Stone",
            ja: "アルカナストーン",
            de: "Arkana-Stein",
            fr: "pierre arcanique",
        },
        (25, 3047) => LocalisedText {
            en: "Verminion Gate",
            ja: "ゲート",
            de: "Tor",
            fr: "portail",
        },
        (25, 3048) => LocalisedText {
            en: "Shield",
            ja: "シールド",
            de: "Schild",
            fr: "bouclier",
        },
        (25, 3049) => LocalisedText {
            en: "Search Eye",
            ja: "サーチ・アイ",
            de: "Suchendes Auge",
            fr: "œil scrutateur",
        },
        (25, 3050) => LocalisedText {
            en: "critter",
            ja: "アニマル",
            de: "Tier",
            fr: "créature",
        },
        (25, 3051) => LocalisedText {
            en: "poppet",
            ja: "ドール",
            de: "Puppe",
            fr: "poupée",
        },
        (25, 3052) => LocalisedText {
            en: "gadget",
            ja: "ガジェット",
            de: "Apparat",
            fr: "mécanisme",
        },
        (25, 3053) => LocalisedText {
            en: "ability",
            ja: "特殊技",
            de: "Technik",
            fr: "technique",
        },
        (25, 3054) => LocalisedText {
            en: "trap",
            ja: "トラップ",
            de: "Falle",
            fr: "piège",
        },
        (25, 3055) => LocalisedText {
            en: "Any Way the Wind Blows",
            ja: "暴風！はないきフンガー",
            de: "Hungaaa, Hungaaa!",
            fr: "Un sacré courant d'air",
        },
        (25, 3056) => LocalisedText {
            en: "Cliffhanger",
            ja: "救出！ミニオンレスキュー",
            de: "Rettet meinen Begleiter!",
            fr: "Mascotte en détresse",
        },
        (25, 3060) => LocalisedText {
            en: "Costa del Sol",
            ja: "コスタ・デル・ソル",
            de: "Sonnenküste",
            fr: "Costa del Sol",
        },
        (25, 3061) => LocalisedText {
            en: "monster",
            ja: "モンスター",
            de: "Monster",
            fr: "Monstre",
        },
        (25, 3062) => LocalisedText {
            en: "Leap of Faith",
            ja: "挑戦！ジャンピングアスレチック",
            de: "Kaktor-Kletterwand-Gekraxel",
            fr: "Haute voltige",
        },
        (25, 3063) => LocalisedText {
            en: "Air Force One",
            ja: "出撃！エアフォースパイロット",
            de: "Luftwaffe, Feuer frei!",
            fr: "As de l'air",
        },
        (25, 3064) => LocalisedText {
            en: "Doman Mahjong",
            ja: "ドマ式麻雀",
            de: "Domanisches Mahjong",
            fr: "mahjong domien",
        },
        (25, 3065) => LocalisedText {
            en: "The Slice Is Right",
            ja: "一閃！ 斬魔・デ・三昧 ",
            de: "Der große Yojinbo",
            fr: "La coupe est pleine",
        },
        (26, 3300) => LocalisedText {
            en: "【Statuses】",
            ja: "【状態変化】",
            de: "【Statusveränderungen】",
            fr: "【Altération d'état】",
        },
        (26, 3301) => LocalisedText {
            en: "paralysis (status)",
            ja: "麻痺",
            de: "Paralyse (Status)",
            fr: "Paralysie (altération)",
        },
        (26, 3302) => LocalisedText {
            en: "silence (status)",
            ja: "沈黙",
            de: "Stumm (Status)",
            fr: "Silence (altération)",
        },
        (26, 3303) => LocalisedText {
            en: "stun (status)",
            ja: "スタン",
            de: "Betäubung (Status)",
            fr: "Étourdissement (altération)",
        },
        (26, 3304) => LocalisedText {
            en: "sleep (status)",
            ja: "睡眠",
            de: "Schlaf (Status)",
            fr: "Sommeil (altération)",
        },
        (26, 3305) => LocalisedText {
            en: "heavy (status)",
            ja: "ヘヴィ",
            de: "Gewicht (Status)",
            fr: "Pesanteur (altération)",
        },
        (26, 3306) => LocalisedText {
            en: "petrification (status)",
            ja: "石化",
            de: "Stein (Status)",
            fr: "Pétrification (altération)",
        },
        (26, 3307) => LocalisedText {
            en: "charmed (status)",
            ja: "魅了",
            de: "Verhext (Status)",
            fr: "Envoûtement (altération)",
        },
        (26, 3308) => LocalisedText {
            en: "drain (status)",
            ja: "吸収",
            de: "Absorption (Status)",
            fr: "Absorption (altération)",
        },
        (26, 3309) => LocalisedText {
            en: "bind (status)",
            ja: "拘束",
            de: "Gefesselt (Status)",
            fr: "Attache (altération)",
        },
        (26, 3310) => LocalisedText {
            en: "transfiguration (status)",
            ja: "変身",
            de: "Verwandlung (Status)",
            fr: "Transformation (altération)",
        },
        (26, 3311) => LocalisedText {
            en: "poison (status)",
            ja: "毒",
            de: "Gift (Status)",
            fr: "Poison (altération)",
        },
        (25, 3313) => LocalisedText {
            en: "Draft",
            ja: "ドラフト",
            de: "Improvisation",
            fr: "Draft",
        },
        (25, 3314) => LocalisedText {
            en: "Open Tournament",
            ja: "オフィシャルトーナメント",
            de: "Manderville-Turnier",
            fr: "Tournoi officiel",
        },
        (8, 3500) => LocalisedText {
            en: "",
            ja: "",
            de: "",
            fr: "",
        },
        (8, 3501) => LocalisedText {
            en: "",
            ja: "",
            de: "",
            fr: "",
        },
        (8, 3502) => LocalisedText {
            en: "",
            ja: "",
            de: "",
            fr: "",
        },
        (8, 3503) => LocalisedText {
            en: "",
            ja: "",
            de: "",
            fr: "",
        },
        (8, 3504) => LocalisedText {
            en: "inner",
            ja: "内周",
            de: "innerer Umkreis",
            fr: "intérieur",
        },
        (8, 3505) => LocalisedText {
            en: "outer",
            ja: "外周",
            de: "äußerer Umkreis",
            fr: "extérieur",
        },
        (8, 3506) => LocalisedText {
            en: "clockwise",
            ja: "時計回り",
            de: "im Uhrzeigersinn",
            fr: "sens des aiguilles d'une montre",
        },
        (8, 3507) => LocalisedText {
            en: "counterclockwise",
            ja: "反時計回り",
            de: "gegen den Uhrzeigersinn",
            fr: "sens inverse des aiguilles d'une montre",
        },
        (8, 3508) => LocalisedText {
            en: "offensive stance",
            ja: "攻撃スタンス",
            de: "Schwert-Eid",
            fr: "position offensive",
        },
        (8, 3509) => LocalisedText {
            en: "defensive stance",
            ja: "防御スタンス",
            de: "Schild-Eid",
            fr: "position défensive",
        },
        (8, 3510) => LocalisedText {
            en: "action indicator",
            ja: "予兆",
            de: "Vorzeichen",
            fr: "signe précurseur",
        },
        (8, 3511) => LocalisedText {
            en: "gaze",
            ja: "視線",
            de: "Blick",
            fr: "regard",
        },
        (8, 3512) => LocalisedText {
            en: "circle",
            ja: "円形",
            de: "rund",
            fr: "en cercle",
        },
        (8, 3513) => LocalisedText {
            en: "fan",
            ja: "扇状",
            de: "fächerförmig",
            fr: "en éventail",
        },
        (8, 3514) => LocalisedText {
            en: "diagonal",
            ja: "対角線",
            de: "diagonal",
            fr: "diagonale",
        },
        (8, 3515) => LocalisedText {
            en: "straight line",
            ja: "直線",
            de: "geradeaus",
            fr: "ligne droite",
        },
        (8, 3516) => LocalisedText {
            en: "donut",
            ja: "ドーナツ範囲",
            de: "ringförmig",
            fr: "en couronne",
        },
        (8, 3517) => LocalisedText {
            en: "line",
            ja: "線",
            de: "Linie",
            fr: "ligne",
        },
        (8, 3518) => LocalisedText {
            en: "sphere",
            ja: "球",
            de: "Kugel",
            fr: "sphère",
        },
        (8, 3519) => LocalisedText {
            en: "ground",
            ja: "地面",
            de: "Boden",
            fr: "sol",
        },
        (8, 3520) => LocalisedText {
            en: "wall",
            ja: "壁",
            de: "Wand",
            fr: "mur",
        },
        (8, 3521) => LocalisedText {
            en: "pillar",
            ja: "柱",
            de: "Pfeiler",
            fr: "pilier",
        },
        (8, 3522) => LocalisedText {
            en: "boulder",
            ja: "岩",
            de: "Felsen",
            fr: "rocher",
        },
        (8, 3523) => LocalisedText {
            en: "flame",
            ja: "炎",
            de: "Feuer (Element)",
            fr: "feu (élément)",
        },
        (8, 3524) => LocalisedText {
            en: "water",
            ja: "水",
            de: "Wasser (Element)",
            fr: "eau (élément)",
        },
        (8, 3525) => LocalisedText {
            en: "ice",
            ja: "氷",
            de: "Eis (Element)",
            fr: "glace (élément)",
        },
        (8, 3526) => LocalisedText {
            en: "",
            ja: "",
            de: "",
            fr: "",
        },
        (8, 3527) => LocalisedText {
            en: "explosion",
            ja: "爆発",
            de: "Explosion",
            fr: "explosion",
        },
        (8, 3528) => LocalisedText {
            en: "simultaneously",
            ja: "同時",
            de: "zugleich",
            fr: "simultanément",
        },
        (8, 3529) => LocalisedText {
            en: "one after the other",
            ja: "交互",
            de: "abwechselnd",
            fr: "alternativement",
        },
        (8, 3531) => LocalisedText {
            en: "combine",
            ja: "合体",
            de: "verschmolzen",
            fr: "fusionnement",
        },
        (8, 3535) => LocalisedText {
            en: "gather",
            ja: "集合",
            de: "Versammeln",
            fr: "rassemblement",
        },
        (8, 3536) => LocalisedText {
            en: "spread out",
            ja: "散開",
            de: "Ausschwärmen",
            fr: "dispersion",
        },
        (8, 3540) => LocalisedText {
            en: "divided damage",
            ja: "頭割りダメージ",
            de: "geteilter Schaden",
            fr: "dégâts partagés",
        },
        (8, 3541) => LocalisedText {
            en: "reflected damage",
            ja: "反射ダメージ",
            de: "reflektierter Schaden",
            fr: "dégâts réfléchis",
        },
        (8, 3542) => LocalisedText {
            en: "damage over time",
            ja: "継続ダメージ",
            de: "Schaden über Zeit",
            fr: "dégâts périodiques",
        },
        (8, 3543) => LocalisedText {
            en: "mode change",
            ja: "モードチェンジ",
            de: "Modus-Wechsel",
            fr: "changement de mode",
        },
        (8, 3544) => LocalisedText {
            en: "phase",
            ja: "フェーズ",
            de: "Phase",
            fr: "phase",
        },
        (8, 3545) => LocalisedText {
            en: "stacked enhancement",
            ja: "スタックバフ",
            de: "gestapelter positiver Status",
            fr: "effet bénéfique cumulé",
        },
        (8, 3546) => LocalisedText {
            en: "stacked enfeeblement",
            ja: "スタックデバフ",
            de: "gestapelter negativer Status",
            fr: "effet néfaste cumulé",
        },
        (8, 3547) => LocalisedText {
            en: "random target",
            ja: "ランダムターゲット",
            de: "zufälliges Ziel",
            fr: "cible aléatoire",
        },
        (27, 3600) => LocalisedText {
            en: "【Deep Dungeon】",
            ja: "【ディープダンジョン】",
            de: "【Tiefes Gewölbe】",
            fr: "【Donjon sans fond】",
        },
        (27, 3601) => LocalisedText {
            en: "Cairn of Passage",
            ja: "転移の石塔",
            de: "Wegleuchte",
            fr: "pierre de téléportation",
        },
        (27, 3602) => LocalisedText {
            en: "Cairn of Return",
            ja: "再生の石塔",
            de: "Totenleuchte",
            fr: "pierre de résurrection",
        },
        (27, 3603) => LocalisedText {
            en: "Pomander of Safety",
            ja: "魔土器:呪印解除",
            de: "Tongefäß des Siegelbruchs",
            fr: "poterie magique de désamorçage",
        },
        (27, 3604) => LocalisedText {
            en: "Pomander of Sight",
            ja: "魔土器:サイトロ",
            de: "Tongefäß der Sicht",
            fr: "poterie magique de localisation",
        },
        (27, 3605) => LocalisedText {
            en: "Pomander of Strength",
            ja: "魔土器:自己強化",
            de: "Tongefäß der Stärkung",
            fr: "poterie magique de puissance",
        },
        (27, 3606) => LocalisedText {
            en: "Pomander of Steel",
            ja: "魔土器:防御強化",
            de: "Tongefäß der Abwehr",
            fr: "poterie magique de protection",
        },
        (27, 3607) => LocalisedText {
            en: "Pomander of Affluence",
            ja: "魔土器:宝箱増加",
            de: "Tongefäß der Schätze",
            fr: "poterie magique de décèlement",
        },
        (27, 3608) => LocalisedText {
            en: "Pomander of Flight",
            ja: "魔土器:敵排除",
            de: "Tongefäß des Feindtods",
            fr: "poterie magique de sécurisation",
        },
        (27, 3609) => LocalisedText {
            en: "Pomander of Alteration",
            ja: "魔土器:敵変化",
            de: "Tongefäß der Feindwandlung",
            fr: "poterie magique d'affaiblissement",
        },
        (27, 3610) => LocalisedText {
            en: "Pomander of Purity",
            ja: "魔土器:解呪",
            de: "Tongefäß der Entzauberung",
            fr: "poterie magique d'anti-maléfice",
        },
        (27, 3611) => LocalisedText {
            en: "Pomander of Fortune",
            ja: "魔土器:運気上昇",
            de: "Tongefäß des Glücks",
            fr: "poterie magique de chance",
        },
        (27, 3612) => LocalisedText {
            en: "Pomander of Witching",
            ja: "魔土器:形態変化",
            de: "Tongefäß der Wandlung",
            fr: "poterie magique de mutation",
        },
        (27, 3613) => LocalisedText {
            en: "Pomander of Serenity",
            ja: "魔土器:魔法効果解除",
            de: "Tongefäß der Enthexung",
            fr: "poterie magique de dissipation",
        },
        (27, 3614) => LocalisedText {
            en: "Pomander of Rage",
            ja: "魔土器:マンティコア化",
            de: "Tongefäß der Manticoren",
            fr: "poterie magique de manticore",
        },
        (27, 3615) => LocalisedText {
            en: "Pomander of Lust",
            ja: "魔土器:サキュバス化",
            de: "Tongefäß der Sukkuben",
            fr: "poterie magique de succube",
        },
        (27, 3616) => LocalisedText {
            en: "Pomander of Intuition",
            ja: "魔土器:財宝感知",
            de: "Tongefäß des Finders",
            fr: "poterie magique d'intuition",
        },
        (27, 3617) => LocalisedText {
            en: "Pomander of Raising",
            ja: "魔土器:リレイズ",
            de: "Tongefäß des Lebens",
            fr: "poterie magique de résurrection",
        },
        (27, 3618) => LocalisedText {
            en: "Pomander of Resolution",
            ja: "魔土器:クリブ化",
            de: "Tongefäß der Kuribu",
            fr: "poterie magique de Kuribu",
        },
        (27, 3619) => LocalisedText {
            en: "Pomander of Frailty",
            ja: "魔土器:敵弱体",
            de: "Tongefäß der Feindschwächung",
            fr: "poterie magique d'incapacité",
        },
        (27, 3620) => LocalisedText {
            en: "Pomander of Concealment",
            ja: "魔土器:バニシュ",
            de: "Tongefäß des Verschwindens",
            fr: "poterie magique d'invisibilité",
        },
        (27, 3621) => LocalisedText {
            en: "Pomander of Petrification",
            ja: "魔土器:敵石化",
            de: "Tongefäß der Feindversteinerung",
            fr: "poterie magique de pétrification",
        },
        (27, 3622) => LocalisedText {
            en: "Inferno magicite",
            ja: "イフリートの魔石",
            de: "Zauberstein des Ifrit",
            fr: "magilithe d'Ifrit",
        },
        (27, 3623) => LocalisedText {
            en: "Crag magicite",
            ja: "タイタンの魔石",
            de: "Zauberstein des Titan",
            fr: "magilithe de Titan",
        },
        (27, 3624) => LocalisedText {
            en: "Vortex magicite",
            ja: "ガルーダの魔石",
            de: "Zauberstein der Garuda",
            fr: "magilithe de Garuda",
        },
        (27, 3625) => LocalisedText {
            en: "elder magicite",
            ja: "オーディンの魔石",
            de: "Zauberstein des Odin",
            fr: "magilithe d'Odin",
        },
        (27, 3626) => LocalisedText {
            en: "Beacon of Passage",
            ja: "転移の灯篭",
            de: "Weglaterne",
            fr: "Lanterne de téléportation",
        },
        (27, 3627) => LocalisedText {
            en: "Beacon of Return",
            ja: "再生の灯篭",
            de: "Totenlaterne",
            fr: "Lanterne de résurrection",
        },
        (17, 4000) => LocalisedText {
            en: "magia board",
            ja: "マギアボード",
            de: "Magia-Tafel",
            fr: "plateau Magia",
        },
        (17, 4001) => LocalisedText {
            en: "magia melder",
            ja: "マギアメルダー",
            de: "Magia-Fusionator",
            fr: "répartiteur Magia",
        },
        (17, 4002) => LocalisedText {
            en: "magicite",
            ja: "魔晶石",
            de: "Maginit",
            fr: "magicite",
        },
        (17, 4003) => LocalisedText {
            en: "aspect value",
            ja: "属性値",
            de: "Elementarwert",
            fr: "affinité élémentaire",
        },
        (17, 4004) => LocalisedText {
            en: "elemental level",
            ja: "エレメンタルレベル",
            de: "Elementarstufe",
            fr: "niveau élémentaire",
        },
        (17, 4005) => LocalisedText {
            en: "logogram",
            ja: "ロゴスシャード",
            de: "Logos-Splitter",
            fr: "éclat Logos",
        },
        (17, 4007) => LocalisedText {
            en: "logos manipulator",
            ja: "ロゴスミキサー",
            de: "Logos-Manipulator",
            fr: "mélangeur Logos",
        },
        (17, 4008) => LocalisedText {
            en: "logos action",
            ja: "ロゴスアクション",
            de: "Logos-Kommando",
            fr: "action Logos",
        },
        (17, 4009) => LocalisedText {
            en: "Eureka weapon",
            ja: "エウレカウェポン",
            de: "Eureka-Waffe",
            fr: "arme Eurêka",
        },
        (17, 4011) => LocalisedText {
            en: "custom match",
            ja: "カスタムマッチ",
            de: "Schaukampf",
            fr: "Combat JcJ personnalisé",
        },
        (17, 4012) => LocalisedText {
            en: "MGP",
            ja: "MGP",
            de: "MGP",
            fr: "PGS",
        },
        (17, 4013) => LocalisedText {
            en: "venture",
            ja: "ベンチャースクリップ",
            de: "Wertmarke",
            fr: "Jeton de tâche",
        },
        (17, 4014) => LocalisedText {
            en: "custom delivery",
            ja: "お得意様取引",
            de: "Wunschlieferung",
            fr: "Commande spéciale",
        },
        (17, 4017) => LocalisedText {
            en: "Ishgardian Restoration",
            ja: "イシュガルド復興",
            de: "Wiederaufbau Ishgards",
            fr: "Reconstruction d'Ishgard",
        },
        (17, 4018) => LocalisedText {
            en: "Allagan tomestone of allegory",
            ja: "アラガントームストーン:奇譚",
            de: "Allagische Steine der Fantastik",
            fr: "mémoquartz allagois fabuleux",
        },
        (17, 4019) => LocalisedText {
            en: "ocean fishing",
            ja: "オーシャンフィッシング",
            de: "Auf großer Fahrt",
            fr: "pêche en mer",
        },
        (17, 4020) => LocalisedText {
            en: "Skysteel tool",
            ja: "スカイスチールツール",
            de: "Himmelsstahl-Werkzeug",
            fr: "outil de Cielacier",
        },
        (17, 4021) => LocalisedText {
            en: "Faux Hollows",
            ja: "ソーチョーの幻想盤",
            de: "Traumtafel",
            fr: "Touché-tourné",
        },
        (17, 4022) => LocalisedText {
            en: "lost action",
            ja: "ロストアクション",
            de: "Verschollenes Kommando",
            fr: "Actions oubliées",
        },
        (17, 4023) => LocalisedText {
            en: "lost finds holster",
            ja: "ロストホルダー",
            de: "Register für Verschollenes",
            fr: "Mémento",
        },
        (17, 4024) => LocalisedText {
            en: "lost finds cache",
            ja: "ロストボックス",
            de: "Kiste für Verschollenes",
            fr: "Malle du rappel",
        },
        (17, 4025) => LocalisedText {
            en: "forgotten fragment",
            ja: "ロストシャード",
            de: "Verschollener Splitter ",
            fr: "éclats oubliés",
        },
        (17, 4026) => LocalisedText {
            en: "mettle",
            ja: "戦果",
            de: "Frontwissen",
            fr: "points de faits d'armes",
        },
        (17, 4027) => LocalisedText {
            en: "Resistance rank",
            ja: "レジスタンスランク",
            de: "Widerstandsrang",
            fr: "rang de résistant",
        },
        (17, 4028) => LocalisedText {
            en: "Explorer Mode",
            ja: "自由探索",
            de: "Erkundung",
            fr: "mode exploration",
        },
        (17, 4029) => LocalisedText {
            en: "Allagan tomestone of revelation",
            ja: "アラガントームストーン:黙示",
            de: "Allagische Steine der Offenbarung",
            fr: "mémoquartz allagois révélateurs",
        },
        (17, 4030) => LocalisedText {
            en: "Bozjan cluster",
            ja: "ボズヤクラスター",
            de: "Bozja-Polykristall",
            fr: "cristaux bozjiens",
        },
        (17, 4031) => LocalisedText {
            en: "fashion accessories",
            ja: "ファッションアクセサリー",
            de: "Modeaccessoires",
            fr: "accessoires de mode",
        },
        (17, 4032) => LocalisedText {
            en: "Resistance weapon",
            ja: "レジスタンス・ウェポン",
            de: "Widerstandswaffe",
            fr: "armes de la résistance",
        },
        (17, 4033) => LocalisedText {
            en: "Resistance Recruitment",
            ja: "ボズヤファインダー",
            de: "Kampfübersicht (Bozja)",
            fr: "affrontement en cours",
        },
        (17, 4034) => LocalisedText {
            en: "Fête",
            ja: "Fête",
            de: "Fête",
            fr: "feste",
        },
        (17, 4035) => LocalisedText {
            en: "Allagan tomestone of aphorism",
            ja: "アラガントームストーン:経典",
            de: "Allagische Steine des Aphorismus",
            fr: "mémoquartz allagois aphoristiques",
        },
        (17, 4036) => LocalisedText {
            en: "Allagan tomestone of astronomy",
            ja: "アラガントームストーン:天文",
            de: "Allagische Steine der Astronomie",
            fr: "mémoquartz allagois astronomiques",
        },
        (17, 4037) => LocalisedText {
            en: "The Unending Codex",
            ja: "愛用の見聞録",
            de: "Reisekodex",
            fr: "Compendium",
        },
        (17, 4038) => LocalisedText {
            en: "Trophy Crystal",
            ja: "トロフィークリスタル",
            de: "Kristalltrophäen",
            fr: "cristaux-trophées",
        },
        (24, 5000) => LocalisedText {
            en: "normal raid",
            ja: "ノーマルレイド",
            de: "Normaler Raid",
            fr: "Raids normaux",
        },
        (24, 5001) => LocalisedText {
            en: "daily challenge",
            ja: "デイリーチャレンジ",
            de: "Tagesherausforderung",
            fr: "Challenges quotidiens",
        },
        (24, 5002) => LocalisedText {
            en: "The Swallow's Compass",
            ja: "ガンエン廟",
            de: "Kompass der Schwalbe",
            fr: "Compas de l'Hirondelle",
        },
        (24, 5003) => LocalisedText {
            en: "The Ridorana Lighthouse",
            ja: "封じられた聖塔 リドルアナ",
            de: "Richtfeuer von Ridorana",
            fr: "Phare de Ridorana",
        },
        (24, 5004) => LocalisedText {
            en: "Castrum Fluminis",
            ja: "ツクヨミ討滅戦",
            de: "Götterdämmerung - Tsukuyomi",
            fr: "Castrum Fluminis",
        },
        (24, 5005) => LocalisedText {
            en: "The Minstrel's Ballad: Tsukuyomi's Pain",
            ja: "極ツクヨミ討滅戦",
            de: "Zenit der Götter - Tsukuyomi",
            fr: "Castrum Fluminis (extrême)",
        },
        (24, 5006) => LocalisedText {
            en: "The Weapon's Refrain (Ultimate)",
            ja: "絶アルテマウェポン破壊作戦",
            de: "Heldenlied von Ultima (fatal)",
            fr: "Fantasmagorie d'Ultima (fatal)",
        },
        (24, 5007) => LocalisedText {
            en: "Heaven-on-High",
            ja: "アメノミハシラ",
            de: "Himmelssäule",
            fr: "Pilier des Cieux",
        },
        (24, 5008) => LocalisedText {
            en: "The Great Hunt",
            ja: "リオレウス狩猟戦",
            de: "Jagd auf Rathalos",
            fr: "Chasse au Rathalos",
        },
        (24, 5009) => LocalisedText {
            en: "The Great Hunt (Extreme)",
            ja: "極リオレウス狩猟戦",
            de: "Jagd auf Rathalos (schwer)",
            fr: "Chasse au Rathalos (extrême)",
        },
        (24, 5010) => LocalisedText {
            en: "Pagos",
            ja: "パゴス編",
            de: "Pagos",
            fr: "Eurêka Pagos",
        },
        (24, 5011) => LocalisedText {
            en: "The Sirensong Sea",
            ja: "セイレーン海",
            de: "Sirenen-See",
            fr: "Mer du Chant des sirènes",
        },
        (24, 5012) => LocalisedText {
            en: "Ala Mhigo",
            ja: "アラミゴ",
            de: "Ala Mhigo",
            fr: "Ala Mhigo",
        },
        (24, 5013) => LocalisedText {
            en: "The Minstrel's Ballad: Thordan's Reign",
            ja: "蒼天幻想ナイツ・オブ・ラウンド討滅戦",
            de: "Heldenlied von Thordans Fall",
            fr: "le règne de Thordan",
        },
        (24, 5014) => LocalisedText {
            en: "The Burn",
            ja: "ザ・バーン",
            de: "Kargland",
            fr: "Escarre",
        },
        (24, 5015) => LocalisedText {
            en: "Hells' Kier",
            ja: "朱雀征魂戦",
            de: "Seelentanz - Suzaku",
            fr: "Nid des Lamentations",
        },
        (24, 5016) => LocalisedText {
            en: "Hells' Kier (Extreme)",
            ja: "極朱雀征魂戦",
            de: "Seelensturm - Suzaku",
            fr: "Nid des Lamentations (extrême)",
        },
        (24, 5017) => LocalisedText {
            en: "Alphascape",
            ja: "アルファ編",
            de: "Alphametrie",
            fr: "Alphastice",
        },
        (24, 5018) => LocalisedText {
            en: "Pyros",
            ja: "ピューロス編",
            de: "Pyros",
            fr: "Eurêka Pyros",
        },
        (24, 5019) => LocalisedText {
            en: "Kugane Castle",
            ja: "クガネ城",
            de: "Schloss Kugane",
            fr: "Château de Kugane",
        },
        (24, 5020) => LocalisedText {
            en: "The Diadem",
            ja: "ディアデム諸島",
            de: "Diadem",
            fr: "Le Diadème",
        },
        (24, 5021) => LocalisedText {
            en: "The Ghimlyt Dark",
            ja: "ギムリトダーク",
            de: "Die Ghimlyt-Finsternis",
            fr: "Ténèbres de Ghimlyt",
        },
        (24, 5022) => LocalisedText {
            en: "The Wreath of Snakes",
            ja: "青龍征魂戦",
            de: "Seelentanz - Seiryu",
            fr: "Îlot des Amertumes",
        },
        (24, 5023) => LocalisedText {
            en: "The Wreath of Snakes (Extreme)",
            ja: "極青龍征魂戦",
            de: "Seelensturm - Seiryu",
            fr: "Îlot des Amertumes (extrême)",
        },
        (24, 5024) => LocalisedText {
            en: "The Orbonne Monastery",
            ja: "楽欲の僧院 オーボンヌ",
            de: "Kloster von Orbonne",
            fr: "Monastère d'Orbonne",
        },
        (24, 5025) => LocalisedText {
            en: "Hydatos",
            ja: "ヒュダトス編",
            de: "Hydatos",
            fr: "Eurêka Hydatos",
        },
        (24, 5026) => LocalisedText {
            en: "Hidden Gorge",
            ja: "ヒドゥンゴージ (機工戦)",
            de: "Verborgene Schlucht",
            fr: "Gorge dérobée (machinerie)",
        },
        (24, 5027) => LocalisedText {
            en: "Holminster Switch",
            ja: "ホルミンスター",
            de: "Holminster",
            fr: "Holminster",
        },
        (24, 5028) => LocalisedText {
            en: "Dohn Mheg",
            ja: "ドォーヌ・メグ",
            de: "Dohn Mheg",
            fr: "Dohn Mheg",
        },
        (24, 5029) => LocalisedText {
            en: "The Qitana Ravel",
            ja: "キタンナ神影洞",
            de: "Irrungen der Qitari",
            fr: "Enchevêtrement des Qitari",
        },
        (24, 5030) => LocalisedText {
            en: "Malikah's Well",
            ja: "マリカの大井戸",
            de: "Malikahs Brunnen",
            fr: "Puits de Malikah",
        },
        (24, 5031) => LocalisedText {
            en: "Mt. Gulg",
            ja: "グルグ火山",
            de: "Der Gulg",
            fr: "Mont Gulg",
        },
        (24, 5032) => LocalisedText {
            en: "Amaurot",
            ja: "アーモロート",
            de: "Amaurot",
            fr: "Amaurote",
        },
        (24, 5033) => LocalisedText {
            en: "The Twinning",
            ja: "シルクス・ツイニング",
            de: "Der Kristallzwilling",
            fr: "Macle de Syrcus",
        },
        (24, 5034) => LocalisedText {
            en: "Akadaemia Anyder",
            ja: "アナイダアカデミア",
            de: "Akadaemia Anyder",
            fr: "Akadaemia Anydre",
        },
        (24, 5035) => LocalisedText {
            en: "The Dancing Plague",
            ja: "ティターニア討滅戦",
            de: "Offenbarung - Titania",
            fr: "Valse du Monarque",
        },
        (24, 5036) => LocalisedText {
            en: "The Dancing Plague (Extreme)",
            ja: "極ティターニア討滅戦",
            de: "Letzte Läuterung - Titania",
            fr: "Valse du Monarque (extrême)",
        },
        (24, 5037) => LocalisedText {
            en: "The Crown of the Immaculate",
            ja: "イノセンス討滅戦",
            de: "Offenbarung - Innozenz",
            fr: "La Couronne de l'Immaculé",
        },
        (24, 5038) => LocalisedText {
            en: "The Crown of the Immaculate (Extreme)",
            ja: "極イノセンス討滅戦",
            de: "Letzte Läuterung - Innozenz",
            fr: "La Couronne de l'Immaculé (extrême)",
        },
        (24, 5039) => LocalisedText {
            en: "The Dying Gasp",
            ja: "ハーデス討滅戦",
            de: "Offenbarung - Hades",
            fr: "Râle de l'Agonie",
        },
        (24, 5040) => LocalisedText {
            en: "Eden",
            ja: "希望の園エデン",
            de: "Eden",
            fr: "Éden",
        },
        (24, 5041) => LocalisedText {
            en: "Eden (Savage)",
            ja: "希望の園エデン零式",
            de: "Eden (episch)",
            fr: "Éden (sadique)",
        },
        (24, 5042) => LocalisedText {
            en: "Eden's Gate",
            ja: "覚醒編",
            de: "Edens Erwachen",
            fr: "L'Éveil d'Éden",
        },
        (24, 5043) => LocalisedText {
            en: "Kugane Ohashi",
            ja: "真ヨウジンボウ討滅戦",
            de: "Duell auf der Kugane-Brücke",
            fr: "Le Pont Ohashi",
        },
        (24, 5044) => LocalisedText {
            en: "The Baldesion Arsenal",
            ja: "バルデシオンアーセナル",
            de: "Baldesions Arsenal",
            fr: "Arsenal de Baldesion",
        },
        (24, 5045) => LocalisedText {
            en: "The Grand Cosmos",
            ja: "グラン・コスモス",
            de: "Chateau Cosmea",
            fr: "Cosmos coruscant",
        },
        (24, 5046) => LocalisedText {
            en: "The Minstrel's Ballad: Hades's Elegy",
            ja: "極ハーデス討滅戦",
            de: "Letzte Läuterung - Hades",
            fr: "Râle de l'Agonie (extrême)",
        },
        (24, 5047) => LocalisedText {
            en: "The Epic of Alexander (Ultimate)",
            ja: "絶アレキサンダー討滅戦",
            de: "Alexander (fatal)",
            fr: "L'Odyssée d'Alexander (fatal)",
        },
        (24, 5048) => LocalisedText {
            en: "The Copied Factory",
            ja: "複製サレタ工場廃墟",
            de: "Die kopierte Fabrik",
            fr: "Réplique de l'usine désaffectée",
        },
        (24, 5049) => LocalisedText {
            en: "Onsal Hakair (Danshig Naadam)",
            ja: "オンサル・ハカイル (終節戦)",
            de: "Onsal Hakair (Danshig Naadam)",
            fr: "Onsal Hakair (Danshig Naadam)",
        },
        (24, 5050) => LocalisedText {
            en: "Anamnesis Anyder",
            ja: "アニドラス・アナムネーシス",
            de: "Anamnesis Anyder",
            fr: "Anamnesis Anydre",
        },
        (24, 5051) => LocalisedText {
            en: "Eden's Verse",
            ja: "共鳴編",
            de: "Edens Resonanz",
            fr: "Les Accords d'Éden",
        },
        (24, 5052) => LocalisedText {
            en: "Cinder Drift",
            ja: "ルビーウェポン破壊作戦",
            de: "Rubinfeuer - Entfesselung",
            fr: "Les Nuées de Brandons",
        },
        (24, 5053) => LocalisedText {
            en: "Cinder Drift (Extreme)",
            ja: "極ルビーウェポン破壊作戦",
            de: "Rubinfeuer - Trauma",
            fr: "Les Nuées de Brandons (extrême)",
        },
        (24, 5054) => LocalisedText {
            en: "Memoria Misera (Extreme)",
            ja: "極シタデル・ボズヤ追憶戦",
            de: "Memoria Misera (extrem)",
            fr: "Memoria Misera (extrême)",
        },
        (24, 5055) => LocalisedText {
            en: "Bozjan southern front",
            ja: "南方ボズヤ戦線",
            de: "Bozja-Südfront",
            fr: "Front sud de Bozja",
        },
        (24, 5056) => LocalisedText {
            en: "The Heroes' Gauntlet",
            ja: "ノルヴラント",
            de: "Schlacht um Norvrandt",
            fr: "La Traversée de Norvrandt",
        },
        (24, 5057) => LocalisedText {
            en: "The Seat of Sacrifice ",
            ja: "ウォーリア・オブ・ライト討滅戦",
            de: "Krieger des Lichts",
            fr: "Le Trône du Sacrifice",
        },
        (24, 5058) => LocalisedText {
            en: "The Seat of Sacrifice (Extreme)",
            ja: "極ウォーリア・オブ・ライト討滅戦",
            de: "Krieger des Lichts (extrem)",
            fr: "Le Trône du Sacrifice (extrême)",
        },
        (24, 5059) => LocalisedText {
            en: "The Puppets' Bunker",
            ja: "人形タチノ軍事基地",
            de: "Die Puppenfestung",
            fr: "Base militaire des Pantins",
        },
        (24, 5061) => LocalisedText {
            en: "skirmish",
            ja: "スカーミッシュ",
            de: "Scharmützel",
            fr: "escarmouche",
        },
        (24, 5062) => LocalisedText {
            en: "critical engagement",
            ja: "クリティカルエンゲージメント",
            de: "Kritisches Gefecht",
            fr: "affrontement crucial",
        },
        (24, 5063) => LocalisedText {
            en: "Save the Queen",
            ja: "セイブ・ザ・クイーン",
            de: "Die Königinnenwache",
            fr: "Garde-la-Reine",
        },
        (24, 5064) => LocalisedText {
            en: "Matoya's Relict",
            ja: "マトーヤのアトリエ",
            de: "Matoyas Atelier",
            fr: "L'Atelier abandonné de Matoya",
        },
        (24, 5065) => LocalisedText {
            en: "Eden's Promise",
            ja: "再生編",
            de: "Edens Verheißung",
            fr: "La Promesse d'Éden",
        },
        (24, 5066) => LocalisedText {
            en: "Castrum Marinum",
            ja: "エメラルドウェポン破壊作戦",
            de: "Smaragdsturm - Entfesselung",
            fr: "Castrum Marinum",
        },
        (24, 5067) => LocalisedText {
            en: "Castrum Marinum (Extreme)",
            ja: "極エメラルドウェポン破壊作戦",
            de: "Smaragdsturm - Trauma",
            fr: "Castrum Marinum (extrême)",
        },
        (24, 5068) => LocalisedText {
            en: "Delubrum Reginae",
            ja: "グンヒルド・ディルーブラム",
            de: "Delubrum Reginae",
            fr: "Delubrum Reginae",
        },
        (24, 5070) => LocalisedText {
            en: "The Battle of Castrum Lacus Litore",
            ja: "カストルム・ラクスリトレ攻城戦",
            de: "Sturm auf Castrum Lacus Litore",
            fr: "Assaut sur Castrum Lacus Litore",
        },
        (24, 5071) => LocalisedText {
            en: "Paglth'an",
            ja: "パガルザン",
            de: "Goldene Ebene von Paglth'an",
            fr: "La grande prairie de Paglth'an",
        },
        (24, 5072) => LocalisedText {
            en: "The Cloud Deck",
            ja: "ダイヤウェポン捕獲作戦",
            de: "Diamantblitz - Entfesselung",
            fr: "Tillac des Cirrus",
        },
        (24, 5073) => LocalisedText {
            en: "The Cloud Deck (Extreme)",
            ja: "極ダイヤウェポン捕獲作戦",
            de: "Diamantblitz - Trauma",
            fr: "Tillac des Cirrus (extrême)",
        },
        (24, 5074) => LocalisedText {
            en: "The Tower at Paradigm's Breach",
            ja: "希望ノ砲台：「塔」",
            de: "Der Turm, Paradigmenbrecher",
            fr: "La tour de la Contingence",
        },
        (24, 5076) => LocalisedText {
            en: "Zadnor",
            ja: "ザトゥノル高原",
            de: "Zadnor",
            fr: "Zadnor",
        },
        (24, 5077) => LocalisedText {
            en: "The Dalriada",
            ja: "旗艦ダル・リアータ攻略戦",
            de: "Sturm auf die Dalriada",
            fr: "Assaut sur le Dalriada",
        },
        (24, 5078) => LocalisedText {
            en: "The Tower of Zot",
            ja: "ゾットの塔",
            de: "Turm von Zot",
            fr: "Tour de Zott",
        },
        (24, 5079) => LocalisedText {
            en: "The Tower of Babil",
            ja: "バブイルの塔",
            de: "Turm von Babil",
            fr: "Tour de Babil",
        },
        (24, 5080) => LocalisedText {
            en: "Vanaspati",
            ja: "ヴァナスパティ",
            de: "Vanaspati",
            fr: "Vanaspati",
        },
        (24, 5081) => LocalisedText {
            en: "Ktisis Hyperboreia",
            ja: "ヒュペルボレア造物院",
            de: "Ktisis Hyperboreia",
            fr: "L'Hyperborée",
        },
        (24, 5082) => LocalisedText {
            en: "The Aitiascope",
            ja: "アイティオン星晶鏡",
            de: "Das Aitiaskop",
            fr: "Prisme de l'Aitia",
        },
        (24, 5083) => LocalisedText {
            en: "The Dead Ends",
            ja: "レムナント",
            de: "Das Sternengrab",
            fr: "L'Issue aux Impasses",
        },
        (24, 5084) => LocalisedText {
            en: "Smileton",
            ja: "スマイルトン",
            de: "Smileton",
            fr: "Risette-sur-lune",
        },
        (24, 5085) => LocalisedText {
            en: "Stigma-4",
            ja: "スティグマ・フォー",
            de: "Stigma-4",
            fr: "Rêve électrique de Stigma-4",
        },
        (24, 5086) => LocalisedText {
            en: "The Dark Inside",
            ja: "ゾディアーク討滅戦",
            de: "Prophetie - Zodiark",
            fr: "Le Cratère des Martyrs",
        },
        (24, 5087) => LocalisedText {
            en: "The Minstrel's Ballad: Zodiark's Fall",
            ja: "極ゾディアーク討滅戦",
            de: "Eschatos - Zodiark",
            fr: "Le Cratère des Martyrs (extrême)",
        },
        (24, 5088) => LocalisedText {
            en: "The Mothercrystal",
            ja: "ハイデリン討滅戦",
            de: "Prophetie - Hydaelyn",
            fr: "Le Cristal-mère",
        },
        (24, 5089) => LocalisedText {
            en: "The Minstrel's Ballad: Hydaelyn's Call",
            ja: "極ハイデリン討滅戦",
            de: "Eschatos - Hydaelyn",
            fr: "Le Cristal-mère (extrême)",
        },
        (24, 5090) => LocalisedText {
            en: "The Final Day",
            ja: "終焉の戦い",
            de: "Prophetie - Endsängerin",
            fr: "Le Répons final",
        },
        (24, 5091) => LocalisedText {
            en: "Pandæmonium",
            ja: "万魔殿パンデモニウム",
            de: "Pandæmonium",
            fr: "Pandæmonium",
        },
        (24, 5092) => LocalisedText {
            en: "Pandæmonium (Savage)",
            ja: "万魔殿パンデモニウム零式",
            de: "Pandæmonium (episch)",
            fr: "Pandæmonium (sadique)",
        },
        (24, 5093) => LocalisedText {
            en: "Asphodelos",
            ja: "辺獄編",
            de: "Asphodelos",
            fr: "Limbes du Pandæmonium",
        },
        (24, 5094) => LocalisedText {
            en: "Alzadaal's Legacy",
            ja: "アルザダール海底遺跡群",
            de: "Alzadaals Vermächtnis",
            fr: "Legs d'Alzadaal",
        },
        (24, 5095) => LocalisedText {
            en: "The Minstrel's Ballad: Endsinger's Aria",
            ja: "終極の戦い",
            de: "Eschatos - Endsängerin",
            fr: "Le Répons final (extrême)",
        },
        (24, 5096) => LocalisedText {
            en: "Ultima's Bane (Unreal)",
            ja: "幻アルテマウェポン破壊作戦",
            de: "Traumprüfung: Heldenlied von Ultima",
            fr: "Le fléau d'Ultima (irréel)",
        },
        (24, 5097) => LocalisedText {
            en: "Aglaia",
            ja: "輝ける神域 アグライア",
            de: "Aglaia",
            fr: "Domaine divin - Aglaé",
        },
        (24, 5098) => LocalisedText {
            en: "Dragonsong's Reprise (Ultimate)",
            ja: "絶竜詩戦争",
            de: "Drachenkrieg (fatal)",
            fr: "Guerre du chant des dragons (fatal)",
        },
        (24, 5099) => LocalisedText {
            en: "Crystalline Conflict",
            ja: "クリスタルコンフリクト",
            de: "Crystalline Conflict",
            fr: "Crystalline Conflict",
        },
        (24, 5100) => LocalisedText {
            en: "The Palaistra",
            ja: "パライストラ",
            de: "Die Palästra",
            fr: "Palestre",
        },
        (24, 5101) => LocalisedText {
            en: "The Volcanic Heart",
            ja: "ヴォルカニック・ハート",
            de: "Das Herz des Vulkans",
            fr: "Cœur volcanique",
        },
        (24, 5102) => LocalisedText {
            en: "Cloud Nine",
            ja: "クラウドナイン",
            de: "Wolke Sieben",
            fr: "Petit Nuage",
        },
    };
}
